#ifndef __FLASH_SPI_NAND_LIST_H__
#define __FLASH_SPI_NAND_LIST_H__


#define GD5F1GQ4UAYIG 0xc8f1c8f1
#define GD5F2GQ4UAYIG 0xc8f2c8f2
#define GD5F1GQ4UBYIG 0xc8d1c8d1

#define MT29F1G01AAADD 0x2c122c12
#define AFD01GWS		0xDDA0DDA0
#define AFD02GWS		0xDDA1DDA1
#define HYF1GQ4UAACAR_C 0xC831C831
#define HYF1GQ4UAACAR	0xC151C151	 //0xC151C151
#define A5U12A21ASC 	0xc8207f7f	 //0x7f7f20c8 0xc8207f7f
#define F50L1G41A		0xc8217f7f	 //0x7f7f21c8 0xc8217f7f
#define W25N01GVZE1G	0xefaa2100
#define MX35LF1GE4AB	0xc212c212
#define TC58CVG1S3HAI	0x98c298cB
#define TC58CVG2S3HAI	0x98cb98cb
#define GD5F2GQ4UBYIG	0xc8d2c8d2


/* ECC bit select (per 512 bytes) */
//#define SPI_N_ECC_6BIT		0x0
//#define SPI_N_ECC_12BIT		0x1
//#define SPI_N_ECC_24BIT		0x2	//


/* Position of initial bad block mark */
#define INITIAL_BB_POS_FIRST	0x0	// in the first page of initial bad block
#define INITIAL_BB_POS_LAST	0xff	// in the last  page of initial bad block
#define INITIAL_BB_POS_BOTH	(INITIAL_BB_POS_FIRST | INITIAL_BB_POS_LAST)	// in both first and last page of initial bad block

typedef enum {
	CACHE_1X,
	CACHE_2X,
	CACHE_4X,
	CACHE_4X_IO,
} cache_mode;

typedef struct /*__attribute__ ((__packed__))*/{
	unsigned int	id;
	unsigned int	PageSize;
	unsigned int	BlockSize;
	unsigned short	oobSize;
	unsigned int	size;	
	unsigned int	initial_bb_pos;
	unsigned int	row_addr_mask;
	unsigned int	bad_tag_offset;
	unsigned int	magic_tag_offset;
    cache_mode      support_cache_mode;
    unsigned int    cache_bit_offset;
	char			*name;
} spi_n_device_type;

static const spi_n_device_type spi_n_device[] =
{
	{GD5F1GQ4UAYIG, 	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"GD5F1GQ4UAYIG" } , 
	{GD5F2GQ4UAYIG, 	   2048, 64*2048, 64,0x10000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"GD5F2GQ4UAYIG" } , 
	{GD5F1GQ4UBYIG, 	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X_IO,0,"GD5F1GQ4UBYIG" } , 
	{MT29F1G01AAADD,	   2048, 64*2048, 64,0x10000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"MT29F1G01AAADD" } ,
	{AFD01GWS,			   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"AFD01GWS"	  } ,	  
	{AFD02GWS,			   2048, 64*2048, 64,0x10000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"AFD02GWS"	  } ,		  
	{HYF1GQ4UAACAR_C,	   2048, 128*2048,64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"HYF1GQ4UAACAR-C"} ,	   
	{HYF1GQ4UAACAR, 	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_1X,0,"HYF1GQ4UAACAR"} ,
	{A5U12A21ASC,		   2048, 64*2048, 64,0x04000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X,0,"A5U12A21ASC" } , 
	{F50L1G41A, 		   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x808,CACHE_4X,0xF,"F50L1G41A" } , 
	{W25N01GVZE1G , 	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X_IO,0,"W25N01GVZE1G" } ,
	{MX35LF1GE4AB , 	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X,0,"MX35LF1GE4AB" } ,
	{TC58CVG1S3HAI ,	   2048, 64*2048, 64,0x08000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X,1,"TC58CVG1S3HAI" } ,
	{TC58CVG2S3HAI, 	   2048, 64*2048, 64,0x10000000, INITIAL_BB_POS_FIRST, 0x7ff,0x800,0x800,CACHE_4X,1,"TC58CVG2S3HAI" } ,
	{GD5F2GQ4UBYIG, 	   2048, 64*2048, 64,0x10000000, INITIAL_BB_POS_FIRST, 0x3ff,0x800,0x800,CACHE_4X_IO,0,"GD5F2GQ4UBYIG" } , 
}; 

#define DEV_SIZE_SPI_N	(sizeof(spi_n_device)/sizeof(spi_n_device_type))

/************************************************************************
 *  define prototypes
 ************************************************************************/
#define SPI_NAND_PROFILE_IDENTIFIER	"REALTEK."		// same as nand solution

typedef struct{
	// identify & revision info
	unsigned char	identifier[8];		    /* byte   0 ~ 7  */	/* identifier */
	unsigned int	revision;		        /* byte   8 ~ 11 */	/* revision */

	// flash info
	unsigned char	manufacturer_id[8];	/* byte  12 ~ 19 */	/* manufacturer id */
	unsigned char   flash_name[20];		    /* byte  20 ~ 39 */	/* flash name */
	unsigned int	flash_size0;			/* byte  40 ~ 43 */	/* flash size 0~31 */
	unsigned int	flash_size1;			/* byte  44 ~ 47 */	/* flash size 32~63 */
	unsigned int	data_per_page;			/* byte  48 ~ 51 */	/* page size */
	unsigned int	oob_per_page;			/* byte  52 ~ 55 */	/* spare size */
	unsigned int	page_per_block;			/* byte  56 ~ 59 */	/* page per block */
	unsigned int	block_per_die;			/* byte  60 ~ 63 */	/* block per die */

	// tag position info
	unsigned int	bb_tag_offset;			/* byte  64 ~ 67 */	/* bad block tag offset */
	unsigned int	special_tag_offset;		/* byte  68 ~ 71 */	/* special block tag offset; eg: hwsetting-0x23 */
	unsigned int	initial_bb_pos;			/* byte  72 ~ 75 */	/* initial bad block tag position*/

	// address mask info
	unsigned int	ra_block_mask;			/* byte  76 ~ 79 */	/* row address <block sector> mask value */
	unsigned int	ra_page_mask;			/* byte  80 ~ 83 */	/* row address <page sector> mask value */
	unsigned int	ca_mask;				/* byte  84 ~ 87 */	/* column address mask value */

	// use plane or not
	unsigned char	use_plane;				/* byte  88 */		/* use plane or not */
	unsigned char	reserved_0[3];			/* byte  89 ~ 91 */	/* reserved*/

	// bbt info sector
	unsigned int	number_of_bbt;		    /* byte  92 ~ 95 */	/* number of bbt */
	unsigned int	bbt_start_block;	    /* byte  96 ~ 99 */	/* bbt start block */
	unsigned int	bbt_end_block;		    /* byte  100 ~ 103 */	/* bbt end block */
	unsigned int	number_of_rsv_block;	/* byte  104 ~ 107 */	/* number of reserved block */
	unsigned int	rsv_start_block;	    /* byte  108 ~ 111 */	/* reserved start block */
	unsigned int	rsv_end_block;		    /* byte  112 ~ 115 */	/* reserved end block */

	// ibt info sector
	unsigned int	number_of_ibt;			/* byte  116 ~ 119 */	/* number of ibt */
	unsigned int	ibt_start_block;		/* byte  120 ~ 123 */	/* ibt start block*/
	unsigned int	ibt_end_block;			/* byte  124 ~ 127 */	/* ibt end block*/
	unsigned char	ibt_buffer_percent[32];	/* byte  128 ~ 159 */	/* ibt buffer percent, can support 32 partition*/

	//cache  mode support
	unsigned int	support_cache_mode; 	/* byte  160 ~ 163 */	/* cache mode*/
	unsigned int	cache_bit_offset;		/* byte  164 ~ 167 */	/*cache bit offset*/
	
	// resvered for future feature
	unsigned char	reserved_1[56]; 		/* byte  168 ~ 223 */	/* reserverd for future feature*/

	
	//checksum
	unsigned char   checksum[32];		    /* byte 224 ~ 255 */
} __attribute__((packed)) t_spi_nand_profile;



#endif /* __FLASH_SPI_NAND_LIST_H__ */
