#ifndef _SE_CMD_H_
#define _SE_CMD_H_
typedef struct _SECMD_WRITE_REGISTER
{
    struct
    {
        uint32_t	OpCode: 4;
        uint32_t	RegisterAddress: 12;
        uint32_t	Clut: 1;
        uint32_t	Reserved: 15;
    } Word1;
    struct
    {
        uint32_t	RegisterValue;
    } Word2;
} SECMD_WRITE_REGISTER;
typedef struct _SECMD_STRETCH
{
    struct
    {
        uint32_t	OpCode: 4;
        uint32_t	OutputX: 12;
        uint32_t	OutputY: 12;
        uint32_t	ColorKey: 1;
        uint32_t	HScalingEnable: 1;
        uint32_t	VScalingEnable: 1;
        uint32_t	SrcInterleaving: 1;
    } Word1;
    struct
    {
		uint32_t	OutputWidth: 12;
		uint32_t	OutputHeight: 12;
		uint32_t	StretchBitblit: 1;
		uint32_t	directFB: 1;
		uint32_t    RopCode1: 2;
		uint32_t	Reserved: 2;
		uint32_t	ScalingAlgorithm: 2;
    } Word2;
    struct
    {
		uint32_t	InputX: 12;
		uint32_t	InputY: 12;
		uint32_t	DestInterleaving: 1;
		uint32_t	SrcAddressIndex: 7;
    } Word3;
    struct
    {
		uint32_t	InputWidth: 13;
		uint32_t	InputHeight: 12;
		uint32_t	DestAddressIndex: 7;
    } Word4;
	struct
	{
        uint32_t	fb_sel_destclr_blend: 4;
        uint32_t    fb_sel_srcclr_blend: 4;
        uint32_t    fb_demultiply: 1;
        uint32_t    fb_blend_alphachannel: 1;
        uint32_t    fb_blend_coloralpha: 1;
        uint32_t    fb_colorize: 1;
        uint32_t    fb_xor: 1;
        uint32_t    fb_src_premultiply: 1;
        uint32_t    fb_src_premulcolor: 1;
        uint32_t    fb_dst_premultiply: 1;
        uint32_t    src_clr_transparency: 1;
        uint32_t    src_clr_transform: 2;
		uint32_t	rop_mode: 1;
        uint32_t    bitblit_color_key: 2;
        uint32_t    sel_src_alpha: 3;
        uint32_t    rop_code: 2;
        uint32_t    sel_output_alpha: 1;
        uint32_t    sync_vo: 1;
        uint32_t    sel_dest_alpha: 3;
	} Word5;

	struct
	{
		uint32_t	Reserved;
	} Word6;
#if 0
	struct
	{
		uint32_t	Reserved;					//IO[31:0]: Reserved
	} Word7;
	struct
	{
		uint32_t	Reserved;					//IO[31:0]: Reserved
	} Word8;
#endif

} SECMD_STRETCH;
typedef struct _SECMD_BITBLT
{
    struct
    {
        uint32_t	OpCode: 4;
        uint32_t	Width: 12;
        uint32_t	Height: 12;
      	uint32_t	InputSelect: 1;
		uint32_t	SelectDestAlpha: 3;
    } Word1;
    struct
    {
        uint32_t	OutputX: 12;
        uint32_t	OutputY: 12;
		uint32_t	StartingCorner: 2;
		uint32_t	RopCode: 2;
		uint32_t	SelectOutputAlpha: 1;
		uint32_t	sync_vo: 1;
		uint32_t	DestAddressIndex: 2;
    } Word2;
    struct
    {
        uint32_t	InputX: 12;
        uint32_t	InputY: 12;
		uint32_t	RopMode: 1;
		uint32_t	ColorKey: 2;
		uint32_t	SelectSrcAlpha: 3;
		uint32_t	SrcAddressIndex: 2;
    } Word3;
    struct
    {
        uint32_t	Reserved: 12;
        uint32_t	fb_sel_destclr_blend: 4;
        uint32_t    fb_sel_srcclr_blend: 4;
        uint32_t    fb_demultiply: 1;
        uint32_t    fb_blend_alphachannel: 1;
        uint32_t    fb_blend_coloralpha: 1;
        uint32_t    fb_colorize: 1;
        uint32_t    fb_xor: 1;
        uint32_t    fb_src_premultiply: 1;
        uint32_t    fb_src_premulcolor: 1;
        uint32_t    fb_dst_premultiply: 1;
        uint32_t    directFB: 1;
        uint32_t    src_clr_transparency: 1;
        uint32_t    src_clr_transform: 2;
    } Word4;

} SECMD_BITBLT;

//=================================================================================================
typedef union _SEREG_PITCH
{
    uint32_t  Value;
    struct
    {
        //Register: PITCH1 (0x1800_C0E0)
        uint32_t  pitch: 16;                //IO[15:0]: Pitch of a SE address set, unit in byte. 0 means 65536 bytes. 16B-align.
        uint32_t  interleave: 1;            //IO[16]: interleaving mode for the picture index.
        uint32_t  Reserved: 15;             //IO[31:17]: Reserved

    } Fields;

} SEREG_PITCH;

//=================================================================================================


//=================================================================================================
typedef union _SEREG_CLR_FMT
{
    uint32_t  Value;
    struct
    {
        uint32_t  Format: 4;
        uint32_t  write_enable1: 1;
        uint32_t  alpha_loc: 1;
        uint32_t  write_enable2: 1;
        uint32_t  Reserved1: 6;
        uint32_t  alpha_loc2: 1;
        uint32_t  write_enable5: 1;
        uint32_t  big_endian_i1: 1;
        uint32_t  write_enable6: 1;
        uint32_t  big_endian_i2: 1;
        uint32_t  write_enable7: 1;
        uint32_t  big_endian_o: 1;
        uint32_t  write_enable8: 1;
        uint32_t  rounding_en: 1;
        uint32_t  write_enable10: 1;
        uint32_t  Reserved2: 9;

    } Fields;

} SEREG_CLR_FMT;
#define UINT32 uint32_t
typedef struct _SECMD_EXT_FMT_CONV
{
	struct
	{
		UINT32	opcode: 4;
		UINT32	rslt_fmt: 5;
		UINT32	lay_ctl: 1;
		UINT32	lay_num: 1;
		UINT32	reserved0: 1;
		UINT32	rslt_swp_chr: 2;
		UINT32	src_swp_chr: 2;
		UINT32	itu_en: 1;
		UINT32	reserved1: 1;
		UINT32	rslt_matx_en: 1;
		UINT32	out_alpha: 1;
		UINT32	rslt_argb_order: 5;
		UINT32	reserved2: 1;
		UINT32	src2_argb_order: 5;
		UINT32	reserved3: 1;
	} Word1;
	struct
	{
		UINT32	rslt_out_x: 13;
		UINT32	rslt_out_y: 13;
		UINT32	reserved0: 2;
		UINT32  rslt_rotate: 2;
		UINT32  rslt_mirror: 2;
	} Word2;
	struct
	{
		UINT32	rslt_chr2_addr_index: 7;
		UINT32	reserved0: 1;
		UINT32	rslt_chr1_addr_index: 7;
		UINT32	reserved1: 1;
		UINT32	rslt_addr_index: 7;
		UINT32	rslt_blk_mode_interleave: 1;
		UINT32	reserved2: 8;
	} Word3;
	struct
	{
		UINT32	src1_inpt_w: 13;
		UINT32	src1_inpt_h: 13;
		UINT32	src1_fmt: 5;
		UINT32	src1_mat_en: 1;
	} Word4;
	struct
	{
		UINT32	src1_inpt_x: 13;
		UINT32	src1_inpt_y: 13;
		UINT32	src1_argb_order: 5;
		UINT32	src1_fcv_stch: 1;
	} Word5;
	struct
	{
		UINT32	src1_chr2_addr_index: 7;
		UINT32	reserved0: 1;
		UINT32	src1_chr1_addr_index: 7;
		UINT32	reserved1: 1;
		UINT32	src1_addr_index: 7;
		UINT32	src1_blk_mode_interleave: 1;
		UINT32	src1_inpt_sel: 1;
		UINT32	src1_lg_en: 1;
		UINT32	reserved2: 6;
	} Word6;
	struct
	{
		UINT32	Reserved: 32;
	} Word7;
	struct
	{
		UINT32	Reserved: 32;
	} Word8;
} SECMD_EXT_FMT_CONV;


typedef struct _SECMD_EXT_STCH_BLIT
{
	struct
	{
		UINT32	opcode: 4;
		UINT32	rslt_fmt: 5;
		UINT32	lay_ctl: 1;
		UINT32	lay_num: 1;
		UINT32	reserved0: 1;
		UINT32	rslt_swp_chr: 2;
		UINT32	src_swp_chr: 2;
		UINT32	itu_en: 1;
		UINT32	reserved1: 1;
		UINT32	rslt_matx_en: 1;
		UINT32	out_alpha: 1;
		UINT32	rslt_argb_order: 5;
		UINT32	reserved2: 1;
		UINT32	src2_argb_order: 5;
		UINT32	reserved3: 1;
	} Word1;
	struct
	{
		UINT32	rslt_out_x: 13;
		UINT32	rslt_out_y: 13;
		UINT32	reserved0: 2;
		UINT32  rslt_rotate: 2;
		UINT32  rslt_mirror: 2;
	} Word2;
	struct
	{
		UINT32	rslt_chr2_addr_index: 7;
		UINT32	reserved0: 1;
		UINT32	rslt_chr1_addr_index: 7;
		UINT32	reserved1: 1;
		UINT32	rslt_addr_index: 7;
		UINT32	rslt_blk_mode_interleave: 1;
		UINT32	reserved2: 8;
	} Word3;
	struct
	{
		UINT32	src1_inpt_w: 13;
		UINT32	src1_inpt_h: 13;
		UINT32	src1_fmt: 5;
		UINT32	src1_mat_en: 1;
	} Word4;
	struct
	{
		UINT32	src1_inpt_x: 13;
		UINT32	src1_inpt_y: 13;
		UINT32	src1_argb_order: 5;
		UINT32	src1_fcv_stch: 1;
	} Word5;
	struct
	{
		UINT32	src1_chr2_addr_index: 7;
		UINT32	reserved0: 1;
		UINT32	src1_chr1_addr_index: 7;
		UINT32	reserved1: 1;
		UINT32	src1_addr_index: 7;
		UINT32	src1_blk_mode_interleave: 1;
		UINT32	src1_inpt_sel: 1;
		UINT32	src1_lg_en: 1;
		UINT32	reserved2: 6;
	} Word6;
	struct
	{
		UINT32	src2_inpt_x: 13;
		UINT32	src2_inpt_y: 13;
		UINT32	src2_fmt: 5;
		UINT32	src2_mat_en: 1;
	} Word7;
	struct
	{
		UINT32	src2_chr2_addr_index: 7;
		UINT32	reserved0: 1;
		UINT32	src2_chr1_addr_index: 7;
		UINT32	reserved1: 1;
		UINT32	src2_addr_index: 7;
		UINT32	src2_blk_mode_interleave: 1;
		UINT32	src2_inpt_sel: 1;
		UINT32	src2_swp_chr: 2;
		UINT32	src2_clip_10b: 1;
		UINT32	reserved2: 4;
	} Word8;
	struct
	{
		UINT32	sel_s2_alpha: 3;
		UINT32	reserved0: 1;
		UINT32	sel_s1_alpha: 3;
		UINT32	reserved1: 1;
		UINT32	lay_num_ext: 3;
		UINT32	reserved2: 21;
	} Word9;
	struct
	{
		UINT32	rop_code_1: 4;
		UINT32	fb_sel_destclr_blend_1: 4;
		UINT32	fb_sel_srcclr_blend_1: 4;
		UINT32	fb_demultiply_1: 1;
		UINT32	fb_blend_alphachannel_1: 1;
		UINT32	fb_blend_coloralpha_1: 1;
		UINT32	fb_colorize_1: 1;
		UINT32	fb_xor_1: 1;
		UINT32	fb_src_premultiply_1: 1;
		UINT32	fb_src_premulcolor_1: 1;
		UINT32	fb_dst_premultiply_1: 1;
		UINT32	src_clr_transparency_1: 1;
		UINT32	src_clr_transform_1: 2;
		UINT32	reserved0: 2;
		UINT32	color_key_1: 2;
		UINT32	blend_sel_1: 4;
		UINT32	reserved1: 1;
	} Word10;
	struct
	{
		UINT32	gl_sel_destclr_1: 6;
		UINT32	reserved0: 2;
		UINT32	gl_sel_srctclr_1: 6;
		UINT32	reserved1: 2;
		UINT32	gl_sel_destalpha_1: 6;
		UINT32	reserved2: 2;
		UINT32	gl_sel_srctalpha_1: 6;
		UINT32	reserved3: 2;
	} Word11;
	struct
	{
		UINT32	stch1_out_w: 13;
		UINT32	stch1_out_h: 13;
		UINT32	stch1_src: 3;
		UINT32	stch1_h_en: 1;
		UINT32	stch1_v_en: 1;
		UINT32	stch1_algo: 1;
	} Word12;
	struct
	{
		UINT32	src2_inpt_w: 13;
		UINT32	src2_inpt_h: 13;
		UINT32	Reserved: 6;
	} Word13;
	struct
	{
		UINT32	src2_out_x: 13;
		UINT32	src2_out_y: 13;
		UINT32	Reserved: 6;
	} Word14;
	struct
	{
		UINT32	stch2_out_w: 13;
		UINT32	stch2_out_h: 13;
		UINT32	stch2_src: 3;
		UINT32	stch2_h_en: 1;
		UINT32	stch2_v_en: 1;
		UINT32	stch2_algo: 1;
	} Word15;
	struct
	{
		UINT32	Reserved: 32;
	} Word16;
} SECMD_EXT_STCH_BLIT;


#endif
