/*
 * (C) Copyright 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/system.h>
#include <asm/arch/io.h>
#ifdef CONFIG_QC_VERIFY
#include "qc_verify_def.h"
#endif

#if !(defined(CONFIG_SYS_ICACHE_OFF) && defined(CONFIG_SYS_DCACHE_OFF))

#if defined(CONFIG_SYS_ARM_CACHE_WRITETHROUGH)
#define CACHE_SETUP	0x1a
#else
#define CACHE_SETUP	0x1e
#endif
#ifdef CONFIG_SYS_SUPPORT_MULTICORE
#define TLB_SCHARED (1 << 16)
#endif
#define SUPER_SECTION    (1 << 18)

DECLARE_GLOBAL_DATA_PTR;

extern int smp_processor_id(unsigned int mpidr);
extern void v7_flush_louis_dcache_all(void);

void __arm_init_before_mmu(void)
{
}
void arm_init_before_mmu(void)
	__attribute__((weak, alias("__arm_init_before_mmu")));

static void cp_delay (void)
{
	volatile int i;

	/* copro seems to need some delay between reading and writing */
	for (i = 0; i < 100; i++)
		nop();
	asm volatile("" : : : "memory");
}

static inline void dram_bank_mmu_setup(int bank)
{
	u32 *page_table = (u32 *)gd->tlb_addr;
	bd_t *bd = gd->bd;
	u32	i;
	u32 page_address;
	u32 page_attr;

	debug("%s: bank: %d\n", __func__, bank);
#ifdef CONFIG_MAX_POWER_MEASURE
	for (i = bd->bi_dram[bank].start >> 20; i < 4096;i++) {
#else
	for (i = bd->bi_dram[bank].start >> 20;
	     i < (bd->bi_dram[bank].start + bd->bi_dram[bank].size) >> 20;
	     i++) {
#endif
		page_address = i << 20;
		page_attr = (i << 20) | (3 << 10);
		// rbus and spi region needs a device memory type
		if (((RBUS_BASE_VIRT) <= page_address ) && page_address < (RBUS_BASE_VIRT+RBUS_BASE_SIZE)) {
                        page_attr |= 0x16;
		}
#ifdef CONFIG_NX_ATTR_FOR_OPTEE_REGION
		else if( (OPTEE_BASE_VIRT <= page_address) && (page_address < (OPTEE_BASE_VIRT + OPTEE_BASE_SIZE) ) )
		{
			page_attr |= 0x10;
		}
#endif
#if 0 // uncache the 1st 1-MB addressable space in SPI
		else if (i == (SPI_FLASH_BASE_VIRT >> 20)) {
			page_attr |= 0x16;
		}
#else // uncahce all SPI addressable space in SPI
		else if( ( (SPI_FLASH_BASE_VIRT) <= page_address ) && page_address < (SPI_FLASH_BASE_VIRT+SPI_FLASH_BASE_SIZE) ) {
			page_attr |= 0x16;
		}
#endif
#ifdef CONFIG_NONCACHE_ALLOCATION
		else if (i == (gd->noncache_addr >> 20)) {
			page_attr |= 0x16;
		}
#endif
#ifdef CONFIG_QC_VERIFY
#if CONFIG_QC_GPU
		else if ((page_address >= QC_GPU_TEST_BASE) && (page_address < (QC_GPU_TEST_BASE + QC_GPU_TEST_SIZE))) {
			page_attr |= 0x16;
		}
#endif
#if CONFIG_QC_VE
/*		else if ((page_address >= QC_VE_TEST_BASE) && (page_address < (QC_VE_TEST_BASE + QC_VE_TEST_SIZE))) {
			page_attr |= 0x16;
		}
*/
#endif
#endif
#ifndef CONFIG_JUMP_TO_64BIT_KERNEL
		else if((page_address >=GICV3_BASE)&&(page_address <(GICV3_BASE + GICV3_SIZE)))
		{
			page_attr |= SUPER_SECTION |GICV3_PHYSICAL| 0x16;
		}
#endif
		else {
#ifndef CONFIG_SYS_SUPPORT_MULTICORE
			page_attr |= CACHE_SETUP;
#else
			page_attr |= TLB_SCHARED | CACHE_SETUP;
#endif
		}

		page_table[i] = page_attr;
	}
}

/* to activate the MMU we need to set up virtual memory: use 1M areas */
static inline void mmu_setup(void)
{
	u32 *page_table = (u32 *)gd->tlb_addr;
	int i;
	u32 reg;

	arm_init_before_mmu();
	/* Set up an identity-mapping for all 4GB, rw for everyone */
	for (i = 0; i < 4096; i++) {
		page_table[i] = i << 20 | (3 << 10) | 0x12;
	}

	for (i = 0; i < CONFIG_NR_DRAM_BANKS; i++) {
		dram_bank_mmu_setup(i);
	}

	/* Copy the page table address to cp15 */
	asm volatile("mcr p15, 0, %0, c2, c0, 0"
		: : "r" (page_table) : "memory");
	/* Set the access control to all-supervisor */
	asm volatile("mcr p15, 0, %0, c3, c0, 0"
		: : "r" (~0));
	/* and enable the mmu */
	reg = get_cr();	/* get control reg. */
	cp_delay();
	set_cr(reg | CR_M);
	asm volatile("ISB": : );
}

#ifdef CONFIG_SYS_SUPPORT_MULTICORE
/* to activate the MMU we need to set up virtual memory: use 1M areas */
static inline void mmu_setup_coreX(void)
{
     u32 reg;
    u32 *page_table = (u32 *)gd->tlb_addr;
   /* Copy the page table address to cp15 */
   asm volatile("mcr p15, 0, %0, c2, c0, 0"
          : : "r" (page_table) : "memory");
   /* Set the access control to all-supervisor */
   asm volatile("mcr p15, 0, %0, c3, c0, 0"
          : : "r" (~0));
   /* and enable the mmu */
   reg = get_cr();   /* get control reg. */
   cp_delay();
   set_cr(reg | CR_M);

   asm volatile("ISB": : );
}
#endif //CONFIG_SYS_SUPPORT_MULTICORE

static int mmu_enabled(void)
{
	return get_cr() & CR_M;
}

/* cache_bit must be either CR_I or CR_C */
static void cache_enable(uint32_t cache_bit)
{
	uint32_t reg;

#if defined(CONFIG_ARM_CA7_MPCORE) || defined(CONFIG_ARM_CA17_MPCORE)
	// ACTLR.SMP must be set before caches and MMU are enabled
	// the L1/L2 data caches are disabled when ACTLR.SMP = 0 regardless of the SCTLR.C bit
	reg = get_cr1(); /* get CP15 ACTLR */
	cp_delay();
	set_cr1(reg | (1 << 6)); /* set ACTLR.SMP bit */
#endif

	reg = get_cr();	/* get control reg. */
	if (reg & CR_C) {
		/* flush L1 dcache all */
		v7_flush_louis_dcache_all();
	}
	else {
		/* invalid L1 dcache all, before RE-enable dcache to join SMP. */
		v7_invalid_louis_dcache_all();
	}

	/* The data cache is not active unless the mmu is enabled too */
	if ((cache_bit == CR_C) && !mmu_enabled()) {

#ifndef CONFIG_LESS_CONSOLE
		puts("To setup mmu...\n");
#endif
		#ifdef CONFIG_SYS_SUPPORT_MULTICORE 
			if (smp_processor_id(get_MPIDR()) == 0)
				mmu_setup();
			else
				mmu_setup_coreX();
		#else
			mmu_setup();
		#endif
	}
	reg = get_cr();	/* get control reg. */
	cp_delay();
	set_cr(reg | cache_bit);
}

/* cache_bit must be either CR_I or CR_C */
static void cache_disable(uint32_t cache_bit)
{
	uint32_t reg;

	reg = get_cr();
	cp_delay();

	if (cache_bit == CR_C) {
		/* if cache isn;t enabled no need to disable */
		if ((reg & CR_C) != CR_C)
			return;
		/* if disabling data cache, disable mmu too */
		//cache_bit |= CR_M;
		//flush_dcache_all();
	}
	set_cr(reg & ~cache_bit);
	flush_dcache_all(); //  after cache disabled, to prevent more data cache allocations.

	
#if 1 // new style for ARMv7
	asm volatile("ISB": : );
#endif

}
#endif

#ifdef CONFIG_SYS_ICACHE_OFF
void icache_enable (void)
{
	return;
}

void icache_disable (void)
{
	return;
}

int icache_status (void)
{
	return 0;					/* always off */
}
#else
void icache_enable(void)
{
	cache_enable(CR_I);
}

void icache_disable(void)
{
	cache_disable(CR_I);
}

int icache_status(void)
{
	return (get_cr() & CR_I) != 0;
}
#endif

#ifdef CONFIG_SYS_DCACHE_OFF
void dcache_enable (void)
{
	return;
}

void dcache_disable (void)
{
	return;
}

int dcache_status (void)
{
	return 0;					/* always off */
}
#elif defined(CONFIG_ARCH_CPU_ARMA53) || defined(CONFIG_ARCH_CPU_ARMA57) || defined(CONFIG_ARCH_CPU_ARMA55)
void __asm_flush_dcache_all(void);
void flush_dcache_all_a5x(void)
{
	__asm_flush_dcache_all();
}

void dcache_enable(void)
{
	int mpidr = get_MPIDR();
	
	cache_enable(CR_C);

	printf("Cache: Enabled, cpu(0x%x)\n", mpidr & 0x0ffffff);  //puts protect is enable only after spinlock working on C/M initialized.

#ifdef CONFIG_NONCACHE_ALLOCATION
	printf("Non-Cache Region: %d MB@0x%08lx, cpu(0x%x)\n", CONFIG_SYS_NON_CACHE_LEN >> 20, gd->noncache_addr, mpidr & 0x0ffffff);
#endif
	
}

void dcache_disable_flush_all(void)
{
	asm volatile(

		/* disable C */
		"mrc	p15, 0, r3, c1, c0, 0\n"
		"bic	r3, r3, #4\n"
		"mcr	p15, 0, r3, c1, c0, 0\n" ::: "r3");
	
	asm volatile(
		/* clean_and_inval all dcache by set/way */
		"dmb\n" //@ ensure ordering with previous memory accesses
		"mrc	 p15, 1, r0, c0, c0, 1\n" //	       @ read clidr
		"mov	 r3, r0, lsr #23\n" //		       @ left align loc bit field
		"ands	 r3, r3, #(0x7<<1)\n" //	       @ extract loc from clidr
		"beq	 finished\n" // 		       @ if loc is 0, then no need to clean
		
		"mov	 r10, #0\n" //			       @ start clean at cache level 0
		
		"flush_levels:\n" //
		"add	 r2, r10, r10, lsr #1\n" //	       @ work out 3x current cache level
		"mov	 r1, r0, lsr r2\n" //		       @ extract cache type bits from clidr
		"and	 r1, r1, #7\n" //		       @ mask of the bits for current cache only
		"cmp	 r1, #2\n" //		       @ see what cache we have at this level
		"blt	 skip\n" //		       @ skip if no cache, or just i-cache

		"mcr	 p15, 2, r10, c0, c0, 0\n" //	       @ select current cache level in cssr
		"isb\n" //				       @ isb to sych the new cssr&csidr
		"mrc	 p15, 1, r1, c0, c0, 0\n" //	       @ read the new csidr

		"and	 r2, r1, #7\n" //		       @ extract the length of the cache lines
		"add	 r2, r2, #4\n" //		       @ add 4 (line length offset)
		"ldr	 r4, =0x400\n" //
		"sub	 r4, r4, #1\n" //
		"ands	 r4, r4, r1, lsr #3\n" //	       @ find maximum number on the way size
		"clz	 r5, r4\n" //		       @ find bit position of way size increment
		"ldr	 r8, =0x8000\n" //
		"sub	 r8, r8, #1\n" //

		"ands	 r8, r8, r1, lsr #13\n" //	       @ extract max number of the index size
		"loop1:\n" //
		"mov	 r9, r8\n" //		       @ create working copy of max way size
		"loop2:\n" //
		"lsl	r6, r4, r5 \n\t"
		"orr	r11, r10, r6 \n\t" //	@ factor way and cache number into r11
		"lsl	r6, r9, r2 \n\t" 
		"orr	r11, r11, r6 \n\t" //	@ factor index number into r11
		"mcr	 p15, 0, r11, c7, c14, 2\n" //	       @ clean & invalidate by set/way
		"subs	 r9, r9, #1\n" //		       @ decrement the way
		"bge	 loop2\n" //
		"subs	 r4, r4, #1\n" //		       @ decrement the index
		"bge	 loop1\n" //
		"skip:\n" //
		"add	 r10, r10, #2\n" //		       @ increment cache number
		"cmp	 r3, r10\n" //
		"bgt	 flush_levels\n" //
		"finished:\n" //
		"mov	 r10, #0\n" //			       @ swith back to cache level 0
		"mcr	 p15, 2, r10, c0, c0, 0\n" //	       @ select current cache level in cssr
		"dsb\n" //
		"isb\n"
		"mov	 r0, #0\n"
		"mcr	 p15, 0, r0, c7, c1, 0\n"

		"isb\n"
		"dsb sy"

		::: "r0", "r1", "r2", "r3", "r4", "r5", "r6", "r8", "r9", "r10","r11");
}



void dcache_disable(void)
{
#if 1 // temp mark for bringup

//	cache_disable(CR_C);

        dcache_disable_flush_all();

#else
	u32 reg;

	reg = get_cr();

	/* if cache isn't enabled no need to disable */
	if (!(reg & CR_C))
		return;

	// FIXME, disable cache first then flush. 
	asm volatile(
		"mvn	r4, #5\n\t"
		"and	%0, %0, r4\n\t"
		"mcr p15, 0, %0, c1, c0, 0 @ set CR"
		: : "r" (reg) : "r4", "cc");
	isb();
	dsb();

	flush_dcache_all_a5x();

#endif
}

int dcache_status(void)
{
	return (get_cr() & CR_C) != 0;
}
#else
void dcache_enable(void)
{
	cache_enable(CR_C);

	puts("Cache: Enabled\n");   //puts protect is enable only after spinlock working on C/M initialized.
	
#ifdef CONFIG_NONCACHE_ALLOCATION
	printf("Non-Cache Region: %d MB@0x%08lx\n", CONFIG_SYS_NON_CACHE_LEN >> 20, gd->noncache_addr);
#endif

}

void dcache_disable(void)
{
	cache_disable(CR_C);
}

int dcache_status(void)
{
	return (get_cr() & CR_C) != 0;
}
#endif
