/*
 * (C) Copyright 2003
 * Texas Instruments <www.ti.com>
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Alex Zuepke <azu@sysgo.de>
 *
 * (C) Copyright 2002-2004
 * Gary Jennejohn, DENX Software Engineering, <garyj@denx.de>
 *
 * (C) Copyright 2004
 * Philippe Robin, ARM Ltd. <philippe.robin@arm.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/proc-armv/ptrace.h>
#include <asm/arch/io.h>
#ifdef CONFIG_USE_IRQ
#include <asm/interrupt.h>

#define GIC_BASE                                          0x18080000
#define GICD_OFFSET                                    0x1000
#define GICC_OFFSET                                    0x2000

#define GICC_BASE                                        (GIC_BASE + GICC_OFFSET)
#define GIC_CPU_CTRL                                  0x00
#define GIC_CPU_PRIMASK                            0x04
#define GIC_CPU_BINPOINT                          0x08
#define GIC_CPU_INTACK	                            0x0c
#define GIC_CPU_EOI                                    0x10
#define GIC_CPU_RUNNINGPRI                      0x14
#define GIC_CPU_HIGHPRI	                            0x18

#define GICD_BASE                                        (GIC_BASE + GICD_OFFSET)
#define GIC_DIST_CTRL                                 0x000
#define GIC_DIST_CTR                                   0x004
#define GIC_DIST_IGROUP	                            0x080
#define GIC_DIST_ENABLE_SET                     0x100
#define GIC_DIST_ENABLE_CLEAR	              0x180
#define GIC_DIST_PENDING_SET	              0x200
#define GIC_DIST_PENDING_CLEAR              0x280
#define GIC_DIST_ACTIVE_SET                     0x300
#define GIC_DIST_ACTIVE_CLEAR                 0x380
#define GIC_DIST_PRI                                   0x400
#define GIC_DIST_TARGET                            0x800
#define GIC_DIST_CONFIG	                            0xc00
#define GIC_DIST_SOFTINT                           0xf00


static irq_func_t irq_func[MAX_IRQ];
static mhal_ipi_t ipi_func[MAX_IPI];
static unsigned int ipi_func_param[MAX_IPI];
#endif
DECLARE_GLOBAL_DATA_PTR;
extern void enable_console(void);

#ifdef CONFIG_USE_IRQ
int interrupt_init (void)
{
	/*
	 * setup up stacks if necessary
	 */
	IRQ_STACK_START = gd->irq_sp - 4;
#ifdef CONFIG_SYS_IRQ_STACK_START_IN_ADDR
	IRQ_STACK_START_IN = CONFIG_SYS_IRQ_STACK_START_IN_ADDR;
#else
	IRQ_STACK_START_IN = gd->irq_sp + 8;
#endif
	FIQ_STACK_START = IRQ_STACK_START - CONFIG_STACKSIZE_IRQ;

	return arch_interrupt_init();
}

/* enable IRQ interrupts */
void enable_interrupts (void)
{
	unsigned long temp;
	__asm__ __volatile__("mrs %0, cpsr\n"
			     "bic %0, %0, #0x80\n"
			     "msr cpsr_c, %0"
			     : "=r" (temp)
			     :
			     : "memory");
}


/*
 * disable IRQ/FIQ interrupts
 * returns true if interrupts had been enabled before we disabled them
 */
int disable_interrupts (void)
{
	unsigned long old,temp;
	__asm__ __volatile__("mrs %0, cpsr\n"
			     "orr %1, %0, #0xc0\n"
			     "msr cpsr_c, %1"
			     : "=r" (old), "=r" (temp)
			     :
			     : "memory");
	return (old & 0x80) == 0;
}

void RtOS_EnableInterrupt(InterruptNum irq_num)
{
	unsigned int offset, irq_bit;

	offset = (irq_num >> 5) * 4;
	irq_bit = irq_num % 32;

	rtd_outl(GICD_BASE + GIC_DIST_ENABLE_SET + offset, 1 << irq_bit);
}

void RtOS_DisableInterrupt(InterruptNum irq_num)
{
	unsigned int offset, irq_bit;

	offset = (irq_num >> 5) * 4;
	irq_bit = irq_num % 32;

	rtd_outl(GICD_BASE + GIC_DIST_ENABLE_CLEAR + offset, 1 << irq_bit);
}

void RtOS_AttachInterrupt(InterruptNum irq_num, irq_func_t func)
{
	if (irq_num < MAX_IRQ)
		irq_func[irq_num] = func;
	else
		printf("%s:[ERROR] irq_num(%d) >= MAX_IRQ(%d)\n", __FUNCTION__, irq_num, MAX_IRQ);
}

void RtOS_DetachInterrupt(InterruptNum irq_num)
{
	if (irq_num < MAX_IRQ)
		irq_func[irq_num] = NULL;
	else
		printf("%s:[ERROR] irq_num(%d) >= MAX_IRQ(%d)\n", __FUNCTION__, irq_num, MAX_IRQ);
}

static u8 rtk_interrupt_gic_get_cpumask(void)
{
	u32 base = GICD_BASE;
	u32 mask, i;

	for (i = mask = 0; i < 32; i += 4) {
		mask = rtd_inl(base + GIC_DIST_TARGET + i);
		mask |= mask >> 16;
		mask |= mask >> 8;
		if (mask)
			break;
	}

	return mask;
}

void rtk_interrupt_gic_dist_init(void)
{
	unsigned int i,j;
	u32 cpumask;
	u32 base = GICD_BASE;
	unsigned int gic_irqs = 96;

	rtd_outl(base + GIC_DIST_CTRL, 0x0);

	/*
	 * Set all global interrupts to be level triggered, active low.
	 */
	for (i = 32; i < gic_irqs; i += 16)
		rtd_outl(base + GIC_DIST_CONFIG + i * 4 / 16, 0x0);

	/*
	 * Set all global interrupts to this CPU only.
	 */
	cpumask = rtk_interrupt_gic_get_cpumask();
	cpumask |= cpumask << 8;
	cpumask |= cpumask << 16;
       for(j=0; j < 4;j++)
       {
            cpumask = cpumask <<j;
	     for (i = 32; i < gic_irqs; i += 4) {
			rtd_outl(base + GIC_DIST_TARGET + i * 4 / 4, cpumask);
	}
       }
	/*
	 * Set priority on all global interrupts.
	 */
	for (i = 32; i < gic_irqs; i += 4)
		rtd_outl(base + GIC_DIST_PRI + i * 4 / 4, 0xa0a0a0a0);

	/*
	 * Disable all interrupts.  Leave the PPI and SGIs alone
	 * as these enables are banked registers.
	 */
	for (i = 32; i < gic_irqs; i += 32)
		rtd_outl(base + GIC_DIST_ENABLE_CLEAR + i * 4 / 32, 0xffffffff);

	rtd_outl(base + GIC_DIST_CTRL, 0x1);
}

void rtk_interrupt_gic_cpu_init(void)
{
	u32 dist_base = GICD_BASE;
	u32 base = GICC_BASE;
	int i;

	rtd_outl(base + GIC_CPU_CTRL, 0x0);

	/*
	 * Deal with the banked PPI and SGI interrupts - disable all
	 * PPI interrupts, ensure all SGI interrupts are enabled.
	 */
	rtd_outl(dist_base + GIC_DIST_ENABLE_CLEAR, 0xffff0000);
	rtd_outl(dist_base + GIC_DIST_ENABLE_SET, 0x0000ffff);

	/*
	 * Set priority on PPI and SGI interrupts
	 */
	for (i = 0; i < 32; i += 4)
		rtd_outl(dist_base + GIC_DIST_PRI + i * 4 / 4, 0xa0a0a0a0);

	rtd_outl(base + GIC_CPU_PRIMASK, 0xf0);
	rtd_outl(base + GIC_CPU_CTRL, 0x1);
}

int arch_interrupt_init(void)
{
	int i = 0;

	// Clear irq func handler
	for (i = 0;i < MAX_IRQ;i++)
	{
		irq_func[i] = NULL;
	}

	rtk_interrupt_gic_dist_init();
	rtk_interrupt_gic_cpu_init();

	return 0;
}
void handle_IRQ(unsigned int irqnr)
{
	// call irq handler
	if (irq_func[irqnr] != NULL)
		irq_func[irqnr]();

	// EOI irq
//	rtk_interrupt_gic_eoi_irq(irqnr);
}

void handle_IPI(unsigned int irqnr)
{
	// call ipi handler
	if (ipi_func[irqnr] != NULL)
		ipi_func[irqnr](irqnr);//(ipi_func_param[irqnr]);
}

void interrupt_do_irq (unsigned int irqnr)
{
//	unsigned int irqstat, cpuid;

//	irqstat = rtd_inl(GICC_BASE + GIC_CPU_INTACK);
//	irqnr = irqstat & ~0x1c00;

	if (irqnr > 15 && irqnr < 1021) { // SPI, PPI
	//	printf("irqnr:%d\n", irqnr);
		handle_IRQ(irqnr);
		return;
	}

	if (irqnr < 16) { // SGI
//		cpuid = irqstat & ~0x3ff;
//		printf("SGI cpuid:%d irqnr:%d\n", cpuid, irqnr);
//		rtk_interrupt_gic_eoi_irq(irqstat);
		handle_IPI(irqnr);
		return;
	}

}

#else
int interrupt_init (void)
{
	/*
	 * setup up stacks if necessary
	 */
#ifdef CONFIG_SYS_IRQ_STACK_START_IN_ADDR
	IRQ_STACK_START_IN = CONFIG_SYS_IRQ_STACK_START_IN_ADDR;
#else
	IRQ_STACK_START_IN = gd->irq_sp + 8;
#endif

	return 0;
}

void enable_interrupts (void)
{
	return;
}
int disable_interrupts (void)
{
	return 0;
}
#endif

spin_lock_t g_exception_lock = INIT_SPIN_LOCK;


void bad_mode (void)
{
	panic ("Resetting CPU ...\n");
	reset_cpu (0);
}

void show_regs (struct pt_regs *regs)
{
	unsigned long flags;
	const char *processor_modes[] = {
	"USER_26",	"FIQ_26",	"IRQ_26",	"SVC_26",
	"UK4_26",	"UK5_26",	"UK6_26",	"UK7_26",
	"UK8_26",	"UK9_26",	"UK10_26",	"UK11_26",
	"UK12_26",	"UK13_26",	"UK14_26",	"UK15_26",
	"USER_32",	"FIQ_32",	"IRQ_32",	"SVC_32",
	"UK4_32",	"UK5_32",	"UK6_32",	"ABT_32",
	"UK8_32",	"UK9_32",	"UK10_32",	"UND_32",
	"UK12_32",	"UK13_32",	"UK14_32",	"SYS_32",
	};
	unsigned int regValue;

	flags = condition_codes (regs);

	printf ("pc : [<%08lx>]	   lr : [<%08lx>]\n"
		"sp : %08lx  ip : %08lx	 fp : %08lx\n",
		instruction_pointer (regs),
		regs->ARM_lr, regs->ARM_sp, regs->ARM_ip, regs->ARM_fp);
	printf ("r10: %08lx  r9 : %08lx	 r8 : %08lx\n",
		regs->ARM_r10, regs->ARM_r9, regs->ARM_r8);
	printf ("r7 : %08lx  r6 : %08lx	 r5 : %08lx  r4 : %08lx\n",
		regs->ARM_r7, regs->ARM_r6, regs->ARM_r5, regs->ARM_r4);
	printf ("r3 : %08lx  r2 : %08lx	 r1 : %08lx  r0 : %08lx\n",
		regs->ARM_r3, regs->ARM_r2, regs->ARM_r1, regs->ARM_r0);
	printf ("Flags: %c%c%c%c",
		flags & CC_N_BIT ? 'N' : 'n',
		flags & CC_Z_BIT ? 'Z' : 'z',
		flags & CC_C_BIT ? 'C' : 'c', flags & CC_V_BIT ? 'V' : 'v');
	printf ("  IRQs %s  FIQs %s  Mode %s%s\n",
		interrupts_enabled (regs) ? "on" : "off",
		fast_interrupts_enabled (regs) ? "on" : "off",
		processor_modes[processor_mode (regs)],
		thumb_mode (regs) ? " (T)" : "");

	/* Read DFAR*/
	asm volatile ("mrc p15, 0, %0, c6, c0, 0" : "=r" (regValue));
	printf ("DFAR: %08x\n", regValue);

	/* Read DFSR*/
	asm volatile ("mrc p15, 0, %0, c5, c0, 0" : "=r" (regValue));
	printf ("DFSR: %08x\n", regValue);

	/* Read IFAR*/
	asm volatile ("mrc p15, 0, %0, c6, c0, 2" : "=r" (regValue));
	printf ("IFAR: %08x\n", regValue);

	/* Read IFSR*/
	asm volatile ("mrc p15, 0, %0, c5, c0, 1" : "=r" (regValue));
	printf ("IFSR: %08x\n", regValue);

}


#define HWSEMA (0xb801A620)
inline int hwsema_trylock(void)
{
	return (rtd_inl(HWSEMA) == 1);
}

inline void hwsema_lock(unsigned long ra)
{
	int tmp = 0;
	while(1) {
		if (hwsema_trylock() == 1) {
			break;
		}
		
		asm volatile (
			"\n\t"
			"mov %0, %1 \n\t"
			"1: subs %0, #1\n\t"
			"bne 1b \n\t"
			::"r"(tmp), "I"(100));
	}
}

inline void hwsema_unlock(unsigned long ra)
{
	rtd_outl(HWSEMA, 0);
}


#if 1
#define smp_printf(fmt...)					\
	({							\
		int ret = 0;					\
		hwsema_lock(get_MPIDR());			\
		ret = printf(fmt);				\
		hwsema_unlock(get_MPIDR());			\
		ret;						\
	})
#else
#define smp_printf(fmt...)    printf(fmt)
#endif

void do_undefined_instruction (struct pt_regs *pt_regs)
{
	//int flags __attribute__((unused));
	enable_console();
	//spin_lock_save(&g_exception_lock, flags);
	hwsema_lock(get_MPIDR());
	printf ("cpu(%x) undefined instruction\n", get_MPIDR());
	show_regs (pt_regs);
	//spin_unlock_restore(&g_exception_lock, flags);
	hwsema_unlock(get_MPIDR());
	bad_mode ();
}

void do_software_interrupt (struct pt_regs *pt_regs)
{
	//int flags;
	
	//spin_lock_save(&g_exception_lock, flags);
	hwsema_lock(get_MPIDR());
	printf ("software interrupt\n");
	show_regs (pt_regs);
	//spin_unlock_restore(&g_exception_lock, flags);
	hwsema_unlock(get_MPIDR());
	bad_mode ();
}

void do_prefetch_abort (struct pt_regs *pt_regs)
{
	//int flags;
	
	enable_console();
	//spin_lock_save(&g_exception_lock, flags);
	hwsema_lock(get_MPIDR());             
	printf ("cpu(%x) prefetch abort\n", get_MPIDR());
	show_regs (pt_regs);
	//spin_unlock_restore(&g_exception_lock, flags);
	hwsema_unlock(get_MPIDR()); 
	bad_mode ();
}

extern void rtk_rbus_timeout_show(void);
void do_data_abort (struct pt_regs *pt_regs)
{
	//int flags;
	
	enable_console();
#ifdef CONFIG_CMD_HW_MONITOR
	extern void ArmWrapper_DataAbort_Check (struct pt_regs *pt_regs);
	extern void RBUS_Timeout_Time_Check (void);

        ArmWrapper_DataAbort_Check(pt_regs);
        RBUS_Timeout_Time_Check();
#endif
	//spin_lock_save(&g_exception_lock, flags);
	hwsema_lock(get_MPIDR());      
	printf ("cpu(%x) data abort\n", get_MPIDR());

	rtk_rbus_timeout_show();

	show_regs (pt_regs);
	//spin_unlock_restore(&g_exception_lock, flags);
	hwsema_unlock(get_MPIDR());

	bad_mode ();
}

void do_not_used (struct pt_regs *pt_regs)
{
	int flags;

	spin_lock_save(&g_exception_lock, flags);
	//hwsema_lock(get_MPIDR());
	printf ("not used\n");
	show_regs (pt_regs);
	spin_unlock_restore(&g_exception_lock, flags);
	//hwsema_unlock(get_MPIDR());
	bad_mode ();
}

void do_fiq (struct pt_regs *pt_regs)
{
	printf ("fast interrupt request\n");
	show_regs (pt_regs);
	bad_mode ();
}

#ifndef CONFIG_USE_IRQ
void do_irq (struct pt_regs *pt_regs)
{
	printf ("interrupt request\n");
	show_regs (pt_regs);
	bad_mode ();
}
#else
void do_irq (unsigned int irqnr)
{
	interrupt_do_irq(irqnr);
}
#endif

#ifdef CONFIG_USE_IRQ
void mhal_ipi_attach(unsigned int ipi_num, mhal_ipi_t isr, unsigned int dat)
{
    if (ipi_num < MAX_IPI)
    {
        ipi_func[ipi_num] = isr;
        ipi_func_param[ipi_num] = dat;
    }
    else
        printf("%s:[ERROR] irq_num(%d) >= MAX_IRQ(%d)\n", __FUNCTION__, ipi_num, MAX_IPI);
}

void mhal_ipi_detach(unsigned int ipi_num)
{
    if (ipi_num < MAX_IPI)
        ipi_func[ipi_num] = NULL;
    else
        printf("%s:[ERROR] irq_num(%d) >= MAX_IRQ(%d)\n", __FUNCTION__, ipi_num, MAX_IPI);
}

void mhal_ipi_mask(unsigned int ipi_num)
{

}

void mhal_ipi_unmask(unsigned int ipi_num)
{

}

void mhal_ipi_send(unsigned int cpu_mask, unsigned int ipi_num)
{
    unsigned int regValue = 0;
    if (cpu_mask > 0xF)
    {
        return;
    }
    if (ipi_num > 0xF)
    {
        return;
    }
    regValue = (0x0 << 24) | ((cpu_mask & 0x0F) << 16) | (ipi_num & 0x0F); // send to the specific cpu
    rtd_outl(GICD_BASE + GIC_DIST_SOFTINT, regValue);
}

void mhal_ipi_broadcast(unsigned int ipi_num)
{
    unsigned int regValue = 0;
    if (ipi_num > 0xF)
    {
        return;
    }
    //	regValue = (0x0 << 24) | (0x0F << 16) | (ipi_num & 0x0F); // send to all cpus
    regValue = (0x1 << 24) | (0x0F << 16) | (ipi_num & 0x0F); // send to all cpus except myself
    rtd_outl(GICD_BASE + GIC_DIST_SOFTINT, regValue);
}

void mhal_ipi_send_target(unsigned int cpu_id, unsigned int ipi_num)
{
    unsigned int regValue = 0;
    if (cpu_id > 0x3)
    {
        return;
    }
    if (ipi_num > 0xF)
    {
        return;
    }
    regValue = (0x0 << 24) | ((1 << cpu_id) << 16) | (ipi_num & 0x0F); // send to the specific cpu
    rtd_outl(GICD_BASE + GIC_DIST_SOFTINT, regValue);
}

void mhal_ipi_send_self(unsigned int ipi_num)
{
    unsigned int regValue = 0;
    if (ipi_num > 0xF)
    {
        return;
    }
    regValue = (0x2 << 24) | (ipi_num & 0x0F); // send to myself
    rtd_outl(GICD_BASE + GIC_DIST_SOFTINT, regValue);
}
#endif
#ifdef TEST_TIMER_INTERRUPT
u32 test_timer_id = 0;
static void test_timer_callback(unsigned int stTimer, unsigned int u32Data)
{
    static int count = 0;
    printf("test oneshot %d count\n", count++);
}
#endif

