/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2012 by C.K. Lai <cklai@realtek.com>
 *
 */

#include <common.h>
#include <command.h>
#include <coproc.h>
#include <asm/system.h>
#include <asm/arch/io.h>
#include <asm/interrupt.h>

#if CONFIG_ARM_MAX_POWER  //fixme 
DECLARE_GLOBAL_DATA_PTR;
extern int ca7_max_power(int, int);
#endif

#if CONFIG_MAX_POWER_MEASURE
int main_maxpower(void);
int maxpower(int data);
#endif //#ifdef CONFIG_MAX_POWER_MEASURE

#if CONFIG_QC_SCPU_PWR_CTRL
extern void cpu_core_pwr_dwn(int val);
#endif
extern int scu_test(void);
extern int mem_test(vu_long *start, vu_long *end, int iteration_limit, vu_long pattern);

#ifdef CONFIG_USE_IRQ
extern void ipi_send_target(unsigned int cpu_id, InterruptNum ipi_num);
#endif

unsigned int cop_mtest_corebits = 1;
unsigned int cop_mtest_base = 0x10000000;
unsigned int cop_mtest_size = 0x00200000;
unsigned int cop_mtest_end  = 0x10200000;
unsigned int cop_mtest_pattern = 0x5a5a55aa;
unsigned int cop_mtest_iteration = 0x10;
unsigned int cop_mtest_dcache_en = 1;
unsigned int cop_mtest_log_en = 1;
#define HWSEMA (0xb801A620)
volatile int swsema = 0; 
inline int hwsema_trylock(void)
{
	return (rtd_inl(HWSEMA) == 1);
}

inline void hwsema_lock(unsigned long ra)
{
	int tmp = 0;
	while(1) {
		if (hwsema_trylock() == 1) {
			break;
		}
		
		asm volatile (
			"\n\t"
			"mov %0, %1 \n\t"
			"1: subs %0, #1\n\t"
			"bne 1b \n\t"
			::"r"(tmp), "I"(100));
	}
}

inline void hwsema_unlock(unsigned long ra)
{
	rtd_outl(HWSEMA, 0);
}


#if 1
#define smp_printf(fmt...)					\
	({							\
		int ret = 0;					\
		hwsema_lock(get_MPIDR());			\
		ret = printf(fmt);				\
		hwsema_unlock(get_MPIDR());			\
		ret;						\
	})
#else
#define smp_printf(fmt...)    printf(fmt)
#endif


/*
 * function executed by core 1~3
 */
void coproc_putc(char c)
{
	while ((rtd_inl(0xb8062314) & _BIT5) == 0)
		;
	rtd_outl(0xb8062300, c);
}

/*
 * function executed by core 1~3
 */
int coproc_handler(void)
{
	unsigned int cpu_id = 0;
	unsigned int regBase = 0;
	unsigned int regValue = 0;
	int ret = 0;

	cpu_id = smp_processor_id(get_MPIDR());
#if  1
	if (cpu_id >= NR_CPUS)
		return 0;
	regBase = MAGIC_TAG_CORE0_reg + (4 * cpu_id);
#else
	switch(cpu_id) {
		case 1: regBase = MAGIC_TAG_CORE1_reg; break;
		case 2: regBase = MAGIC_TAG_CORE2_reg; break;
		case 3: regBase = MAGIC_TAG_CORE3_reg; break;
		case 4: regBase = MAGIC_TAG_CORE4_reg; break;
		default: return 0;
	}
#endif
	console_protect_set(1);
	
//	asm volatile("mov r8, %0" : : "r" (_irq_stack_start) : "cc");	// for multi-core use printf issue

#ifndef CONFIG_SYS_DCACHE_OFF	
	//flush_dcache_all();  //flush cache ? anyway, flush under cache disabled is more rational for ARM.
	enable_caches();    // this function also enable M 
#endif //+#ifndef CONFIG_SYS_DCACHE_OFF

	rtd_outl(regBase, MAGIC_COPROC_OP_GET_READY);

	
	while (1) {
		/* if smp/cache on, then regbase should be sharable and coreherent.
		 * otherwise, noncacheable access will also get fresh data.
		 */
#ifndef CONFIG_SYS_DCACHE_OFF		
		invalidate_dcache_range(MAGIC_TAG_CORE0_reg, MAGIC_TAG_CORE0_reg + (4 * (NR_CPUS + 1)));
#endif //#ifndef CONFIG_SYS_DCACHE_OFF
		regValue = rtd_inl(regBase);

		
		if (regValue == MAGIC_COPROC_OP_GET_READY)
		{
#ifdef CONFIG_USE_IRQ
			asm volatile("WFI");
#else
			asm volatile("WFE");
#endif
			continue;
		}

		console_protect_set(1);
		switch (regValue) {
			case MAGIC_COPROC_OP_SCU_TEST:
				coproc_putc(0x24 + cpu_id); // print [%&'] to debug
#ifdef CONFIG_CMD_SCUTEST
				scu_test();
#endif
				break;

			case MAGIC_COPROC_OP_MEM_TEST:
				//coproc_putc(0x27 + cpu_id); // print [()*] to debug

				{
					int core_idx = 0, i;
					for (i=0; i<cpu_id; i++) 
						core_idx += (cop_mtest_corebits & (1<<i)) ? 1 : 0;
					if(cop_mtest_log_en)
					{
						smp_printf("=> Core %d do mtest %x %x %x pat:%x\n",
							cpu_id,
							(cop_mtest_base + (core_idx) * cop_mtest_size),
							(cop_mtest_base + (core_idx+1) * cop_mtest_size),
							cop_mtest_iteration,
							cop_mtest_pattern);
					}
					
#ifndef CONFIG_SYS_DCACHE_OFF
				if (cop_mtest_dcache_en != 0)
					dcache_enable();
				else 
					dcache_disable();
#endif
	
					ret = mem_test((vu_long *)(cop_mtest_base + (core_idx) * cop_mtest_size),
						       (vu_long *)(cop_mtest_base + (core_idx+1) * cop_mtest_size),
						       cop_mtest_iteration,
						       cop_mtest_pattern);
#ifndef CONFIG_SYS_DCACHE_OFF
					dcache_enable();
#endif
				}
				// notify the result to core 0
				if (ret == 0) {
					rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_OK);
				}
				else {
					rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_FAIL);
				}

				// wait for core 0 got mtest result
				while(rtd_inl(regBase) != MAGIC_COPROC_MEM_TEST_DONE)
					;

				break;
#if CONFIG_MAX_POWER_MEASURE
			case MAGIC_COPROC_OP_MAX_POWER_TEST:
                       maxpower(cpu_id);

                   break;
#endif
#ifdef CONFIG_ARM_MAX_POWER
			case MAGIC_COPROC_OP_MAX_POWER_TEST:
			#ifndef CONFIG_SYS_ICACHE_OFF
   			icache_disable();
			#endif
			#ifndef CONFIG_SYS_DCACHE_OFF
   			dcache_disable();
			#endif

			ret = ca7_max_power(gd->ram_size + CONFIG_SYS_SDRAM_BASE, 0);
			if (ret == 0) {
				rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_OK);
         }
			else {
				rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_FAIL);
			}

			#ifndef CONFIG_SYS_ICACHE_OFF
   			icache_enable();
			#endif
			#ifndef CONFIG_SYS_DCACHE_OFF
        		dcache_enable();
			#endif

				break;
#endif	//CONFIG_ARM_MAX_POWER
#if CONFIG_QC_SCPU_PWR_CTRL
			case MAGIC_COPROC_OP_POWER_CTRL:
				cpu_id = smp_processor_id(get_MPIDR());
				cpu_core_pwr_dwn(cpu_id);
			break;
#endif
			default:
				break;
		}

		rtd_outl(regBase, MAGIC_COPROC_OP_GET_READY);
	}

	return 0;
}

int do_coprocessor_start(unsigned int coproc_bitmap)
{
	return smp_boot_secondary(coproc_bitmap);
}

int do_coprocessor_status(unsigned int coproc_bitmap)
{
	unsigned int regValue = 0;
	unsigned int regBase = 0;
	int i = 0;

	for (i = 1; i < NR_CPUS; i++)
	{
#if 1
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0 << i )) {
			continue;
		}
		else {
			regBase = MAGIC_TAG_CORE0_reg + (4 *  i);
		}
#else
		// decide which core
		if (i == 1) {
			if ((coproc_bitmap & _BIT1) != _BIT1)
				continue;
			else
				regBase = MAGIC_TAG_CORE1_reg;
		}
		else if (i == 2) {
			if ((coproc_bitmap & _BIT2) != _BIT2)
				continue;
			else
				regBase = MAGIC_TAG_CORE2_reg;
		}
		else if (i == 3) {
			if ((coproc_bitmap & _BIT3) != _BIT3)
				continue;
			else
				regBase = MAGIC_TAG_CORE3_reg;
		}
		else if (i == 4) {
			if ((coproc_bitmap & _BIT4) != _BIT4)
				continue;
			else
				regBase = MAGIC_TAG_CORE4_reg;
		}
#endif
		// get coreX current status
		regValue = rtd_inl(regBase);

		switch (regValue) {
			case MAGIC_COPROC_IN_REDIRECT:
				printf("Core %d: loop in redirect\n", i);
				break;

			case MAGIC_COPROC_OP_GET_READY:
				printf("Core %d: loop in uboot\n", i);
				break;

			case MAGIC_COPROC_OP_SCU_TEST:
				printf("Core %d: in scu test\n", i);
				break;

			case MAGIC_COPROC_OP_MEM_TEST:
				printf("Core %d: in mem test\n", i);
				break;
			case MAGIC_COPROC_NOT_SUPPORT:
				printf("Core %d not support \n",i);
				break;
			default:
				printf("Core %d: unknown status: 0x%x\n", i, regValue);
				break;
		}	
	}
	return 0;
}

#ifdef CONFIG_CMD_SCUTEST
int do_coprocessor_scu_test(unsigned int coproc_bitmap)
{
	//unsigned int regValue = 0;
	unsigned int regBase = 0;
	int i = 0;
	//unsigned int bit_i = 0;

	for (i = 1; i < NR_CPUS; i++)
	{
#if 0
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0 << i)) {
			continue;
		}
		else {
			regBase = MAGIC_TAG_CORE0_reg + (4 * i);
		}
#else
		// decide which core
		if (i == 1) {
			if ((coproc_bitmap & _BIT1) != _BIT1)
				continue;
			else {
				regBase = MAGIC_TAG_CORE1_reg;
				//bit_i = _BIT1;
			}
	
		}
		else if (i == 2)
 		{
			if ((coproc_bitmap & _BIT2) != _BIT2)
				continue;
			else {
				regBase = MAGIC_TAG_CORE2_reg;
				//bit_i = _BIT2;
			}
		}
		else if (i == 3) {
			if ((coproc_bitmap & _BIT3) != _BIT3)
				continue;
			else {
				regBase = MAGIC_TAG_CORE3_reg;
				//bit_i = _BIT3;
			}
		}
		else if (i == 4) {
			if ((coproc_bitmap & _BIT4) != _BIT4)
				continue;
			else {
				regBase = MAGIC_TAG_CORE4_reg;
				//bit_i = _BIT4;
			}
		}
#endif
		printf("Core %d do scu_test\n", i);

		// start coreX scu_test
		rtd_outl(regBase, MAGIC_COPROC_OP_SCU_TEST);

	
	}

	if ((coproc_bitmap & _BIT0) == _BIT0){
		printf("Core 0 do scu_test\n");

		scu_test();
	}

	return 0;

}
#endif	//CONFIG_CMD_SCUTEST

int do_coprocessor_mem_test(unsigned int coproc_bitmap)
{
	unsigned int regValue = 0;
	unsigned int regBase, base[] = {-1, MAGIC_TAG_CORE1_reg, MAGIC_TAG_CORE2_reg, MAGIC_TAG_CORE3_reg, MAGIC_TAG_CORE4_reg};
	unsigned char cpuid =0;
	int i = 0;
	//unsigned int bit_i = 0;
	int ret = 0;

	flush_dcache_all();

	
	for (i = 1; i < NR_CPUS; i++)
	{
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0 << i))
			continue;

		regBase = base[i];
		cpuid = i;

#if (CONFIG_FT == 0)
		int idx = 0;
		int c;
		for (c=0; c<cpuid; c++) 
			idx += (coproc_bitmap & (1<<c)) ? 1 : 0;
		
			smp_printf("Core %d do mtest %x %x %x %x\n",
				   cpuid,
				   cop_mtest_base + (idx) * cop_mtest_size,
				   cop_mtest_base + (idx+1) * cop_mtest_size,
				   cop_mtest_iteration,
				   cop_mtest_pattern);
#endif

		if (cpuid == 0)
			continue;
		
		// start coreX mtest
		rtd_outl(regBase, MAGIC_COPROC_OP_MEM_TEST);
		flush_dcache_range(regBase, regBase+4*sizeof(int));
              
  
#ifdef CONFIG_USE_IRQ
		ipi_send_target(cpuid,2);
#else
		asm volatile("SEV");
#endif
	}

	if (coproc_bitmap & _BIT0)
	{
#ifndef CONFIG_SYS_DCACHE_OFF
		if (cop_mtest_dcache_en != 0)
			dcache_enable();
		else 
			dcache_disable();
#endif
		
		// start core0 mtest
		ret = mem_test((vu_long *)(cop_mtest_base),
				(vu_long *)(cop_mtest_base + cop_mtest_size),
				cop_mtest_iteration,
				cop_mtest_pattern);

#ifndef CONFIG_SYS_DCACHE_OFF
		dcache_enable();
#endif

		if (ret == 0) {
			smp_printf("Core 0 OK~~~~~~\n");
	
		}
		else {
			smp_printf("Core 0 FAILED!!!!!!\n");
		}
	
	}

	for (i = 1; i < NR_CPUS; i++)
	{
		// decide which core
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0<<i))
			continue;

		regBase = base[i];

		// got coreX mtest result
		while(1)
		{
			regValue = rtd_inl(regBase);
			if (regValue == MAGIC_COPROC_MEM_TEST_OK)
			{
#if (CONFIG_FT == 0)
				smp_printf("Core %d OK~~~~~~\n", i);
#endif
				break;
		
			}
			else if (regValue == MAGIC_COPROC_MEM_TEST_FAIL) {
				smp_printf("Core %d FAILED!!!!!!\n", i);
				ret = 1;
				break;
			}
		}

		// tell coreX done
		rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_DONE);
	
	}
	return ret;
}

int do_coprocessor_max_power_a55(unsigned int coproc_bitmap)
{
	int ret = 0;
#if CONFIG_MAX_POWER_MEASURE
    ret |= main_maxpower();
#endif
	return ret;
}

#ifdef CONFIG_ARM_MAX_POWER
int do_coprocessor_max_power(unsigned int coproc_bitmap)
{
	unsigned int regValue = 0;
	unsigned int regBase = 0;
	int i = 0, ret = 0, cpunb = 1;
	unsigned int bit_i = 0;

	for (i = 0; i < NR_CPUS; i++)
	{
#if 1
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0 << i)){
			continue;
		}
		else {
			regBase = MAGIC_TAG_CORE0_reg + (4 * i);
			bit_i = _BIT0 < i;
			cpunb++;
		}
#else
		// decide which core
		if (i == 0) {
			if ((coproc_bitmap & _BIT1) != _BIT1)
				continue;
			else {
				regBase = MAGIC_TAG_CORE1_reg;
				bit_i = _BIT1;
				cpunb++;
			}
		}
		else if (i == 1) {
			if ((coproc_bitmap & _BIT2) != _BIT2)
				continue;
			else {
				regBase = MAGIC_TAG_CORE2_reg;
				bit_i = _BIT2;
				cpunb++;
			}
		}
		else if (i == 2) {
			if ((coproc_bitmap & _BIT3) != _BIT3)
				continue;
			else {
				regBase = MAGIC_TAG_CORE3_reg;
				bit_i = _BIT3;
				cpunb++;
			}
		}
#endif
		printf("Core %d do max_power\n", i + 1);

		// start coreX scu_test
		rtd_outl(regBase, MAGIC_COPROC_OP_MAX_POWER_TEST);

	}

	if (coproc_bitmap & _BIT0 == _BIT0) {
		printf("Core 0 do max_power\n");

	#ifndef CONFIG_SYS_ICACHE_OFF
   	icache_disable();
	#endif
	#ifndef CONFIG_SYS_DCACHE_OFF
   	dcache_disable();
	#endif

		printf("cpunb is .....%d\n", cpunb);
		ret = ca7_max_power(gd->ram_size + CONFIG_SYS_SDRAM_BASE, cpunb);

	#ifndef CONFIG_SYS_ICACHE_OFF
   	icache_enable();
	#endif
	#ifndef CONFIG_SYS_DCACHE_OFF
      dcache_enable();
	#endif

	if (ret == 0)
		printf("Core 0 OK~~~~~~\n");
   	else
		printf("Core 0 FAILED!!!!!!\n");

	for (i = 1; i < NR_CPUS; i++)
	{
#if 1
		if ((coproc_bitmap & (_BIT0 << i)) != (_BIT0 << i)) {
			continue;
		}
		else {
			regBase = MAGIC_TAG_CORE0_reg + (4 * i);
		}
#else
      		// decide which core
      		if (i == 1) {
         		if ((coproc_bitmap & _BIT1) != _BIT1)
            			continue;
         		else {
            			regBase = MAGIC_TAG_CORE1_reg;
         		}
      		}
      		else if (i == 2) {
	         	if ((coproc_bitmap & _BIT2) != _BIT2)
		            	continue;
         		else {
            			regBase = MAGIC_TAG_CORE2_reg;
         		}
      		}
		else if (i == 3) {
			if ((coproc_bitmap & _BIT3) != _BIT3)
				continue;
			else {
				regBase = MAGIC_TAG_CORE3_reg;
			}
		}
		else if (i == 4) {
			if ((coproc_bitmap & _BIT4) != _BIT4)
				continue;
			else {
				regBase = MAGIC_TAG_CORE4_reg;
			}
		}
#endif
	      // got coreX mtest result
      	while(1) {
         	regValue = rtd_inl(regBase);
         	if (regValue == MAGIC_COPROC_MEM_TEST_OK) {
				#if (CONFIG_FT == 0)
            	printf("Core %d OK~~~~~~\n", i + 1);
				#endif
            	break;
         	}
         	else if (regValue == MAGIC_COPROC_MEM_TEST_FAIL) {
            	printf("Core %d FAILED!!!!!!\n", i + 1);
            	ret = 1;
            	break;
         	}
      	}

      	// tell coreX done
      	rtd_outl(regBase, MAGIC_COPROC_MEM_TEST_DONE);
   	}
	}
	return ret;
}
#endif	//CONFIG_ARM_MAX_POWER

int do_coprocessor(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	unsigned int coproc_bitmap = (_BIT0 | _BIT1 | _BIT2 | _BIT3 | _BIT4); // means all cores
	int ret = 0;

#ifdef CONFIG_CMD_MTEST_MULTICORE	
	int coproc_num = 1;
	int count;
#endif

	if (argc == 1) {
		return CMD_RET_USAGE;
	}

	if (argc > 2) {
		coproc_bitmap = simple_strtoul(argv[2], NULL, 16);
		if ((coproc_bitmap == 0x0) || (coproc_bitmap > 0x1F)) {
			printf("Warning: core_bitmap out of range\n");
			return CMD_RET_SUCCESS;
		}
	}

	if (strncmp(argv[1], "start", 5) == 0) {
		ret = do_coprocessor_start(coproc_bitmap);
	}
	else if (strncmp(argv[1], "status", 6) == 0) {
		ret = do_coprocessor_status(coproc_bitmap);
	}
	else if (strncmp(argv[1], "scutest", 7) == 0) {
#ifdef CONFIG_CMD_SCUTEST
		ret = do_coprocessor_scu_test(coproc_bitmap);
#else
		printf("Warning: CONFIG_CMD_SCUTEST is disabled\n");
#endif
	}
	else if (strncmp(argv[1], "mtest", 5) == 0) {
#ifdef CONFIG_CMD_MTEST_MULTICORE
		if (argc > 9) {
			return CMD_RET_USAGE;
		}
		else {
			if (argc > 2) 
			{
				cop_mtest_corebits = coproc_bitmap;
				for (count = 1; count < NR_CPUS; count++)
					coproc_num += (coproc_bitmap & (_BIT0 << count)) ? 1 : 0;
			}
			if (argc > 3)
				cop_mtest_dcache_en = (unsigned int)simple_strtoul(argv[3], NULL, 16);
			if (argc > 4) 
				cop_mtest_iteration = (unsigned int)simple_strtoul(argv[4], NULL, 16);
			if (argc > 6)
			{
				cop_mtest_base = (unsigned int)simple_strtoul(argv[5], NULL, 16);
				cop_mtest_end = (unsigned int)simple_strtoul(argv[6], NULL, 16);
				cop_mtest_size = (cop_mtest_end - cop_mtest_base)/coproc_num;
				cop_mtest_size = cop_mtest_size  & ~0x3; //aligned to 4
			}
			if (argc > 7)
				cop_mtest_pattern = (unsigned int)simple_strtoul(argv[7], NULL, 16);
			if (argc > 8)
				cop_mtest_log_en = (unsigned int)simple_strtoul(argv[8], NULL, 16);
			if(cop_mtest_log_en)
			{
				printf("mtest argu(%d): bits:0x%x, dcache_en:%d, iter:%d s:%x e:%x pcs_size:%x pat:%x\n",
			       argc,
			       cop_mtest_corebits,
			       cop_mtest_dcache_en,
			       cop_mtest_iteration,
			       cop_mtest_base, 
			       cop_mtest_end,
			       cop_mtest_size,
			       cop_mtest_pattern);
			}
			
			ret = do_coprocessor_mem_test(coproc_bitmap);
		}
#else
		printf("Warning: CONFIG_CMD_MTEST_MULTICORE is disabled\n");
#endif
	}
	else if (strncmp(argv[1], "max_power", 5) == 0) {
#if CONFIG_MAX_POWER_MEASURE
		ret = do_coprocessor_max_power_a55(coproc_bitmap);
#else
		printf("Warning: CONFIG_MAX_POWER_MEASURE is disabled\n");
#endif

#ifdef CONFIG_ARM_MAX_POWER
		ret = do_coprocessor_max_power(coproc_bitmap);
#else
		printf("Warning: CONFIG_ARM_MAX_POWER is disabled\n");
#endif
	}
	else {
		printf("Warning: Unknown option: %s\n", argv[1]);
		return CMD_RET_USAGE;
	}

	if (ret) {
		return CMD_RET_FAILURE;
	}

	return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	coproc,	CONFIG_SYS_MAXARGS,	1,	do_coprocessor,
	"control co-processor 1~3",
	"[start/status/scutest/mtest/max_power] [core_bitmap] [dcache_en] [iteration] [base end pattern] [log_en]\n"
	"  start   - start each coprocessor\n"
	"  status  - display each coprocessor status\n"
	"  scutest - core [0~4] enter scutest\n"
	"  mtest   - core [0~4] enter mtest\n"
	"  max_power   - core [0~4] enter max_power\n"
	"    [core_bitmap] - 1~F: [core4][core3|core2|core1|core0]\n"
	"    [dcache_en] [iteration] [base end pattern] [log_en] - mtest region"
);

