#include <common.h>
#include <command.h>
#include <io.h>
#include <rbus/dc_sys_reg.h>
#include <rbus/sys_reg_reg.h>

#define FLAG_READ 1
#define FLAG_WRITE 2
/*avcpu test related*/
#define MIPS_KSEG1BASE  0xa0000000
#define BYTE(d,n)       (((d) >> ((n) << 3)) & 0xFF)
#define SWAPEND32(d)    ((BYTE(d,0) << 24) | (BYTE(d,1) << 16) | (BYTE(d,2) << 8) | (BYTE(d,3) << 0))

#define DC_MT_ENTRY_START                    0
#define DC_MT2_ENTRY_START                  4
#define DC_MTEX_ENTRY_START                8
#define DC_MTEX2_ENTRY_START             12
#define DC_MT_ENTRY_COUNT                   16

extern uint *VideoFlag_ptr_V1;
extern uint *VideoFW_entry_ptr_V1;

extern uint64_t smd_memcpy(uint32_t addrDst, uint32_t addrSrc, int32_t len);
extern int sb2_dcmt_vcpu_test(unsigned int addr,unsigned int read_write);

#define msleep(a)       udelay(a * 1000)

typedef struct dcmt_test_desc
{
        char name[64];//test case name
        unsigned int misc;
        unsigned int type;
        unsigned int saddr;
        unsigned int eaddr;
        unsigned int mode;
        unsigned int table0;//module
        unsigned int table1;//module for mtex
        unsigned int hi0;//rw
        unsigned int hi1;//rw for mtex
        unsigned int int_en;
        unsigned int int_en_sys2;
        unsigned int int_en_sys3;
        unsigned int sys2_mux;
        unsigned int sys3_mux;
        unsigned int intr_routing;
} dcmt_test_desc;

typedef struct dcmt_addr_desc
{
        char name[64];//addr desc name
        unsigned int int_offset; //intr pend bit offset
        /*setting addr*/
        unsigned int misc_addr;
        unsigned int type_addr;
        unsigned int saddr_addr;
        unsigned int eaddr_addr;
        unsigned int mode_addr;
        unsigned int table0_addr;//module
        unsigned int table1_addr;//module for mtex
        unsigned int hi0_addr;//rw
        unsigned int hi1_addr;//rw for mtex
        unsigned int int_en_addr;
        unsigned int int_en_sys2_addr;
        unsigned int int_en_sys3_addr;
        unsigned int sys2_mux_addr;
        unsigned int sys3_mux_addr;
        unsigned int intr_routing_addr;
        /*status addr*/
        unsigned int int_status_addr;
        unsigned int int_status_sys2_addr;
        unsigned int int_status_sys3_addr;
        //hi0_addr : already decalred above
        unsigned int lo_addr;
        unsigned int sa_addr;

        unsigned int hi0_sys2_addr;
        unsigned int lo_sys2_addr;
        unsigned int sa_sys2_addr;

        unsigned int hi0_sys3_addr;
        unsigned int lo_sys3_addr;
        unsigned int sa_sys3_addr;
} dcmt_addr_desc;

typedef struct trash_info
{
        unsigned int check_bit; //0:mem_set, 1:module_id, 2:rw_tpe, 3:sb_type,
                                          //4:trash_addr
                                          //8:sys1 9:sys2 10:sys3
        int mem_set;
        int module_id;
        int rw_type;
        int sb_type;
        unsigned int start_addr;
        unsigned int end_addr;
}trash_info;

enum TRASH_CHECK_BIT
{
        TRASH_CHECK_MEMSET = 0,
        TRASH_CHECK_MODULE,
        TRASH_CHECK_RWTYPE,
        TRASH_CHECK_SBTYPE,
        TRASH_CHECK_ADDR,
        TRASH_CHECK_SYS1=8,
        TRASH_CHECK_SYS2,
        TRASH_CHECK_SYS3
};

typedef enum SYS_PEND_MASKBIT
{
        SYS1_PEND_BIT = 0,
        SYS2_PEND_BIT,
        SYS3_PEND_BIT
} SYS_PEND_MASKBIT;


#define EVER_TRAP_BIT 22
#define MODULE_ID_SYS_BIT 18
#define MODULE_ID_SYS_MASK 0xf
#define MODULE_ID_SUB_BIT 14
#define MODULE_ID_SUB_MASK 0xf

dcmt_test_desc testcase_universal;
dcmt_test_desc testcase_modpos;
dcmt_test_desc testcase_wmonitor;
dcmt_test_desc testcase_mdtest;
dcmt_test_desc testcase_mode2;
dcmt_test_desc testcase_mode3;
dcmt_test_desc testcase_regiontest;
dcmt_test_desc testcase_video;
dcmt_test_desc testcase_video_mode2;
dcmt_test_desc testcase_video_mode3;
dcmt_test_desc testcase_dcmt_sb2;

dcmt_addr_desc mt_addr[DC_MT_ENTRY_COUNT];
//dcmt_addr_desc mt2_addr[4];
//dcmt_addr_desc mtex_addr[8];
//dcmt_addr_desc mtex2_addr[4];

/*************************************init code***************************************/

/*
init mt&mt2&mtex&mtex2 reg addr
*/
void init_mt_addr(void)
{
        int i;
        for(i = DC_MT_ENTRY_START; i < DC_MT2_ENTRY_START; ++i)
        {
                memset(&mt_addr[i].name, 0, sizeof(mt_addr[i].name));
                snprintf((char *)(&mt_addr[i].name), sizeof(mt_addr[i].name), "mt_%d_regs", i);
                mt_addr[i].int_offset = i * 3;

                mt_addr[i].misc_addr = DC_SYS_DC_MT_MISC_reg;
                mt_addr[i].type_addr = DC_SYS_DC_MT_TYPE_0_reg + i * 4;
                mt_addr[i].saddr_addr = DC_SYS_DC_MT_SADDR_0_reg + i * 4;
                mt_addr[i].eaddr_addr = DC_SYS_DC_MT_EADDR_0_reg + i * 4;
                mt_addr[i].mode_addr = DC_SYS_DC_MT_MODE_0_reg + i * 4;
                mt_addr[i].table0_addr = DC_SYS_DC_MT_TABLE_0_reg + i * 4;
                mt_addr[i].table1_addr = 0;
                mt_addr[i].hi0_addr = DC_SYS_DC_MT_ADDCMD_HI_0_reg + i * 4;
                mt_addr[i].hi1_addr = 0;
                mt_addr[i].int_en_addr = DC_SYS_DC_int_enable_reg;
                mt_addr[i].int_en_sys2_addr = DC_SYS_DC_int_enable_SYS2_reg;
                mt_addr[i].int_en_sys3_addr = DC_SYS_DC_int_enable_SYS3_reg;
                mt_addr[i].sys2_mux_addr = DC_SYS_dc_sys2_mux_reg;
                mt_addr[i].sys3_mux_addr = DC_SYS_dc_sys3_mux_reg;
                mt_addr[i].intr_routing_addr = SYS_REG_INT_CTRL_SCPU_reg;

                mt_addr[i].int_status_addr = DC_SYS_DC_int_status_reg;
                mt_addr[i].int_status_sys2_addr = DC_SYS_DC_int_status_SYS2_reg;
                mt_addr[i].int_status_sys3_addr = DC_SYS_DC_int_status_SYS3_reg;
                mt_addr[i].lo_addr = DC_SYS_DC_MT_ADDCMD_LO_0_reg + i * 4;
                mt_addr[i].sa_addr = DC_SYS_DC_MT_ADDCMD_SA_0_reg + i * 4;
                //SYS2
                mt_addr[i].hi0_sys2_addr = DC_SYS_DC_MT_ADDCMD_HI_SYS2_0_reg + i * 4;
                mt_addr[i].lo_sys2_addr = DC_SYS_DC_MT_ADDCMD_LO_SYS2_0_reg + i * 4;
                mt_addr[i].sa_sys2_addr = DC_SYS_DC_MT_ADDCMD_SA_SYS2_0_reg + i * 4;
                //SYS3
                mt_addr[i].hi0_sys3_addr = DC_SYS_DC_MT_ADDCMD_HI_SYS3_0_reg + i * 4;
                mt_addr[i].lo_sys3_addr = DC_SYS_DC_MT_ADDCMD_LO_SYS3_0_reg + i * 4;
                mt_addr[i].sa_sys3_addr = DC_SYS_DC_MT_ADDCMD_SA_SYS3_0_reg + i * 4;
        }
}

void init_mt2_addr(void)
{
        int i;
        for(i = DC_MT2_ENTRY_START; i < DC_MTEX_ENTRY_START; ++i)
        {
                memset(&mt_addr[i].name, 0, sizeof(mt_addr[i].name));
                snprintf((char *)(&mt_addr[i].name), sizeof(mt_addr[i].name), "mt_%d_regs", i);
                mt_addr[i].int_offset = (i-DC_MT2_ENTRY_START) * 3;

                mt_addr[i].misc_addr = DC_SYS_DC_MT_MISC_reg;
                mt_addr[i].type_addr = DC_SYS_DC_MT2_TYPE_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].saddr_addr = DC_SYS_DC_MT2_SADDR_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].eaddr_addr = DC_SYS_DC_MT2_EADDR_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].mode_addr = DC_SYS_DC_MT2_MODE_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].table0_addr = DC_SYS_DC_MT2_TABLE_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].table1_addr = 0;
                mt_addr[i].hi0_addr = DC_SYS_DC_MT2_ADDCMD_HI_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].hi1_addr = 0;
                mt_addr[i].int_en_addr = DC_SYS_DC_MT2_int_enable_reg;
                mt_addr[i].int_en_sys2_addr = DC_SYS_DC_MT2_int_enable_SYS2_reg;
                mt_addr[i].int_en_sys3_addr = DC_SYS_DC_MT2_int_enable_SYS3_reg;
                mt_addr[i].sys2_mux_addr = DC_SYS_dc_sys2_mux_reg;
                mt_addr[i].sys3_mux_addr = DC_SYS_dc_sys3_mux_reg;
                mt_addr[i].intr_routing_addr = SYS_REG_INT_CTRL_SCPU_reg;

                mt_addr[i].int_status_addr = DC_SYS_DC_MT2_int_status_reg;
                mt_addr[i].int_status_sys2_addr = DC_SYS_DC_MT2_int_status_SYS2_reg;
                mt_addr[i].int_status_sys3_addr = DC_SYS_DC_MT2_int_status_SYS3_reg;
                mt_addr[i].lo_addr = DC_SYS_DC_MT2_ADDCMD_LO_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].sa_addr = DC_SYS_DC_MT2_ADDCMD_SA_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                //SYS2
                mt_addr[i].hi0_sys2_addr = DC_SYS_DC_MT2_ADDCMD_HI_SYS2_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].lo_sys2_addr = DC_SYS_DC_MT2_ADDCMD_LO_SYS2_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].sa_sys2_addr = DC_SYS_DC_MT2_ADDCMD_SA_SYS2_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                //SYS3
                mt_addr[i].hi0_sys3_addr = DC_SYS_DC_MT2_ADDCMD_HI_SYS3_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].lo_sys3_addr = DC_SYS_DC_MT2_ADDCMD_LO_SYS3_0_reg + (i-DC_MT2_ENTRY_START) * 4;
                mt_addr[i].sa_sys3_addr = DC_SYS_DC_MT2_ADDCMD_SA_SYS3_0_reg + (i-DC_MT2_ENTRY_START) * 4;
        }
}
void init_mtex_addr(void)
{
        int i;
        for(i = DC_MTEX_ENTRY_START; i < DC_MTEX2_ENTRY_START; ++i)
        {
                memset(&mt_addr[i].name, 0, sizeof(mt_addr[i].name));
                snprintf((char *)(&mt_addr[i].name), sizeof(mt_addr[i].name), "mtex_%d_regs", i);
                mt_addr[i].int_offset = (i-DC_MTEX_ENTRY_START) * 3;

                mt_addr[i].misc_addr = DC_SYS_DC_MT_MISC_reg;
                mt_addr[i].type_addr = DC_SYS_DC_MTEX_TYPE_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].saddr_addr = DC_SYS_DC_MTEX_SADDR_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].eaddr_addr = DC_SYS_DC_MTEX_EADDR_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].mode_addr = DC_SYS_DC_MTEX_MODE_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].table0_addr = DC_SYS_DC_MTEX_TABLE0_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].table1_addr = DC_SYS_DC_MTEX_TABLE1_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].hi0_addr = DC_SYS_DC_MTEX_ADDCMD_HI0_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].hi1_addr = DC_SYS_DC_MTEX_ADDCMD_HI1_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].int_en_addr = DC_SYS_DC_MTEX_int_enable_reg;
                mt_addr[i].int_en_sys2_addr = DC_SYS_DC_MTEX_int_enable_SYS2_reg;
                mt_addr[i].int_en_sys3_addr = DC_SYS_DC_MTEX_int_enable_SYS3_reg;
                mt_addr[i].sys2_mux_addr = DC_SYS_dc_sys2_mux_reg;
                mt_addr[i].sys3_mux_addr = DC_SYS_dc_sys3_mux_reg;
                mt_addr[i].intr_routing_addr = SYS_REG_INT_CTRL_SCPU_reg;

                mt_addr[i].int_status_addr = DC_SYS_DC_MTEX_int_status_reg;
                mt_addr[i].int_status_sys2_addr = DC_SYS_DC_MTEX_int_status_SYS2_reg;
                mt_addr[i].int_status_sys3_addr = DC_SYS_DC_MTEX_int_status_SYS3_reg;
                mt_addr[i].lo_addr = DC_SYS_DC_MTEX_ADDCMD_LO_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].sa_addr = DC_SYS_DC_MTEX_ADDCMD_SA_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                //SYS2
                mt_addr[i].hi0_sys2_addr = DC_SYS_DC_MTEX_ADDCMD_HI0_SYS2_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].lo_sys2_addr = DC_SYS_DC_MTEX_ADDCMD_LO_SYS2_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].sa_sys2_addr = DC_SYS_DC_MTEX_ADDCMD_SA_SYS2_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                //SYS3
                mt_addr[i].hi0_sys3_addr = DC_SYS_DC_MTEX_ADDCMD_HI0_SYS3_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].lo_sys3_addr = DC_SYS_DC_MTEX_ADDCMD_LO_SYS3_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
                mt_addr[i].sa_sys3_addr = DC_SYS_DC_MTEX_ADDCMD_SA_SYS3_0_reg + (i-DC_MTEX_ENTRY_START) * 4;
        }
}

void init_mtex2_addr(void)
{
        int i;
        for(i = DC_MTEX2_ENTRY_START; i < DC_MT_ENTRY_COUNT; ++i)
        {
                memset(&mt_addr[i].name, 0, sizeof(mt_addr[i].name));
                snprintf((char *)(&mt_addr[i].name), sizeof(mt_addr[i].name), "mtex_%d_regs", i);
                mt_addr[i].int_offset = (i-DC_MTEX2_ENTRY_START) * 3;

                mt_addr[i].misc_addr = DC_SYS_DC_MT_MISC_reg;
                mt_addr[i].type_addr = DC_SYS_DC_MTEX2_TYPE_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].saddr_addr = DC_SYS_DC_MTEX2_SADDR_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].eaddr_addr = DC_SYS_DC_MTEX2_EADDR_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].mode_addr = DC_SYS_DC_MTEX2_MODE_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].table0_addr = DC_SYS_DC_MTEX2_TABLE0_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].table1_addr = DC_SYS_DC_MTEX2_TABLE1_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].hi0_addr = DC_SYS_DC_MTEX2_ADDCMD_HI0_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].hi1_addr = DC_SYS_DC_MTEX2_ADDCMD_HI1_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].int_en_addr = DC_SYS_DC_MTEX2_int_enable_reg;
                mt_addr[i].int_en_sys2_addr = DC_SYS_DC_MTEX2_int_enable_SYS2_reg;
                mt_addr[i].int_en_sys3_addr = DC_SYS_DC_MTEX2_int_enable_SYS3_reg;
                mt_addr[i].sys2_mux_addr = DC_SYS_dc_sys2_mux_reg;
                mt_addr[i].sys3_mux_addr = DC_SYS_dc_sys3_mux_reg;
                mt_addr[i].intr_routing_addr = SYS_REG_INT_CTRL_SCPU_reg;

                mt_addr[i].int_status_addr = DC_SYS_DC_MTEX2_int_status_reg;
                mt_addr[i].int_status_sys2_addr = DC_SYS_DC_MTEX2_int_status_SYS2_reg;
                mt_addr[i].int_status_sys3_addr = DC_SYS_DC_MTEX2_int_status_SYS3_reg;
                mt_addr[i].lo_addr = DC_SYS_DC_MTEX2_ADDCMD_LO_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].sa_addr = DC_SYS_DC_MTEX2_ADDCMD_SA_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                //SYS2
                mt_addr[i].hi0_sys2_addr = DC_SYS_DC_MTEX2_ADDCMD_HI0_SYS2_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].lo_sys2_addr = DC_SYS_DC_MTEX2_ADDCMD_LO_SYS2_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].sa_sys2_addr = DC_SYS_DC_MTEX2_ADDCMD_SA_SYS2_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                //SYS3
                mt_addr[i].hi0_sys3_addr = DC_SYS_DC_MTEX2_ADDCMD_HI0_SYS3_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].lo_sys3_addr = DC_SYS_DC_MTEX2_ADDCMD_LO_SYS3_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
                mt_addr[i].sa_sys3_addr = DC_SYS_DC_MTEX2_ADDCMD_SA_SYS3_0_reg + (i-DC_MTEX2_ENTRY_START) * 4;
        }
}

/*
this test case monitor ANY(RW) , region 0x00200000 - 0x17ffffff
*/
void testcase_universal_init(void)
{
        memset(&testcase_universal.name, 0, sizeof(testcase_universal.name));
        snprintf((char *)(&testcase_universal.name), sizeof(testcase_universal.name), "testcase_universal");

        testcase_universal.misc = 1;
        testcase_universal.type = 0xc0000000;//sb
        testcase_universal.saddr = 0x1a000000;
        testcase_universal.eaddr = 0x1fffffff;
        testcase_universal.mode = 0x5;//mode1
        testcase_universal.table0 = 0xff000000;//module : ANY
        testcase_universal.table1 = 0;
        testcase_universal.hi0 = 0xc0000000;
        testcase_universal.hi1 = 0;
        testcase_universal.int_en = 0x00001fff;
        testcase_universal.int_en_sys2 = 0x00001fff;
        testcase_universal.int_en_sys3 = 0x00001fff;
        testcase_universal.sys2_mux = 0x2001000f;
        testcase_universal.sys3_mux = 0x1000801f;
        testcase_universal.intr_routing = 0x00000800;
}

/*
this test case monitor ANY(RW) , region 0x00200000 - 0x17ffffff
*/
void testcase_modpos_init(void)
{
        memset(&testcase_modpos.name, 0, sizeof(testcase_modpos.name));
        snprintf((char *)(&testcase_modpos.name), sizeof(testcase_modpos.name), "testcase_modpos");

        testcase_modpos.misc = 1;
        testcase_modpos.type = 0xc0000000;//sb
        testcase_modpos.saddr = 0x1a000000;
        testcase_modpos.eaddr = 0x1fffffff;
        testcase_modpos.mode = 0x5;//mode1
        testcase_modpos.table0 = 0; //module : md
        testcase_modpos.table1 = 0;
        testcase_modpos.hi0 = 0;
        testcase_modpos.hi1 = 0;
        testcase_modpos.int_en = 0x00001fff;
        testcase_modpos.int_en_sys2 = 0x00001fff;
        testcase_modpos.int_en_sys3 = 0x00001fff;
        testcase_modpos.sys2_mux = 0x2001000f;
        testcase_modpos.sys3_mux = 0x1000801f;
        testcase_modpos.intr_routing = 0x00000800;
}

void testcase_mdtest_init(void)
{
        memset(&testcase_mdtest.name, 0, sizeof(testcase_mdtest.name));
        snprintf((char *)(&testcase_mdtest.name), sizeof(testcase_mdtest.name), "md testcase");

        testcase_mdtest.misc = 1;
        testcase_mdtest.type = 0xc0000000;//sb
        testcase_mdtest.saddr = 0x1a000000;
        testcase_mdtest.eaddr = 0x1fffffff;
        testcase_mdtest.mode = 0x5;//mode1
        testcase_mdtest.table0 = 0x29000000;//module : SCPU_MD
        testcase_mdtest.table1 = 0;
        testcase_mdtest.hi0 = 0x40000000;//test read
        testcase_mdtest.hi1 = 0;
        testcase_mdtest.int_en = 0x00001fff;
        testcase_mdtest.int_en_sys2 = 0x00001fff;
        testcase_mdtest.int_en_sys3 = 0x00001fff;
        testcase_mdtest.sys2_mux = 0x2001000f;
        testcase_mdtest.sys3_mux = 0x1000801f;
        testcase_mdtest.intr_routing = 0x00000800;
}

void testcase_regiontest_init(void)
{
        memset(&testcase_regiontest.name, 0, sizeof(testcase_regiontest.name));
        snprintf((char *)(&testcase_regiontest.name), sizeof(testcase_regiontest.name), "region test");

        testcase_regiontest.misc = 1;
        testcase_regiontest.type = 0xc0000000;//sb
        testcase_regiontest.saddr = 0x1a000000;
        testcase_regiontest.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_regiontest.mode = 0x5;//mode1
        testcase_regiontest.table0 = 0x29000000;//module : SCPU_MD
        testcase_regiontest.table1 = 0;
        testcase_regiontest.hi0 = 0x40000000;//test read
        testcase_regiontest.hi1 = 0;
        testcase_regiontest.int_en = 0x00001fff;
        testcase_regiontest.int_en_sys2 = 0x00001fff;
        testcase_regiontest.int_en_sys3 = 0x00001fff;
        testcase_regiontest.sys2_mux = 0x2001000f;
        testcase_regiontest.sys3_mux = 0x1000801f;
        testcase_regiontest.intr_routing = 0x00000800;
}

void testcase_video_init(void)
{
        memset(&testcase_video.name, 0, sizeof(testcase_video.name));
        snprintf((char *)(&testcase_video.name), sizeof(testcase_video.name), "video test");

        testcase_video.misc = 1;
        testcase_video.type = 0xc0000000;//sb
        testcase_video.saddr = 0x1a000000;
        testcase_video.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_video.mode = 0x5;//mode1
        testcase_video.table0 = 0x33000000;//module : A1,A2,V1,V2
        testcase_video.table1 = 0;
        testcase_video.hi0 = 0x40000000;//test read
        testcase_video.hi1 = 0;
        testcase_video.int_en = 0x00001fff;
        testcase_video.int_en_sys2 = 0x00001fff;
        testcase_video.int_en_sys3 = 0x00001fff;
        testcase_video.sys2_mux = 0x2001000f;
        testcase_video.sys3_mux = 0x1000801f;
        testcase_video.intr_routing = 0x00000800;
}

void testcase_video_mode2_init(void)
{
        memset(&testcase_video_mode2.name, 0, sizeof(testcase_video_mode2.name));
        snprintf((char *)(&testcase_video_mode2.name), sizeof(testcase_video_mode2.name), "video mode2 test");

        testcase_video_mode2.misc = 1;
        testcase_video_mode2.type = 0xc0000000;//sb
        testcase_video_mode2.saddr = 0x1a000000;
        testcase_video_mode2.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_video_mode2.mode = 0x6;//mode2
        testcase_video_mode2.table0 = 0x33000000;//module : video
        testcase_video_mode2.table1 = 0;
        testcase_video_mode2.hi0 = 0x40000000;//test read
        testcase_video_mode2.hi1 = 0;
        testcase_video_mode2.int_en = 0x00001fff;
        testcase_video_mode2.int_en_sys2 = 0x00001fff;
        testcase_video_mode2.int_en_sys3 = 0x00001fff;
        testcase_video_mode2.sys2_mux = 0x2001000f;
        testcase_video_mode2.sys3_mux = 0x1000801f;
        testcase_video_mode2.intr_routing = 0x00000800;
}

void testcase_video_mode3_init(void)
{
        memset(&testcase_video_mode3.name, 0, sizeof(testcase_video_mode3.name));
        snprintf((char *)(&testcase_video_mode3.name), sizeof(testcase_video_mode3.name), "video mode3 test");

        testcase_video_mode3.misc = 1;
        testcase_video_mode3.type = 0xc0000000;//sb
        testcase_video_mode3.saddr = 0x80;
        testcase_video_mode3.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_video_mode3.mode = 0x7;//mode3
        testcase_video_mode3.table0 = 0x33000000;//module : video
        testcase_video_mode3.table1 = 0;
        testcase_video_mode3.hi0 = 0xc0000000;//test rw
        testcase_video_mode3.hi1 = 0;
        testcase_video_mode3.int_en = 0x00001fff;
        testcase_video_mode3.int_en_sys2 = 0x00001fff;
        testcase_video_mode3.int_en_sys3 = 0x00001fff;
        testcase_video_mode3.sys2_mux = 0x2001000f;
        testcase_video_mode3.sys3_mux = 0x1000801f;
        testcase_video_mode3.intr_routing = 0x00000800;
}

void testcase_mode2_init(void)
{
        memset(&testcase_mode2.name, 0, sizeof(testcase_mode2.name));
        snprintf((char *)(&testcase_mode2.name), sizeof(testcase_mode2.name), "mode2 test");

        testcase_mode2.misc = 1;
        testcase_mode2.type = 0xc0000000;//sb
        testcase_mode2.saddr = 0x1a000000;
        testcase_mode2.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_mode2.mode = 0x6;//mode2
        testcase_mode2.table0 = 0x29000000;//module : md
        testcase_mode2.table1 = 0;
        testcase_mode2.hi0 = 0x40000000;//test read
        testcase_mode2.hi1 = 0;
        testcase_mode2.int_en = 0x00001fff;
        testcase_mode2.int_en_sys2 = 0x00001fff;
        testcase_mode2.int_en_sys3 = 0x00001fff;
        testcase_mode2.sys2_mux = 0x2001000f;
        testcase_mode2.sys3_mux = 0x1000801f;
        testcase_mode2.intr_routing = 0x00000800;
}

void testcase_mode3_init(void)
{
        memset(&testcase_mode3.name, 0, sizeof(testcase_mode3.name));
        snprintf((char *)(&testcase_mode3.name), sizeof(testcase_mode3.name), "mode3 test");

        testcase_mode3.misc = 1;
        testcase_mode3.type = 0xc0000000;//sb
        testcase_mode3.saddr = 0x16000000;
        testcase_mode3.eaddr = 0x3affffff;//must end of 0xff,other wise test code will be error
        testcase_mode3.mode = 0x7;//mode3
        testcase_mode3.table0 = 0x29000000;//module : md
        testcase_mode3.table1 = 0;
        testcase_mode3.hi0 = 0xc0000000;//test rw
        testcase_mode3.hi1 = 0;
        testcase_mode3.int_en = 0x00001fff;
        testcase_mode3.int_en_sys2 = 0x00001fff;
        testcase_mode3.int_en_sys3 = 0x00001fff;
        testcase_mode3.sys2_mux = 0x2001000f;
        testcase_mode3.sys3_mux = 0x1000801f;
        testcase_mode3.intr_routing = 0x00000800;
}

void testcase_dcmt_sb2_init(void)
{
        memset(&testcase_dcmt_sb2.name, 0, sizeof(testcase_dcmt_sb2.name));
        snprintf((char *)(&testcase_dcmt_sb2.name), sizeof(testcase_dcmt_sb2.name), "dcmt_sb2 test");

        testcase_dcmt_sb2.misc = 1;
        testcase_dcmt_sb2.type = 0xc0000000;//sb
        testcase_dcmt_sb2.saddr = 0x16000000;
        testcase_dcmt_sb2.eaddr = 0x1affffff;//must end of 0xff,other wise test code will be error
        testcase_dcmt_sb2.mode = 0x5;//mode1
        testcase_dcmt_sb2.table0 = 0x33000000;//module : vcpu
        testcase_dcmt_sb2.table1 = 0;
        testcase_dcmt_sb2.hi0 = 0xc0000000;//test rw
        testcase_dcmt_sb2.hi1 = 0;
        testcase_dcmt_sb2.int_en = 0x00001fff;
        testcase_dcmt_sb2.int_en_sys2 = 0x00001fff;
        testcase_dcmt_sb2.int_en_sys3 = 0x00001fff;
        testcase_dcmt_sb2.sys2_mux = 0x2001000f;
        testcase_dcmt_sb2.sys3_mux = 0x1000801f;
        testcase_dcmt_sb2.intr_routing = 0x00000800;
}


#if 0
void mt_testcase_init(void)
{
        testcase_universal_init();
}
#endif

void dcmt_init(void)
{
        //mt_testcase_init(); mark it because no need to init at init flow. init it in detail test flow.
        init_mt_addr();
        init_mt2_addr();
        init_mtex_addr();
        init_mtex2_addr();
}

/*************************************setting code***************************************/

static void disable_dcmt_mode(dcmt_addr_desc * pDesc)
{
        rtd_outl(pDesc->mode_addr, 0x4);
}

static int _mt_reg_set(dcmt_addr_desc * pAddrDesc, dcmt_test_desc *pTestDesc)
{
        //disable mode
        disable_dcmt_mode(pAddrDesc);
        //other setting
        rtd_outl(pAddrDesc->misc_addr, pTestDesc->misc);
        rtd_outl(pAddrDesc->type_addr, pTestDesc->type);
        rtd_outl(pAddrDesc->saddr_addr, pTestDesc->saddr);
        rtd_outl(pAddrDesc->eaddr_addr, pTestDesc->eaddr);
        rtd_outl(pAddrDesc->table0_addr, pTestDesc->table0);
        if(pAddrDesc->table1_addr) // mt has no table1
        {
                rtd_outl(pAddrDesc->table1_addr, pTestDesc->table1);
        }
        rtd_outl(pAddrDesc->hi0_addr, pTestDesc->hi0);
        if(pAddrDesc->hi1_addr) // mt has no hi1
        {
                rtd_outl(pAddrDesc->hi1_addr, pTestDesc->hi1);
        }
        rtd_outl(pAddrDesc->int_en_addr, pTestDesc->int_en);
        rtd_outl(pAddrDesc->int_en_sys2_addr, pTestDesc->int_en_sys2);
        rtd_outl(pAddrDesc->int_en_sys3_addr, pTestDesc->int_en_sys3);
        rtd_outl(pAddrDesc->sys2_mux_addr, pTestDesc->sys2_mux);
        rtd_outl(pAddrDesc->sys3_mux_addr, pTestDesc->sys3_mux);
        rtd_outl(pAddrDesc->intr_routing_addr, pTestDesc->intr_routing);
        //set mode
        rtd_outl(pAddrDesc->mode_addr, pTestDesc->mode);

        return 0;
}

int mt_set(int id, dcmt_test_desc * pDesc)
{
        if((id < DC_MT_ENTRY_START) || (id > (DC_MTEX_ENTRY_START-1)))
        {
                printf("%s id:%d set fail!\n", __FUNCTION__, id);
                return -1;
        }

        //set regs
        _mt_reg_set(&mt_addr[id], pDesc);

        return 0;
}

int mtex_set(int id, dcmt_test_desc * pDesc)
{
        if((id < DC_MTEX_ENTRY_START) || (id > (DC_MT_ENTRY_COUNT-1)))
        {
                printf("%s id:%d set fail!\n", __FUNCTION__, id);
                return -1;
        }

        //set regs
        _mt_reg_set(&mt_addr[id], pDesc);

        return 0;
}

int mt_mem_set(int id, dcmt_test_desc * pDesc)
{
        if((id < DC_MT_ENTRY_START) || (id > (DC_MT_ENTRY_COUNT-1)))
        {
                printf("%s id:%d set fail!\n", __FUNCTION__, id);
                return -1;
        }

        if(id < DC_MTEX_ENTRY_START)
        {
                return mt_set(id, pDesc);
        }
        else
        {
                return mtex_set(id, pDesc);
        }
}

/*************************************check&dump code***************************************/

int verify_trash_info(trash_info *info,dcmt_addr_desc * pDesc)
{
        volatile unsigned int addcmd_hi = 0;
        volatile unsigned int addcmd_lo = 0;
        volatile unsigned int sys1_pend = 0;
        volatile unsigned int sys2_pend = 0;
        volatile unsigned int sys3_pend = 0;
        volatile unsigned int trap_addr = 0;
        int ret = -1;

        //get
        sys1_pend = rtd_inl(pDesc->int_status_addr) & (0xe << pDesc->int_offset);
        if(sys1_pend)
        {
                addcmd_hi = rtd_inl(pDesc->hi0_addr);
                addcmd_lo = rtd_inl(pDesc->lo_addr);
                trap_addr = rtd_inl(pDesc->sa_addr);
        }
        sys2_pend = rtd_inl(pDesc->int_status_sys2_addr) & (0xe << pDesc->int_offset);
        if(sys2_pend)
        {
                addcmd_hi = rtd_inl(pDesc->hi0_sys2_addr);
                addcmd_lo = rtd_inl(pDesc->lo_sys2_addr);
                trap_addr = rtd_inl(pDesc->sa_sys2_addr);
        }
        sys3_pend = rtd_inl(pDesc->int_status_sys3_addr) & (0xe << pDesc->int_offset);
        if(sys3_pend)
        {
                addcmd_hi = rtd_inl(pDesc->hi0_sys3_addr);
                addcmd_lo = rtd_inl(pDesc->lo_sys3_addr);
                trap_addr = rtd_inl(pDesc->sa_sys3_addr);
        }

        if(info->check_bit & (1<<TRASH_CHECK_MEMSET))//check
        {
                if((pDesc - info->mem_set) != &mt_addr[0])
                {
                        printf("check memset failed! (%d) \n",__LINE__);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_MODULE))//check
        {
                if( ((addcmd_hi>>14)&0xff) != info->module_id )
                {
                        printf("check module_id( 0x%02x(reg) vs 0x%02x(expected) ) failed!\n",((addcmd_hi>>14)&0xff), info->module_id);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_RWTYPE))//check
        {
                if( ((addcmd_lo & (0X1<<1))>>1) != info->rw_type )
                {
                        printf("check rw type( %d(reg) vs %d(expected) ) failed!\n",((addcmd_lo & (0X1<<1))>>1), info->rw_type);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_SBTYPE))//check
        {
                if( (addcmd_lo & 0X1) != info->sb_type )
                {
                        printf("check sb type( %d(reg) vs %d(expected) ) failed!\n",(addcmd_lo & 0X1), info->sb_type);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_ADDR))//check
        {
                if(( trap_addr < info->start_addr )||( trap_addr > info->end_addr ))
                {
                        printf("check trap addr( 0x%08x(reg) vs [0x%08x - 0x%08x](expected) ) failed!\n",trap_addr,info->start_addr,info->end_addr);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_SYS1))//check
        {
                if( sys1_pend == 0 )
                {
                        printf("check sys1_pend failed! (%d) \n",__LINE__);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_SYS2))//check
        {
                if( sys2_pend == 0 )
                {
                        printf("check sys2_pend failed! (%d) \n",__LINE__);
                        return ret;
                }
        }

        if(info->check_bit & (1<<TRASH_CHECK_SYS3))//check
        {
                if( sys3_pend == 0 )
                {
                        printf("check sys3_pend failed! (%d) \n",__LINE__);
                        return ret;
                }
        }

        ret = 0;
        printf("[%s] trash verify pass!\n",pDesc->name);
        return ret;
}

int clear_dcmt_trash_status(dcmt_addr_desc * pDesc)
{
        volatile unsigned int addcmd_hi = rtd_inl(pDesc->hi0_addr);
        //clear status
        rtd_outl(pDesc->hi0_addr, addcmd_hi & (~0x00ffffff));
        rtd_outl(pDesc->lo_addr, 0);
        rtd_outl(pDesc->sa_addr, 0);
        //clear intr pending
        rtd_outl(pDesc->int_status_addr, (0xe << pDesc->int_offset)); //sys1 intr
        rtd_outl(pDesc->int_status_sys2_addr, (0xe << pDesc->int_offset)); //sys2 intr
        rtd_outl(pDesc->int_status_sys3_addr, (0xe << pDesc->int_offset)); //sys3 intr(for merlin2 only)

        return 0;
}

/*
addcmd format:
0       : 0-> seq 1->block
1       : 0->read 1->write
30-2  : start addr, 29bit, Unit-8Byte
38-31: seq burst len,8bit, Unit-8Byte
*/

/*
sys_mask: sys3_mask(bit2) | sys2_mask(bit1) | sys1_mask(bit0)
               0 : don't detect ; 1 : do detect

disable: 0 : clear intr status and go on
            1 : clear intr status and disable DCMT mode
*/
int is_dcmt_trash(dcmt_addr_desc * pDesc, int sys_mask, int disable)
{
        volatile unsigned int addcmd_hi = 0;
        volatile unsigned int addcmd_lo = 0;
        volatile unsigned int sys1_pend = 0;
        volatile unsigned int sys2_pend = 0;
        volatile unsigned int sys3_pend = 0;
        volatile unsigned int intr_pend = 0;
        volatile unsigned int trap_addr = 0;
        int ret = 0;

        //get
        intr_pend = 0;
        sys1_pend = rtd_inl(pDesc->int_status_addr);
        if(sys1_pend & (0xe << pDesc->int_offset) & (sys_mask << (pDesc->int_offset+1)))
        {
                addcmd_hi = rtd_inl(pDesc->hi0_addr);
                addcmd_lo = rtd_inl(pDesc->lo_addr);
                trap_addr = rtd_inl(pDesc->sa_addr);
                intr_pend = 1;
        }
        sys2_pend = rtd_inl(pDesc->int_status_sys2_addr);
        if(sys2_pend & (0xe << pDesc->int_offset) & (sys_mask << (pDesc->int_offset+1)))
        {
                addcmd_hi = rtd_inl(pDesc->hi0_sys2_addr);
                addcmd_lo = rtd_inl(pDesc->lo_sys2_addr);
                trap_addr = rtd_inl(pDesc->sa_sys2_addr);
                intr_pend = 1;
        }
        sys3_pend = rtd_inl(pDesc->int_status_sys3_addr);
        if(sys3_pend & (0xe << pDesc->int_offset) & (sys_mask << (pDesc->int_offset+1)))
        {
                addcmd_hi = rtd_inl(pDesc->hi0_sys3_addr);
                addcmd_lo = rtd_inl(pDesc->lo_sys3_addr);
                trap_addr = rtd_inl(pDesc->sa_sys3_addr);
                intr_pend = 1;
        }

        if(intr_pend)
        {
                //read related reg when intr pending is pulled
                printf("\n[DCMT] %s trash detected, module : %01x(sys_id)-%01x(sub_id)\n",
                       pDesc->name,
                       (addcmd_hi & ((unsigned int)MODULE_ID_SYS_MASK << MODULE_ID_SYS_BIT)) >> MODULE_ID_SYS_BIT,
                       (addcmd_hi & ((unsigned int)MODULE_ID_SUB_MASK << MODULE_ID_SUB_BIT)) >> MODULE_ID_SUB_BIT);
                printf("[DCMT] %-16s : 0x%08x\n", "addcmd high", addcmd_hi);
                printf("[DCMT] %-16s : 0x%08x\n", "addcmd low", addcmd_lo);
                printf("[DCMT] %-16s : 0x%08x\n", "trap addr", trap_addr);
                printf("[DCMT] %-16s : %s\n", "rw type", addcmd_lo & (0x1 << 1) ? "Write" : "Read");
                printf("[DCMT] %-16s : 0x%08x\n", "sys1 intr", sys1_pend);
                printf("[DCMT] %-16s : 0x%08x\n", "sys2 intr", sys2_pend);
                printf("[DCMT] %-16s : 0x%08x\n", "sys3 intr", sys3_pend);
                //check disable or not
                if(disable)
                {
                        disable_dcmt_mode(pDesc);
                }

                ret = 1;
        }

        return ret;
}

void dump_dcmt_regs(dcmt_addr_desc * pDesc)
{
        printf("\n\ndump [%s] regs:\n", pDesc->name);
        printf("%-16s : %08x @ %08x\n", "MODE", rtd_inl(pDesc->mode_addr), pDesc->mode_addr);
        printf("%-16s : %08x @ %08x\n", "TYPE", rtd_inl(pDesc->type_addr), pDesc->type_addr);
        printf("%-16s : %08x @ %08x\n", "START", rtd_inl(pDesc->saddr_addr), pDesc->saddr_addr);
        printf("%-16s : %08x @ %08x\n", "END", rtd_inl(pDesc->eaddr_addr), pDesc->eaddr_addr);
        printf("%-16s : %08x @ %08x\n", "MODULE", rtd_inl(pDesc->table0_addr), pDesc->table0_addr);
        if(pDesc->table1_addr)
        {
                printf("%-16s : %08x @ %08x\n", "MODULE_EX", rtd_inl(pDesc->table1_addr), pDesc->table1_addr);
        }
        printf("%-16s : %08x @ %08x\n", "RW", rtd_inl(pDesc->hi0_addr), pDesc->hi0_addr);
        if(pDesc->hi1_addr)
        {
                printf("%-16s : %08x @ %08x\n", "RW_EX", rtd_inl(pDesc->hi1_addr), pDesc->hi1_addr);
        }

        printf("%-16s : %08x @ %08x\n", "INTR EN", rtd_inl(pDesc->int_en_addr), pDesc->int_en_addr);
        printf("%-16s : %08x @ %08x\n", "INTR SYS2 EN", rtd_inl(pDesc->int_en_sys2_addr), pDesc->int_en_sys2_addr);
        printf("%-16s : %08x @ %08x\n", "INTR SYS3 EN", rtd_inl(pDesc->int_en_sys3_addr), pDesc->int_en_sys3_addr);
        printf("%-16s : %08x @ %08x\n", "INTR ROUTING", rtd_inl(pDesc->intr_routing_addr), pDesc->intr_routing_addr);

        printf("dump status regs:\n");
        printf("%-16s : %08x @ %08x\n", "ADDCMD_HI", rtd_inl(pDesc->hi0_addr), pDesc->hi0_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_LO", rtd_inl(pDesc->lo_addr), pDesc->lo_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_SA", rtd_inl(pDesc->sa_addr), pDesc->sa_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_HI_SYS2", rtd_inl(pDesc->hi0_sys2_addr), pDesc->hi0_sys2_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_LO_SYS2", rtd_inl(pDesc->lo_sys2_addr), pDesc->lo_sys2_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_SA_SYS2", rtd_inl(pDesc->sa_sys2_addr), pDesc->sa_sys2_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_HI_SYS3", rtd_inl(pDesc->hi0_sys3_addr), pDesc->hi0_sys3_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_LO_SYS3", rtd_inl(pDesc->lo_sys3_addr), pDesc->lo_sys3_addr);
        printf("%-16s : %08x @ %08x\n", "ADDCMD_SA_SYS3", rtd_inl(pDesc->sa_sys3_addr), pDesc->sa_sys3_addr);
        printf("%-16s : %08x @ %08x\n", "INTR PEND", rtd_inl(pDesc->int_status_addr), pDesc->int_status_addr);
        printf("%-16s : %08x @ %08x\n", "INTR SYS2 PEND", rtd_inl(pDesc->int_status_sys2_addr), pDesc->int_status_sys2_addr);
        printf("%-16s : %08x @ %08x\n", "INTR SYS3 PEND", rtd_inl(pDesc->int_status_sys3_addr), pDesc->int_status_sys3_addr);
}

/*************************************test case code***************************************/
int dcmt_universal_test(void)
{
        int timeout = 5000000;
        int i;
        int fail_cnt = 0;
        testcase_universal_init();

        for(i = 0; i < 8; ++i)
        {
                mt_mem_set(i, &testcase_universal);
                //dump_dcmt_regs(&mt_addr[i]);
                timeout = 5000000;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[i], 0x7, 1))
                        {
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        fail_cnt --;
                }
        }

        return fail_cnt;
}

int dcmt_modpos_test(void)
{
        int timeout;
        int i, j;
        trash_info info;

        printf("-------------------------%s started-------------------------\n",__FUNCTION__);
        testcase_modpos_init();

        for(i = DC_MT_ENTRY_START; i < DC_MTEX_ENTRY_START; ++i)
        {
                for(j = 0; j < 4; ++j)
                {
                        //test read
                        msleep(100);
                        testcase_modpos.table0 = 0x29000000 >> (j*8);
                        testcase_modpos.hi0 = 0x40000000 >> (j*2);
                        testcase_modpos.table1 = 0;
                        testcase_modpos.hi1 = 0;
                        mt_mem_set(i, &testcase_modpos);
                        smd_memcpy(testcase_modpos.saddr,testcase_modpos.eaddr-0x100,0x20);
                        //dump_dcmt_regs(&mt_addr[i]);
                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[i], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = i;
                                        info.module_id = 0x29;
                                        info.rw_type = 0;//read
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_modpos.eaddr-0x100-32;
                                        info.end_addr = testcase_modpos.eaddr+32;
                                        if(verify_trash_info(&info,&mt_addr[i]))
                                        {
                                                printf("verify md read failed! (%d) \n",__LINE__);
                                                return -1;
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                printf("[%s] test mem[%d]-ip[%d] write test timeout!\n",__FUNCTION__,i,j);
                                return -1;
                        }
                        clear_dcmt_trash_status(&mt_addr[i]);
                        printf("[%s] test mem[%d]-ip[%d] read test pass!\n",__FUNCTION__,i,j);

                        //test write
                        msleep(100);
                        testcase_modpos.table0 = 0x29000000 >> (j*8);
                        testcase_modpos.hi0 = 0x80000000 >> (j*2);
                        testcase_modpos.table1 = 0;
                        testcase_modpos.hi1 = 0;
                        mt_mem_set(i, &testcase_modpos);
                        smd_memcpy(testcase_modpos.saddr,testcase_modpos.eaddr-0x100,0x20);
                        //dump_dcmt_regs(&mt_addr[i]);
                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[i], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = i;
                                        info.module_id = 0x29;
                                        info.rw_type = 1;//write
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_modpos.saddr;
                                        info.end_addr = testcase_modpos.saddr+0x100+32;
                                        if(verify_trash_info(&info,&mt_addr[i]))
                                        {
                                                printf("verify md write failed! (%d) \n",__LINE__);
                                                return -1;
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                printf("[%s] test mem[%d]-ip[%d] write test timeout!\n",__FUNCTION__,i,j);
                                return -1;
                        }
                        clear_dcmt_trash_status(&mt_addr[i]);
                        printf("[%s] test mem[%d]-ip[%d] write test pass!\n",__FUNCTION__,i,j);
                }
        }

        for(i = DC_MTEX_ENTRY_START; i < DC_MT_ENTRY_COUNT; ++i)
        {
                for(j = 0; j < 8; ++j)
                {
                        //test read
                        msleep(100);
                        if(j < 4)
                        {
                                testcase_modpos.table0 = 0x29000000 >> (j*8);
                                testcase_modpos.hi0 = 0x40000000 >> (j*2);
                                testcase_modpos.table1 = 0;
                                testcase_modpos.hi1 = 0;
                        }
                        else
                        {
                                testcase_modpos.table0 = 0;
                                testcase_modpos.hi0 = 0;
                                testcase_modpos.table1 = 0x29000000 >> ((j-4)*8);
                                testcase_modpos.hi1 = 0x40000000 >> ((j-4)*2);
                        }
                        mt_mem_set(i, &testcase_modpos);
                        smd_memcpy(testcase_modpos.saddr,testcase_modpos.eaddr-0x100,0x20);
                        //dump_dcmt_regs(&mt_addr[i]);
                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[i], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = i;
                                        info.module_id = 0x29;
                                        info.rw_type = 0;//read
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_modpos.eaddr-0x100-32;
                                        info.end_addr = testcase_modpos.eaddr+32;
                                        if(verify_trash_info(&info,&mt_addr[i]))
                                        {
                                                printf("verify md read failed! (%d) \n",__LINE__);
                                                return -1;
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                printf("[%s] test mem[%d]-ip[%d] write test timeout!\n",__FUNCTION__,i,j);
                                return -1;
                        }
                        clear_dcmt_trash_status(&mt_addr[i]);
                        printf("[%s] test mem[%d]-ip[%d] read test pass!\n",__FUNCTION__,i,j);

                        //test write
                        msleep(100);
                        if(j < 4)
                        {
                                testcase_modpos.table0 = 0x29000000 >> (j*8);
                                testcase_modpos.hi0 = 0x80000000 >> (j*2);
                                testcase_modpos.table1 = 0;
                                testcase_modpos.hi1 = 0;
                        }
                        else
                        {
                                testcase_modpos.table0 = 0;
                                testcase_modpos.hi0 = 0;
                                testcase_modpos.table1 = 0x29000000 >> ((j-4)*8);
                                testcase_modpos.hi1 = 0x80000000 >> ((j-4)*2);
                        }
                        mt_mem_set(i, &testcase_modpos);
                        smd_memcpy(testcase_modpos.saddr,testcase_modpos.eaddr-0x100,0x20);
                        //dump_dcmt_regs(&mt_addr[i]);
                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[i], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = i;
                                        info.module_id = 0x29;
                                        info.rw_type = 1;//write
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_modpos.saddr;
                                        info.end_addr = testcase_modpos.saddr+0x100+32;
                                        if(verify_trash_info(&info,&mt_addr[i]))
                                        {
                                                printf("verify md write failed! (%d) \n",__LINE__);
                                                return -1;
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                printf("[%s] test mem[%d]-ip[%d] write test timeout!\n",__FUNCTION__,i,j);
                                return -1;
                        }
                        clear_dcmt_trash_status(&mt_addr[i]);
                        printf("[%s] test mem[%d]-ip[%d] write test pass!\n",__FUNCTION__,i,j);
                }
        }

        return 0;
}

int dcmt_md_test(void)
{
        int timeout = 0x100;
        trash_info info;
        int entry_id;

        printf("-------------------------%s started-------------------------\n",__FUNCTION__);
        testcase_mdtest_init();

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //test read
                msleep(100);
                testcase_mdtest.hi0 = 0x40000000;
                mt_mem_set(entry_id, &testcase_mdtest);
                smd_memcpy(testcase_mdtest.saddr,testcase_mdtest.eaddr-0x100,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mdtest.eaddr-0x100-32;
                                info.end_addr = testcase_mdtest.eaddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        dump_dcmt_regs(&mt_addr[entry_id]);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //test write
                msleep(100);
                testcase_mdtest.hi0 = 0x80000000;
                mt_mem_set(entry_id, &testcase_mdtest);
                smd_memcpy(testcase_mdtest.saddr,testcase_mdtest.eaddr-0x100,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mdtest.saddr;
                                info.end_addr = testcase_mdtest.saddr+0x100+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }

                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;

}

/*test dcmt monitor area [0-start] [start-end] [end-4G]*/
int dcmt_region_test(void)
{
        unsigned int rtest_src[3];
        unsigned int rtest_dst[3];
        unsigned int wtest_src[3];
        unsigned int wtest_dst[3];
        int timeout = 0x100;
        trash_info info;
        int entry_id;
        int i;
        unsigned int rwlen = 0x20;
        //test 8 sets : mem0-7 -> test 2 access type : read test,write test -> test 3 area : [0-start] [start-end] [end-4G]

        printf("-------------------------%s started-------------------------\n",__FUNCTION__);
        testcase_regiontest_init();
        //read before, write between, monitor read.
        rtest_src[0] = testcase_regiontest.saddr - rwlen;
        rtest_dst[0] = testcase_regiontest.saddr;
        //read between, write between, monitor read.
        rtest_src[1] = testcase_regiontest.saddr;
        rtest_dst[1] = testcase_regiontest.saddr + rwlen;
        //read after, write between, monitor read.
        rtest_src[2] = testcase_regiontest.eaddr + 1;
        rtest_dst[2] = testcase_regiontest.saddr;

        //read between, write before, monitor write.
        wtest_src[0] = testcase_regiontest.saddr;
        wtest_dst[0] = testcase_regiontest.saddr - rwlen;
        //read between, write between, monitor write.
        wtest_src[1] = testcase_regiontest.saddr;
        wtest_dst[1] = testcase_regiontest.saddr + rwlen;
        //read between, write after, monitor write.
        wtest_src[2] = testcase_regiontest.saddr;
        wtest_dst[2] = testcase_regiontest.eaddr + 1;

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //test read
                msleep(100);
                testcase_regiontest.hi0 = 0x40000000;
                mt_mem_set(entry_id, &testcase_regiontest);

                for(i = 0;i < 3;++i)//0 : before, 1 : between, 2:after
                {
                        smd_memcpy(rtest_dst[i], rtest_src[i], rwlen);
                        //dump_dcmt_regs(&mt_addr[entry_id]);

                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = entry_id;
                                        info.module_id = 0x29;
                                        info.rw_type = 0;//read
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_regiontest.saddr;
                                        info.end_addr = testcase_regiontest.eaddr;
                                        if(verify_trash_info(&info,&mt_addr[entry_id]))
                                        {
                                                if(i == 1)
                                                {
                                                        printf("verify md read between region failed! (%d) \n",__LINE__);
                                                        return -1;
                                                }
                                        }
                                        else
                                        {
                                                if( (i == 0) || (i == 2) )
                                                {
                                                        printf("verify md read out of region failed! (%d) \n",__LINE__);
                                                        return -1;
                                                }
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                if(i == 1)
                                {
                                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                                        return -1;
                                }
                        }
                        clear_dcmt_trash_status(&mt_addr[entry_id]);
                }

                //test write
                msleep(100);
                testcase_regiontest.hi0 = 0x80000000;
                mt_mem_set(entry_id, &testcase_regiontest);

                for(i = 0;i < 3;++i)//0 : before, 1 : between, 2:after
                {
                        smd_memcpy(wtest_dst[i], wtest_src[i], rwlen);
                        //dump_dcmt_regs(&mt_addr[entry_id]);

                        timeout = 0x100;
                        while(timeout > 0)
                        {
                                timeout--;
                                msleep(1);
                                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                                {
                                        //check dcmt trash info
                                        memset(&info,0,sizeof(trash_info));
                                        info.check_bit = 0x11f;
                                        info.mem_set = entry_id;
                                        info.module_id = 0x29;
                                        info.rw_type = 1;//write
                                        info.sb_type = 0;//seq
                                        info.start_addr = testcase_regiontest.saddr;
                                        info.end_addr = testcase_regiontest.eaddr+1;
                                        if(verify_trash_info(&info,&mt_addr[entry_id]))
                                        {
                                                if(i == 1)
                                                {
                                                        printf("verify md write between region failed! (%d) \n",__LINE__);
                                                        return -1;
                                                }
                                        }
                                        else
                                        {
                                                if( (i == 0) || (i == 2) )
                                                {
                                                        printf("verify md write out of region failed! (%d) \n",__LINE__);
                                                        return -1;
                                                }
                                        }
                                        break;
                                }
                        }

                        if(timeout <= 0)
                        {
                                if(i == 1)
                                {
                                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                                        return -1;
                                }
                        }
                        clear_dcmt_trash_status(&mt_addr[entry_id]);
                }
        }

        return 0;
}

extern int vcpu_rtd_api (unsigned int addr,int rw);

int dcmt_sb2_test(void)
{
extern int sb2_dcmt_test_init(void);

        int timeout;
        trash_info info;
        int entry_id = 0;
        unsigned int trash_addr = 0x16000000;
        int ret;

	sb2_dcmt_test_init();
        testcase_dcmt_sb2_init();
        //set dcmt
        testcase_dcmt_sb2.hi0 = 0x80000000;//test write
        mt_mem_set(entry_id, &testcase_dcmt_sb2);
	ret = sb2_dcmt_vcpu_test(trash_addr,1);
        if(ret != 1)
        {
                printf("sb2 verify vcpu write failed! (%d) \n",__LINE__);
                return -1;
        }
        //check dcmt
        timeout = 500;
        while(timeout)
        {
                timeout--;
                msleep(1);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        //check dcmt trash info
                        memset(&info,0,sizeof(trash_info));
                        info.check_bit = 0x11f;
                        info.mem_set = entry_id;
                        info.module_id = 0x33;
                        info.rw_type = 1;//write
                        info.sb_type = 0;//seq
                        info.start_addr = trash_addr;
                        info.end_addr = trash_addr+32;
                        if(verify_trash_info(&info,&mt_addr[entry_id]))
                        {
                                printf("verify vcpu write failed! (%d) \n",__LINE__);
                                return -1;
                        }
                        break;
                }
        }

        if(timeout <= 0)
        {
                printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                return -1;
        }
        clear_dcmt_trash_status(&mt_addr[entry_id]);

        //set dcmt
        testcase_dcmt_sb2.hi0 = 0x40000000;//test read
        mt_mem_set(entry_id, &testcase_dcmt_sb2);
	ret = sb2_dcmt_vcpu_test(trash_addr,0);
        if(ret != 1)
        {
                printf("sb2 verify vcpu read failed! (%d) \n",__LINE__);
                return -1;
        }
        //check dcmt
        timeout = 500;
        while(timeout)
        {
                timeout--;
                msleep(1);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        //check dcmt trash info
                        memset(&info,0,sizeof(trash_info));
                        info.check_bit = 0x11f;
                        info.mem_set = entry_id;
                        info.module_id = 0x33;
                        info.rw_type = 0;//read
                        info.sb_type = 0;//seq
                        info.start_addr = trash_addr;
                        info.end_addr = trash_addr+32;
                        if(verify_trash_info(&info,&mt_addr[entry_id]))
                        {
                                printf("verify vcpu read failed! (%d) \n",__LINE__);
                                return -1;
                        }
                        break;
                }
        }

        if(timeout <= 0)
        {
                printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                return -1;
        }
        clear_dcmt_trash_status(&mt_addr[entry_id]);

        return 0;
}

int dcmt_video_test(void)
{
        int timeout;
        trash_info info;
        int entry_id;
        unsigned int trash_addr;

        testcase_video_init();
        trash_addr = testcase_video.saddr;
        vcpu_rtd_api(testcase_video.saddr - 0x20,FLAG_READ);

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                testcase_video.hi0 = 0x40000000;
                mt_mem_set(entry_id, &testcase_video);

                vcpu_rtd_api(testcase_video.saddr - 0x20,FLAG_READ);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch read out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.eaddr+1,FLAG_READ);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch read out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.saddr - 0x20,FLAG_WRITE);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch write out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.eaddr+1,FLAG_WRITE);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch write out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(trash_addr,FLAG_READ);
                msleep(1000);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = trash_addr;
                                info.end_addr = trash_addr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                testcase_video.hi0 = 0x80000000;
                mt_mem_set(entry_id, &testcase_video);

                vcpu_rtd_api(testcase_video.saddr - 0x20,FLAG_READ);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch read out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.eaddr+1,FLAG_READ);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch read out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.saddr - 0x20,FLAG_WRITE);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch write out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(testcase_video.eaddr+1,FLAG_WRITE);
                msleep(1000);
                if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                {
                        printf("error : dcmt catch write out of region!\n");
                        return -1;
                }

                vcpu_rtd_api(trash_addr,FLAG_WRITE);
                msleep(1000);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = trash_addr;
                                info.end_addr = trash_addr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;
}

int dcmt_video_mode2_test(void)
{
        int timeout;
        trash_info info;
        int entry_id;
        //unsigned int trash_addr;

        testcase_video_mode2_init();

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //case1: mode2, video, read ; test video read
                testcase_video_mode2.table0 = 0x33000000;//module : video
                testcase_video_mode2.hi0 = 0x40000000;//allow read
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_READ);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify video read failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case1: mode2, video, read ; test video write
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_WRITE);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode2.saddr;
                                info.end_addr = testcase_video_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify audio write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);


                //case2: mode2, video, write ; test video write
                testcase_video_mode2.table0 = 0x33000000;//module : video
                testcase_video_mode2.hi0 = 0x80000000;//allow write
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_WRITE);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify video write failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case2: mode2, video, write ; test video read
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_READ);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode2.saddr;
                                info.end_addr = testcase_video_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify video read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode2, md, read&write ; test video read
                testcase_video_mode2.table0 = 0x29000000;//module : md
                testcase_video_mode2.hi0 = 0xc0000000;//allow read&write
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_READ);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode2.saddr;
                                info.end_addr = testcase_video_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify video read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode2, vcpu, read&write ; test audio write
                mt_mem_set(entry_id, &testcase_video_mode2);
                vcpu_rtd_api(testcase_video_mode2.saddr,FLAG_WRITE);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode2.saddr;
                                info.end_addr = testcase_video_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify video write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;
}


int dcmt_video_mode3_test(void)
{
        int timeout;
        trash_info info;
        int entry_id;
        unsigned int trash_addr = 0x14000000;

        testcase_video_mode3_init();

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //case1: rw between region
                mt_mem_set(entry_id, &testcase_video_mode3);
                vcpu_rtd_api(trash_addr,FLAG_READ);
                msleep(10000);
                vcpu_rtd_api(trash_addr,FLAG_WRITE);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify video read failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case2: mode3, video ; test video write out
                mt_mem_set(entry_id, &testcase_video_mode3);
                vcpu_rtd_api(testcase_video_mode3.eaddr+1,FLAG_WRITE);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode3.eaddr+1;
                                info.end_addr = testcase_video_mode3.eaddr+1+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify video write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode3, video ; test video read out
                mt_mem_set(entry_id, &testcase_video_mode3);
                vcpu_rtd_api(testcase_video_mode3.eaddr+1,FLAG_READ);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x33;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_video_mode3.eaddr+1;
                                info.end_addr = testcase_video_mode3.eaddr+1+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify video read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;
}

int dcmt_mode2_test(void)
{
        int timeout;
        trash_info info;
        int entry_id;
        //unsigned int trash_addr;

        testcase_mode2_init();

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //case1: mode2, md, read ; test md read
                testcase_mode2.table0 = 0x29000000;//module : md
                testcase_mode2.hi0 = 0x40000000;//monitor read
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.eaddr+1,testcase_mode2.saddr,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify md read failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case1: mode2, md, read ; test md write
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.saddr,testcase_mode2.eaddr+1,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode2.saddr;
                                info.end_addr = testcase_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);


                //case2: mode2, md, write ; test md write
                testcase_mode2.table0 = 0x29000000;//module : md
                testcase_mode2.hi0 = 0x80000000;//monitor write
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.saddr,testcase_mode2.eaddr+1,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify md write failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case2: mode2, md, read ; test md read
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.eaddr+1,testcase_mode2.saddr,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode2.saddr;
                                info.end_addr = testcase_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode2, test md write
                testcase_mode2.table0 = 0x33000000;//module : vcpu
                testcase_mode2.hi0 = 0xc0000000;//monitor write
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.saddr,testcase_mode2.eaddr+1,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode2.saddr;
                                info.end_addr = testcase_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode2, read&write ; test md read
                mt_mem_set(entry_id, &testcase_mode2);
                smd_memcpy(testcase_mode2.eaddr+1,testcase_mode2.saddr,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 0;//read
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode2.saddr;
                                info.end_addr = testcase_mode2.saddr+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;
}


int dcmt_mode3_test(void)
{
        int timeout;
        trash_info info;
        int entry_id;
        //unsigned int trash_addr;

        testcase_mode3_init();

        for(entry_id = DC_MT_ENTRY_START; entry_id < DC_MT_ENTRY_COUNT; ++entry_id)
        {
                //case1: rw between region
                mt_mem_set(entry_id, &testcase_mode3);
                smd_memcpy(testcase_mode3.saddr,testcase_mode3.eaddr-0x40,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                printf("verify md read failed! (%d) \n",__LINE__);
                                return -1;
                        }
                }

                //case2: mode3, md ; test md read out
                mt_mem_set(entry_id, &testcase_mode3);
                smd_memcpy(testcase_mode3.saddr,testcase_mode3.eaddr+1,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 0;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode3.eaddr+1;
                                info.end_addr = testcase_mode3.eaddr+1+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md read failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d read timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);

                //case3: mode3, md ; test md write out
                mt_mem_set(entry_id, &testcase_mode3);
                smd_memcpy(testcase_mode3.eaddr+1,testcase_mode3.saddr,0x20);
                //dump_dcmt_regs(&mt_addr[entry_id]);

                timeout = 0x100;
                while(timeout > 0)
                {
                        timeout--;
                        msleep(1);
                        if(is_dcmt_trash(&mt_addr[entry_id], 0x7, 1))
                        {
                                //check dcmt trash info
                                memset(&info,0,sizeof(trash_info));
                                info.check_bit = 0x11f;
                                info.mem_set = entry_id;
                                info.module_id = 0x29;
                                info.rw_type = 1;//write
                                info.sb_type = 0;//seq
                                info.start_addr = testcase_mode3.eaddr+1;
                                info.end_addr = testcase_mode3.eaddr+1+32;
                                if(verify_trash_info(&info,&mt_addr[entry_id]))
                                {
                                        printf("verify md write failed! (%d) \n",__LINE__);
                                        return -1;
                                }
                                break;
                        }
                }

                if(timeout <= 0)
                {
                        printf("[%s] test mem%d write timeout!\n",__FUNCTION__,entry_id);
                        return -1;
                }
                clear_dcmt_trash_status(&mt_addr[entry_id]);
        }

        return 0;
}

/*************************************main loop code***************************************/
int dcmt_autotest_loop( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        //int rst = 0;
        int tmp_rst = 0;
        int i = 0;
        //init
        dcmt_init();
        for(i = 0;i < DC_MT_ENTRY_COUNT;++i)
        {
                dump_dcmt_regs(&(mt_addr[i]));
        }


        tmp_rst = dcmt_md_test();//mem0-15 test,r/w test,ip:scpu_md test,
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_md_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_md_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_modpos_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_modpos_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_modpos_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_region_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_region_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_region_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_sb2_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_sb2_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_sb2_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_video_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_video_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_video_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_video_mode2_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_video_mode2_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_video_mode2_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_video_mode3_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_video_mode3_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_video_mode3_test passed!++++++++++++\n");
        }
        
        tmp_rst = dcmt_mode2_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_mode2_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_mode2_test passed!++++++++++++\n");
        }

        tmp_rst = dcmt_mode3_test();
        if(tmp_rst)
        {
                printf("***************[ERROR] dcmt_mode3_test failed!***************\n");
                return -1;
        }
        else
        {
                printf("++++++++++++[PASS] dcmt_mode3_test passed!++++++++++++\n");
        }

        printf("++++++++++++[PASS] DCMT BOOTCODE TEST PASS!++++++++++++\n");
        return tmp_rst;
}

/* -------------------------------------------------------------------- */

U_BOOT_CMD(
        dcmt,   2,      1,      dcmt_autotest_loop,
        "start dcmt monitor",
        ""
);

