/*
 * (C) Copyright 2000-2010
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <linux/stddef.h>
#include <search.h>
#include <errno.h>
#include <malloc.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/factorylib.h>
#include <asm/arch/factorylib_ro.h>
#ifdef Config_EXT4_Load_Takeover_TRUE
#include <asm/arch/takeoverlib_ext4.h>
#endif
#include <asm/arch/bootparam.h>

#ifdef CONFIG_REALTEK_EERPOM_TV010_A
#ifndef UINT8
#define UINT8 unsigned char 
#endif
#ifndef UINT32
#define UINT32 unsigned int 
#endif

extern int getCustomIrCode(UINT8 *code);
extern UINT32 ConvertIrCode(UINT8 TmpIrIndex);
extern int getPanelNameFromIniByIndex(char *filestart, size_t filelen,unsigned int panelindex,char *panelname);
extern int getPanelIndex(UINT32 *index);
#endif 

#include <asm/arch/sys_flash_layout.h>
#include "cmd_irda.h"

#if defined(CONFIG_SYS_RTK_NAND_FLASH)
 #include <linux/mtd/nand.h>		//surport nand flash driver
 #include <nand.h>
#elif defined(CONFIG_SYS_RTK_SPI_NAND)
 #include "../drivers/mtd/spi/rtk_spi_nand.h"
#elif defined CONFIG_SYS_RTK_SPI_FLASH
 #include <flashdev_s.h>
 #undef SYS_PARAM_BOOT_FLASH
 #undef SYS_PARAM_FACTORY_SIZE
 #undef SYS_PARAM_BOOTCODE_START
 #undef SYS_PARAM_BOOTCODE_SIZE
 #undef SYS_PARAM_FACTORY_RO_START
 #undef SYS_PARAM_FACTORY_RO_SIZE
 #undef SYS_PARAM_EMCU_START
 #undef SYS_PARAM_EMCU_SIZE
 #undef SYS_PARAM_RESCUE_START
 #undef SYS_PARAM_RESCUE_SIZE
 #include <asm/arch/sys_param_emmc_tv002.h>
#endif


DECLARE_GLOBAL_DATA_PTR;

#define PANEL_BIN_FILENAME_IN_FACTORY	FACTORY_HEADER_FILE_NAME"bin_panel/panel.bin"
#define PANEL_BIN_FILE_PREFIX			FACTORY_HEADER_FILE_NAME"bin_panel/panel/"
extern void enable_console(void);
extern void disable_console(void);
extern void CusApi_env_relocate(void);
extern uint boot_part;
extern int boot_power_on_mode;
extern uint *uart_reg_base_ptr;
extern _BOARD_VERSION board_version;
#if defined(CONFIG_CUSTOMER_TV002) //factory XML format
extern int CusApi_env_readxml_key_value_from_buf(const char *pFileBuf,const char *pKey, char *pValue, size_t valueLen);
#endif

extern struct IRDA_PROTOCOL irda_proto_list[IRDA_NUM_REMOTER];
/************************************************************************
 * Default settings to be used when no valid environment is found
 */
#define XMK_STR(x)	#x
#define MK_STR(x)	XMK_STR(x)

const uchar default_environment[] = {
#if (1)	// Chuck for DS-5
#ifdef	CONFIG_BOOTARGS
	"bootargs="	CONFIG_BOOTARGS			"\0"
#endif
#ifdef  CONFIG_BOOTARGS_EX
	"bootargs_ex="  CONFIG_BOOTARGS_EX              "\0"
#endif
#ifdef	CONFIG_BOOTCOMMAND
	"bootcmd="	CONFIG_BOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_RAMBOOTCOMMAND
	"ramboot="	CONFIG_RAMBOOTCOMMAND		"\0"
#endif
#ifdef	CONFIG_NFSBOOTCOMMAND
	"nfsboot="	CONFIG_NFSBOOTCOMMAND		"\0"
#endif
#if defined(CONFIG_BOOTDELAY) && (CONFIG_BOOTDELAY >= 0)
	"bootdelay="	MK_STR(CONFIG_BOOTDELAY)	"\0"
#endif
#if defined(CONFIG_BAUDRATE) && (CONFIG_BAUDRATE >= 0)
	"baudrate="	MK_STR(CONFIG_BAUDRATE)		"\0"
#endif
#ifdef	CONFIG_LOADS_ECHO
	"loads_echo="	MK_STR(CONFIG_LOADS_ECHO)	"\0"
#endif
#ifdef	CONFIG_ETHADDR
	"ethaddr="	MK_STR(CONFIG_ETHADDR)		"\0"
#endif
#ifdef	CONFIG_ETH1ADDR
	"eth1addr="	MK_STR(CONFIG_ETH1ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH2ADDR
	"eth2addr="	MK_STR(CONFIG_ETH2ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH3ADDR
	"eth3addr="	MK_STR(CONFIG_ETH3ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH4ADDR
	"eth4addr="	MK_STR(CONFIG_ETH4ADDR)		"\0"
#endif
#ifdef	CONFIG_ETH5ADDR
	"eth5addr="	MK_STR(CONFIG_ETH5ADDR)		"\0"
#endif
#ifdef	CONFIG_IPADDR
	"ipaddr="	MK_STR(CONFIG_IPADDR)		"\0"
#endif
#ifdef	CONFIG_SERVERIP
	"serverip="	MK_STR(CONFIG_SERVERIP)		"\0"
#endif
#ifdef	CONFIG_SYS_AUTOLOAD
	"autoload="	CONFIG_SYS_AUTOLOAD		"\0"
#endif
#ifdef	CONFIG_PREBOOT
	"preboot="	CONFIG_PREBOOT			"\0"
#endif
#ifdef	CONFIG_ROOTPATH
	"rootpath="	CONFIG_ROOTPATH			"\0"
#endif
#ifdef	CONFIG_GATEWAYIP
	"gatewayip="	MK_STR(CONFIG_GATEWAYIP)	"\0"
#endif
#ifdef	CONFIG_NETMASK
	"netmask="	MK_STR(CONFIG_NETMASK)		"\0"
#endif
#ifdef	CONFIG_HOSTNAME
	"hostname="	MK_STR(CONFIG_HOSTNAME)		"\0"
#endif
#ifdef	CONFIG_BOOTFILE
	"bootfile="	CONFIG_BOOTFILE			"\0"
#endif
#ifdef	CONFIG_LOADADDR
	"loadaddr="	MK_STR(CONFIG_LOADADDR)		"\0"
#endif
#ifdef	CONFIG_CLOCKS_IN_MHZ
	"clocks_in_mhz=1\0"
#endif
#if defined(CONFIG_PCI_BOOTDELAY) && (CONFIG_PCI_BOOTDELAY > 0)
	"pcidelay="	MK_STR(CONFIG_PCI_BOOTDELAY)	"\0"
#endif
#ifdef	CONFIG_ENV_VARS_UBOOT_CONFIG
	"arch="		CONFIG_SYS_ARCH			"\0"
	"cpu="		CONFIG_SYS_CPU			"\0"
	"board="	CONFIG_SYS_BOARD		"\0"
#ifdef CONFIG_SYS_VENDOR
	"vendor="	CONFIG_SYS_VENDOR		"\0"
#endif
#ifdef CONFIG_SYS_SOC
	"soc="		CONFIG_SYS_SOC			"\0"
#endif
#endif
#ifdef CONFIG_SILENT_CONSOLE
	"silent=1\0"
#endif
#ifdef CONFIG_CHECK_IMG_VERSION
        "fwver=0.0.0\0"
#endif
#ifdef	CONFIG_EXTRA_ENV_SETTINGS
	CONFIG_EXTRA_ENV_SETTINGS
#endif
    "PanelID_out_list=0\0"  //for tv002
#ifdef CONFIG_VERIFY_BOOT_P
	"OEMLock=on\0"
#endif
#endif	//Chuck for DS-5
	"\0"
};

struct hsearch_data env_htab;

static uchar __env_get_char_spec(int index)
{
	return *((uchar *)(gd->env_addr + index));
}
uchar env_get_char_spec(int)
	__attribute__((weak, alias("__env_get_char_spec")));

static uchar env_get_char_init(int index)
{
	/* if crc was bad, use the default environment */
	if (gd->env_valid)
		return env_get_char_spec(index);
	else
		return default_environment[index];
}

uchar env_get_char_memory(int index)
{
	return *env_get_addr(index);
}

uchar env_get_char(int index)
{
	/* if relocated to RAM */
	if (gd->flags & GD_FLG_RELOC)
		return env_get_char_memory(index);
	else
		return env_get_char_init(index);
}

const uchar *env_get_addr(int index)
{
	if (gd->env_valid)
		return (uchar *)(gd->env_addr + index);
	else
		return &default_environment[index];
}

void set_default_env(const char *s)
{
	if (sizeof(default_environment) > ENV_SIZE) {
		puts("*** Error - default environment variables is too large\n\n");
		return;
	}

	if (s) {
		if (*s == '!') {
			printf("*** Warning - %s, "
				"using default environment variables\n\n",
				s + 1);
		} else {
			puts(s);
		}
	} else {
		puts("[ENV] Using default environment variables\n\n");
	}

	if (himport_r(&env_htab, (char *)default_environment,
			sizeof(default_environment), '\0', 0) == 0)
		error("Environment import failed: errno = %d\n", errno);

	gd->flags |= GD_FLG_ENV_READY;
}

/*
 * Check if CRC is valid and (if yes) import the environment.
 * Note that "buf" may or may not be aligned.
 */
int env_import(const char *buf, int check)
{
	env_t *ep = (env_t *)buf;

	if (check) {
		uint32_t crc;

		memcpy(&crc, &ep->crc, sizeof(crc));

		if (crc32(0, ep->data, ENV_SIZE) != crc) {
			//set_default_env("!bad CRC");
			set_default_env(NULL); // ck modify that prevent serious error messages
			return 0;
		}
	}

	if (himport_r(&env_htab, (char *)ep->data, ENV_SIZE, '\0', 0)) {
		gd->flags |= GD_FLG_ENV_READY;
		return 1;
	}

	error("Cannot import environment: errno = %d\n", errno);

	set_default_env("!import failed");

	return 0;
}

/* Porting from Darwin Env*/

#if 0 /* Boot Revision Define Comments that sync from kernel */
/* The format of "bootrev" is like "0001.0203.4567". The second two bytes of it stands for board id, and the upper half byte of board id stands for CPU id */
typedef enum {
	realtek_299x_qa_board				= 0x000700,
	realtek_299x_demo_ab_board			= 0x000701,
	realtek_299x_demo_c_4ddr_1g_board	= 0x000702,
	realtek_299x_demo_c_4ddr_2g_board	= 0x000703,
	realtek_299x_demo_d_4ddr_2g_board	= 0x000704,
	C10_2995_v1_board					= 0x100700,
	C05_2993_l4300_board				= 0x050700,
} _board_id_t;

/* The format of "bootrev" is like "0001.0203.4567". The first two byte of it stands for company id */
typedef enum {
	company_realtek	= 0x00,
	company_alpha	= 0x01,
	company_C02		= 0x02,
	company_C03		= 0x03,
	company_C04		= 0x04,
	company_C05		= 0x05,
	company_C06		= 0x06,
	company_C07		= 0x07,
	company_C08		= 0x08,
	company_C09		= 0x09,
	/* skip 0x0a ~ 0x0f */
	company_C10		= 0x10,
	company_C11 	= 0x11,
	company_C12 	= 0x12,
	company_C13 	= 0x13,
	company_C14 	= 0x14,
	company_C15 	= 0x15,
	company_C16 	= 0x16,
	company_C17 	= 0x17,
	company_C18 	= 0x18,
	company_C19 	= 0x19,
	/* skip 0x1a ~ 0x1f */
	company_C20 	= 0x20,
} _company_id_t;

/* Defines of CPU ID */
typedef enum {
	realtek_299x_cpu	= 0x07,
} _cpu_id_t;
#endif

static char sys_chiprev_str[64];

//#define BOOT_PARAMS_SIZE		1536	// sync with linux/arch/arm/kernel/atags.c
//#define ENV_VAR_ADDRESS			(LINUX_KERNEL_ADDR + 0x100 + BOOT_PARAMS_SIZE)
#define K_BYTES 1024
#define ENVP_RESERVED_KBYTES 32
__attribute__((unused, section(".envp_cmd"),aligned(4))) char envp_array[ENVP_RESERVED_KBYTES*K_BYTES];
#define BOOT_PARAMS_SIZE 		0x4000
#define ENV_VAR_ADDRESS			(((char *)&envp_array) + 0x100 + BOOT_PARAMS_SIZE)
#define ENV_VAR_INFO_ADDRESS	(ENV_VAR_ADDRESS + sizeof(t_env_var) * (SYS_USER_ENVIRONMENT_MAX_INDEX + 2))

//static t_env_var       	env_vars[SYS_USER_ENVIRONMENT_MAX_INDEX + 2];
//static t_env_var_info  	env_var_info[SYS_USER_ENVIRONMENT_MAX_INDEX + 1];
static t_env_var       	*env_vars = (t_env_var *)ENV_VAR_ADDRESS;
static t_env_var_info  	*env_var_info = (t_env_var_info *)ENV_VAR_INFO_ADDRESS;

unsigned char env_setup_env_panel(uchar *panel_filename);

#ifdef CONFIG_CUSTOM_SN_PANEL
unsigned char env_setup_env_panel_sn(uchar *panel_filename);
#endif


extern unsigned int env_setup_env_pcb(void);

void *env_get_config_boot_param_base(void)
{
	return (((char *)&envp_array) + 0x100);
}


/*
 * Get export env pointer
 */
t_env_var *env_get_export_all(void)
{
	return env_vars;
}

/*
 * export env
 */
int env_export(void)
{
	char *buf = NULL;
	int len = 0;
	int buf_i = 0, data_j =0;
	int index = 0;
	unsigned char bParseName = 1; /* when parsing name, we should find the first '=' */

	env_vars = (t_env_var *)ENV_VAR_ADDRESS;
	env_var_info = (t_env_var_info *)ENV_VAR_INFO_ADDRESS;

	memset(env_vars, 0, sizeof(t_env_var) * (SYS_USER_ENVIRONMENT_MAX_INDEX + 2));
	memset(env_var_info, 0, sizeof(t_env_var_info) * (SYS_USER_ENVIRONMENT_MAX_INDEX + 1));

	len = hexport_r(&env_htab, '\0', &buf, 0, 0, NULL);
	if (len < 0) {
		error("%s: Cannot export environment: errno = %d\n", __func__, errno);
		return -1;
	}

	/* Parse export string */
	/* Ex. name1=value1\0name2=value2\0name3=value3\0name4= ... \0\0 */
	for (buf_i = 0;buf_i < len && buf[buf_i] != '\0';)
	{
		data_j = 0;
		bParseName = 1;

		/* Match an env item */
		while (buf[buf_i] != '\0') {

			if (bParseName && buf[buf_i] == '=') {
				/* Separate name and value */
				env_var_info[index].data[data_j] = '\0';
				env_vars[index].name	= &(env_var_info[index].data[0]);
				env_vars[index].val	= &(env_var_info[index].data[data_j + 1]); /* skip '=' */

				bParseName = 0; /* change to 0 that means start to parse value */
			}
			else {
				/* Save data */
				env_var_info[index].data[data_j] = buf[buf_i];
			}

			data_j++;
			buf_i++;
		}

		/* data terminate */
		env_var_info[index].data[data_j] = '\0';

		index++;
		buf_i++;

		if (index >= (SYS_USER_ENVIRONMENT_MAX_INDEX + 1)) {
			printf("%s: Over max index (%d,%d)\n", __func__, index, (SYS_USER_ENVIRONMENT_MAX_INDEX + 1));
			break;
		}
	}

	/* env terminate */
	env_vars[index].name = NULL;
	env_vars[index].val = NULL;

	free(buf);

#if 0 // dump result
	index = 0;

	printf("dump env_export() result\n");

	while ((env_vars[index].name != NULL) && (env_vars[index].val != NULL))
	{
		printf("[%d] %s %s\n", index, env_vars[index].name, env_vars[index].val);

		index++;
	}
#endif

	return len;
}
#if defined(CONFIG_ENV_IS_NOWHERE)
int saveenv(void)
{
    return 0;
}
#endif
void env_relocate(void)
{
#ifdef CONFIG_SYS_RTK_NAND_FLASH
//	int dev = nand_curr_device;
//	nand_info_t *nand =&nand_info[dev]; 	//nand_info[0];
//	struct nand_chip *this = (struct nand_chip *) nand->priv;
#endif
	char		*raw = NULL;
	int			i = 0;
	char		*msg;
//    unsigned int size;
#if defined(CONFIG_REALTEK_EERPOM_TV010_A)
#elif defined(CONFIG_SYS_FACTORY_READ_ONLY)
	char		 mac_ptr[18] = {0};
#if defined(CONFIG_CUSTOMER_TV002)
#ifdef Config_EXT4_Load_Takeover_TRUE
    char		*file_buf;
    int 	   buf_len;
#endif
#else
	char		*file_buf;
    int 	   buf_len;
#endif
	int 		rc = 0;

#endif

	msg = (char *)malloc(128);

#if defined(CONFIG_NEEDS_MANUAL_RELOC)
	env_reloc();
#endif
	if (gd->env_valid == 0) {
#if defined(CONFIG_ENV_IS_NOWHERE)	/* Environment not changable */
		set_default_env(NULL);
#else
		bootstage_error(BOOTSTAGE_ID_NET_CHECKSUM);
		set_default_env("!bad CRC");
#endif
	} else {
		env_relocate_spec();
	}


	if (board_version == BOARD_C)
	{
		setenv("mmc_param_1", "emmc_en_hs400");
		setenv("emmc_clock", "200000000");
	}


	if(getenv("print") == NULL)
	{
#ifndef Config_Boot_Type_RELEASE
		setenv("print", "on");
#else
		setenv("print", "off");
#endif
#if defined(CONFIG_CMD_SAVEENV) && !defined(CONFIG_ENV_IS_NOWHERE)
		saveenv();
#endif
	}

	if (!strcmp(getenv("print"), "off")) 
	{
		disable_console();
		setenv("silent", "1");
#ifdef CONFIG_REALTEK_EERPOM_TV010_A
// tv010 UART1 use for factory tool
		//*uart_reg_base_ptr = SWAPEND32(UART1_BASE | 0xb0000000);
#else
		*uart_reg_base_ptr = SWAPEND32(UART1_BASE | 0xb0000000);
#endif
	}
	else 
	{
		enable_console();
		setenv("silent", NULL);
		display_options(); /* let user can see the U-boot version info */
	}

/*
 *********************************************************
 * Realtek Patch:
 *    Assign platform related environment variables.
 *********************************************************
 */

	/* Bootcode revision */
	memset(sys_chiprev_str, 0x0, 64);
	if (strlen(BOOT_REV_STRING) < 64)
		strcpy(sys_chiprev_str, BOOT_REV_STRING);
	else
		strncpy(sys_chiprev_str, BOOT_REV_STRING, 63);

	for (i = 0; i < 63; i++)
	{
		if( (sys_chiprev_str[i] == 0x58) || (sys_chiprev_str[i] == 0x78)) //rev_str[i] == 'X' || 'x'
		{
			/* FIXME */
			sys_chiprev_str[i] = 0x32;
			break;
		}
	}

	if (setenv("bootrev", sys_chiprev_str))
		;

#if defined(CONFIG_REALTEK_EERPOM_TV010_A)
	if(boot_power_on_mode != POWER_OFF)
	{
// mark it, change to panel.c's panel_misc()
//		Eeprom_SetMacAddressToKernel_Read();
//		Eeprom_SetMacAddressToKernel_Apply();
	}
#elif defined(CONFIG_SYS_FACTORY_READ_ONLY)
	if(boot_power_on_mode != POWER_OFF) {

#if defined(CONFIG_MAC_ADDRESS_FROM_FACTORY)
		/* Mac Address from factory */
#if defined(CONFIG_CUSTOMER_TV002) //factory XML format
#ifdef Config_EXT4_Load_Takeover_TRUE
				rc = takeover_read(TAKEOVER_MAC_XML_BIN, &file_buf, &buf_len);
#endif
#else	//factory TXT format
				rc = factory_read(FACTORY_MAC_BIN, &file_buf, &buf_len);
#endif
#else	//factory_ro txt format
				rc = factory_ro_read(FACTORY_RO_MAC_BIN, &file_buf, &buf_len);
#endif
				
				/* Mac Address parse */
				if(!rc) {
#if defined(CONFIG_CUSTOMER_TV002) //factory XML format
#ifdef Config_EXT4_Load_Takeover_TRUE
					if(CusApi_env_readxml_key_value_from_buf(file_buf,"macAddress", mac_ptr,sizeof(mac_ptr)-1))
					{
						printf("fail to parse %s\n",TAKEOVER_MAC_XML_BIN);
					}		
#endif
#else
					memcpy(mac_ptr, file_buf, sizeof(mac_ptr)-1);
#endif
		
				/* Mac Address save to env */
					memset(msg, 0, 20); // real length is 17
					strncpy(msg, mac_ptr,sizeof(mac_ptr)-1);			
					if (!setenv("ethaddr", msg))
						printf("Mac Address use %s in factory\n", msg);
					else
						printf("Mac Address use %s in env\n", getenv("ethaddr"));
				}
			}	 
#endif


	/* Handle bootargs use ttyS0 or ttyS1 issue */
	setenv("bootargs", CONFIG_BOOTARGS);

	/* Prompt */
	if (setenv("prompt", NULL)) // clear prompt
		;

	//for irda
	{
	        char *tmP = NULL;
		int ir_found         = 0;
		int ir_proto_id      = 0;
		char ir_remote_name[64] = {0};
		char ir_value[64]    = {0};
#ifdef Config_Board_TV030 
		char ir_value_proto[20] = {0};
		int ir_i =0;
#endif
		if(NULL==getenv("irda_select_value"))
		{
		    setenv("irda_select_value", "DefaultSetting(none)");
		}
		//printf("roger env_common ir getenv(irda_default_setting)=%s %d\n"
		//		,getenv("irda_default_value")
		//		,(NULL==getenv("irda_default_value")));
		if(strlen(bootparam.ir_default_remoter[0]))
		{
			//printf("roger env_common ir enter bootparam.ir_default_remoter[0]=%s\n"
			//	,bootparam.ir_default_remoter[0]);
                        memcpy(ir_remote_name, bootparam.ir_default_remoter[0], sizeof(bootparam.ir_default_remoter[0]));
                        ir_remote_name[sizeof(bootparam.ir_default_remoter[0])] = 0;
                        tmP = strstr(ir_remote_name, "(");
                        if(tmP != NULL) {
#ifdef Config_Board_TV030 
				ir_i = strlen(tmP);
				strncpy(ir_value_proto ,tmP+1,ir_i-2);
#endif				
                            tmP[0] = 0;
                            if(strlen(ir_remote_name) > 0)
                                ir_found++;
                        }

			for(i=1;i<IRDA_NUM_REMOTER;i++)
			{
#ifdef Config_Board_TV030 
				if(strcmp(irda_proto_list[i].proto_name, ir_value_proto) == 0)
#else
				if(NULL != strstr(bootparam.ir_default_remoter[0],irda_proto_list[i].proto_name))
#endif	
				{
					ir_found++;
					ir_proto_id = irda_proto_list[i].proto_id;
					//printf("roger env_common ir[0] found%d (proto_id)name=%s id=%d\n"
					//		,ir_found
					//		,irda_proto_list[i].proto_name
					//		,irda_proto_list[i].proto_id);
					break;
				}
			}


			//if ir_found < 2, use default remoter
			if(2 <= ir_found)
			{
				memset(ir_value,0,64);
				snprintf(ir_value, 64, "%d-%s",ir_proto_id,ir_remote_name);
				setenv("irda_default_value", ir_value);
			}

			//printf("roger set ir[0] found%d irda_default_value=%s\n"
			//			,ir_found
			//			,ir_value);
			if(strlen(&(bootparam.ir_power_up_key[0][0])))
			{
				setenv("irda_powerup",bootparam.ir_power_up_key[0]);
			}
			if(strlen(&(bootparam.irda_filter[0])))
			{
				setenv("irda_filter",bootparam.irda_filter);
			}
		}
		if(strlen(bootparam.ir_default_remoter[1]))
		{
			ir_found         = 0;
			ir_proto_id      = 0;
                        //printf("roger env_common ir enter bootparam.ir_default_remoter[1]=%s\n"
			//	,bootparam.ir_default_remoter[1]);
                        memcpy(ir_remote_name, bootparam.ir_default_remoter[1], sizeof(bootparam.ir_default_remoter[1]));
                        ir_remote_name[sizeof(bootparam.ir_default_remoter[1])] = 0;
                        tmP = strstr(ir_remote_name, "(");
                        if(tmP != NULL) {
                            tmP[0] = 0;
                            if(strlen(ir_remote_name) > 0)
                                ir_found++;
                        }

			//printf("roger env_common ir[1] enter bootparam.ir_default_remoter[1]=%s\n"
			//	,bootparam.ir_default_remoter[1]);
			for(i=1;i<IRDA_NUM_REMOTER;i++)
			{
				if(NULL != strstr(bootparam.ir_default_remoter[1]
							,irda_proto_list[i].proto_name))
				{
					ir_found++;
					ir_proto_id = irda_proto_list[i].proto_id;
					//printf("roger env_common ir[1] found%d (proto_id)name=%s id=%d\n"
					//		,ir_found
					//		,irda_proto_list[i].proto_name
					//		,irda_proto_list[i].proto_id);
					break;
				}
			}

                        

			//if ir_found < 2, use default remoter
			if(2 <= ir_found)
			{
				memset(ir_value,0,64);
				snprintf(ir_value, 64, "%d-%s"
						,ir_proto_id,ir_remote_name);
				setenv("irda_default_value1", ir_value);
			}
			else if (0==strcmp(bootparam.ir_default_remoter[1],"DefaultSetting(none)"))
				setenv("irda_default_value1", "DefaultSetting(none)");

			if(strlen(&(bootparam.ir_power_up_key[1][0]))
				&& 0!=strcmp(bootparam.ir_power_up_key[1],"none"))
			{
				setenv("irda_powerup1",bootparam.ir_power_up_key[1]);
			}
		}
	}

	// Tuner selection
	// Get from bootparam(ini setting) first
	char tp_id[50];
	char tp_default_config[50];
	int tp_index;
	char* g_default_config;
	for (tp_index = 0; tp_index < TP_MAX_NUM; tp_index++)
	{
		memset(tp_default_config, 0, 50);
		snprintf(tp_default_config, 100, "default_boot_TP%d", tp_index);
		g_default_config = getenv(tp_default_config);
		if(g_default_config != NULL){
			if (strcmp(g_default_config, "y") == 0)
			{
				// aplly default bootcode setting; do nothing
				continue;
			}
		}else{
			printf("\n\033[1;33m<<<env_relocate g_default_config ==null>>>\033[m\n");
		}

		if (strlen(bootparam.tp_fe_setting[tp_index]))
		{
			memset(tp_id, 0, 50);
			snprintf(tp_id, 50, "TP%d_FE", tp_index);
			// In the DEFAULT case, set NULL to clear env value
			if (strcmp(bootparam.tp_fe_setting[tp_index], "DEFAULT") == 0)
				setenv(tp_id, NULL);
			else
				setenv(tp_id, bootparam.tp_fe_setting[tp_index]);
			printf("%s %s\n", tp_id, bootparam.tp_fe_setting[tp_index]);
		}
	}

#if 0 // BUG, wait for jerry fix
	if(!strcmp(bootparam.ethaddr," 10:20:30:40:50:60 \n"))
	{
             printf("MAC_ADDR not setting\n");
	}
	else
	{
		char *macaddr;
		macaddr = strim(bootparam.ethaddr);
		setenv("ethaddr",macaddr);
	}
#endif

#ifdef CONFIG_SYS_BOOT_PARM
	if (strlen((char *)bootparam.install_file_name) > 0)
		sprintf( msg, "tv_system=%s imgfile_pattern=%s",
			SYS_PARAM_TV_SYSTEM,
			bootparam.install_file_name
		);
	else
#endif
		sprintf( msg, "tv_system=%s",
			SYS_PARAM_TV_SYSTEM
		);

	/* system_parameters_1 pass parameter start address to kernel */
	raw = msg; /* Always overwrite parameters with default settings. */

	if (setenv("system_parameters_1", raw))
		;

    /* This is for telling the installer what target flash type and flash info */
	sprintf( msg, "boot_flash=%s factory_start=%x factory_size=%x",
		SYS_PARAM_BOOT_FLASH,
		SYS_PARAM_FACTORY_START,
		SYS_PARAM_FACTORY_SIZE
	);
	printf("Factory Address:0x%x, Size:0x%x!\n", SYS_PARAM_FACTORY_START, SYS_PARAM_FACTORY_SIZE);

	if (setenv("system_parameters_2", raw))
		;

#if defined CONFIG_SYS_RTK_EMMC_FLASH
	sprintf(raw, "bootcode_start=%x bootcode_size=%x ss_start=%x ss_size=%x reserved_size=%x",
		SYS_PARAM_BOOTCODE_START,
		SYS_PARAM_BOOTCODE_SIZE,
		SYS_PARAM_SECURE_STORE_START,
		SYS_PARAM_SECURE_STORE_SIZE,
		SYS_PARAM_RESERVED_SIZE
	);
#elif defined(CONFIG_SYS_RTK_NAND_FLASH) || defined(CONFIG_SYS_RTK_SPI_NAND)
#ifdef CONFIG_SAFE_UPGRADE
		sprintf(raw, "bootcode_start=%x bootcode_size=%x ss_start=%x ss_size=%x reserved_size=%x ssu=1",
			SYS_PARAM_BOOTCODE_START,
			SYS_PARAM_BOOTCODE_SIZE,
			SYS_PARAM_SECURE_STORE_START,
			SYS_PARAM_SECURE_STORE_SIZE,
			SYS_PARAM_RESERVED_SIZE
		);

#else
		sprintf(raw, "bootcode_start=%x bootcode_size=%x ss_start=%x ss_size=%x reserved_size=%x",
			SYS_PARAM_BOOTCODE_START,
			SYS_PARAM_BOOTCODE_SIZE,
			SYS_PARAM_SECURE_STORE_START,
			SYS_PARAM_SECURE_STORE_SIZE,
			SYS_PARAM_RESERVED_SIZE
		);
#endif

#elif defined CONFIG_SYS_RTK_SPI_FLASH
	sprintf(raw, "bootcode_start=%x bootcode_size=%x ss_start=%x ss_size=%x reserved_size=%x",
		SYS_PARAM_BOOTCODE_START,
		SYS_PARAM_BOOTCODE_SIZE,
		SYS_PARAM_SECURE_STORE_START,
		SYS_PARAM_SECURE_STORE_SIZE,
		SYS_PARAM_RESERVED_SIZE
	);
#endif

	if (setenv("system_parameters_3", raw))
		;

	/* system_parameters_4 pass bootpart parameter to kernel */
#if defined(CONFIG_SYS_RTK_EMMC_FLASH) || defined(CONFIG_SYS_RTK_SPI_FLASH)
	sprintf( msg, "boot_part=%d factory_ro_start=%x factory_ro_size=%x eeprom_start=%x eeprom_size=%x",
			boot_part,
			SYS_PARAM_FACTORY_RO_START,
			SYS_PARAM_FACTORY_RO_SIZE,
			SYS_PARAM_FACTORY_EEPROM_START,
			SYS_PARAM_FACTORY_EEPROM_SIZE
	);
#else
	sprintf( msg, "boot_part=%d factory_ro_start=%x factory_ro_size=%x",
			boot_part,
			SYS_PARAM_FACTORY_RO_START,
			SYS_PARAM_FACTORY_RO_SIZE
	);
#endif

	printf("Factory Read Only Address:0x%x, Size:0x%x!\n", SYS_PARAM_FACTORY_RO_START, SYS_PARAM_FACTORY_RO_SIZE);

	if (setenv("system_parameters_4", raw))
		;

	/* system_parameters_5 pass emcu and rescue parameter to kernel */
#ifdef Config_EXT4_Factory_Enable_TRUE
	sprintf( msg, "emcu_s=%x rescue_s=%x fac_ext4=1", SYS_PARAM_EMCU_START,SYS_PARAM_RESCUE_START);
#else
	sprintf( msg, "emcu_s=%x rescue_s=%x", SYS_PARAM_EMCU_START,SYS_PARAM_RESCUE_START);
#endif

	if (setenv("system_parameters_5", raw))
		;

#if defined(Config_SELinux_Disable)
	/* set SELinux disable to bootargs_ex parameter */
	char *msg1;
	msg1 = (char *)malloc(100);
	memset(msg1, 0, 100);
	strcat(msg1, "no_console_suspend androidboot.selinux=disabled");
	setenv("bootargs_ex", msg1);
#endif

#ifdef CONFIG_REALTEK_EERPOM_TV010_A
	char *SetCustomIrCodeCmd = (char *)malloc(128);
	unsigned char tmpIrIndex = 0x0E;
	memset(SetCustomIrCodeCmd,0,128);
	getCustomIrCode(&tmpIrIndex);
	sprintf(SetCustomIrCodeCmd,"%4X",ConvertIrCode(tmpIrIndex));
	setenv("Custom_Ir_Code", SetCustomIrCodeCmd);
	free(SetCustomIrCodeCmd);
	SetCustomIrCodeCmd = NULL;
#endif

	CusApi_env_relocate(); // call customized function

#ifdef CONFIG_SYS_PANEL_PARAMETER
	/* panel parameters setting */
  #ifdef PANEL_CONFIG_FROM_FACTORY_PANEL_BIN
    #ifdef CONFIG_REALTEK_EERPOM_TV010_A
	char *dst_addr;
	int dst_length = 0;
	unsigned int tmpindex = 0;
	char tmppanelname[50];
	char tmppanelbinpath[128];

	getPanelIndex(&tmpindex);
	int rc = factory_read(PANEL_INI_FILE_NAME, &dst_addr, &dst_length);
	if ((!rc) && (0 != dst_length)) {
		memset(tmppanelname,0,50);
		getPanelNameFromIniByIndex(dst_addr,dst_length,tmpindex,tmppanelname);
		memset(tmppanelbinpath,0,128);
		sprintf(tmppanelbinpath,"%s%s_panel.bin",PANEL_BIN_FILE_PREFIX,tmppanelname);
		printf("[%s]get tmppanelbinpath: %s\n",__FUNCTION__,tmppanelbinpath);
		env_setup_env_panel((uchar *)tmppanelbinpath);
	}
	else {
		env_setup_env_panel((uchar *)PANEL_BIN_FILENAME_IN_FACTORY); // assign panel.bin value
	}
    #else
	env_setup_env_panel((uchar *)PANEL_BIN_FILENAME_IN_FACTORY); // assign panel.bin value
    #endif
  #else
    #ifdef CONFIG_CUSTOM_SN_PANEL
    env_setup_env_panel_sn((uchar *)PANEL_BIN_FILENAME_IN_FACTORY); 
    #elif defined(CONFIG_SYS_PANEL_SUPPORT_INI_FILE)
	env_setup_env_panel((uchar *)PANEL_INI_FILE_NAME);
	#else
    env_setup_env_panel((uchar *)NULL); // assign default value
    #endif
  #endif
#endif

#ifdef CONFIG_SYS_PCB_MANAGER
	/* PCB Manager */
	env_setup_env_pcb();
#endif
        if(getenv("dvfs_low") == NULL)
		setenv("dvfs_low", "0xC6"); // init dvfs to 2
	
        if(getenv("dvfs_high") == NULL)
		setenv("dvfs_high", "0xFD"); // init dvfs to 2

	free(msg);
}

#ifdef CONFIG_AUTO_COMPLETE
int env_complete(char *var, int maxv, char *cmdv[], int bufsz, char *buf)
{
	ENTRY *match;
	int found, idx;

	idx = 0;
	found = 0;
	cmdv[0] = NULL;

	while ((idx = hmatch_r(var, idx, &match, &env_htab))) {
		int vallen = strlen(match->key) + 1;

		if (found >= maxv - 2 || bufsz < vallen)
			break;

		cmdv[found++] = buf;
		memcpy(buf, match->key, vallen);
		buf += vallen;
		bufsz -= vallen;
	}

	qsort(cmdv, found, sizeof(cmdv[0]), strcmp_compar);

	if (idx)
		cmdv[found++] = "...";

	cmdv[found] = NULL;
	return found;
}
#endif
