#include <common.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>

#include <drivers/lsadc/lsadc.h>
#include <platform_lib/board/pcb_mgr.h>
#include <platform_lib/board/pcb.h>

#include "cus_common.h"

unsigned long long last_lsadc_key = 0;

#if CONFIG_ANDROID_RECOVERY
#define BOOT_RECOVERY_FILE_NAME	FACTORY_HEADER_FILE_NAME"recovery"

typedef struct _bootloader_message {
    char command[32];
    char status[32];
    char recovery[1024];
} bootloader_message;

void writeRecoveryCrashCmd(void)
{
	factory_delete(BOOT_RECOVERY_FILE_NAME);

	bootloader_message boot;
	memset(&boot, 0, sizeof(boot));
	sprintf(boot.command, "boot-recovery");
	sprintf(boot.recovery,"recovery\n--system_crash");

	printf("writeRecoveryCrashCmd, command: %s \n", boot.command);
	printf("writeRecoveryCrashCmd, recovery: %s \n", boot.recovery);

	factory_write(BOOT_RECOVERY_FILE_NAME, (char *)&boot, sizeof(boot));
	factory_save();
}
void writeRecoveryWipeCmd(void)
{
	factory_delete(BOOT_RECOVERY_FILE_NAME);

	bootloader_message boot;
	memset(&boot, 0, sizeof(boot));
	sprintf(boot.command, "boot-recovery");
	sprintf(boot.recovery,"recovery\n--wipe_data");

	printf("writeRecoveryWipeCmd, command: %s \n", boot.command);
	printf("writeRecoveryWipeCmd, recovery: %s \n", boot.recovery);

	factory_write(BOOT_RECOVERY_FILE_NAME, (char *)&boot, sizeof(boot));
	factory_save();
}
#endif

bool check_lsadc_input(unsigned long long Lsadc_Key_info)
{
	int index = -1;
	int hitvalue = -1;
	int torrence  = -1;
	int lsadc_key = -1;

	index     = GET_PIN_INDEX(Lsadc_Key_info);
	hitvalue  = GET_LSADC_HITVALUE(Lsadc_Key_info);
	torrence  = GET_LSADC_TOLERANCE(Lsadc_Key_info);
	lsadc_key = lsadc_get(index);

	if(ABS(lsadc_key, hitvalue) < torrence)
		return true;

	return false;
}

void upgrade_led_ctrl(void)
{
#if 1
//refine by Ravi  
//Auxcode disbale EMCU clk by CLKEN=0 > RSTN=0
	rtd_outl(REG_MAGIC_51, 0);
	rtd_outl(REG_EMCU_CLKEN, _BIT9);
	rtd_outl(REG_EMCU_RESET, _BIT9);

#endif

#ifdef PIN_LED_ON_OFF
	mdelay(100);
	IO_PWM_SetFreq(PIN_LED_ON_OFF, 3); //Timing_CTRL
	IO_PWM_SetDuty(PIN_LED_ON_OFF, 0x80); //pwm_set
#endif	
}


void CusApi_main_loop_before_enter_rescue(void)
{
	upgrade_led_ctrl();
}

BOOT_MODE CusApi_abortboot(void)
{
	BOOT_MODE ret = BOOT_NORMAL_MODE;

#if CONFIG_ANDROID_RECOVERY
	unsigned long long lsadc_key_info;
	int nConfirmCount=0;
	
	//if( boot_ac_on == 1 )
	{
		printf("Check Recovery Mode\n");
		
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_PWR", &lsadc_key_info) == 0)
		{
			if(lsadc_key_info != 0)
			{	
				if(check_lsadc_input(lsadc_key_info))
				{
					if(last_lsadc_key != lsadc_key_info)
					{
						last_lsadc_key = lsadc_key_info;
						nConfirmCount = 0;
					}
					
					if(++nConfirmCount==5)
					{
						//writeRecoveryCmd();
						setenv("rescue_cmd", "go r -tab");
						ret = BOOT_RESCUE_MODE;
					}
					udelay(10000);
				}else {
					nConfirmCount=0;
				}
			}
		}
		else if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_UP", &lsadc_key_info) == 0)
		{
			if(lsadc_key_info != 0)
			{
				if(check_lsadc_input(lsadc_key_info))
				{
					if(last_lsadc_key != lsadc_key_info)
					{
						last_lsadc_key = lsadc_key_info;
						nConfirmCount = 0;
					}
					
					if(++nConfirmCount==5)
					{
						writeRecoveryWipeCmd();
						setenv("rescue_cmd", "go r -tab");
						ret = BOOT_RESCUE_MODE;	
					}
					udelay(10000);
				}else {
					nConfirmCount=0;
				}
			}
		}
		else if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_DOWN", &lsadc_key_info) == 0)
		{
			if(lsadc_key_info != 0)
			{
				if(check_lsadc_input(lsadc_key_info))
				{
					if(last_lsadc_key != lsadc_key_info)
					{
						last_lsadc_key = lsadc_key_info;
						nConfirmCount = 0;
					}

					if(++nConfirmCount==5)
					{
						writeRecoveryCrashCmd();
						setenv("rescue_cmd", "go r -tab");
						ret = BOOT_RESCUE_MODE;
					}
					udelay(10000);
				}else {
					nConfirmCount=0;
				}
			}
		}
	}
#endif

    return ret;
}
