#include <qc_verify_common.h>
#include <drivers/mcp/rtk_mcp_scpu.h>

#if 0 // Copy form "uboot/common/cmd_rtk_cp.c"

/*----------------------------------------------------------------------
 * Func : mcp_dump_mem
 *
 * Desc : dump data in memory
 *
 * Parm : data : start address of data
 *        len  : length of data
 *
 * Retn : N/A
 *----------------------------------------------------------------------*/
void mcp_dump_mem(
	unsigned char* 		data,
	unsigned int            len )
{
    int i;
    for (i=0; i<len; i++)
    {
        if ((i & 0xF)==0)
            printf(" %04x | ", i);
        printf("%02x ", data[i]);
    }
    printf("\n");
}


#define mcp_dump_data_with_text(data, len ,fmt, args...)\
do {\
        printf(fmt, ## args);\
        mcp_dump_mem(data, len);\
}while(0)

int PrintCheckReuslt(unsigned int enc, unsigned char* pRefData, unsigned char* pOutData, unsigned int OutDataLen, const char *msg, const char *fun){
/*
    if(enc == 1){
	mcp_dump_data_with_text(pRefData, OutDataLen, "Expect  : ");
	mcp_dump_data_with_text(pOutData, OutDataLen, "Encrypt : ");
    }else{
 	mcp_dump_data_with_text(pOutData, OutDataLen, "Decrypt : ");
    }
*/
    if (strncmp(pRefData, pOutData, OutDataLen) != 0 ) /* output not equual to except output */
    {
    	printf("\033[0;32;31m""%s %s %s Fail!\n""\033[m", msg, fun, enc? "Enc":"Dec");
        return 1;
    }
    else
    {
        printf("\033[0;32;34m""%s %s %s Sucess!\n""\033[m", msg, fun, enc? "Enc":"Dec");
    }

    return 0;
}

/*------------------------------------------------------------------
* Func : MCP_DES_ECB_Test
*
* Desc : DES ECB Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_DES_ECB_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xa3, 0x2a, 0xd8, 0xac, 0xd2, 0x0c, 0x38, 0xfc, 0x28, 0x51, 0x07, 0x7e, 0x16, 0xf6, 0x38, 0x75};
    unsigned char  Key[7];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= DES ECB ===============================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    DES_ECB_encrypt(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[1.1.1]", __FUNCTION__);

    DES_ECB_decrypt(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[1.1.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_DES_CBC_Test
*
* Desc : DES CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_DES_CBC_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x03, 0xd3, 0x76, 0x73, 0xa2, 0x79, 0x50, 0xd1, 0x76, 0x7a, 0x78, 0x3c, 0x47, 0x5b, 0x6a, 0x0a};
    unsigned char  Key[7];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= DES CBC ===============================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    DES_CBC_encrypt_IV(Data, BlockSize, EncData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[1.2.1]", __FUNCTION__);

    DES_CBC_decrypt_IV(EncData, BlockSize, DecData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[1.2.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_TDES_ECB_Test
*
* Desc : TDES ECB Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_TDES_ECB_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xa3, 0x2a, 0xd8, 0xac, 0xd2, 0x0c, 0x38, 0xfc, 0x28, 0x51, 0x07, 0x7e, 0x16, 0xf6, 0x38, 0x75};
    unsigned char  Key[21];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= TDES ECB ==============================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    TDES_ECB_encrypt(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[2.1.1]", __FUNCTION__);

    TDES_ECB_decrypt(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[2.1.2] ", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_TDES_CBC_Test
*
* Desc : TDES CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_TDES_CBC_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x03, 0xd3, 0x76, 0x73, 0xa2, 0x79, 0x50, 0xd1, 0x76, 0x7a, 0x78, 0x3c, 0x47, 0x5b, 0x6a, 0x0a};
    unsigned char  Key[21];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= TDES CBC ==============================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    TDES_CBC_encrypt_IV(Data, BlockSize, EncData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[2.2.1] ", __FUNCTION__);

    TDES_CBC_decrypt_IV(EncData, BlockSize, DecData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[2.2.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_ECB_Test
*
* Desc : AES-128 ECB Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_128_ECB_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x45, 0xd7, 0x4d, 0xac, 0xb3, 0x52, 0xf9, 0x3f, 0xd3, 0x09, 0xbf, 0x8b, 0x91, 0x59, 0xdf, 0x58};
    unsigned char  Key[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-128 ECB ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_ECB_encrypt(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.1.1]", __FUNCTION__);

    AES_ECB_decrypt(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.1.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_128_ECB_CW_Test
*
* Desc : AES-128 ECB CW Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/

int MCP_AES_128_ECB_CW_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x9a, 0x1f, 0xe1, 0xf0, 0xe8, 0xb0, 0xf4, 0x9b, 0x5b, 0x42, 0x16, 0xae, 0x79, 0x6d, 0xa0, 0x62};
    unsigned char  Key[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= AES-128 ECB CW ========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_ECB_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.1.1]", __FUNCTION__);

    AES_ECB_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.1.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_CBC_Test
*
* Desc : AES-128 CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_128_CBC_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x9a, 0x6c, 0x86, 0xce, 0xc3, 0x31, 0xdd, 0xb0, 0x14, 0x2c, 0xfb, 0x37, 0x06, 0x25, 0xbf, 0xfe};
    unsigned char  Key[16];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-128 CBC ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CBC_encrypt_IV(Data, BlockSize, EncData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.2.1]", __FUNCTION__);

    AES_CBC_decrypt_IV(EncData, BlockSize, DecData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.2.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_CBC_Test
*
* Desc : AES-128 CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_128_CBC_NO_IV_Test(void) //error
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x45, 0xd7, 0x4d, 0xac, 0xb3, 0x52, 0xf9, 0x3f, 0xd3, 0x09, 0xbf, 0x8b, 0x91, 0x59, 0xdf, 0x58};
    unsigned char  Key[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-128 CBC NO IV ====================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CBC_encrypt(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.2.1]", __FUNCTION__);

    AES_CBC_decrypt(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.2.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_CBC_CW_Test
*
* Desc : AES-128 CBC CW Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/

int MCP_AES_128_CBC_CW_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x83, 0x33, 0x82, 0x32, 0x2e, 0x08, 0xb6, 0xce, 0xb6, 0x9e, 0x2e, 0x0a, 0x1d, 0xcb, 0x81, 0x30};
    unsigned char  Key[16];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= AES-128 CBC CW ========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CBC_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.2.1]", __FUNCTION__);

    AES_CBC_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.2.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_CTR_CW_Test
*
* Desc : AES-128 CTR Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_128_CTR_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xd7, 0x9d, 0xe0, 0x8a, 0xbe, 0xa9, 0xa2, 0xc8, 0xf7, 0xf1, 0x85, 0xc7, 0x72, 0x57, 0x2e, 0xb5};
    unsigned char  Key[16];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= AES-128 CTR ===========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CTR_encrypt_IV(Data, BlockSize, EncData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.3.1]", __FUNCTION__);

    AES_CTR_decrypt_IV(EncData, BlockSize, DecData, (unsigned int *)Key, (unsigned int *)IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.3.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_128_CBC_CW_Test
*
* Desc : AES-128 CBC CW Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_128_CTR_CW_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x7d, 0xb9, 0x71, 0xac, 0x60, 0x11, 0xff, 0x4d, 0xfb, 0xd1, 0x37, 0x41, 0x7b, 0xbd, 0x66, 0xf5};
    unsigned char  Key[16];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================= AES-128 CTR CW ========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CTR_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[3.3.1]", __FUNCTION__);

    AES_CTR_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[3.3.2]", __FUNCTION__);

    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_AES_192_ECB_Test
*
* Desc : AES-192 ECB Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_192_ECB_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x3f, 0xf7, 0xcf, 0xbb, 0x1b, 0xde, 0x3b, 0xf9, 0xf4, 0x35, 0x76, 0xd3, 0xfb, 0xc7, 0xc0, 0x71};
    unsigned char  Key[24];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-192 ECB ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_ECB_192_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[4.1.1]", __FUNCTION__);

    AES_ECB_192_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[4.1.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_192_CBC_Test
*
* Desc : AES-192 CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_192_CBC_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x35, 0x6c, 0xf4, 0x8b, 0x96, 0xd6, 0x37, 0x1f, 0x01, 0x35, 0x8f, 0xeb, 0x10, 0x57, 0x87, 0x61};
    unsigned char  Key[24];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-192 CBC ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CBC_192_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[4.2.1]", __FUNCTION__);

    AES_CBC_192_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[4.2.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_192_CTR_Test
*
* Desc : AES-192 CTR Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_192_CTR_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xd6, 0xf9, 0x82, 0x2f, 0xe4, 0x45, 0x84, 0x34, 0x66, 0x2b, 0x47, 0x32, 0x16, 0x5f, 0x4c, 0x27};
    unsigned char  Key[24];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-192 CTR ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CTR_192_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[4.3.1]", __FUNCTION__);

    AES_CTR_192_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[4.3.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_256_ECB_Test
*
* Desc : AES-256 ECB Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_256_ECB_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0x1d, 0xe1, 0xf2, 0x24, 0x63, 0x73, 0x35, 0x27, 0x9c, 0x16, 0xc4, 0x37, 0x80, 0x7d, 0xd6, 0x08};
    unsigned char  Key[32];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-256 ECB ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_ECB_256_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[5.1.1]", __FUNCTION__);

    AES_ECB_256_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[5.1.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_256_CBC_Test
*
* Desc : AES-256 CBC Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_256_CBC_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xb6, 0x5f, 0x06, 0x43, 0x31, 0x4a, 0x8b, 0x3a, 0x2d, 0x1a, 0x2b, 0x8f, 0xbe, 0x98, 0x54, 0xc9};
    unsigned char  Key[32];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-256 CBC ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CBC_256_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[5.2.1]", __FUNCTION__);

    AES_CBC_256_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[5.2.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_AES_256_CTR_Test
*
* Desc : AES-256 CTR Encrypt Decrypt Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_AES_256_CTR_Test(void)
{
    unsigned int   BlockSize = 16;
    unsigned char  Data[16]={0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00};
    unsigned char  RefD[16]={0xb4, 0x56, 0x81, 0x07, 0x02, 0xe0, 0x7b, 0xc3, 0x05, 0xf2, 0xb6, 0x80, 0x63, 0x82, 0xdd, 0xc7};
    unsigned char  Key[32];
    unsigned char  IV[16];
    unsigned char  EncData[16];
    unsigned char  DecData[16];
    int ret = 0;

    //printf("========================== AES-256 CTR ==========================\n");
    mcp_dump_data_with_text(Data, BlockSize, "Input   : ");
    memset(Key, 0xff, sizeof(Key));
    memset(IV , 0x22, BlockSize);
    memset(EncData, 0x00, BlockSize);
    memset(DecData, 0x00, BlockSize);

    AES_CTR_256_encrypt_CW(Data, BlockSize, EncData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(1, RefD, EncData, BlockSize, "[5.3.1]", __FUNCTION__);

    AES_CTR_256_decrypt_CW(EncData, BlockSize, DecData, (unsigned int *)Key, IV);
    ret += PrintCheckReuslt(0, Data, DecData, BlockSize, "[5.3.2]", __FUNCTION__);

    return ret;
}
/*------------------------------------------------------------------
* Func : MCP_SHA1_HASH_Test
*
* Desc : SHA1 HASH Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_SHA1_HASH_Test(void)
{
    unsigned int error = 0;
    unsigned char EncData[20]; // hash value calculated by CP engine
    unsigned int pad_sz;
    unsigned char RefD[] = {0x44, 0xee, 0x7a, 0x16, 0x54, 0xda, 0xfe, 0x62, 0x9f, 0xe0, 0xa3, 0xd8, 0x5a, 0x00, 0x2b, 0x18, 0xcd, 0xd6, 0xc2, 0x45};
    unsigned char Data[256 + 64];
    int ret = 0;

    //printf("========================== SHA1 HASH ============================\n");

    memset(Data, 0x33, 256);
    mcp_dump_data_with_text(Data, 20, "Input   : ");

    pad_sz = SHA1_padding((unsigned char *)&Data[256], 64, 256);

    flush_dcache_all();

    error = SHA1_hash((unsigned char *)Data, 256 + pad_sz, (unsigned char *)EncData, NULL);
    if( error ) {
	printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
	return 1;
    }

    ret = PrintCheckReuslt(1, RefD, EncData, 20, "[7.1]", __FUNCTION__);
    return ret;
}

/*------------------------------------------------------------------
* Func : MCP_SHA256_HASH_Test
*
* Desc : SHA256 HASH Test
*
* Parm : Void
*
* Retn : Fail Count
*------------------------------------------------------------------*/
int MCP_SHA256_HASH_Test(void)
{
    unsigned int error = 0;
    unsigned char EncData[32]; // hash value calculated by CP engine
    unsigned int length=32;
    unsigned char RefD[] = {0xde, 0xb0, 0xe3, 0x8c, 0xed, 0x1e, 0x41, 0xde, 0x6f, 0x92, 0xe7, 0x0e, 0x80, 0xc4, 0x18, 0xd2, 0xd3, 0x56, 0xaf, 0xaa};
    unsigned char Data[256];
    int ret = 0;
    //printf("========================== SHA-256 HASH =========================\n");

    memset(Data, 0x33, 256);
    mcp_dump_data_with_text(Data, 20, "Input   : ");

    length += SHA256_padding((unsigned char *)Data+length, length);

    //EncData = (unsigned char *)SECURE_IMAGE2HASH_BUF; // temp use this address (use malloc is better?)
    flush_dcache_all();

    error = SHA256_hash((unsigned char *)Data, length, (unsigned char *)EncData, NULL);
    if( error ) {
 	printf("[ERR] %s: caculate hash1 fail(%d)\n", __FUNCTION__, error );
	return 1;
    }
    ret = PrintCheckReuslt(1, RefD, EncData, 20, "[7.2]", __FUNCTION__);
    return ret;
}
#endif

typedef struct _MCPTestItem
{
    int (*func)(void);
    char TestName[64];
} TestItem;

#if CONFIG_FT

#define TEST_ITEM_NUM       18

TestItem _gMCPTestItem[TEST_ITEM_NUM]=
{
    {MCP_DES_CBC_Test,            "MCP DES CBC Test"},
    {MCP_TDES_CBC_Test,           "MCP TDES CBC Test"},
    {MCP_AES_128_CBC_Test,        "MCP AES 128 CBC Test"},
    {MCP_AES_128_CTR_Test,        "MCP AES 128 CTR Test"},
    {MCP_AES_192_CBC_DDR_Test,    "MCP AES 192 CBC Test"},
    {MCP_AES_192_CTR_DDR_Test,    "MCP AES 192 CTR Test"},
    {MCP_AES_256_CBC_DDR_Test,    "MCP AES 256 CBC Test"},
    {MCP_AES_256_CTR_DDR_Test,    "MCP AES 256 CTR Test"},
    {MCP_CAM_128_CBC_Test,        "MCP CAM 128 CBC Test"},
    {MCP_CAM_128_CTR_Test,        "MCP CAM 128 CTR Test"},
    {MCP_CAM_192_CBC_DDR_Test,    "MCP CAM 192 CBC Test"},
    {MCP_CAM_192_CTR_DDR_Test,    "MCP CAM 192 CTR Test"},
    {MCP_CAM_256_CBC_DDR_Test,    "MCP CAM 256 CBC Test"},
    {MCP_CAM_256_CTR_DDR_Test,    "MCP CAM 256 CTR Test"},
    {MCP_SHA1_HASH_Test,          "MCP SHA1 HASH Test"},
    {MCP_SHA256_HASH_Test,        "MCP SHA256 HASH Test"},
    {MCP_AES_HASH_Test,           "MCP AES-HASH Test"},
};

#else
#define TEST_ITEM_NUM       26

/* not test with CW */
TestItem _gMCPTestItem[TEST_ITEM_NUM]=
{
    {MCP_DES_ECB_Test,            "MCP DES ECB Test"},
    {MCP_DES_CBC_Test,            "MCP DES CBC Test"},
    {MCP_TDES_ECB_Test,           "MCP TDES ECB Test"},
    {MCP_TDES_CBC_Test,           "MCP TDES CBC Test"},
    {MCP_AES_128_ECB_Test,        "MCP AES 128 ECB Test"},
    {MCP_AES_128_CBC_Test,        "MCP AES 128 CBC Test"},
    {MCP_AES_128_CBC_NO_IV_Test,  "MCP AES 128 CBC NO IV Test"},
    {MCP_AES_128_CTR_Test,        "MCP AES 128 CTR Test"},
    {MCP_AES_192_ECB_DDR_Test,    "MCP AES 192 ECB Test"},
    {MCP_AES_192_CBC_DDR_Test,    "MCP AES 192 CBC Test"},
    {MCP_AES_192_CTR_DDR_Test,    "MCP AES 192 CTR Test"},
    {MCP_AES_256_ECB_DDR_Test,    "MCP AES 256 ECB Test"},
    {MCP_AES_256_CBC_DDR_Test,    "MCP AES 256 CBC Test"},
    {MCP_AES_256_CTR_DDR_Test,    "MCP AES 256 CTR Test"},
    {MCP_CAM_128_ECB_Test,        "MCP CAM 128 ECB Test"},
    {MCP_CAM_128_CBC_Test,        "MCP CAM 128 CBC Test"},
    {MCP_CAM_128_CTR_Test,        "MCP CAM 128 CTR Test"},
    {MCP_CAM_192_ECB_DDR_Test,    "MCP CAM 192 ECB Test"},
    {MCP_CAM_192_CBC_DDR_Test,    "MCP CAM 192 CBC Test"},
    {MCP_CAM_192_CTR_DDR_Test,    "MCP CAM 192 CTR Test"},
    {MCP_CAM_256_ECB_DDR_Test,    "MCP CAM 256 ECB Test"},
    {MCP_CAM_256_CBC_DDR_Test,    "MCP CAM 256 CBC Test"},
    {MCP_CAM_256_CTR_DDR_Test,    "MCP CAM 256 CTR Test"},
    {MCP_SHA1_HASH_Test,          "MCP SHA1 HASH Test"},
    {MCP_SHA256_HASH_Test,        "MCP SHA256 HASH Test"},
    {MCP_AES_HASH_Test,           "MCP AES-HASH Test"},
};
#endif
int QC_Verifier_CP(void)
{
    int i;
    int nRet = SUCCESS;

    printf("\r\n\r\n");
    printf("===========================================\r\n");
    printf("              MCP QC Conclusion            \r\n");
    printf("===========================================\r\n");

    for(i = 0 ;i < TEST_ITEM_NUM ;i++)
    {
        //printf(_gMCPTestItem[i].TestName);
        if(_gMCPTestItem[i].func() == 1){

            nRet = FAILED;
            printf("\033[0;32;31m"": FAILED!! \r\n""\033[m");
        }// else
            //printf(": Passed!! \r\n");
    }

    return nRet;
} 
