//----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
// (C) COPYRIGHT 2006-2007 ARM Limited.
// ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Checked In : $Date: 2012-06-08 13:05:57 +0100 (Fri, 08 Jun 2012) $
// Revision : $Revision: 131396 $
//
//----------------------------------------------------------------------------

//#include <stdio.h>
//#include <stdlib.h>
//#include <strings.h>
//#include <string.h>
//#include <unistd.h>
//#include <errno.h>
//#include <ctype.h> /* isalnum(int c).. */
#include "test_vector/config.h"
#include <command.h>
#include <common.h>
#include <asm/arch/system.h>
#include <linux/string.h>
#include <linux/ctype.h>
#include <qc_verify_common.h>
#include <malloc.h>
/* Definitions */

// Needed only for the size_t and WORDSIZE type definition below
#include <stddef.h>
#include "tb_util.h"
#include "internal.h"
#include "backend.h"
#if defined(CONFIG_SYS_RTK_NAND_FLASH)
 #include <linux/mtd/nand.h>
 #include <nand.h>
#endif


#define TIMEOUT_DEFAULT_VAL 1001
#define NAND_QC_GCPU1_BLK_ADDR			80	
#define NAND_QC_GCPU2_BLK_ADDR			208
#define BLOCK_QC_GPU1				0x90
#define BLOCK_QC_GPU2				0x91
#define NAND_TMP_BUFFER				0x15000000

int timeoutval=TIMEOUT_DEFAULT_VAL;
volatile int irq_counter_mali=0;
volatile int waitfor_irq_counter_mali=0;

extern unsigned long __mali_phys_addr_mem;
// Set 'remap' to 1 to remap the virtual address 0x0 containted in the config.txt
// files to the physical address of the Mali memory
unsigned int remap = 0;

int debug_print = 0;

cmd_function cmd_exec;

cmd_function cmd_buslog;
cmd_function cmd_check_reg;
cmd_function cmd_clear_mem;
cmd_function cmd_compare_mem;
cmd_function cmd_compare_reg;
cmd_function cmd_dump_mem;
cmd_function cmd_dump_reg;
cmd_function cmd_inject_read_error;
cmd_function cmd_inject_write_error;
cmd_function cmd_load_mem;
cmd_function cmd_load_reg;
cmd_function cmd_quit;
cmd_function cmd_readlatency;
cmd_function cmd_reset;
cmd_function cmd_timeout;
cmd_function cmd_wait;
cmd_function cmd_writelatency;
cmd_function cmd_writeln;
cmd_function cmd_writereg;


typedef struct name_function{
    char * name;
    cmd_function * function;
} name_function;

#define COMMAND_GEN(NAME)  { #NAME, cmd_ ##NAME }
#define CMD_LIST_SIZE 19
name_function cmd_list[CMD_LIST_SIZE] =
{
    COMMAND_GEN(buslog),
    COMMAND_GEN(check_reg),
    COMMAND_GEN(clear_mem),
    COMMAND_GEN(compare_mem),
    COMMAND_GEN(compare_reg),
    COMMAND_GEN(dump_mem),
    COMMAND_GEN(dump_reg),
    COMMAND_GEN(inject_read_error),
    COMMAND_GEN(inject_write_error),
    COMMAND_GEN(load_mem),
    COMMAND_GEN(load_reg),
    COMMAND_GEN(quit),
    COMMAND_GEN(readlatency),
    COMMAND_GEN(reset),
    COMMAND_GEN(timeout),
    COMMAND_GEN(wait),
    COMMAND_GEN(writelatency),
    COMMAND_GEN(writeln),
    COMMAND_GEN(writereg),
};


void closing(test_vector * vector)
{
//    fclose(mali_file);
    printf("Closing\n");
}

void close_err(test_vector * vector)
{
    printf("Closing due to an error\n");
    closing(vector);
    //exit(1);
}

#define SYS_REG_GPU_CLK_OFFSET              0xb8000114
#define SYS_REG_GPU_RESET_OFFSET            0xb8000104
#define REG_GPU_INTERRUPT_ENABLE_OFFSET     0xb810e050
#define REG_GPU_RESET_OFFSET                0xb810f000

#define DELAY_C(val)  udelay(val)
#define WAIT_C(addr,val,mask) do {  }while(rtd_inl(addr)&mask != val)
#define CHECK_RD_REG_C(addr,val,mask) printf("[check register] addr:0x%x value:0x%x expected value:0x%x\n",addr,rtd_inl(addr)&mask,val);

static int mali_platform_gpu_resume(void)
{
    unsigned long uReg;
	printf("mali_platform_gpu_resume\n");
        //dv
	rtd_outl(0xb8000300,0x00000018); //reg_main_voltage_strobe_en[4],reg_pllemmc_voltage_strobe_en[3]
	udelay(1000);
        //dv
//	rtd_outl(0xb810e0f0,0x1);//power control
//	udelay(1000);
  // Enable GPU clock
    //uReg = rtd_inl(SYS_REG_GPU_CLK_OFFSET);
    rtd_outl(SYS_REG_GPU_CLK_OFFSET, 0x09);
    udelay(100);

	

    // Disable GPU reset
    //uReg = rtd_inl(SYS_REG_GPU_RESET_OFFSET);
    rtd_outl(SYS_REG_GPU_RESET_OFFSET, 0x09);
    udelay(100);

	rtd_outl(0xb810e048, 0x01);
		
    // Release GPU core reset
    uReg = rtd_inl(REG_GPU_RESET_OFFSET);
    rtd_outl(REG_GPU_RESET_OFFSET, uReg | 0x01);


    // Enable GPU interrupt
  //  uReg = rtd_inl(REG_GPU_INTERRUPT_ENABLE_OFFSET);
  //  rtd_outl(REG_GPU_INTERRUPT_ENABLE_OFFSET, uReg | 0xff);

    //Enable GPU interrupt route to SCPU
//    rtd_outl(SYS_REG_INT_CTRL_SCPU, 0x100001);
	//----arm gpu power up ---
//	rtd_outl(0xb810200c,0x00000001);
//	rtd_outl(0xb8102000,0xffffffff);
//	WAIT_C(0xb810e000,0x40,0x40);
//	rtd_outl(0xb8102018,0x00000001);
//	CHECK_RD_REG_C(0xb8101004,0x07100206,0xffffffff);
//	CHECK_RD_REG_C(0xb8111004,0x00000000,0xffffffff);
//	rtd_outl(0xb8113000,0x000000ff);
//	CHECK_RD_REG_C(0xb8102000,0x00000000,0xffffffff);
//	CHECK_RD_REG_C(0xb8101004,0x07100206,0xffffffff);
    return 0;


}
static void init(char *mali_core, unsigned char nb_pp, unsigned char pmu, unsigned int remap)
{
    //mali_register tmp_mali;
    /*Registering signal handler */
    //(void) signal(SIGUSR2,handle_gp_interrupt);
    //(void) signal(SIGUSR1,handle_pp_interrupt);

    /* Opens the mp device file */
    //if (! (mali_file= fopen(MALI_MP_FILE_NAME, "r+")) ){
    //    fprintf(stderr, "! Error: Could not open the media processor device file\n");
    //    close_err();
    //}
    //ioctl( fileno(mali_file), MALI_IOC_SET_SIGNAL_ME, (unsigned long)getpid());
    //ioctl( fileno(mali_file), MALI_MP_IOC_GET_LAST_IRQ_RAWSTAT, &tmp_mali);
    //ioctl( fileno(mali_file), MALI_MP_IOC_ENABLE_SIGNALS );
    _backend_init(mali_core, nb_pp, pmu, remap);
}


/**
 * Helper-function for the following commands:
 * - cmd_compare_mem
 * - cmd_dump_mem
 */
mali_error_code _do_mem_dump( char* pattern, unsigned long * offset, int append, unsigned long mem_addr_from, unsigned long mem_addr_to)
{
    mali_error_code ret_value;
    unsigned long mem_addr_current;
    unsigned long mem_addr_diff;
    unsigned long * buffer = NULL;
    unsigned long outword[4];
    char * current, *nxt_ptr ;
    unsigned long i = 0 ;
    unsigned long mask;
    if (!offset){	
	printf("Error: offset pointer is NULL!\n");
	return MALI_ERROR_FUNCTION_FAILED;
    }else{
    	current = pattern+*offset;
    }
    /* Dump actual memory */
    /* NOTE! This part can probably be optimized later to read larger */
    /* chunks of memory, currently it's just reading one 32bit word at a time. */
    mem_addr_current = mem_addr_from;
    while (mem_addr_current < mem_addr_to)
    {
        unsigned long wordnr;
	char print_buf_start[128];
	char pattern_buf[128]={0};
	char * print_buf;

	wordnr = (mem_addr_current>>2)&3;
        mem_addr_diff = mem_addr_to - mem_addr_current;
        if (mem_addr_diff > 3)
        {
            for (int i=0; i < 4; i++)
                mask |= 1<<((mem_addr_current+i)&0xf);
        } else {
            /* Less than four bytes (32 bits) left to care about */
            for (int i=0; i < mem_addr_diff; i++)
                mask |= 1<<((mem_addr_current+i)&0xf);
        }

        /* Read out whole, aligned, words */
        ret_value = _backend_memory_read_word(mem_addr_current&0xFFFFFFFC, &outword[wordnr]);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error while reading from memory, error code: %d\n", ret_value);
            return MALI_ERROR_FUNCTION_FAILED;
        }

        /* Print out */
        if ( (wordnr==3) || (((mem_addr_current&0xFFFFFFFC)+4) >= mem_addr_to) )
        {
            print_buf = print_buf_start;
            if (remap)
            	// using physical addresses
                _internal_dump_write_line(print_buf, &print_buf, mem_addr_current, &outword[0], mask);
            else
            	// using virtual addresse => let's add the base Mali memory physical address
                _internal_dump_write_line(print_buf, &print_buf, mem_addr_current + __mali_phys_addr_mem, &outword[0], mask);

	    /*Compare memory*/	
	    nxt_ptr = strpbrk(current,"\n");
	    if(!nxt_ptr){
			printf("Unexpected end of file!\n");
			return MALI_ERROR_FUNCTION_FAILED;
	    }
	    strncpy(pattern_buf, current, nxt_ptr-current+1);
	    if(strncmp(pattern_buf,print_buf,nxt_ptr-current+1)){
			printf("Unmatched !!!\n");
			printf("%s<==>%s\n",pattern_buf,print_buf);
			return MALI_ERROR_FUNCTION_FAILED;
	    }
	    current = nxt_ptr + 1;
            mask = 0;
        }

        /* Aligned */
        mem_addr_current = (mem_addr_current&0xFFFFFFFC)+4;
    }
    *offset =(unsigned long) current -(unsigned long) pattern;
    return MALI_ERROR_NO_ERROR;
}


/**
 * Command:     buslog <value>
 * Description: This command enables the buslog in the testbench, and
 *              since this is a testbench-related feature it's not
 *              available on FPGA-platforms.
 */
mali_error_code cmd_buslog(char * buf, test_vector * vector)
{
    IPRINTF("*** BUSLOG not available. Params: %s\n", buf);
    return 0;
}

/**
 * Command:     check_reg <hexaddr> <op> <hexvalue> [ >|>> <file>]
 * Description: Compares value of register at <hexaddr> to <hexvalue>
 *              with <op>. <op> can be '<','>' or '='. Writes
 *              'TRUE' or 'FALSE' to <file> (STDOUT if not specified).
 */
mali_error_code cmd_check_reg(char * buf, test_vector * vector)
{
#if 1
    mali_error_code ret_value;
    unsigned long addr_reg;
    unsigned long value_reg_check;
    unsigned long value_reg_read;
    unsigned int comp;
    char *operand, *filename;
    char *current, *nxt_ptr;
    int append;
    unsigned long index;
    char print_buf[16]={0};
    char pattern_buf[16] = {0};
    /* default is output to STDOUT */
    append = 1;

    ret_value = _internal_parse_arg_int(buf, &buf, &addr_reg, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_str(buf, &buf, &operand);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &value_reg_check, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading argument, error code: %d.\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_redirect(buf, &buf, &filename, &append);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading redirect-argument, error code: %d.\n", ret_value);
        return ret_value;
    }

    ret_value = _backend_read_reg(addr_reg, &value_reg_read);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading register-value, error code: %d.\n", ret_value);
        return ret_value;
    }

    if (strlen(filename) > 0)
    {
	index = strtoul(filename,NULL,10);
    }

    if (strstr(operand, "<") != NULL)
    {
        comp = value_reg_read < value_reg_check;
    } else if (strstr(operand, ">") != NULL)
    {
        comp = value_reg_read > value_reg_check;
    } else if (strstr(operand, "=") != NULL)
    {
        comp = (value_reg_read == value_reg_check);
    } else if (strstr(operand, ">=") != NULL)
    {
        comp = value_reg_read >= value_reg_check;
    } else if (strstr(operand, "<=") != NULL)
    {
        comp = value_reg_read <= value_reg_check;
    } else if (strstr(operand, "!=") != NULL)
    {
        comp = value_reg_read != value_reg_check;
    } else
    {
         return MALI_ERROR_FUNCTION_FAILED;
    }

    sprintf(print_buf, "%s\n", comp ? _INTERNAL_STR_TRUE : _INTERNAL_STR_FALSE);
    if(append){
	current = vector->pattern[index]+vector->pattern_offset[index];
    }else{
    	current = vector->pattern[index];
    }
    nxt_ptr = strpbrk(current,"\n");
    strncpy(pattern_buf, current, nxt_ptr-current+1);
    if(strncmp(pattern_buf,print_buf,nxt_ptr-current+1)){
		printf("Unmatched !!!\n");
		printf("%s<==>%s\n",pattern_buf,print_buf);
		vector->pattern_offset[index]+=nxt_ptr-current+1;
		IPRINTF("CHECK_REG addr:%04lx op:\"%s\" ref:%08lx read:%08lx: %s\n",
            addr_reg, operand, value_reg_check, value_reg_read, comp ? _INTERNAL_STR_TRUE : _INTERNAL_STR_FALSE);
		return MALI_ERROR_FUNCTION_FAILED;
    }
    vector->pattern_offset[index]+=nxt_ptr-current+1;
    IPRINTF("CHECK_REG addr:%04lx op:\"%s\" ref:%08lx read:%08lx: %s\n",
            addr_reg, operand, value_reg_check, value_reg_read, comp ? _INTERNAL_STR_TRUE : _INTERNAL_STR_FALSE);
    return MALI_ERROR_NO_ERROR;
#else
    IPRINTF("*** CHECKREG not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
#endif
}

/**
 * Command:     clear_mem <start_hexaddr> <end_hexaddr> [<hexval>=0]
 * Description: Clears the memory, starting from start_hexaddr ending
 *              with end_hexaddr-1.  The memory is cleared with hexval
 *              which defaults to 0 if not set.
 */
mali_error_code cmd_clear_mem(char * buf, test_vector* vector)
{
    mali_error_code ret_value;
    unsigned long addr_start;
    unsigned long addr_end;
    unsigned long fill_value;

    /* Set default values */
    fill_value = 0;
    addr_start = 0;
    addr_end = 0;

    ret_value = _internal_parse_arg_int(buf, &buf, &addr_start, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Failed to read start_hexaddr, error code: %d\n", ret_value);
        return MALI_ERROR_FUNCTION_FAILED;
    }
    if (!remap)
    {
		// using physical addresses
        if (! (addr_start >= __mali_phys_addr_mem) )
        {
            PRINTERR("start_hexaddr is not inside the range of malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        addr_start = addr_start - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &addr_end, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Failed to read end_hexaddr, error code: %d\n", ret_value);
        return MALI_ERROR_FUNCTION_FAILED;
    }
    if (!remap)
    {
        if (! (addr_end >= __mali_phys_addr_mem) )
        {
            PRINTERR("end_hexaddr is not inside the range of malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        addr_end = addr_end - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &fill_value, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Failed to read hexval, error code: %u\n", ret_value);
        return MALI_ERROR_FUNCTION_FAILED;
    }

    if (remap)
    {
		// using physical addresses
        IPRINTF("CLEAR_MEM start_addr:%04lx end_addr:%04lx fillvalue:%lx\n",
                addr_start, addr_end, fill_value);
    } else {
		// using addresses relative to the physical Mali memory base address
        IPRINTF("CLEAR_MEM start_addr:%04lx end_addr:%04lx fillvalue:%lx\n",
                __mali_phys_addr_mem + addr_start, __mali_phys_addr_mem + addr_end, fill_value);
    }

    /* Fill memory with value  */
    _backend_memset(addr_start, addr_end - addr_start, fill_value);

    return MALI_ERROR_NO_ERROR;
}


/**
 * Command:     compare_mem <hexaddr0, original> <hexaddr1, new> <length, hex> [ > | >> <file> ]
 * Description: Compares two areas of memory, original and new, if they are equal.
 *              Writes the complete new area to <file> (stdout if not specified).
 */
mali_error_code cmd_compare_mem(char * buf, test_vector* vector)
{
#if 0
    mali_error_code ret_value;
    unsigned long original_addr;
    unsigned long new_addr;
    unsigned long mem_length;
    char *filename;
    FILE *file_dump;
    int append;
    int close_file;
    int compare_result;

    /* Set default values */
    file_dump = stdout;
    close_file = 0;
    append = 1;

    /* Parse arguments */
    ret_value = _internal_parse_arg_int(buf, &buf, &original_addr, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading hexaddr0, error code: %d\n", ret_value);
        return ret_value;
    }
    if (! remap)
    {
        if (! (original_addr >= __mali_phys_addr_mem) )
        {
            PRINTERR("Error; original_addr not inside malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        original_addr = original_addr - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &new_addr, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading hexaddr1, error code: %d\n", ret_value);
        return ret_value;
    }
    if (! remap)
    {
        if (! (new_addr >= __mali_phys_addr_mem) )
        {
            PRINTERR("Error; new_addr not inside malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        new_addr = new_addr - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &mem_length, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading mem_length, error_code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_redirect(buf, &buf, &filename, &append);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading redirect-argument, error code: %d\n", ret_value);
        return ret_value;
    }

    /* Do the compare of memory-regions */
    ret_value = _backend_compare_mem(original_addr, new_addr, mem_length, &compare_result);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error doing memory compare, error code: %d\n", ret_value);
        return ret_value;
    }

    /* If result is equal, don't do anything about the content. */
    if (compare_result != 0)
    {
        ret_value = _do_mem_dump(filename, append, close_file, new_addr, new_addr+mem_length);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error while dumping memory to file / stdout, error code: %d.\n", ret_value);
            return ret_value;
        }
    }

    if (remap)
    {
        IPRINTF("COMPARE_MEM original_addr:%04lx new_addr:%08lx length:%04lx: %d\n",
                original_addr, new_addr, mem_length, compare_result);
    } else {
        IPRINTF("COMPARE_MEM original_addr:%04lx new_addr:%08lx length:%04lx: %d\n",
                __mali_phys_addr_mem + original_addr, __mali_phys_addr_mem + new_addr, mem_length, compare_result);
    }

    return MALI_ERROR_NO_ERROR;
#else
    IPRINTF("*** COMPAREMEM not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
#endif
}

/**
 * Command:     compare_reg <hexaddr0> <hexaddr1> [ >|>> <file> ]
 * Description: Compares two registers to check if they are equal.
 *              Writes TRUE or FALSE to <file> (stdout if not specified).
 */
mali_error_code cmd_compare_reg(char * buf, test_vector* vector)
{
#if 1
    mali_error_code ret_value;
    unsigned long addr_0;
    unsigned long addr_1;
    unsigned long value_reg_0;
    unsigned long value_reg_1;
    char *filename;
    int comp;
    char *current, *nxt_ptr;
    int append;
    unsigned long index;
    char print_buf[16]={0};
    char pattern_buf[16] = {0};
    /* Set default values */
    append = 1;

    /* Parse arguments */
    ret_value = _internal_parse_arg_int(buf, &buf, &addr_0, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading hexaddr_0, error code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &addr_1, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading hexaddr_1, error code: %d.\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_redirect(buf, &buf, &filename, &append);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading redirect-argument, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Read values from hardware */
    ret_value = _backend_read_reg(addr_0, &value_reg_0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading register-value from reg0 (%04lx), error code: %d.\n", addr_0, ret_value);
        return ret_value;
    }

    ret_value = _backend_read_reg(addr_1, &value_reg_1);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading register-value from reg1 (%04lx), error code: %d.\n", addr_1, ret_value);
        return ret_value;
    }

    /* Check if we're supposed to redirect to file, setup if we are. */
    if (strlen(filename) > 0)
    {
	index = strtoul(filename,NULL,10);
    }

    comp = value_reg_0 == value_reg_1;
    sprintf(print_buf, "%s\n", comp ? _INTERNAL_STR_TRUE : _INTERNAL_STR_FALSE);
    if(append){
	current = vector->pattern[index]+vector->pattern_offset[index];
    }else{
    	current = vector->pattern[index];
    }
    nxt_ptr = strpbrk(current,"\n");
    strncpy(pattern_buf, current, nxt_ptr-current+1);
    if(strncmp(pattern_buf,print_buf,nxt_ptr-current+1)){
		printf("Unmatched !!!\n");
		printf("%s<==>%s\n",pattern_buf,print_buf);
		vector->pattern_offset[index]+=nxt_ptr-current+1;
		return MALI_ERROR_FUNCTION_FAILED;
    }
    vector->pattern_offset[index]+=nxt_ptr-current+1;
    IPRINTF("COMPARE_REG addr_0:%04lx read:%08lx addr_1:%04lx read:%08lx: %s\n",
            addr_0, value_reg_0, addr_1, value_reg_1, comp ? _INTERNAL_STR_TRUE : _INTERNAL_STR_FALSE);

    return MALI_ERROR_NO_ERROR;
#else
    IPRINTF("*** COMPAREREG not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
#endif
}

/**
 * Command:     dump_mem <from_hexaddr> <to_hexaddr> [>|>> <file>]
 * Description: Dumps memory. From address is inclusive, to
 *              address is exclusive. Both support byte alignments.
 *              Dump goes to stdout, or can be redirected to or
 *              appended to a file.
 */

mali_error_code cmd_dump_mem(char * buf,  test_vector * vector)
{
    mali_error_code ret_value;
    unsigned long mem_addr_from;
    unsigned long mem_addr_to;
    unsigned long offset;
    int append;
    int close_file;
    char * filename;
    char * pattern;
    unsigned long index;
    /* Set default values */

    /* Parse arguments */
    ret_value = _internal_parse_arg_int(buf, &buf, &mem_addr_from, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading mem_addr_from, error code: %d\n", ret_value);
        return ret_value;
    }
    if (!remap)
    {
        if (! (mem_addr_from >= __mali_phys_addr_mem) )
        {
            PRINTERR("Error; mem_addr_from not in malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        mem_addr_from = mem_addr_from - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &mem_addr_to, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading mem_addr_to, error code: %d.\n", ret_value);
        return ret_value;
    }
    if (!remap)
    {
		// using physical addresses
        if (! (mem_addr_to >= __mali_phys_addr_mem) )
        {
            PRINTERR("Error; mem_addr_to not in malimemory, aborting.\n");
            return MALI_ERROR_FUNCTION_FAILED;
        }
        mem_addr_to = mem_addr_to - __mali_phys_addr_mem;
    }

    ret_value = _internal_parse_arg_redirect(buf, &buf, &filename, &append);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading redirect-argument, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Status-log prinout */
    if (remap)
    {
        IPRINTF("DUMP_MEM addr_from:%04lx addr_to:%04lx\n",
                mem_addr_from, mem_addr_to);
    } else {
        IPRINTF("DUMP_MEM addr_from:%04lx addr_to:%04lx\n",
                __mali_phys_addr_mem + mem_addr_from, __mali_phys_addr_mem + mem_addr_to);
    }
#if ONLY_PRINT
    printf("dump_mem:0x%x,0x%x\n",mem_addr_from,mem_addr_to);
#else
    /* This is shared by cmd_compare_mem */
    index = strtoul(filename,NULL,10);
    pattern = vector->pattern[index];
    ret_value = _do_mem_dump(pattern,&vector->pattern_offset[index],append,mem_addr_from, mem_addr_to);
#endif
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error while dumping memory to file / stdout, error code: %d.\n", ret_value);
        return ret_value;
    }


    return MALI_ERROR_NO_ERROR;
}


/**
 * Command:     dump_reg <from_hexaddr> <to_hexaddr> [>|>> <file>]
 * Description: Dumps register values. From address is inclusive, to
 *              address is exclusive. Both support byte alignments.
 *              Dump goes to stdout, or can be redirected to or
 *              appended to a file.
 */
mali_error_code cmd_dump_reg(char * buf, test_vector * vector)
{
 mali_error_code ret_value;
    unsigned long reg_addr_from;
    unsigned long reg_addr_to;
    unsigned long reg_addr_current;
    unsigned long reg_addr_diff;
    unsigned long outword[4];
    unsigned long mask;
    char * filename;
    char * pattern_start;
    char * current;
    char * nxt_ptr;
    int append;
    unsigned long index;
    /* Set default values */
    append = 1;
    mask = 0;
    for (int i = 0; i < 4; i++)
    {
        outword[i] = 0;
    }

    /* Parse arguments */
    ret_value = _internal_parse_arg_int(buf, &buf, &reg_addr_from, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading reg_addr_from, error code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &reg_addr_to, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading reg_addr_to, error code: %d.\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_redirect(buf, &buf, &filename, &append);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading redirect-argument, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Status-log prinout */
    IPRINTF("DUMP_REG addr_from:%04lx addr_to:%04lx\n",
            reg_addr_from, reg_addr_to);

    /* Check if we're supposed to redirect to file, setup is we are. */
    if (!strlen(filename) )
    {
	printf("Error: filename invalid!\n");
	return MALI_ERROR_FUNCTION_FAILED;
    }
	
    index = strtoul(filename,NULL,10);
    current = vector->pattern[index]+vector->pattern_offset[index];
	
    /* Dump actual registers */
    /* NOTE!  This code currently allows for byte-selection of registers, it */
    /* shouldn't, as registers are always 32 bits. */
    reg_addr_current = reg_addr_from;
    while (reg_addr_current < reg_addr_to)
    {
        unsigned int wordnr;
        char print_buf_start[128]={0};
	char pattern_buf[128]={0};
	char *pattern_current;
        char *print_buf;

        wordnr = (reg_addr_current>>2)&3;
        reg_addr_diff = reg_addr_to - reg_addr_current;
        if (reg_addr_diff > 3)
        {
            for (int i=0; i < 4; i++)
                mask |= 1<<((reg_addr_current+i)&0xf);
        } else {
            /* Less than four bytes (32 bits) left to care about */
            for (int i=0; i < reg_addr_diff; i++)
                mask |= 1<<((reg_addr_current+i)&0xf);
        }

        /* Read out whole, aligned, words */
        ret_value = _backend_read_reg(reg_addr_current&0xFFFFFFFC, &outword[wordnr]);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error while reading from register, error code: %d\n", ret_value);
            return MALI_ERROR_FUNCTION_FAILED;
        }

        /* Print out */
        if ( (wordnr==3) || (((reg_addr_current&0xFFFFFFFC)+4) >= reg_addr_to) )
        {
            print_buf = print_buf_start;
            _internal_dump_write_line(print_buf, &print_buf, reg_addr_current, &outword[0], mask);
		/*compare with pattern*/
	   nxt_ptr = strpbrk(current,"\n");
	   if ( !nxt_ptr ){
	   	PRINTERR("Error unexpected end of pattern!\n");
		return MALI_ERROR_FUNCTION_FAILED;
	   }
	   strncpy(pattern_buf,current,nxt_ptr-current+1);
	   #if ONLY_PRINT
	   printf("pattern_buf:%s\n",pattern_buf);
	   printf("read_buff:%s\n",print_buf);
	   #else
	   if(strncmp(pattern_buf,print_buf,nxt_ptr-current+1)){
			printf("Unmatched !!!\n");
			printf("%s<==>%s\n",pattern_buf,print_buf);
			return MALI_ERROR_FUNCTION_FAILED;
	    }
	   #endif
	    current = nxt_ptr + 1;
            mask = 0;
        }

        /* Aligned */
        reg_addr_current = (reg_addr_current&0xFFFFFFFC)+4;
    }
    vector->pattern_offset[index] = current - vector->pattern[index];


    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     inject_read_error
 * Description: Makes the memorymodel in the testbench return ERROR on
 *              a read access, since this is a testbench-related
 *              feature it's not available on FPGA-platforms.
 */
mali_error_code cmd_inject_read_error(char * buf, test_vector * vector)
{
    IPRINTF("*** INJECT_READ_ERROR not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     inject_write_error
 * Description: Makes the memorymodel in the testbench return ERROR on
 *              a write access, since this is a testbench-related
 *              feature it's not available on FPGA-platforms.
 */
mali_error_code cmd_inject_write_error(char * buf, test_vector * vector)
{
    IPRINTF("*** INJECT_WRITE_ERROR not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     load_mem <file> [<hex_address_offset>]
 * Description: Loads a memory dump from file.
 *              Optionally, the file may be loaded at an offset
 *              from the address stored in the file.
 */
mali_error_code cmd_load_mem(char * buf, test_vector * vector)
{
    mali_error_code ret_value;
    unsigned long mem_offset;
    unsigned long mem_cur_address;
    unsigned long mem_value;
    unsigned long inword[4];
    unsigned long mask;
    unsigned long cur_mask;
    unsigned long expanded_mask;
    unsigned long read_elements;
#define READ_BUFFER_SIZE    128
    char read_buffer[READ_BUFFER_SIZE];
    char *read_ptr;
    char *nxt_ptr;
    char *parse_ptr;
    char * filename;
    char tmps[READ_BUFFER_SIZE];
    unsigned int line_no;
    unsigned int buf_start;
    unsigned long index;
    char * load_pattern ;

    /* Set default values */
    mask = 0;
    mem_offset = 0;
    line_no = 0;
    for (int i = 0; i < 4; i++)
    {
        inword[i] = 0;
    }
    ret_value = _internal_parse_arg_str(buf, &buf, &filename);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading filename, error code: %d\n", ret_value);
        return ret_value;
    }
    if (strlen(filename) > 0){
    	index = strtoul(filename,NULL,10);
    }
    load_pattern = vector->load_pattern[index];
    ret_value = _internal_parse_arg_int(buf, &buf, &mem_offset, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading address_offset, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Read file and write to memory */
    memset(read_buffer, 0, READ_BUFFER_SIZE);
    read_ptr = NULL;
    nxt_ptr = load_pattern;
   
   while( 1 ){
   	read_ptr = nxt_ptr;
	nxt_ptr = strpbrk(nxt_ptr, "\n");
	if (!nxt_ptr)
		break;
	strncpy(read_buffer,read_ptr, nxt_ptr-read_ptr);
	read_buffer[nxt_ptr-read_ptr]='\0';
	nxt_ptr++;
	line_no++;
        if ( (strlen(read_buffer) < 2) || (read_buffer[0] == '#') )
        {
            /* skip empty/commented lines... */
            continue;
        }

        strncpy(tmps, (char *) &read_buffer[10], 35);

        /* Parse line */
        ret_value = _internal_parse_dump_line(read_buffer, &mem_cur_address, &inword[0], &mask);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing dump_line, error code: %d.\n", ret_value);
            return ret_value;
        }
        mem_cur_address += mem_offset;

        if (!remap)
        {
			// using physical addresses
            if (! (mem_cur_address >= __mali_phys_addr_mem) )
            {
                IPRINTF("*** %s\n", read_ptr);
                PRINTERR("Error; mem_cur_address (%08lx) on line #%d not in malimemory (%08d), aborting.\n", mem_cur_address, line_no, __mali_phys_addr_mem);
                return MALI_ERROR_FUNCTION_FAILED;
            }
            // substract the Mali base address to switch to relative addresses
            mem_cur_address = mem_cur_address - __mali_phys_addr_mem;
        }

        /* Do any (if any, that is) read, modify before we write it all */
        if (debug_print)
            printf("%08lx: %s  => ", mem_cur_address, tmps);
        for (int cur_word = 0; cur_word < 4; cur_word++)
        {
            cur_mask = (mask & (0xf << (cur_word*4))) >> (cur_word*4);
            if (cur_mask < 0xf)
            {
                /* We need to read and modify. */
                _backend_memory_read_word(mem_cur_address + (cur_word*4), &mem_value);
                expanded_mask = 0;
                for (int cur_byte = 0; cur_byte < 4; cur_byte++)
                    expanded_mask += (cur_mask & 1<<cur_byte) ? (0xff << (cur_byte*8)) : 0x00;
                inword[cur_word] |= mem_value & ~expanded_mask;
            }
            if (debug_print)
                printf(" %08lx", inword[cur_word]);
	    #if ONLY_PRINT
            printf("_backend_memory_write_word: addr:0x%x value:0x%x\n",mem_cur_address + (cur_word*4),inword[cur_word]);
	    #else
            _backend_memory_write_word(mem_cur_address + (cur_word*4), inword[cur_word]);
	    #endif
        }
        if (debug_print)
            printf("\n");
    }

    return MALI_ERROR_NO_ERROR;
  }


/**
 * Command:     load_reg <file> [<hex_address_offset>]
 * Description: Loads a register dump from file.
 *              Optionally, the file may be loaded at an offset
 *              from the address stored in the file.
 */
mali_error_code cmd_load_reg(char * buf, test_vector * vector)
{
    mali_error_code ret_value;
    unsigned long reg_offset;
    unsigned long reg_cur_address;
    unsigned long inword[4];
    unsigned long mask;
    unsigned long cur_mask;
    unsigned long index;
    char read_buffer[128];
    char *read_ptr;
    char *nxt_ptr;
    char *filename;
    char tmps[128];
    char * load_pattern;

    /* Set default values */
    mask = 0;
    reg_offset = 0;
    for (int i = 0; i < 4; i++)
    {
        inword[i] = 0;
    }

    /* Parse arguments */
    ret_value = _internal_parse_arg_str(buf, &buf, &filename);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading filename, error code: %d\n", ret_value);
        return ret_value;
    }
    index = strtoul(filename,NULL,10);
    load_pattern = vector->load_pattern[index];

    ret_value = _internal_parse_arg_int(buf, &buf, &reg_offset, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading reg_address_offset, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Read file and write to memory */
    nxt_ptr = load_pattern;
    while (1)
    {
        read_ptr = nxt_ptr;
        nxt_ptr = strpbrk(nxt_ptr,"\n");
	if (!nxt_ptr){
	    break;
	}
	strncpy(read_buffer,read_ptr,nxt_ptr-read_ptr);
	read_buffer[nxt_ptr-read_ptr]='\0';
        if ( (strlen(read_buffer) < 2) ||
             (read_buffer[0] == '#') )
        {
            /* Skip empty/commented lines... */
            continue;
        }
	nxt_ptr++;
        strncpy(tmps, (char *) &read_buffer[10], 35);

        /* Parse line */
        ret_value = _internal_parse_dump_line(read_buffer, &reg_cur_address, &inword[0], &mask);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing dump_line, error code: %d.\n", ret_value);
            return ret_value;
        }
        reg_cur_address += reg_offset;

        /* Only write registers with a mask of 0xf, */
        /* no read/modify/write on registers.       */
        if (debug_print)
            printf("%08lx: %s  => ", reg_cur_address, tmps);
        for (int cur_word = 0; cur_word < 4; cur_word++)
        {
            cur_mask = (mask & (0xf << (cur_word*4))) >> (cur_word*4);
            if (cur_mask == 0xf)
            {
                if (debug_print)
                    printf(" %08lx", inword[cur_word]);
                _backend_write_reg(reg_cur_address + (cur_word*4), inword[cur_word]);
            } else {
                if (debug_print)
                    printf(" skipping");
            }
        }
        if (debug_print)
            printf("\n");
    }

    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     quit
 * Description: quits the program / parser.
 */
mali_error_code cmd_quit(char * buf, test_vector * vector)
{
    IPRINTF("QUIT\n");
    closing(vector);
    //exit(0);
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     readlatency <decvalue>
 * Description: Changes number of cycles between read access is
 *              accepted till data is returned to <decvalue>
 */
mali_error_code cmd_readlatency(char * buf, test_vector * vector)
{
    IPRINTF("*** READLATENCY not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     reset
 * Description: Asserts the nRESET signal for several cycles.
 *              May have side effects like loading of an MMU
 *              page table.
 */
mali_error_code cmd_reset(char * buf, test_vector * vector)
{
    IPRINTF("RESET core(s)\n");
    return _backend_reset_cores(remap);
}

/**
 * Command:     timeout <decvalue>
 * Description: Sets the max simulation timeout limit to <decvalue>
 *              clock cycles.
 */
mali_error_code  cmd_timeout(char *buf, test_vector * vector)
{
    mali_error_code ret_value;
    unsigned long new_value;

    ret_value = _internal_parse_arg_int(buf, &buf, &new_value, 10, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error parsing timeout-value, error code: %d.\n", ret_value);
        return ret_value;
    }

    if (new_value)
    {
        timeoutval = new_value;
    }

    IPRINTF("TIMEOUT value set to %u\n", timeoutval);
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     wait [posedge|negedge] irq
 *              wait [posedge|negedge] clk <decvalue>
 *              wait irq <mask> [<value>]
 * Description: Stalls script execution waiting for the given event. The
 *              event can be a positive or negatie edge of any irq, a
 *              <decvalue> number of clock cycles, or a specific
 *              interrupt mask value
 *              - The "wait irq" command uses a 32-bit interrupt
 *                vector defined as:
 *                - [31:24] Others (Currently [24] is MMU)
 *                - [23:16] Geometry processors 7-0
 *                - [15: 0] Pixels processors 15-0
 *              - irq <mask> waits until the masked interrupt vector
 *                is non-zero.
 *              - irq <mask> <value> waits until the masked interrupts
 *                equals the specified value.
 */
mali_error_code cmd_wait(char * buf, test_vector * vector)
{
    mali_error_code ret_value;
    char *strarg_1, *strarg_2;
    unsigned long irq_mask, core_mask;
    unsigned long clk_value;
    unsigned int edge_triggered;

    /* defaults */
    irq_mask = 0;
    core_mask = 0;
    clk_value = 0;
    edge_triggered = 0;

    ret_value = _internal_parse_arg_str(buf, &buf, &strarg_1);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error parsing first argument of wait-command, error code: %d.\n", ret_value);
        return ret_value;
    }

    if ((strncmp(strarg_1, "posedge",7)==0) ||
        (strncmp(strarg_1, "negedge",7)==0))
    {
        /* wait posedge/negedge irq/clk will be caught inside here. */
        edge_triggered = 1;

        /* Trigger-edge specified, read string arg */
        ret_value = _internal_parse_arg_str(buf, &buf, &strarg_2);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing trigger-type argument (should be irq or clk), error code: %d.\n", ret_value);
            return ret_value;
        }

        /* Only clk will have this argument, just try to read it anyway. */
        ret_value = _internal_parse_arg_int(buf, &buf, &clk_value, 10, 0);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing decvalue for clock cycles, error code: %d.\n", ret_value);
            return ret_value;
        }

    } else if (strncmp(strarg_1, "clk",3)==0)
    {
        /* wait clk will be caught inside here. */

        /* Try to read clock cycles */
        ret_value = _internal_parse_arg_int(buf, &buf, &clk_value, 10, 0);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing decvalue for clock cycles, error code: %d.\n", ret_value);
            return ret_value;
        }

    } else if (strncmp(strarg_1, "irq",3)==0)
    {
        /* wait irq / wait irq <mask> [<value>] will be caught inside here. */

        ret_value = _internal_parse_arg_int(buf, &buf, &irq_mask, 16, 0);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing irq_mask, error code: %d.\n", ret_value);
            return ret_value;
        }

        ret_value = _internal_parse_arg_int(buf, &buf, &core_mask, 16, 0);
        if (ret_value != MALI_ERROR_NO_ERROR)
        {
            PRINTERR("Error parsing core_mask, error code: %d.\n", ret_value);
            return ret_value;
        }
    }

    if (edge_triggered)
    {
        if (clk_value)
        {
            /* wait [pos/neg]edge clk <decvalue> */
            IPRINTF("WAIT %s clk %lu\n", strarg_1, clk_value);
            _backend_wait_cycles(clk_value);
        } else
        {
            /* wait [pos/neg]edge irq */
            IPRINTF("WAIT %s irq\n", strarg_1);
            _backend_wait_irq();
        }
    } else if (clk_value)
    {
        /* wait clk <decvalue> */
        IPRINTF("WAIT clk %lu\n", clk_value);
        _backend_wait_cycles(clk_value);
    } else if (!irq_mask)
    {
        /* wait irq */
        IPRINTF("WAIT irq\n");
        _backend_wait_irq();
    } else if (irq_mask)
    {
        if (!core_mask)
        {
            /* wait irq <mask> */
            IPRINTF("WAIT irq mask:%08lx\n", irq_mask);
            _backend_wait_irqmask( irq_mask, 0 );  // No specific core select, wait for any core
        } else
        {
            /* wait irq <mask> <value> */
            IPRINTF("WAIT irq mask:%08lx value:%08lx\n", irq_mask, core_mask);
            _backend_wait_irqmask( irq_mask, core_mask );
        }
    } else
    {
        PRINTERR("Undefined wait-statement, this could be a bug.\n");
        return MALI_ERROR_FUNCTION_FAILED;
    }

    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     writelatency <decvalue>
 * Description: Changes number of cycles between write data valid
 *              on the bus till data is accepted to <decvalue>.
 */
mali_error_code cmd_writelatency(char * buf, test_vector * vector)
{
    IPRINTF("*** WRITELATENCY not available on platform.\n");
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     writelen [text]
 * Description: Writes [text] to STDOUT.
 */
mali_error_code cmd_writeln(char * buf, test_vector * vector)
{
    printf("  %s\n", buf);
    IPRINTF("WRITELN %s\n", buf);
    return MALI_ERROR_NO_ERROR;
}

/**
 * Command:     writereg <hexaddr> <hexvalue>
 * Description: Writes <hexvalue> to a register at <hexaddr>.
 *              Side effects may occur.
 */
mali_error_code cmd_writereg(char * buf, test_vector * vector)
{
    mali_error_code ret_value;
    unsigned long reg_addr;
    unsigned long reg_value;

    /* Parse arguments */
    ret_value = _internal_parse_arg_int(buf, &buf, &reg_addr, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading reg_addr, error code: %d\n", ret_value);
        return ret_value;
    }

    ret_value = _internal_parse_arg_int(buf, &buf, &reg_value, 16, 0);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error reading reg_value, error code: %d.\n", ret_value);
        return ret_value;
    }

    /* Status-log printout */
    IPRINTF("WRITEREG reg_addr:%04lx reg_value:%08lx\n",
            reg_addr, reg_value);

    ret_value = _backend_write_reg(reg_addr, reg_value);
    if (ret_value != MALI_ERROR_NO_ERROR)
    {
        PRINTERR("Error writing value to register, error code: %d.\n", ret_value);
    }

    return MALI_ERROR_NO_ERROR;
}

/**
 * Reads config.txt (or a file with same syntax).
 */
mali_error_code cmd_read_configtxt(test_vector *vector)
{
    mali_error_code ret_value;
    char parse_txt[128];
    char * cmd_start;
    char *parse_current;
    char *nxt_ptr;
    unsigned long line = 0;
    IPRINTF("Executing commands in test \"%s\"\n",vector->vector_name);

    if (!vector->cmd_string)
    {
        printf("! There is no cmd sequence.\n");
        print_help();
        return MALI_ERROR_FUNCTION_FAILED;
    }
    nxt_ptr = vector->cmd_string;
    while (1)
    {
        parse_current = nxt_ptr;
        nxt_ptr = strpbrk(nxt_ptr,"\n");
	if(nxt_ptr == NULL){
		break;
	}
	strncpy(parse_txt, parse_current, nxt_ptr-parse_current);
	parse_txt[nxt_ptr-parse_current] = '\0';
	cmd_start = parse_txt;
	nxt_ptr++;
        while (isspace(*cmd_start))  cmd_start++;
        if (strlen(cmd_start) == 0)
        {
            continue;
        }
	DPRINTF("\nline%d:%s\n-------------------------------------------------------\n",++line,cmd_start);
	ret_value = cmd_exec(cmd_start,vector);
	if(ret_value != MALI_ERROR_NO_ERROR){
		_backend_flush();
		return ret_value;
	}
        _backend_flush();
    }
    return MALI_ERROR_NO_ERROR;
}



/**
 * Execute-function responsible of dispatching
 * call to correct cmd_-function
 */
mali_error_code cmd_exec(char * cmd, test_vector * vector)
{
    int i;
    int len;
    char *param;

    if (*cmd=='#') {
	return 0;
    }
#if 0
    while(1)
    {
        param = strpbrk(cmd, "\t\r\n\"\';");
        if(!param) break;
        *param = ' ';
    }
    /*strcspn(cmd, " .,=;:\0\t\r\n");*/
#endif
    for (i=0; i< CMD_LIST_SIZE; i++)
    {
        len = strlen(cmd_list[i].name);
        if (! strncmp(cmd_list[i].name,cmd,len) )
        {
            if ( isgraph(*(cmd+len)) ) continue;
            param = cmd+len;
            while( isspace(*param) ) param++;

            DPRINTF("  Running cmd: \"%s\", with param: \"%s\"\n", cmd_list[i].name, param);
            return cmd_list[i].function(param,vector);
        }
    }

    printf("!!!  Command not found:  %s\n", cmd);
    return MALI_ERROR_FUNCTION_FAILED;
}


void print_help()
{
    printf("  ***********************************************************)\n");
    printf("  *  Test Bench Util (tb_util)                              *)\n");
    printf("  *                                                         *)\n");
    printf("  *                                                         *)\n");
    printf("  *  A program that can control Mali PP+GP through the      *)\n");
    printf("  *  command line, or a file with a command list.           *)\n");
    printf("  *                                                         *)\n");
    printf("  *  This program will execute                              *)\n");
    printf("  *  commands found in the file \"config.txt\" in the       *)\n");
    printf("  *  current directory.                                     *)\n");
    printf("  *                                                         *)\n");
    printf("  *  The type of the Mali core used MUST be specified with  *)\n");
    printf("  *  the option:                                            *)\n");
    printf("  *    -c <mali_core> (<mali_core> = mali200, mali300       *)\n");
    printf("  *    mali400, mali450 or mali470.                         *)\n");
    printf("  *  If <mali_core> = mali400, then the number of Pixel     *)\n");
    printf("  *  Processors must be provided with -nb_pp <n>,           *)\n");
    printf("  *  (1 <= <n> <= 4)                                        *)\n");
    printf("  *  If <mali_core> = mali450 or mali470, then the number   *)\n");
    printf("  *  of Pixel Processors must be provided with -nb_pp <n>,  *)\n");
    printf("  *  (1 <= <n> <= 8).                                       *)\n");
    printf("  *                                                         *)\n");
    printf("  *  For mali300, mali400, mali450 or mali470 the -p switch *)\n");
    printf("  *  will enable PMU support.                               *)\n");
    printf("  *                                                         *)\n");
    printf("  *  If you write -d,--d or \"debug\" on the cmd line, the  *)\n");
    printf("  *  program will show some additional debug info.          *)\n");
    printf("  *                                                         *)\n");
    printf("  *  To execute an other cmd list file than \"config.txt\"  *)\n");
    printf("  *  type -f --f or \"file\" and the file name.             *)\n");
    printf("  *                                                         *)\n");
    printf("  *  To use the program with virtual addresses starting     *)\n");
    printf("  *  from 0x0 and allow the program to set up the MMU table *)\n");
    printf("  *  and configure the MMU(s), type -r.                     *)\n");
    printf("  *  The address 0x0 will then be remapped to the Mali      *)\n");
    printf("  *  memory physical base address.                          *)\n");
    printf("  *                                                         *)\n");
    printf("  *  Text after a \"#\" will not be parsed. All nrs are hex.*)\n");
    printf("  *  All commands are case insensitive.                     *)\n");
    printf("  *                                                         *)\n");
    printf("  *  Valid commmands are:                                   *)\n");
    printf("  *   buslog <value>                                        *)\n");
    printf("  *   check_reg <hexaddr> <op> <hexvalue> [ >|>> <file>]    *)\n");
    printf("  *   clear_mem <start_hexaddr> <end_hexaddr> [<hexval>=0]  *)\n");
    printf("  *   compare_mem <origaddr0> <newaddr1> <len> [>|>> <file>]*)\n");
    printf("  *   compare_reg <hexaddr0> <hexaddr1> [ >|>> <file> ]     *)\n");
    printf("  *   dump_mem <from_hexaddr> <to_hexaddr> [>|>> <file>]    *)\n");
    printf("  *   dump_reg <from_hexaddr> <to_hexaddr> [>|>> <file>]    *)\n");
    printf("  *   inject_read_error                                     *)\n");
    printf("  *   inject_write_error                                    *)\n");
    printf("  *   load_mem <file> [<hex_address_offset>]                *)\n");
    printf("  *   load_reg <file> [<hex_address_offset>]                *)\n");
    printf("  *   quit                                                  *)\n");
    printf("  *   readlatency <decvalue>                                *)\n");
    printf("  *   reset                                                 *)\n");
    printf("  *   timeout [timeunits]                                   *)\n");
    printf("  *   wait [posedge|negedge] irq                            *)\n");
    printf("  *   wait [posedge|negedge] clk <decvalue>                 *)\n");
    printf("  *   wait irq <mask> [<value>]                             *)\n");
    printf("  *   writelatency <decvalue>                               *)\n");
    printf("  *   writeln [text]                                        *)\n");
    printf("  *   writereg <hexaddr> <hexvalue>                         *)\n");
    printf("  ***********************************************************)\n");
    printf("\n");
}

/***************************************************************/

/* Definitions */


unsigned long gpu_pattern_size = 0;
unsigned long gpu_pattern_golden_size = 0;
mali_error_code QC_Copy_emmc_Pattern_data(unsigned int qc_buf_1,unsigned int qc_buf_2)
{
	int ret = 0;
	char cmd[256] = {0};
	unsigned int *ptr;
	unsigned int index, block_no, data_size;
	unsigned int block_cnt;

	IPRINTF("======================%s start copy test pattern  ======================\n", __FUNCTION__);

	/* read from eMMC */
	//read header

	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", EMMC_TMP_BUFFER, EMMC_QC_EXTERND_DATA_1_BLK_ADDR, 1);
	DPRINTF("run cmd => %s\n", cmd);	
	if ((ret = run_command(cmd, 0))) {
		PRINTERR("command error\n");
		goto copy_end;
	}	

	ptr = (unsigned int *)(EMMC_TMP_BUFFER);

	if (*ptr++ != 0x12345678) {
		PRINTERR("error magic no.\nno GPU code on flash\n");
		goto copy_end;
	}
	data_size = *ptr;
	gpu_pattern_size = data_size;
	block_cnt = data_size%EMMC_BLOCK_SIZE?data_size/EMMC_BLOCK_SIZE+1:data_size/EMMC_BLOCK_SIZE;
	block_no = EMMC_QC_EXTERND_DATA_1_BLK_ADDR + 1; // body = header +1

	DPRINTF("index = 0x%x, block_no = 0x%x, block_cnt = 0x%x, data_size = 0x%x\n", index, block_no, block_cnt, data_size);


	//read body
	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", qc_buf_1, block_no, block_cnt);
	DPRINTF("run cmd ===> %s\n", cmd);
	if ((ret = run_command(cmd, 0))) {
		PRINTERR("command error\n");
		goto copy_end;
	}
	IPRINTF("======================%s end copy test pattern  %d======================\n", __FUNCTION__, ret);
	IPRINTF("======================%s start copy golden pattern  ======================\n", __FUNCTION__);
	/* read from eMMC */
	//read header

	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", EMMC_TMP_BUFFER, EMMC_QC_EXTERND_DATA_2_BLK_ADDR, 1);
	DPRINTF("run cmd => %s\n", cmd);	
	if ((ret = run_command(cmd, 0))) {
		PRINTERR("command error\n");
		goto copy_end;
	}	
	
	ptr = (unsigned int *)(EMMC_TMP_BUFFER);
	
	if (*ptr++ != 0x12345678) {
		PRINTERR("error magic no.\nno VE code on flash\n");
		goto copy_end;
	}
	data_size = *ptr;
	gpu_pattern_golden_size = data_size;
	block_cnt = data_size%EMMC_BLOCK_SIZE?data_size/EMMC_BLOCK_SIZE+1:data_size/EMMC_BLOCK_SIZE;
	block_no = EMMC_QC_EXTERND_DATA_2_BLK_ADDR + 1; // body = header +1	
	DPRINTF("index = 0x%x, block_no = 0x%x, block_cnt = 0x%x, data_size = 0x%x\n", index, block_no, block_cnt, data_size);
	//read body
	snprintf(cmd, sizeof(cmd), "mmc read 0x%x 0x%x 0x%x", qc_buf_2, block_no, block_cnt);
	DPRINTF("run cmd ===> %s\n", cmd);
	if ((ret = run_command(cmd, 0))) {
		PRINTERR("command error\n");
		goto copy_end;
	}	
	IPRINTF("======================%s end copy golden pattern  %d======================\n", __FUNCTION__, ret);
copy_end:

	return (ret == 0) ? MALI_ERROR_NO_ERROR : MALI_ERROR_FUNCTION_FAILED;
}

mali_error_codeQC_Copy_nand_Pattern_data(unsigned int qc_buf_1,unsigned int qc_buf_2)
{
	int ret = 0;
#ifdef CONFIG_SYS_RTK_NAND_FLASH
	int blk_addr = 0,hdr_size = 0;
	unsigned int start_addr = 0,*qc_ptr = NULL,buf_size = 0;
	nand_info_t *nand =&nand_info[0];
	/*read header content*/
	blk_addr = nand->scan_block_tag(nand, NAND_QC_GCPU1_BLK_ADDR,20,0,BLOCK_QC_GPU1);
	if(blk_addr == 0){
		printf("[ERR] %s %d: Scan block-tag failed!!!\n", __FUNCTION__,__LINE__);
		return -1;
	}else{
		printf("[OK] %s %d:Scan %d block-tag 0x%x ok!!\n", __FUNCTION__,__LINE__,
			NAND_QC_GCPU1_BLK_ADDR,BLOCK_QC_GPU1);
	}
	hdr_size = ((512+nand->oobblock-1)/nand->oobblock)*nand->oobblock;
	start_addr = blk_addr* (nand->erasesize);
	if((ret = nand->rescue_image_read(nand, 
		(unsigned char *)NAND_TMP_BUFFER, start_addr, 
			blk_addr, 0, BLOCK_QC_GPU1)) < 0){
		printf("[ERR] %s: %d get image content err!!\n", __FUNCTION__,__LINE__);
		return ret;
	}
	
	/*read buffer content*/
	qc_ptr = (unsigned int *)NAND_TMP_BUFFER;
	if(*qc_ptr++ != 0x12345678)
	{
		printf("error magic no.\nno qc_ve_pattern on flash\n");
		return ret;
	}
	gpu_pattern_size =  *qc_ptr;
	buf_size = *qc_ptr;
	ret = nand->rescue_image_read(nand, (unsigned char *)NAND_TMP_BUFFER, start_addr, 
			hdr_size+buf_size, 0, BLOCK_QC_GPU1);
	if(ret == 0){
		memcpy((unsigned char *)qc_buf_1,(unsigned char *)(NAND_TMP_BUFFER+hdr_size),
				buf_size);
	}else{
		return ret;
	}

	/*read header content*/
	blk_addr = nand->scan_block_tag(nand, NAND_QC_GCPU2_BLK_ADDR,20,0,BLOCK_QC_GPU2);
	if(blk_addr == 0){
		printf("[ERR] %s %d: Scan block-tag failed!!!\n", __FUNCTION__,__LINE__);
		return -1;
	}else{
		printf("[OK] %s %d:Scan %d block-tag 0x%x ok!!\n", __FUNCTION__,__LINE__,
			NAND_QC_GCPU2_BLK_ADDR,BLOCK_QC_GPU2);
	}
	hdr_size = ((512+nand->oobblock-1)/nand->oobblock)*nand->oobblock;
	start_addr = blk_addr* (nand->erasesize);
	if((ret = nand->rescue_image_read(nand, 
		(unsigned char *)NAND_TMP_BUFFER, start_addr, 
			blk_addr, 0, BLOCK_QC_GPU2)) < 0){
		printf("[ERR] %s: %d get image content err!!\n", __FUNCTION__,__LINE__);
		return ret;
	}
	
	/*read buffer content*/
	qc_ptr = (unsigned int *)NAND_TMP_BUFFER;
	if(*qc_ptr++ != 0x12345678)
	{
		printf("error magic no.\nno qc_ve_pattern on flash\n");
		return ret;
	}
	buf_size = *qc_ptr;
	gpu_pattern_golden_size = buf_size;
	ret = nand->rescue_image_read(nand, (unsigned char *)NAND_TMP_BUFFER, start_addr, 
			hdr_size+buf_size, 0, BLOCK_QC_GPU2);
	if(ret == 0){
		memcpy((unsigned char *)qc_buf_2,(unsigned char *)(NAND_TMP_BUFFER+hdr_size),
				buf_size);
	}
#endif
	return (ret == 0) ? MALI_ERROR_NO_ERROR : MALI_ERROR_FUNCTION_FAILED;
}

mali_error_code QC_Copy_Pattern_data(unsigned int qc_buf_1,unsigned int qc_buf_2)
{
	int ret = 0;
#ifdef CONFIG_SYS_RTK_NAND_FLASH
	ret = QC_Copy_nand_Pattern_data(qc_buf_1,qc_buf_2);
#else
	ret = QC_Copy_emmc_Pattern_data(qc_buf_1,qc_buf_2);
#endif
	return (ret == 0) ? MALI_ERROR_NO_ERROR : MALI_ERROR_FUNCTION_FAILED;
}

int QC_Verifier_GPU_PATTERN(void)
{
    char mali_core[8];
    unsigned char nb_pp = 1;
    unsigned char pmu = 0;
    mali_error_code ret_value;
    int i = 0;
    	
   /* config options */
   nb_pp = 3;
   strncpy(mali_core,"mali470",8);/*mali200,mali300,mali400,mali450,mali470*/
   pmu = 1;//control whether to enable PMU
   debug_print = 0;//control whether to enable debug print
   remap = 1;//control whether to enable MMU remap
   

    if (strncmp(mali_core, "mali200", 7) && strncmp(mali_core, "mali400", 7) && strncmp(mali_core, "mali450", 7) && strncmp(mali_core, "mali470", 7) && strncmp(mali_core, "mali300", 7)) {
		PRINTERR("Please provide the Mali core type with -c.\n");
		return 0;
	}

     if (!strncmp(mali_core, "mali200", 7) && pmu) {
                PRINTERR("Mali200 doesn't have PMU support. Please remove the PMU switch.\n");
                return 0;
        }


    if ((!strncmp(mali_core, "mali400", 7)) && nb_pp < 1 && nb_pp > 4) {
		PRINTERR("Please provide the number of Pixel Processors used with -nb_pp (1 <= n <= 4).\n");
		return 0;
	}

    if ((!strncmp(mali_core, "mali450", 7) || !strncmp(mali_core, "mali470", 7)) && nb_pp < 1 && nb_pp > 8) {
		PRINTERR("Please provide the number of Pixel Processors used with -nb_pp (1 <= n <= 8).\n");
		return 0;
	}
    mali_platform_gpu_resume();
    init(mali_core, nb_pp, pmu, remap);
#if GPU_WRAPPER_TEST
    ret_value = gpu_wrapper_test();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
                printf("\n !!!!Test gpu_wrapper_test failed!!!!\n");
                return ret_value;
    }
#endif

#if GP_M470_MP_F000_MMU_APB
    ret_value = gp_m470_mp_F000_MMU_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test gp_m470_mp_F000_MMU_APB failed!!!!\n");
		return ret_value;
    }
#endif

#if GP_M470_MP_F001_GP2_APB
    ret_value = gp_m470_mp_F001_GP2_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test gp_m470_mp_F001_GP2_APB failed!!!!\n");
		return ret_value;
    }
#endif

#if GP_M470_MP_F013_MMU_GP_IRQ
    ret_value = gp_m470_mp_F013_MMU_GP_IRQ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test gp_m470_mp_F013_MMU_GP_IRQ failed!!!!\n");
		return ret_value;
    }
#endif


#if GP_M470_MP_F052_GP_IRQ
    ret_value = gp_m470_mp_F052_GP_IRQ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test gp_m470_mp_F052_GP_IRQ failed!!!!\n");
		return ret_value;
    }
#endif

#if I2C_F044_L2C0_APB
    ret_value = i2c_F044_L2C0_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test i2c_F044_L2C0_APB failed!!!!\n");
		return ret_value;
    }
#endif


#if MP_M470_MP_DLBXBASIC_0
    ret_value = mp_m470_MP_DLBXBASIC_0();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_MP_DLBXBASIC_0 failed!!!!\n");
		return ret_value;
    }
#endif

#if MP_M470_MP_F048_BDC_APB
    ret_value = mp_m470_mp_F048_BDC_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_mp_F048_BDC_APB failed!!!!\n");
		return ret_value;
    }
#endif

#if MP_M470_MP_F049_DLB_APB
    ret_value = mp_m470_mp_F049_DLB_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_mp_F049_DLB_APB failed!!!!\n");
		return ret_value;
    }
#endif

#if MP_M470_MP_F050_PMU_APB
    ret_value = mp_m470_mp_F050_PMU_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_mp_F050_PMU_APB failed!!!!\n");
		return ret_value;
    }
#endif

#if MP_M470_MP_F051_PMU_IRQ
    ret_value = mp_m470_mp_F051_PMU_IRQ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_mp_F051_PMU_IRQ failed!!!!\n");
		return ret_value;
    }
#endif

#if MP_M470_MP_F069_BDC_IRQ
    ret_value = mp_m470_mp_F069_BDC_IRQ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test mp_m470_mp_F069_BDC_IRQ failed!!!!\n");
		return ret_value;
    }
#endif


#if POWER_INDICATIVE_PEAK_POWER
    ret_value = power_indicative_peak_power();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test power_indicative_peak_power failed!!!!\n");
		return ret_value;
    }
#endif

#if POWER_MIN
    ret_value = power_min();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test power_min failed!!!!\n");
		return ret_value;
    }
#endif

#if POWER_SUSTAINED_HIGH_POWER
    ret_value = power_sustained_high_power();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test power_sustained_high_power failed!!!!\n");
		return ret_value;
    }
#endif

#if POWER_TYP
    ret_value = power_typ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test power_typ failed!!!!\n");
		return ret_value;
    }
#endif


#if PP_M470_MP_F010_PP0_MMU_APB
    ret_value = pp_m470_mp_F010_PP0_MMU_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test pp_m470_mp_F010_PP0_MMU_APB failed!!!!\n");
		return ret_value;
    }
#endif


#if PP_M470_MP_F014_MMU_PP0_IRQ
    ret_value = pp_m470_mp_F014_MMU_PP0_IRQ();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test pp_m470_mp_F014_MMU_PP0_IRQ failed!!!!\n");
		return ret_value;
    }
#endif


#if PP_M470_MP_F061_PP0_APB
    ret_value = pp_m470_mp_F061_PP0_APB();
    if ( MALI_ERROR_NO_ERROR != ret_value ){
		printf("\n !!!!Test pp_m470_mp_F061_PP0_APB failed!!!!\n");
		return ret_value;
    }
#endif
    return ret_value;
}
