/*-----------------------------------------------------------------------------
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from ARM Limited or its affiliates.
 *
 *            (C) COPYRIGHT 2010-2017 ARM Limited or its affiliates.
 *                ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from ARM Limited or its affiliates.
 *
 *      Release Information : r0p0-00eac0
 *
 *-----------------------------------------------------------------------------
 *
 *-----------------------------------------------------------------------------
 * Abstract :Declaration of Mali functions
 *-----------------------------------------------------------------------------
 * Overview
 *----------
 *
 **************************************************************/
//#include <stdio.h>
#include <qc_verify_common.h>

#define  DVALIN 1

#ifdef VE
#define  MALI_BASE 0x2D000000
#else
#define  MALI_BASE 0xb8100000
#endif

#define SYS_CLKEN1_reg    0xb8000114
#define SYS_SRST1_reg     0xb8000104
#define SYS_INT_CTRL_SCPU 0xb8000290

#define REGS_GPU_WRAPPER_MODULE_START (0xb810e000)
#define REGS_GPU_WRAPPER_IRQ_STAT (0xb810e000)
#define REGS_GPU_WRAPPER_RESET  (0xb810F000)
#define REGS_GPU_WRAPPER_ENDIAN (0xb810E048)
#define REGS_GPU_WRAPPER_INT_EN (0xb810E050)
/****************************************************************************
 * SYSTEM SPECIFIC FUNCTIONS                                                  
 ****************************************************************************/
void CPU_EnableInterrupts(void);
void CPU_DisableInterrupts(void);
void CPU_InitialiseInterrupts(void);

void Mali_WaitForClk(int);
void Mali_WaitForInterrupt(void);
void Mali_InstallIntHandlers(void (*callback_mali_gpu)(int, int), 
                             void (*callback_mali_mmu)(int, int), 
                             void (*callback_mali_job)(int, int),
                             void (*callback_mali_event)(int, int));
void Mali_Message(char *s);
void Mali_SetupOutputFrame(void);
void Mali_JobPartDone(void);

/****************************************************************************
 * MALI REGISTER READ / WRITE
 ****************************************************************************/
void Mali_Reset(void);
void *Mali_LdMem(void *srcptr,int size,int ttb_base);
void Mali_WrReg(int regnum,int value);
int  Mali_RdReg(int regnum);
unsigned int Mali_PollReg (unsigned int regnum, unsigned int bit_location, unsigned int val, char * name_str);

void Mali_DisplayReg(int regnum);

void Mali_Identify(void);

void Mali_CheckReg(int regnum, int value);
int Mali_InterruptCheck(int i_mask, int i_value);
int Mali_AnyInterruptCheck(int type, int i_mask, int i_value);
void Mali_MaskAllInterrupts(void);

void Mali_SetInterruptErrMask(int type, int e_mask);

/****************************************************************************
 * MALI PERFORMANCE COUNTERS
 ****************************************************************************/
void Mali_InitPerfCounters(void);
void Mali_ReadPerfCounters(void);

/****************************************************************************
 * UTILITY FUNCTIONS FOR COPYING DATA INTO MALI MEMORY
 ****************************************************************************/
int Mali_MemCpy(unsigned int *malidata_page,
                unsigned int mali_va,
                unsigned int *refdata,
                unsigned int refoffset,
                unsigned int len);

int Mali_MemCpyMasked(unsigned int *malidata_page,
                      unsigned int mali_va,
                      unsigned int *refdata,
                      unsigned int *refmask,
                      unsigned int refoffset,
                      unsigned int len);

void Mali_Barrier(void);

/****************************************************************************
 * UTILITY FUNCTIONS FOR CHECKING TEST RESULTS
 ****************************************************************************/
int Mali_MemCmp(volatile unsigned int *malidata_page,
                unsigned int mali_va,
                unsigned int *refdata,
                unsigned int refoffset,
                unsigned int len);

int Mali_MemCmpMasked(volatile unsigned int *malidata_page,
                      unsigned int mali_va,
                      unsigned int *refdata,
                      unsigned int *refmask,
                      unsigned int refoffset,
                      unsigned int len);

int Mali_CompareRegs(unsigned int *reference_ptr, 
                      int mali_unit,
                     int mali_core,
                     int lowreg, 
                     int highreg);

/****************************************************************************
 * FUNCTIONS THAT ARE USEFUL FOR DEBUG
 ****************************************************************************/
void Mali_PrintMem       (volatile unsigned int * memory_address, unsigned int word_count );
void Mali_ReadDescriptor (volatile unsigned int * descriptor_address                      );
