/*-----------------------------------------------------------------------------
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from ARM Limited or its affiliates.
 *
 *            (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
 *                ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from ARM Limited or its affiliates.
 *
 *      Release Information : r0p0-00eac0
 *
 *-----------------------------------------------------------------------------
 * Abstract :Toplevel file implementing Main()
 *-----------------------------------------------------------------------------
 * Overview
 *----------
 *
 **************************************************************/
#include <common.h>
#include "config.h"
#include "MaliFns.h"

#define TEST 1
void call_dmb(void){


};

#ifdef TESTNAME
#else
#define TESTNAME RunMaliTest_test
#endif

typedef int (*MaliFunctionPtr)(int init);
int TESTNAME (int init);

/* Interrupt call backs */
void MaliIntCallBack (int IntNum, int IntSrc);
void MaliIntMMU (int IntNum, int IntSrc);

static MaliFunctionPtr mali_currentfunction_ptr;

static volatile int Mali_FinishedFlag = 0;
static int Mali_ReturnCode   = 0;

/********************************************************************************
 * Main routine for Mali Test.
 *
 * 1. Setup all interrupts
 * 2. Run first part of test. Remainder of test parts operate in interrupt handler.
 * 3. Spin until FinishedFlag is set.
 ********************************************************************************/
int QC_Verifier_GPU_PATTERN (void) {
#if  TEST
   Mali_Reset();
   printf("[GPU][Debug] MALI_BASE: rtd_inl(0x%x)=0x%x\n", MALI_BASE,rtd_inl(MALI_BASE));
   return 0;
#else
  //printf("Installing interrupt handler\n");

  /* 1. Disable CPU interrupts; and initalise any interrupt controller */
  /* 2. Install mali interrupt handlers */
  /* 3. Install interrupt handlers for mali interrupts */
  CPU_DisableInterrupts();
  #if 0
  CPU_InitialiseInterrupts();
  Mali_InstallIntHandlers(MaliIntCallBack,
                          MaliIntCallBack,
                          MaliIntCallBack,
                          MaliIntCallBack
                          );
  #endif
  unsigned int val;
  Mali_Reset();
  val = rtd_inl(MALI_BASE);
  printf("[GPU][Debug] MALI_BASE: rtd_inl(0x%x)=0x%x\n", MALI_BASE,val);
  /* 4. Identify GPU core */
  Mali_Identify();

  /* Raise a fault either GPU or MMU raise an interrupt */
  Mali_SetInterruptErrMask(1,0xFFFFFFFF);
  Mali_SetInterruptErrMask(2,0xFFFFFFFF);

  /* 5. Run the first part of this test. */
  /*    Note this can trigger an interrupt; hence we run this whilst interrupts */
  /*    are disabled. If interrupt masking cannot be done - then create another */
  /*    system to mali interrupts until after this routine has returned.        */
  /* 6. Enable interrupts */
  mali_currentfunction_ptr = &TESTNAME;
  Mali_ReturnCode = (*mali_currentfunction_ptr)(1);
  if ((Mali_ReturnCode != 254) & (Mali_ReturnCode != 255)) {
    Mali_FinishedFlag = 1;
  };
  CPU_EnableInterrupts();

  /* Do whatever you like - mali now runs from within interrupt handler
     For reference we have an infinite loop with a wait for interrupt instruction
     inside - waiting until all Mali jobs are finished.
     ............
   */
  
  call_dmb();

  while (!Mali_FinishedFlag) {
    Mali_WaitForInterrupt();
    MaliIntCallBack(0,0);
  };

  if (Mali_ReturnCode != 0) {
    printf ("TEST FAILED\n");
    return -1;
  } else {
    printf ("TEST PASSED\n");
    return 0;
  };
#endif
};

/********************************************************************************
 * This is the mali interrupt handler for expected interrupts
 *
 * 1. acknowledge the interrupt 
 * 2. call the next part in this test 
 * 3. set finished flag if not returning with wait for interrupt return code
 ********************************************************************************/
void MaliIntCallBack (int IntNum, int IntSrc) {
  /* Only run next step if we have not finished */
  /* This ensures that if the callback is executed when no mali interrupts */
  /* are actually pending; the test still works */
  /* Debug */
  // printf (">>>Start of MaliIntCallBack IntNum=%d IntSrc=%d Mali_FinishedFlag=%d >>>\n", IntNum, IntSrc, Mali_FinishedFlag); 
  if (Mali_FinishedFlag == 0) {
   do {
      Mali_ReturnCode = (*mali_currentfunction_ptr)(0);
      printf (">>> MaliIntCallBack Mali_Returncode=%d <<<\n", Mali_ReturnCode);
    } while(Mali_ReturnCode == 254);
  };
  if (Mali_ReturnCode != 255) {
    Mali_FinishedFlag = 1;
  };
  /* Debug */
  // printf ("<<< End of MaliIntCallBack IntNum=%d IntSrc=%d Mali_FinishedFlag=%d Mali_ReturnCode=%d <<<\n", IntNum, IntSrc, Mali_FinishedFlag, Mali_ReturnCode); 
};
