#ifndef _TUNER_ATV_BASE_H
#define _TUNER_ATV_BASE_H

#ifdef __cplusplus
extern "C" {
#endif

#define YES 1
#define NO 0
#ifndef bool
#define bool char
#endif

typedef struct ATV_TUNER_MODULE_TAG ATV_TUNER_MODULE;
typedef struct SYS_MODULE_TAG SYS_MODULE;
typedef struct I2C_MODULE_TAG I2C_MODULE;

typedef enum {
	I2C_NOADDR=0,
	I2C_ADDR_8=1,
	I2C_ADDR_16=2,
	I2C_CMD_CONTINUE=3, // the i2c command continual sending
} I2C_ADDR_TYPE;

typedef enum TUNER_INPUT_SEL {
        TUNER_INPUT_SEL_CATV = 0,
	 TUNER_INPUT_SEL_AIR = 1,
}TUNER_INPUT_SEL;

typedef enum TV_SCAN_FLOW_TYPE {
        TV_SCAN_FLOW_TYPE_ATV_DTV_indep = 0, // scan all RF freq (40M ~ 900M) in ATV mode, then scan all RF freq in DTV mode
	 TV_SCAN_FLOW_TYPE_ADAD = 1, // scan alternativlye between ATV and DTV per frequency
}TV_SCAN_FLOW_TYPE;

typedef enum {
	TUNER_ATV_VIDEO_MODE_NTSC=0,
	TUNER_ATV_VIDEO_MODE_PAL=1,
	TUNER_ATV_VIDEO_MODE_SECAM,
	TUNER_ATV_VIDEO_MODE_SECAMLA,
	TUNER_ATV_VIDEO_MODE_SCAN=0xfe,
	TUNER_ATV_VIDEO_MODE_UNKNOWN=0xff,
} TUNER_ATV_VIDEO_MODE;

typedef enum {
	TUNER_ATV_AUDIO_MODE_MN=0,
	TUNER_ATV_AUDIO_MODE_B=1,
	TUNER_ATV_AUDIO_MODE_GH,
	TUNER_ATV_AUDIO_MODE_I,
	TUNER_ATV_AUDIO_MODE_DK,
	TUNER_ATV_AUDIO_MODE_L,
	TUNER_ATV_AUDIO_MODE_LA,
	TUNER_ATV_AUDIO_MODE_FM_RADIO,
	TUNER_ATV_AUDIO_MODE_UNKNOWN=0xff,
} TUNER_ATV_AUDIO_MODE;

//Function pointer definition
//*******************************************************
//
// Tuner function
//
//*******************************************************
typedef void
(*ATV_TUNER_FP_GET_TUNER_TYPE)(
	ATV_TUNER_MODULE *pTuner,
	UINT8 *pTunerType
	);
typedef void
(*ATV_TUNER_FP_GET_DEVICE_ADDR)(
	ATV_TUNER_MODULE *pTuner,
	UINT8 *pDeviceAddr
	);
typedef UINT8
(*ATV_TUNER_FP_INITIALIZE)(
	ATV_TUNER_MODULE *pTuner
	);
typedef UINT8
(*ATV_TUNER_FP_SET_RF_FREQ_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 RfFreqHz
	);
typedef UINT8
(*ATV_TUNER_FP_GET_RF_FREQ_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 *pRfFreqHz
	);
typedef UINT8
(*ATV_TUNER_FP_SET_RF_FREQ_SEARCH_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 RfFreqHz
	);
typedef UINT8
(*ATV_TUNER_FP_GET_RF_FREQ_SEARCH_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 *pRfFreqHz
	);
typedef UINT8
(*ATV_TUNER_FP_SET_RF_FREQ_AFC_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 RfFreqHz
	);

typedef UINT8
(*ATV_TUNER_FP_GET_IF_FREQ_HZ)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 *pIfFreqHz
	);
typedef UINT8
(*ATV_TUNER_FP_GET_DIVER_RATIO)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 RfFreqHz,
	UINT16 *DiverRatio
	);
typedef UINT8
(*ATV_TUNER_FP_GET_RATIO_IN_STEPSIZE)(
	ATV_TUNER_MODULE *pTuner,
	UINT16 *RatioInStepSize
	);
typedef UINT8
(*ATV_TUNER_FP_CHECK_BAND)(
	ATV_TUNER_MODULE *pTuner,
	UINT32 RfFreqHz,
	UINT8 *Band
	);
typedef UINT8
(*ATV_TUNER_FP_SET_STD_MODE)(
	ATV_TUNER_MODULE *pTuner,
	TUNER_ATV_VIDEO_MODE video_mode,
	TUNER_ATV_AUDIO_MODE audio_mode
	);
typedef UINT8
(*ATV_TUNER_FP_GET_STD_MODE)(
	ATV_TUNER_MODULE *pTuner,
	TUNER_ATV_VIDEO_MODE* video_mode,
	TUNER_ATV_AUDIO_MODE* audio_mode
	);
//*******************************************************
//
// I2C function
//
//*******************************************************
typedef UINT8
(*I2C_FP_WRITE)(
	I2C_MODULE *pI2CModule,
	UINT8 *pWriteArray
	);
typedef UINT8
(*I2C_FP_READ)(
	I2C_MODULE *pI2CModule, 
	UINT8 *pReadArray);

//*******************************************************
//
// System function
//
//*******************************************************
#if 1
typedef void
(*TIMER_FP_DELAY_MS)(
	UINT32 ms
	);

#else
typedef bool
(*TIMER_FP_DELAY_MS)(
	UINT32 ms
	);
#endif


struct I2C_MODULE_TAG
{
	UINT8 master_id;
	UINT8 device_id;
	UINT16 slave_addr;
	UINT8 len;
	UINT8 addrtype;//0: no addr 1: 8bit 2:16bit

	//I2C interface function
	I2C_FP_WRITE i2c_write;
	I2C_FP_READ i2c_read;
};


struct SYS_MODULE_TAG
{
	//System function
	TIMER_FP_DELAY_MS timer_delay_ms;
};

////////////////////////////////////////////////////////////////////////////////////////////////
//
// NXP TDA18273
//
////////////////////////////////////////////////////////////////////////////////////////////////

// TDA18273 extra module
typedef struct TDA18273_EXTRA_MODULE_TAG TDA18273_EXTRA_MODULE;

// TDA18273 power mode setting function pointer
typedef int
(*TDA18273_FP_SET_POWER_MODE)(
	ATV_TUNER_MODULE *pTuner,
	int PowerMode
	);

// TDA18273 power mode getting function pointer
typedef int
(*TDA18273_FP_GET_POWER_MODE)(
	ATV_TUNER_MODULE *pTuner,
	int *pPowerMode
	);

// TDA18273 IF frequency getting function pointer
typedef int
(*TDA18273_FP_GET_IF_FREQ_HZ)(
	ATV_TUNER_MODULE *pTuner,
	unsigned long *pIfFreqHz
	);

// TDA18273 extra module
struct TDA18273_EXTRA_MODULE_TAG
{
	// TDA18273 extra variables
	unsigned long CrystalFreqHz;
	int UnitNo;
	int IfOutputVppMode;
	int StandardMode;
	int IsStandardModeSet;
	int PowerMode;
	int IsPowerModeSet;

	// TDA18273 extra function pointers
	TDA18273_FP_SET_POWER_MODE                SetPowerMode;
	TDA18273_FP_GET_POWER_MODE                GetPowerMode;
	TDA18273_FP_GET_IF_FREQ_HZ                GetIfFreqHz;
};

////////////////////////////////////////////////////////////////////////////////////////////////
//
// RF TUNER
//
////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct RFTUNER_EXTRA_MODULE_TAG RFTUNER_EXTRA_MODULE;

// RFTUNER extra module
struct RFTUNER_EXTRA_MODULE_TAG
{
	// RFTUNER extra variables
	// RFTUNER extra function pointers
};

struct ATV_TUNER_MODULE_TAG
{
	// Private variables
	UINT8   TunerType;									///<   Tuner type
	UINT8   DeviceAddr;									///<   Tuner I2C device address
	UINT32  RfFreqHz;										///<   Tuner RF frequency in Hz
	UINT8   IsRfFreqHzSet;											///<   Tuner RF frequency in Hz (setting status)
	UINT32 RfFreqHz_Max;
	UINT32 RfFreqHz_Min;
	TUNER_INPUT_SEL RfInput_Sel;
	UINT8 video_mode;
	UINT8 audio_mode;
	
	union														///<   Tuner extra module used by driving module
	{
		RFTUNER_EXTRA_MODULE    rf_tuner;	
		TDA18273_EXTRA_MODULE    Tda18273;
	}
	Extra;
	
	SYS_MODULE *pSysModule;						///<   Base interface module
	I2C_MODULE *pI2cModule;								///<   I2C bridge module

	// Tuner manipulating functions
	ATV_TUNER_FP_GET_TUNER_TYPE    GetTunerType;				///<   Tuner type getting function pointer
	ATV_TUNER_FP_GET_DEVICE_ADDR   GetDeviceAddr;				///<   Tuner I2C device address getting function pointer

	ATV_TUNER_FP_INITIALIZE        Initialize;					///<   Tuner initializing function pointer
	ATV_TUNER_FP_SET_RF_FREQ_HZ    SetRfFreqHz;					///<   Tuner RF frequency setting function pointer
	ATV_TUNER_FP_GET_RF_FREQ_HZ    GetRfFreqHz;					///<   Tuner RF frequency getting function pointer
	ATV_TUNER_FP_SET_RF_FREQ_SEARCH_HZ SetRfFreqSearchHz;
	ATV_TUNER_FP_GET_RF_FREQ_SEARCH_HZ    GetRfFreqSearchHz;	
	ATV_TUNER_FP_SET_RF_FREQ_AFC_HZ SetRfFreqAfcHz;
	ATV_TUNER_FP_SET_STD_MODE    SetStandardMode;
	ATV_TUNER_FP_GET_STD_MODE    GetStandardMode;

	ATV_TUNER_FP_GET_IF_FREQ_HZ			GetIfFreqHz;
	ATV_TUNER_FP_GET_DIVER_RATIO			GeDiverRatio;
	ATV_TUNER_FP_GET_RATIO_IN_STEPSIZE	GetRatioInStepSize;
	ATV_TUNER_FP_CHECK_BAND				CheckBand;
};

#ifdef __cplusplus
}
#endif

#endif

