#include "qc_verify.h"
#include "./inc/crt_sys_reg.h"
#include <qc_verify_common.h>
#include <rtd_types_qc.h>
#include "./inc/mhl_function.h"
#include "./inc/mhl_tx_cbus.h"
#include "./inc/mhl_tx_cbus_reg.h"

#define QCINFO_CMD_BUFFER_SIZE  128
enum mhl_cbus_cmd {
    MHL_CBUS_CMD_TEST_UNKNOWN = 0,
    MHL_CBUS_CMD_TEST_EXIT,
    MHL_CBUS_CMD_TEST_PROTOCOL,
    MHL_CBUS_CMD_TEST_CONNECT,
    MHL_CBUS_CMD_TEST_DISCONNECT,
    MHL_CBUS_CMD_TEST_AUTO_RUN,
    MHL_CBUS_CMD_SET_REG,
    MHL_CBUS_CMD_SET_BIT,
    MHL_CBUS_CMD_SET_MASK,
    MHL_CBUS_CMD_GET_REG_VALUE,
    MHL_CBUS_CMD_READ_EDID,
    MHL_CBUS_CMD_RXSENSE_IMPEDANCE,
    MHL_CBUS_CMD_ITEM_NUM
};

typedef struct {
    int err_code;
    char* msg;
} CBUS_CMD_ERR_ST;

static INT8* cmd_str_array[MHL_CBUS_CMD_ITEM_NUM] = {"unknown", "exit", "cbus", "con", "disc", "auto",
                                                     "setreg", "setbit", "setmask", "getreg", "edid",
                                                     "rxsense"};

static CBUS_CMD_ERR_ST cmd_err_array[] = {
    {ERRCODE_MSC_NORMAL, "pass"},
    {ERRCODE_MSC_RETRY_EXCEED, "retry_exceed"},
    {ERRCODE_MSC_PROTOCOL_ERR, "protocol_err"},
    {ERRCODE_MSC_TIMED_OUT, "timeout"},
    {ERRCODE_MSC_INVALID_OP_CODE, "invalid_op_code"},
    {ERRCODE_MSC_BAD_OFFSET, "bad_offset"},
    {ERRCODE_MSC_BUSY, "busy"},
    NULL,
};

INT32 mhl_cbus_argc = 0;
INT8 **mhl_cbus_argv = NULL;
UINT8 edid_buf[16] = {0};

//###########################################################################################################################
static void check_return_err_code(int value)
{
    UINT16 i;

    for (i = 0; (&cmd_err_array[0] + i) != NULL; i++)
    {
        if (cmd_err_array[i].err_code == value)
        {
            printf("- return msg = %s\n", cmd_err_array[i].msg);
            break;
        }
    }
}

static int mhl_tx_cbus_connect(void)
{
    CBUS_MSC_CMD_PARAM_ST param;
    int ret, rst = 0, cnt = 5000;
    UINT32 path_en_cnt = 100;
    UINT32 regValue = 0, regValue1 = 0, regValue2 = 0;

    if (!(MHL_CBUS_DIS1_get_both_path_en(rtd_inl(MHL_CBUS_DIS1_reg))))
    {
        /* R10K enable sel HW mode */
        rtd_maskl(MHL_CBUS_TEST_reg, ~MHL_CBUS_TEST_fw_control_r10k_mask, MHL_CBUS_TEST_fw_control_r10k(0));

        rtd_outl(MHL_CBUS_DIS4_reg, 0);

        printk("- status = 0x%08x, cbus_phy0 = 0x%08x\n", rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY0_reg));

        //CBUS
        rtd_maskl(MHL_FW0_REQ_01_reg, ~MHL_FW0_REQ_01_check_fw0_msg_processing_mask, MHL_FW0_REQ_01_check_fw0_msg_processing(0));

        rtd_maskl(MHL_CBUS_reg, ~MHL_CBUS_enable_mask, MHL_CBUS_enable(1));


        printk("- after cbus enable status = 0x%08x, 1k stus = 0x%08x, dect = 0x%08x\n", rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg), rtd_inl(MHL_CBUS_PHY0_reg));
        while (1)
        {
            regValue = rtd_inl(MHL_CBUS_DIS4_reg);
            if ((MHL_CBUS_DIS4_get_src3_10(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_09(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_08(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_07(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_06(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_05(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_04(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_03(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_02(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_01(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_00(regValue) == 1))
            {
                printk("- src3 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                //printk("- first off 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src1(regValue) == 1)
            {
                printk("- src1 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src2(regValue) == 1)
            {
                printk("- src2 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src0(regValue) == 1)
            {
                printk("- src0 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 0) && (MHL_CBUS_DIS4_get_src4(regValue) == 1))
            {
                printk("- src4 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 1) && (MHL_CBUS_DIS4_get_src5(regValue) == 0))
            {
                printk("- happen src3 / src4 time out to src7, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                rtd_maskl(MHL_CBUS_DIS4_reg, ~MHL_CBUS_DIS4_irq_src6_7_mask, MHL_CBUS_DIS4_irq_src6_7(0));
                printk("- first on 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 1) && (MHL_CBUS_DIS4_get_src5(regValue) == 1))
            {
                printk("- happen src5 time out to src7, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                rtd_maskl(MHL_CBUS_DIS4_reg, ~MHL_CBUS_DIS4_irq_src6_7_mask, MHL_CBUS_DIS4_irq_src6_7(0));
                printk("- first on 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 0) && (MHL_CBUS_DIS4_get_src5(regValue) == 1))
            {
                printk("- src5 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                break;
            }
            // cnt--;
        }

        printk("- wait path enable / dcap / hpd\n");
        while (path_en_cnt)
        {
            regValue = rtd_inl(MHL_MSC_REG_31_reg);
            regValue1 = rtd_inl(MHL_CBUS_MSC_06_reg);
            regValue2 = rtd_inl(MHL_MSC_REG_30_reg);
            if ((MHL_MSC_REG_31_get_path_en(regValue) && MHL_CBUS_MSC_06_get_irq_path_en_chg(regValue1)) ||
                (MHL_MSC_REG_30_get_dcap_rdy(regValue2) && MHL_CBUS_MSC_06_get_irq_dcap_rdy_chg(regValue1)) ||
                MHL_CBUS_MSC_06_get_irq_set_hpd(regValue1))
            {
                printk("- path enable flag = %d, rdy flag = %d\n, hpd set flag = %d\n",
                       MHL_MSC_REG_31_get_path_en(regValue), MHL_MSC_REG_30_get_dcap_rdy(regValue2), MHL_CBUS_MSC_06_get_irq_set_hpd(regValue1));
                rtd_maskl(MHL_CBUS_MSC_06_reg,
                          ~(MHL_CBUS_MSC_06_irq_path_en_chg_mask|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg_mask|
                            MHL_CBUS_MSC_06_irq_set_hpd_mask),
                           (MHL_CBUS_MSC_06_irq_path_en_chg(0)|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg(0)|
                            MHL_CBUS_MSC_06_irq_set_hpd(0)));
                break;
            }
            path_en_cnt--;
        }

        // param.cmd = MSC_CMD_WRITE_STAT_SET_INT;
        // param.offset = 0x31;
        // param.data0 = 0x8;
        // param.data1 = 0;
        // param.data_len = 0;
        // ret = set_msc_cmd_case0(&param);
        // check_return_err_code(ret);

        //CBUS both_en = 1
        if (path_en_cnt == 0)
        {
            printk("- wait path enable / dcap / hpd timeout\n");
            rst = -1;
        }

        rtd_maskl(MHL_CBUS_DIS1_reg, ~MHL_CBUS_DIS1_both_path_en_mask, MHL_CBUS_DIS1_both_path_en(1));

        printk("- receive path enable, entry src6\n");
        while ((rtd_inl(MHL_CBUS_DIS4_reg) & MHL_CBUS_DIS4_src6_mask) == 0);
        printk("- in src6 state\n");
    }

    return rst;
}

static void mhl_tx_cbus_disconnect(void)
{
    UINT32 regValue;

    regValue = rtd_inl(MHL_CBUS_DIS1_reg);
    if (MHL_CBUS_DIS1_get_both_path_en(regValue))
    {
    /*    //CBUS RxSense by fw
        dis0.regValue = rtd_inl(CBUS_DIS0_VADDR);
        dis0.rxsense_fw_value = 0;
        rtd_outl(CBUS_DIS0_VADDR, dis0.regValue);*/

        printk("- wait src7\n");
        while ((rtd_inl(MHL_CBUS_DIS4_reg) & MHL_CBUS_DIS4_src7_mask) == 0)
        {
            printk("    - 1k det = 0x%08x, status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
        }

        //CBUS both_en = 0
        rtd_maskl(MHL_CBUS_DIS1_reg, ~MHL_CBUS_DIS1_both_path_en_mask, MHL_CBUS_DIS1_both_path_en(0));

        printk("- wait src0\n");
        while ((rtd_inl(MHL_CBUS_DIS4_reg) & MHL_CBUS_DIS4_src0_mask) == 0)
        {
            printk("    - 1k det = 0x%08x, status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
        }

    }
}

static void mhl_tx_cbus_auto_run(void)
{
    UINT32 loop;
    loop = strtoul((const INT8 *) (mhl_cbus_argv[1]), (INT8 **) NULL, 16);
    loop = (mhl_cbus_argc) ? loop : 1;
    do {
        printk("\n- loop %d\n", loop);
        mhl_tx_cbus_connect();
        mhl_tx_cbus_disconnect();
    } while (--loop);

    printk("- loop end\n");

}

static void process_register(UINT16 type)
{
    UINT32 address;
    UINT32 mask;
    UINT32 bit;
    UINT32 value;

    address = strtoul((const INT8 *) (mhl_cbus_argv[1]), (INT8 **) NULL, 16);
    switch (type) {

        case MHL_CBUS_CMD_SET_REG:
            value = strtoul((const INT8 *) (mhl_cbus_argv[2]), (INT8 **) NULL, 16);
            printf("- set reg(0x%08x) = 0x%08x\n", address, value);
            rtd_outl(address, value);
            break;
        case MHL_CBUS_CMD_SET_BIT:
            bit = strtoul((const INT8 *) (mhl_cbus_argv[2]), (INT8 **) NULL, 10);
            value = strtoul((const INT8 *) (mhl_cbus_argv[3]), (INT8 **) NULL, 10);
            printf("- set reg(0x%08x)[%d] = %d\n", address, bit, value);
            rtd_maskl(address, ~(0x00000001 << bit), (value << bit));
            break;
        case MHL_CBUS_CMD_SET_MASK:
            mask = strtoul((const INT8 *) (mhl_cbus_argv[2]), (INT8 **) NULL, 16);
            value = strtoul((const INT8 *) (mhl_cbus_argv[3]), (INT8 **) NULL, 16);
            printf("- set reg(0x%08x)[0x%08x] = 0x%08x\n", address, mask, value);
            rtd_maskl(address, ~mask, value);
            break;
        case MHL_CBUS_CMD_GET_REG_VALUE:
            value = rtd_inl(address);
            printf("- get reg(0x%08x) value = 0x%08x\n", address, value);
            break;
        default:
            printf("- do nothing\n");
            break;
    }
}

static int get_mhl_cbus_cmd_enum(INT8 *str)
{
    int i, ret = MHL_CBUS_CMD_TEST_UNKNOWN;
    for (i = 0; i < MHL_CBUS_CMD_ITEM_NUM; i++)
    {
        if (strcmp(cmd_str_array[i], str) == 0)
        {
            ret = i;
            break;
        }
    }
    return ret;
}

static int cbus_ddc_cmd(void)
{
    UINT8 cmd_type;
    int ret = 0;
    CBUS_DDC_CMD_PARAM_ST param;

    memset(&param, 0, sizeof(CBUS_DDC_CMD_PARAM_ST));
    cmd_type = param.cmd = strtoul((const INT8 *) (mhl_cbus_argv[2]), (INT8 **) NULL, 16);
    param.offset = strtoul((const INT8 *) (mhl_cbus_argv[3]), (INT8 **) NULL, 16);
    param.seg_value = strtoul((const INT8 *) (mhl_cbus_argv[4]), (INT8 **) NULL, 16);
    param.addr_0x60 = strtoul((const INT8 *) (mhl_cbus_argv[5]), (INT8 **) NULL, 16);
    param.addr_w = strtoul((const INT8 *) (mhl_cbus_argv[6]), (INT8 **) NULL, 16);
    param.addr_r = strtoul((const INT8 *) (mhl_cbus_argv[7]), (INT8 **) NULL, 16);;
    param.data_len = strtoul((const INT8 *) (mhl_cbus_argv[8]), (INT8 **) NULL, 16);

    //printk("- send cmd = %d, offset = %d, seg = %d, data_len = %d\n",
    //       param.cmd, param.offset, param.seg_value, param.data_len);
    switch (cmd_type) {
        case DDC_CMD_READ:
            ret = set_ddc_cmd_case0(&param);
            break;
        case DDC_CMD_SEGMENT_READ:
            ret = set_ddc_cmd_case1(&param);
            break;
        case DDC_CMD_SHORT_CURRENT_READ:
            ret = set_ddc_cmd_case2(&param);
            break;
        case DDC_CMD_WRITE:
            ret = test_send_ddc_case_3(&param);
            break;
        default:
            printk("not exit the ddc case\n");
            break;
    }
    check_return_err_code(ret);

    return ret;
}

static UINT8 get_cbus_msc_cmd_type(UINT8 cmd)
{
    UINT8 ret;
    switch (cmd) {
        case MSC_CMD_WRITE_STAT_SET_INT:
            ret = 0;
            break;

        case MSC_CMD_READ_DEVCAP:
            ret = 1;
            break;

        case MSC_CMD_GET_STATE:
        case MSC_CMD_GET_VENDOR_ID:
        case MSC_CMD_GET_DDC_ERRORCODE:
        case MSC_CMD_GET_MSC_ERRORCODE:
            ret = 2;
            break;

        case MSC_CMD_MSC_MSG:
            ret = 4;
            break;

        case MSC_CMD_WRITE_BURST:
            ret = 5;
            break;

        default:
            ret = 0xff;
            break;
    }
    return ret;
}

static int cbus_msc_cmd(void)
{
    UINT8 cmd_type;
    int ret = 0;
    CBUS_MSC_CMD_PARAM_ST param;

    memset(&param, 0, sizeof(CBUS_MSC_CMD_PARAM_ST));
    param.cmd = strtoul((const INT8 *) (mhl_cbus_argv[2]), (INT8 **) NULL, 16);
    param.offset = strtoul((const INT8 *) (mhl_cbus_argv[3]), (INT8 **) NULL, 16);
    param.data0 = strtoul((const INT8 *) (mhl_cbus_argv[4]), (INT8 **) NULL, 16);
    param.data1 = strtoul((const INT8 *) (mhl_cbus_argv[5]), (INT8 **) NULL, 16);
    param.data_len = strtoul((const INT8 *) (mhl_cbus_argv[6]), (INT8 **) NULL, 16);
    cmd_type = get_cbus_msc_cmd_type(param.cmd);

    //printk("- send cmd = %d, offset = %d, data0 = %d, data1 = %d, data_len = %d\n",
    //       param.cmd, param.offset, param.data0, param.data1, param.data_len);
    switch (cmd_type) {
        case 0:
            ret = set_msc_cmd_case0(&param);
            break;
        case 1:
            ret = set_msc_cmd_case1(&param);
            break;
        case 2:
            ret = set_msc_cmd_case2(&param);
            break;
        case 3:
            //ret = set_msc_cmd_case3(&param);
            break;
        case 4:
            ret = set_msc_cmd_case4(&param);
            break;
        case 5:
            ret = set_msc_cmd_case5(&param);
            break;
        default:
            printk("not exit the msc case\n");
            break;
    }
    check_return_err_code(ret);

    return ret;
}

static void mhl_tx_cbus_test(void)
{
    unsigned int header, err_code = 0;
    header = strtoul((const INT8 *) (mhl_cbus_argv[1]), (INT8 **) NULL, 16);

    mhl_tx_cbus_connect();

    switch (header) {
        case 0:
            err_code = cbus_ddc_cmd();
            break;
        case 1:
            //rsvd
            break;
        case 2:
            err_code = cbus_msc_cmd();
            break;
        case 3:
            //rsvd
            break;
        default:
            break;
    }

    if (err_code)
    {
        printk("- cbus test error\n");
    }

    mhl_tx_cbus_disconnect();
}

static void read_edid(void)
{
    UINT16 blk_num, seg_num, offset_cnt;
    UINT16 i, j, k;
    CBUS_DDC_CMD_PARAM_ST param;
    UINT8 is_segment_read;
    int ret = 0;

    mhl_tx_cbus_connect();

    is_segment_read = strtoul((const INT8 *) (mhl_cbus_argv[1]), (INT8 **) NULL, 16);

    if (is_segment_read)
    {
        blk_num = 2;
        seg_num = blk_num / 2;
        offset_cnt = 256 / 16;
        memset(&param, 0, sizeof(CBUS_DDC_CMD_PARAM_ST));
        param.cmd = DDC_CMD_SEGMENT_READ;
        param.addr_0x60 = 0x60;
        param.addr_w = 0xa0;
        param.addr_r = 0xa1;
        for (i = 0; i < seg_num; i++)
        {
            printk("[cbus][info] edid segment idx = %d\n", i);
            printk("%s\n", "==================================\n");
            param.seg_value = i;
            for (j = 0; j < offset_cnt; j++)
            {
                param.offset = 16 * j;
                param.data_len = 15;
                ret = set_ddc_cmd_case1(&param);
                check_return_err_code(ret);
                get_data_from_fifo_to_buf(param.data_len, &edid_buf[0]);
                for (k = 0; k < 16; k ++)
                {
                    printk("0x%02x ", edid_buf[k]);
                }

                printk("\n");
            }
            printk("%s\n", "==================================\n\n");
        }
    }
    else
    {
        blk_num = 2;
        offset_cnt = 128 / 16;
        param.cmd = DDC_CMD_READ;
        param.addr_w = 0xa0;
        param.addr_r = 0xa1;
        for (i = 0; i < blk_num; i++)
        {
            printk("[cbus][info] edid blk idx = %d\n", i);
            printk("%s\n", "==================================\n");
            for (j = 0; j < offset_cnt; j++)
            {
                param.offset = (i * 128) + 16 * j;
                param.data_len = 15;
                ret = set_ddc_cmd_case1(&param);
                check_return_err_code(ret);
                get_data_from_fifo_to_buf(param.data_len, &edid_buf[0]);
                for (k = 0; k < 16; k ++)
                {
                    printk("0x%02x ", edid_buf[k]);
                }

                printk("\n");
            }
            printk("%s\n", "==================================\n\n");

        }
    }
}

static void mhl_tx_cbus_rxsense_connect(void)
{
    CBUS_MSC_CMD_PARAM_ST param;
    int ret, rst = 0, cnt = 5000;
    UINT32 regValue = 0, regValue1 = 0;

    if (!(MHL_CBUS_DIS1_get_both_path_en(rtd_inl(MHL_CBUS_DIS1_reg))))
    {
        /* R10K enable sel HW mode */
        rtd_maskl(MHL_CBUS_TEST_reg, ~MHL_CBUS_TEST_fw_control_r10k_mask, MHL_CBUS_TEST_fw_control_r10k(0));

        rtd_outl(MHL_CBUS_DIS4_reg, 0);

        printk("- status = 0x%08x, cbus_phy0 = 0x%08x\n", rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY0_reg));

        //CBUS
        rtd_maskl(MHL_FW0_REQ_01_reg, ~MHL_FW0_REQ_01_check_fw0_msg_processing_mask, MHL_FW0_REQ_01_check_fw0_msg_processing(0));

        rtd_maskl(MHL_CBUS_reg, ~MHL_CBUS_enable_mask, MHL_CBUS_enable(1));


        printk("- after cbus enable status = 0x%08x, 1k stus = 0x%08x, dect = 0x%08x\n", rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg), rtd_inl(MHL_CBUS_PHY0_reg));
        while (1)
        {
            regValue = rtd_inl(MHL_CBUS_DIS4_reg);
            if ((MHL_CBUS_DIS4_get_src3_10(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_09(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_08(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_07(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_06(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_05(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_04(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_03(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_02(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_01(regValue) == 1) ||
                (MHL_CBUS_DIS4_get_src3_00(regValue) == 1))
            {
                printk("- src3 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                //printk("- first off 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src1(regValue) == 1)
            {
                printk("- src1 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src2(regValue) == 1)
            {
                printk("- src2 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if (MHL_CBUS_DIS4_get_src0(regValue) == 1)
            {
                printk("- src0 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 0) && (MHL_CBUS_DIS4_get_src4(regValue) == 1))
            {
                printk("- src4 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 1) && (MHL_CBUS_DIS4_get_src5(regValue) == 0))
            {
                printk("- happen src3 / src4 time out to src7, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                rtd_maskl(MHL_CBUS_DIS4_reg, ~MHL_CBUS_DIS4_irq_src6_7_mask, MHL_CBUS_DIS4_irq_src6_7(0));
                printk("- first on 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 1) && (MHL_CBUS_DIS4_get_src5(regValue) == 1))
            {
                printk("- happen src5 time out to src7, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                rtd_maskl(MHL_CBUS_DIS4_reg, ~MHL_CBUS_DIS4_irq_src6_7_mask, MHL_CBUS_DIS4_irq_src6_7(0));
                printk("- first on 1k det = 0x%08x, then status = 0x%08x, 1k stus = 0x%08x\n", rtd_inl(MHL_CBUS_PHY0_reg), rtd_inl(MHL_CBUS_DIS4_reg), rtd_inl(MHL_CBUS_PHY1_reg));
            }
            else if ((MHL_CBUS_DIS4_get_irq_src6_7(regValue) == 0) && (MHL_CBUS_DIS4_get_src5(regValue) == 1))
            {
                printk("- src5 state, status = 0x%08x, 1k stus = 0x%08x\n", regValue, rtd_inl(MHL_CBUS_PHY1_reg));
                break;
            }
            // cnt--;
        }

        printk("- wait path enable\n");
        while (1)
        {
            regValue = rtd_inl(MHL_MSC_REG_31_reg);
            regValue1 = rtd_inl(MHL_CBUS_MSC_06_reg);
            if (MHL_MSC_REG_31_get_path_en(regValue) && 
                MHL_CBUS_MSC_06_get_irq_path_en_chg(regValue1))
            {
                rtd_maskl(MHL_CBUS_MSC_06_reg,
                          ~(MHL_CBUS_MSC_06_irq_path_en_chg_mask|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg_mask|
                            MHL_CBUS_MSC_06_irq_set_hpd_mask),
                           (MHL_CBUS_MSC_06_irq_path_en_chg(0)|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg(0)|
                            MHL_CBUS_MSC_06_irq_set_hpd(0)));
                param.cmd = MSC_CMD_WRITE_STAT_SET_INT;
                param.offset = 0x31;
                param.data0 = 0x8;
                param.data1 = 0;
                param.data_len = 0;
                ret = set_msc_cmd_case0(&param);
                check_return_err_code(ret);
                break;
            }
        }

        rtd_maskl(MHL_CBUS_DIS1_reg, ~MHL_CBUS_DIS1_both_path_en_mask, MHL_CBUS_DIS1_both_path_en(1));

        printk("- receive path enable, entry src6\n");
        while ((rtd_inl(MHL_CBUS_DIS4_reg) & MHL_CBUS_DIS4_src6_mask) == 0);
        printk("- in src6 state\n");

        printk("- wait path disable\n");
        while (1)
        {
            regValue = rtd_inl(MHL_MSC_REG_31_reg);
            regValue1 = rtd_inl(MHL_CBUS_MSC_06_reg);
            if ((MHL_MSC_REG_31_get_path_en(regValue) == 0)&& 
                 MHL_CBUS_MSC_06_get_irq_path_en_chg(regValue1))
            {
                rtd_maskl(MHL_CBUS_MSC_06_reg,
                          ~(MHL_CBUS_MSC_06_irq_path_en_chg_mask|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg_mask|
                            MHL_CBUS_MSC_06_irq_set_hpd_mask),
                           (MHL_CBUS_MSC_06_irq_path_en_chg(0)|
                            MHL_CBUS_MSC_06_irq_dcap_rdy_chg(0)|
                            MHL_CBUS_MSC_06_irq_set_hpd(0)));
                param.cmd = MSC_CMD_WRITE_STAT_SET_INT;
                param.offset = 0x31;
                param.data0 = 0x0;
                param.data1 = 0;
                param.data_len = 0;
                ret = set_msc_cmd_case0(&param);
                check_return_err_code(ret);
                break;
            }
        }
    }

    return rst;
}

static void test_rxsense_impedance(void)
{
    mhl_tx_cbus_rxsense_connect();
    mhl_tx_cbus_disconnect();
}

static void system_init(void)
{
    printk("[CBUS TX] CBUS setting init!!\n");

    //CRT setting
    rtd_maskl(0xb8000104, 0xffffb9ff, 0x4600);  //CBUS, HDMI RX, HDMI TX reset release
    rtd_maskl(0xb800010c, 0xffffefff, 0x1000);  //HDMI TX PLL reset release

    rtd_maskl(0xb8000114, 0xffffb9ff, 0x4600);  //CBUS, HDMI RX, HDMI TX clk enable

    rtd_maskl(0xb8060034, 0xffffffcf, 0x30);  //ST CBUS TX RX reset release
    rtd_maskl(0xb8060044, 0xffffffcf, 0x30);  //ST CBUS TX and RX clk enable
    rtd_maskl(0xb8060044, 0xfffffbff, 0x400);  //ST CBUS TX IP clk enable

    //CBUS PHY
    //rtd_outl(0xb8062b1c, 0x00000000); // CBUS_EN_TX (en_driver_tx) FW mode control enable

    rtd_outl(0xb8062b30, 0x00000a03); // CBUS bias and LDO enable, LDO=0x10
    rtd_outl(0xb8062b38, 0x08CAB637); // BMOS enable, GPIO_CBUS sel, CBUS enable , strength sel 0x3

    //CBUS MAC
    rtd_outl(0xb8062c04, 0x0363E896); //Tsrc_float = 52ms
    rtd_outl(0xb8062c00, 0x00000100); //MHL disable , CBUS clk cycle time
    rtd_outl(0xb8062c00, 0x80000100); //MHL enable , CBUS clk cycle time
    rtd_outl(0xb8062c28, 0x22); //Tsrc_float = 52ms


#if 0   //for CBUS connect RXsense function
    //HDMI TX PHY setting
    rtd_outl(0xb803de20, 0xbc468759);  //TX PLL parameter set, LDO_1V, VCO gain, internal LDO
    rtd_outl(0xb803de24, 0x084000e8);  //P2S LDO, preemphasis, termination R and C
    rtd_outl(0xb803de28, 0x77745550);  //driving, pre-em driving current
    rtd_outl(0xb803de2c, 0xf0071000);  //PLLDISP parameter, PLLDISP output disnable, PLLDISP power off
    rtd_outl(0xb803de30, 0x30022002);  //PLLDISP N code, tmds div, pixel div
    rtd_outl(0xb803de34, 0x00000000);  //Reserved
    //rtd_outl(0xb803de0c, 0x93680013);  //PLLDISP N code = 19+3
    rtd_outl(0xb803de0c, 0x9b780013);  //PLLDISP N code = 19+3
    rtd_outl(0xb803de10, 0x1e200284);  //PLLDISP N code load
    rtd_outl(0xb803de10, 0x1e200285);  //PLLDISP N code load

    rtd_outl(0xb803de2c, 0xf0071001);  //PLLDISP parameter, PLLDISP output disnable, PLLDISP power on
    rtd_outl(0xb803de2c, 0x70071001);  //PLLDISP parameter, PLLDISP output ennable, PLLDISP power on
    rtd_outl(0xb803de20, 0xfc468759);  //TX PLL parameter set, LDO_1V, VCO gain, TX PLL power on

    rtd_outl(0xb803de24, 0x084200e8);  //P2S LDO, preemphasis, termination R and C, MHL mode enable

    rtd_outl(0xb8000214, 0x00001100);  //HDMI_TMDS_DIV = 2
#else
    rtd_outl(0xb803de24, 0x08600000);  //HDMI TX bias enable, B CH lane output enable, B channel MHL mode enable, for RXsense normal work
#endif
}

int QC_Verifier_MHL_TX_CBUS(void)
{

    UINT8 receive_buf[QCINFO_CMD_BUFFER_SIZE];
    int i, option, tNum;
    bool loop = TRUE;

    system_init();
    while (loop) {
        printk("type mhl tx cbus cmd >> ");
        memset(receive_buf, 0, QCINFO_CMD_BUFFER_SIZE);
        GetLine(receive_buf, QCINFO_CMD_BUFFER_SIZE, 1);
        mhl_cbus_argc = GetArgc((const INT8 *) receive_buf);
        mhl_cbus_argv = (INT8 **) GetArgv((const INT8 *) receive_buf);
        /*
        for (i = 0; i < mhl_cbus_argc; i++)
            printk("\nreceive data %d %s\n", i, *(&mhl_cbus_argv[i]));
        */
        option = get_mhl_cbus_cmd_enum(*(&mhl_cbus_argv[0]));
        printk("\n");
        switch (option) {
            case MHL_CBUS_CMD_TEST_PROTOCOL:
                printk("[test cbus protocol]\n");
                mhl_tx_cbus_test();
                break;
            case MHL_CBUS_CMD_TEST_CONNECT:
                printk("[test cbus connect]\n");
                mhl_tx_cbus_connect();
                break;
            case MHL_CBUS_CMD_TEST_DISCONNECT:
                printk("[test cbus disconnect]\n");
                mhl_tx_cbus_disconnect();
                break;
            case MHL_CBUS_CMD_TEST_AUTO_RUN:
                printk("[test auto run]\n");
                mhl_tx_cbus_auto_run();
                break;
            case MHL_CBUS_CMD_TEST_EXIT:
                loop = FALSE;
                printk("[test exit]\n");
                break;
            case MHL_CBUS_CMD_SET_REG:
            case MHL_CBUS_CMD_SET_BIT:
            case MHL_CBUS_CMD_SET_MASK:
            case MHL_CBUS_CMD_GET_REG_VALUE:
                printk("[process register]\n");
                process_register(option);
                break;
            case MHL_CBUS_CMD_READ_EDID:
                printk("[test read edid]\n");
                read_edid();
                break;
            case MHL_CBUS_CMD_RXSENSE_IMPEDANCE:
                printk("[test 3_1_1_11 : RxSense impedance]\n");
                test_rxsense_impedance();
                break;
            default:
                printk("[unknown cmd]\n");
                break;
        }
    }
    return SUCCESS;
}
