#include "./inc/mhl_function.h"
#include "./inc/mhl_tx_cbus_reg.h"
#include "./inc/mhl_tx_cbus.h"
#include <qc_verify_common.h>

#define MSC_CASE_DATA_LEN 0x20
UINT16 msc_case5_data_table_1[MSC_CASE_DATA_LEN] =
{
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X','Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f'
};

UINT16 ddc_write_offset_table_1[] = {0x10, 0x12};
UINT16 ddc_write_offset_table_2[] = {0x15};
UINT16 ddc_write_offset_table_3[] = {0x18, 0x1B, 0x1E};

static void BusyWaiting(UINT32 count)
{
    UINT32 i=0;
    if(count>10000)
        count=10000;
    while(count>0)
    {
        i=5; //about 200uS
        while(i>0) i--;
        count--;
    }
}

static UINT32 CheckCBUSBusy0(void)
{
	UINT32 cbus_busy, regValue;
	// MHL_CBUS_WDOG cbus_wdog;
	// cbus_wdog.regValue = rtd_inl(MHL_CBUS_WDOG_reg);
	// cbus_busy=cbus_wdog.cbus_busy;
	regValue= rtd_inl(MHL_CBUS_WDOG_reg);
	cbus_busy=MHL_CBUS_WDOG_get_cbus_busy(regValue);
	return cbus_busy;
}

static UINT32 CheckCBUSBusy(void)
{
	while(CheckCBUSBusy0())
	{
		BusyWaiting(1000);
	}
	return 0;
}

static UINT32 CheckFW0Req0(void)
{
	UINT32 FW0_Req, regValue;
	regValue = rtd_inl(MHL_FW0_REQ_00_reg);
	FW0_Req = MHL_FW0_REQ_00_get_fw0_req(regValue);
	//printf("FW0_Req=%d\n",FW0_Req);
	return FW0_Req;
}

static UINT32 CheckFW0Req(void)
{
	while(CheckFW0Req0())
	{
		//printf("FW0 Req not ready\n");
		BusyWaiting(1000);
	}
	return 0;
}

static UINT32 SetFW0TXCase_Cmd1(UINT32 fx0_tx_case, UINT32 fx0_cmd1)
{
	//Set FW0 Request Case, case0, case1, case2, ... case 5
	rtd_maskl(MHL_FW0_REQ_01_reg, 
		      ~(MHL_FW0_REQ_01_fw0_tx_case_mask|MHL_FW0_REQ_01_fw0_cmd1_mask),
		      (MHL_FW0_REQ_01_fw0_tx_case(fx0_tx_case)|MHL_FW0_REQ_01_fw0_cmd1(fx0_cmd1)));
	return 0;
}

static UINT32 SetFW0Offset_Data1(UINT32 fx0_offset,UINT32 fx0_data1)
{
	//FW0 Offset : FW0 Offset byte
	//printf("SetFW0Offset fx0_offset=0x%x=%d\n",fx0_offset,fx0_offset);
	rtd_maskl(MHL_FW0_REQ_03_reg, 
		      ~(MHL_FW0_REQ_03_fw0_offset_mask|MHL_FW0_REQ_03_fw0_data1_mask),
		      (MHL_FW0_REQ_03_fw0_offset(fx0_offset)|MHL_FW0_REQ_03_fw0_data1(fx0_data1)));
	return 0;
}

static UINT32 TriggerFW0Req(void)
{

	rtd_maskl(MHL_FW0_REQ_00_reg, ~MHL_FW0_REQ_00_fw0_req_mask, MHL_FW0_REQ_00_fw0_req(1));
	return 0;
}

static UINT32 WaitFW0Req(void)
{
	while(CheckFW0Req0())
	{
		//printf("FW0 Req not ready\n");
		BusyWaiting(1000);
	}
	return 0;
}

static UINT32 CheckReceiveCommand0(void)
{
	//Read : FW0 Requester has received reply Command byte
	UINT32 irq_fw0_cmd, regValue;
	regValue = rtd_inl(MHL_FW0_REQ_09_reg);
	irq_fw0_cmd=MHL_FW0_REQ_09_get_irq_fw0_cmd(regValue);
	return irq_fw0_cmd;
}

static UINT32 GetReceiveCommand(void)
{
	//Read : Reply Command From FW0 Requester
	UINT32 fw0_cmd_rcv, regValue;
	regValue = rtd_inl(MHL_FW0_REQ_06_reg);
	fw0_cmd_rcv=MHL_FW0_REQ_06_get_fw0_cmd_rcv(regValue);
	return fw0_cmd_rcv;
}

UINT32 CheckReceiveCommand(void)
{
	UINT32 cmd_rcv;
	printf("- wait receive cmd\n");
	while(!CheckReceiveCommand0())
	{
		//printf("FW0 does not receive command\n");
		BusyWaiting(1000);
	}
	cmd_rcv=GetReceiveCommand();
	/*
	if(cmd_rcv==MSC_CMD_ACK)
		printf("ACK\n");
	else if(cmd_rcv==MSC_CMD_NACK)
		printf("NACK\n");
	else if(cmd_rcv==MSC_CMD_ABORT)
		printf("ABORT\n");
	else if(cmd_rcv==MSC_CMD_EOF)
		printf("ACK\n");
	else
		printf("unknown receive command = 0x%x\n",cmd_rcv);
	*/
	return cmd_rcv;
}

static UINT32 CheckMSCErrCode(void)
{
	UINT32 MSC_Error_Code, regValue;
	regValue = rtd_inl(MHL_CBUS_MSC_0C_reg);
	MSC_Error_Code=MHL_CBUS_MSC_0C_get_msc_err_code(regValue);
	/*
	printf("MSC_Error_Code=0x%x=%d\n",MSC_Error_Code,MSC_Error_Code);
	switch (MSC_Error_Code)
	{
		case ERRCODE_MSC_NORMAL:
			printf("Normal response, no error.\n");
			break;
		case ERRCODE_MSC_RETRY_EXCEED:
			printf("Re-try threshold exceeded.\n");
			break;
		case ERRCODE_MSC_PROTOCOL_ERR:
			printf("Protocol Error.\n");
			break;
		case ERRCODE_MSC_TIMED_OUT:
			printf("Peer Timed Out.\n");
			break;
		case ERRCODE_MSC_INVALID_OP_CODE:
			printf("Invalid Op Code.\n");
			break;
		case ERRCODE_MSC_BAD_OFFSET:
			printf("Bad Offset in Command\n");
			break;
		case ERRCODE_MSC_BUSY:
			printf("Peer is busy, cannot process new request.\n");
			break;
		default:
			printf("Unknown MSC_Error_Code=0x%x=%d\n",MSC_Error_Code,MSC_Error_Code);
		break;
	}
	*/
	return MSC_Error_Code;
}

static UINT32 SetFW0Offset(UINT32 fx0_offset)
{
	//FW0 Offset : FW0 Offset byte
	//printf("SetFW0Offset fx0_offset=0x%x=%d\n",fx0_offset,fx0_offset);
	rtd_maskl(MHL_FW0_REQ_03_reg, ~MHL_FW0_REQ_03_fw0_offset_mask, MHL_FW0_REQ_03_fw0_offset(fx0_offset));
	return 0;
}

static UINT32 CheckReceiveData0(void)
{
	//Read : FW0 Requester has received reply Data byte
	UINT32 irq_fw0_data, regValue;
	regValue = rtd_inl(MHL_FW0_REQ_09_reg);
	irq_fw0_data=MHL_FW0_REQ_09_get_irq_fw0_data(regValue);
	//TODO: original : return irq_fw0_data;
	return 0x1;
}

static UINT32 GetReceiveData(void)
{
	//Read : Reply Data From FW0 Requester
	UINT32 fw0_data_rcv, regValue;
	regValue = rtd_inl(MHL_FW0_REQ_07_reg);
	fw0_data_rcv=MHL_FW0_REQ_07_get_fw0_data_rcv(regValue);
	return fw0_data_rcv;
}

static UINT32 CheckReceiveData(void)
{
	UINT32 data_rcv;
	while(!CheckReceiveData0())
	{
		//printf("FW0 does not receive data\n");
		BusyWaiting(1000);
	}
	data_rcv=GetReceiveData();
	//printf("Get Data = 0x%x\n",data_rcv);
	return data_rcv;
}

static UINT32 SetFW0Data1_Data2(UINT32 fx0_data1, UINT32 fx0_data2)
{
	//FW0 Data1
	//FW0 Data2 for case4
	rtd_maskl(MHL_FW0_REQ_03_reg, 
		      ~(MHL_FW0_REQ_03_fw0_data1_mask|MHL_FW0_REQ_03_fw0_data2_mask),
		       (MHL_FW0_REQ_03_fw0_data1(fx0_data1)|MHL_FW0_REQ_03_fw0_data2(fx0_data2)));
	return 0;
}

static void Test_SendMSCCommand_case_0_2_3a(void)
{
	CBUS_MSC_CMD_PARAM_ST param;

	param.cmd=MSC_CMD_WRITE_STAT_SET_INT;				//SET_INT 0x60;
	param.offset=Device_Int_DEVICE_REGISTER_CHANGE;			//0x20
	param.data0=4;		//REQ_WRT=1	bit[4:0]=0b00100=4
	param.data1=0;		//useless

	//printf("Test_SendMSCCommand_case_0_2_3\n");
	set_msc_cmd_case0(&param);
}

static UINT32 SetFW0_FIFO_clr(void)
{
	rtd_maskl(MHL_FW0_REQ_05_reg, ~MHL_FW0_REQ_05_fw0_fifo_clr_mask, MHL_FW0_REQ_05_fw0_fifo_clr(1));
	rtd_inl(MHL_FW0_REQ_05_reg);
	rtd_maskl(MHL_FW0_REQ_05_reg, ~MHL_FW0_REQ_05_fw0_fifo_clr_mask, MHL_FW0_REQ_05_fw0_fifo_clr(0));
	return 0;
}

static UINT32 SetFW0Cmd2(UINT32 fx0_cmd2)
{
	//FW0 CMD2 : FW0 second Command byte
	//printf("SetFW0Cmd2 fx0_cmd2=0x%x=%d\n",fx0_cmd2,fx0_cmd2);
	rtd_maskl(MHL_FW0_REQ_02_reg, ~MHL_FW0_REQ_02_fw0_cmd2_mask, MHL_FW0_REQ_02_fw0_cmd2(fx0_cmd2));
	return 0;
}

static UINT32 SetFW0Data(UINT32 fx0_data)
{
	//FW0 Data Port, write data into FIFO
	//printf("SetFW0Data fx0_data=0x%x=%d\n",fx0_data,fx0_data);
	rtd_maskl(MHL_FW0_REQ_04_reg, ~MHL_FW0_REQ_04_fw0_data_mask, MHL_FW0_REQ_04_fw0_data(fx0_data));
	return 0;
}

static UINT32 SetFW0DataToFifo(UINT32 fx0_data_length)
{
	UINT32 i;
	for(i=0;i<fx0_data_length;i++)
	{
		SetFW0Data(msc_case5_data_table_1[i]);
	}
	return 0;
}

static UINT32 Test_SendMSCCommand_case_0_2_2a(void)
{
	CBUS_MSC_CMD_PARAM_ST param;

	param.cmd=MSC_CMD_WRITE_STAT_SET_INT;				//SET_INT 0x60;
	param.offset=Device_Int_DEVICE_REGISTER_CHANGE;			//0x20
	param.data0=2;		//DSCR_CHG=1	bit[4:0]=0b00010=2
	param.data1=0;		//useless

	//printf("Test_SendMSCCommand_case_0_2_2\n");
	set_msc_cmd_case0(&param);
	return 0;
}

static UINT32 CheckDDCReq0(void)
{
	UINT32 DDC_Req, regValue;
	regValue = rtd_inl(MHL_CBUS_DDC_00_reg);
	DDC_Req=MHL_CBUS_DDC_00_get_ddc_req(regValue);
	//printf("DDC_Req=%d\n",DDC_Req);
	return DDC_Req;
}

static UINT32 CheckDDCReq(void)
{
	while(CheckDDCReq0())
	{
		//printf("DDC Req not ready\n");
		BusyWaiting(1000);
	}
	return 0;
}

static UINT32 SetDDCReqType(UINT32 ddc_req_type)
{
	rtd_maskl(MHL_CBUS_DDC_02_reg, ~MHL_CBUS_DDC_02_ddc_req_type_mask, MHL_CBUS_DDC_02_ddc_req_type(ddc_req_type));
	return 0;
}

static UINT32 FillDDCOffset(UINT32 ddc_offset)
{
	rtd_maskl(MHL_CBUS_DDC_06_reg, ~MHL_CBUS_DDC_06_ddc_offset_mask, MHL_CBUS_DDC_06_ddc_offset(ddc_offset));
	return 0;
}

static UINT32 FillDDCAddrR(UINT32 ddc_addr_r)
{
	rtd_maskl(MHL_CBUS_DDC_01_reg, ~MHL_CBUS_DDC_01_ddc_addr_r_mask, MHL_CBUS_DDC_01_ddc_addr_r(ddc_addr_r));
	return 0;
}

static UINT32 FillDDCAddrW_AddrR(UINT32 ddc_addr_w, UINT32 ddc_addr_r)
{
	rtd_maskl(MHL_CBUS_DDC_01_reg, 
		      ~(MHL_CBUS_DDC_01_ddc_addr_w_mask|MHL_CBUS_DDC_01_ddc_addr_r_mask),
		      (MHL_CBUS_DDC_01_ddc_addr_w(ddc_addr_w)|MHL_CBUS_DDC_01_ddc_addr_r(ddc_addr_r)));
	return 0;
}

static UINT32 SetFW0_FIFO_len(UINT32 fw0_fifo_len)
{
	rtd_maskl(MHL_FW0_REQ_05_reg, ~MHL_FW0_REQ_05_fw0_fifo_len_mask, MHL_FW0_REQ_05_fw0_fifo_len(fw0_fifo_len));
	return 0;
}

static UINT32 TriggerDDCReq(void)
{
	rtd_maskl(MHL_CBUS_DDC_00_reg, ~MHL_CBUS_DDC_00_ddc_req_mask, MHL_CBUS_DDC_00_ddc_req(1));
	return 0;
}

static UINT32 WaitDDCReq(void)
{
	while(CheckDDCReq0())
	{
		//printf("DDC Req not ready\n");
		BusyWaiting(1000);
	}
	return 0;
}

static UINT32 CheckDDCErrCode(void)
{
	UINT32 DDC_Error_Code, regValue;

	regValue = rtd_inl(MHL_CBUS_DDC_00_reg);
	DDC_Error_Code=MHL_CBUS_DDC_00_get_ddc_err_code(regValue);
	/*
	printf("DDC_Error_Code=0x%x=%d\n",DDC_Error_Code,DDC_Error_Code);
	switch (DDC_Error_Code)
	{
		case ERRCODE_DDC_NORMAL:
			printf("Normal response, no error.\n");
			break;
		case ERRCODE_DDC_RETRY_EXCEED:
			printf("Re-try threshold exceeded.\n");
			break;
		case ERRCODE_DDC_PROTOCOL_ERR:
			printf("Protocol Error.\n");
			break;
		case ERRCODE_DDC_TIMED_OUT:
			printf("Peer Timed Out.\n");
			break;
		default:
			printf("Unknown DDC_Error_Code=0x%x=%d\n",DDC_Error_Code,DDC_Error_Code);
		break;
	}
	*/
	return DDC_Error_Code;
}

static UINT32 FillDDCAddr_0x60_SegVal_Offset(UINT32 ddc_addr_0x60, UINT32 ddc_seg_val, UINT32 ddc_offset)
{
	// MHL_CBUS_DDC_06 cbus_ddc_06_reg;
	// cbus_ddc_06_reg.regValue = rtd_inl(MHL_CBUS_DDC_06_reg);
	// cbus_ddc_06_reg.ddc_offset=ddc_offset;
	// cbus_ddc_06_reg.ddc_seg_val=ddc_seg_val;
	// cbus_ddc_06_reg.ddc_addr_0x60=ddc_addr_0x60;
	// rtd_outl(MHL_CBUS_DDC_06_reg, cbus_ddc_06_reg.regValue);
	rtd_maskl(MHL_CBUS_DDC_06_reg,
		      ~(MHL_CBUS_DDC_06_ddc_offset_mask|MHL_CBUS_DDC_06_ddc_seg_val_mask|MHL_CBUS_DDC_06_ddc_addr_0x60_mask),
		      (MHL_CBUS_DDC_06_ddc_offset(ddc_offset)|MHL_CBUS_DDC_06_ddc_seg_val(ddc_seg_val)|MHL_CBUS_DDC_06_ddc_addr_0x60(ddc_addr_0x60)));
	return 0;
}

static UINT32 FillDDCAddrW(UINT32 ddc_addr_w)
{
	// MHL_CBUS_DDC_01 cbus_ddc_01_reg;
	// cbus_ddc_01_reg.regValue = rtd_inl(MHL_CBUS_DDC_01_reg);
	// cbus_ddc_01_reg.ddc_addr_w=ddc_addr_w;
	// rtd_outl(MHL_CBUS_DDC_01_reg, cbus_ddc_01_reg.regValue);
	rtd_maskl(MHL_CBUS_DDC_01_reg, ~MHL_CBUS_DDC_01_ddc_addr_w_mask, MHL_CBUS_DDC_01_ddc_addr_w(ddc_addr_w));
	return 0;
}

int set_msc_cmd_case0(CBUS_MSC_CMD_PARAM_ST *param)
{
	UINT8 cmd_case = 0;
	//UINT32 ret;
	printf("%s \n", "- msc case 0 test");
	//case0: ->cmd, ->offset, ->data,		<-ack
	CheckCBUSBusy();
	CheckFW0Req();
	//SetFW0_FIFO_clr();
	//GetMSCCommandCase(cmd_case);

	//SetFW0TXCase(cmd_case); merge to one function
	//SetFW0Cmd1(fx0_cmd1); merge to one function
	SetFW0TXCase_Cmd1(cmd_case,param->cmd);

	//SetFW0Offset(fx0_offset);	merge to one function
	//SetFW0Data1(fx0_data1);	merge to one function
	SetFW0Offset_Data1(param->offset,param->data0);

	//SetFW0Data2(fx0_data2);
	TriggerFW0Req();
	WaitFW0Req();
	CheckReceiveCommand();		// <-ack
	//CheckReceiveData();				// <-data
	return CheckMSCErrCode();
}

int set_msc_cmd_case1(CBUS_MSC_CMD_PARAM_ST *param)
{
	UINT8 cmd_case = 1;
	//UINT32 ret;
	printf("%s \n", "- msc case 1 test");
	//case1: ->cmd, ->offset,             	<-ack, <-data
	CheckCBUSBusy();
	CheckFW0Req();
	//SetFW0_FIFO_clr();
	//GetMSCCommandCase(cmd_case);
	//SetFW0TXCase(cmd_case); merge to one function
	//SetFW0Cmd1(fx0_cmd1); merge to one function
	SetFW0TXCase_Cmd1(cmd_case,param->cmd);

	SetFW0Offset(param->offset);
	//SetFW0Data1(fx0_data1);
	//SetFW0Data2(fx0_data2);
	TriggerFW0Req();
	WaitFW0Req();
	CheckReceiveCommand();		// <-ack
	CheckReceiveData();				// <-data
	return CheckMSCErrCode();
}

int set_msc_cmd_case2(CBUS_MSC_CMD_PARAM_ST *param)
{
	UINT8 cmd_case = 2;
	//UINT32 ret;
	printf("%s \n", "- msc case 2 test");
	//case2: ->cmd, 				<-data
	CheckCBUSBusy();
	CheckFW0Req();
	//SetFW0_FIFO_clr();
	//GetMSCCommandCase(cmd_case);
	//SetFW0TXCase(cmd_case); merge to one function
	//SetFW0Cmd1(fx0_cmd1); merge to one function
	SetFW0TXCase_Cmd1(cmd_case,param->cmd);
	//SetFW0Offset(fx0_offset);
	//SetFW0Data1(fx0_data1);
	//SetFW0Data2(fx0_data2);
	TriggerFW0Req();
	WaitFW0Req();
	//CheckReceiveCommand();		// <-ack
	CheckReceiveData();			// <-data
	return CheckMSCErrCode();
}

int set_msc_cmd_case4(CBUS_MSC_CMD_PARAM_ST *param)
{
	UINT8 cmd_case = 4;
	//UINT32 ret;
	printf("%s \n", "- msc case 4 test");
	//case4: ->cmd, ->data1,  ->data2,			<-ack
	CheckCBUSBusy();
	CheckFW0Req();
	//SetFW0_FIFO_clr();
	//GetMSCCommandCase(cmd_case);
	//SetFW0TXCase(cmd_case); merge to one function
	//SetFW0Cmd1(fx0_cmd1); merge to one function
	SetFW0TXCase_Cmd1(cmd_case,param->cmd);
	//SetFW0Offset(fx0_offset);
	//SetFW0Data1(fx0_data1); merge to one function
	//SetFW0Data2(fx0_data2); merge to one function
	SetFW0Data1_Data2(param->data0,param->data1);
	TriggerFW0Req();
	WaitFW0Req();
	CheckReceiveCommand();		// <-ack
	//CheckReceiveData();				// <-data
	return CheckMSCErrCode();
}

int set_msc_cmd_case5(CBUS_MSC_CMD_PARAM_ST *param)
{
	UINT8 cmd_case = 5;
	//UINT32 ret;
	printf("%s \n", "- msc case 5 test");
	Test_SendMSCCommand_case_0_2_3a();
	////Delay(300);
	BusyWaiting(5000);

	//case5: ->cmd, ->offset, ->data, ->data, ... -> eof,      	<-ack
	//case5: starting offset : 0x40
	param->offset+=0x40;
	CheckCBUSBusy();
	CheckFW0Req();
	SetFW0_FIFO_clr();
	//GetMSCCommandCase(cmd_case);
	//SetFW0TXCase(cmd_case); merge to one function
	//SetFW0Cmd1(fx0_cmd1); merge to one function
	SetFW0TXCase_Cmd1(cmd_case,param->cmd);
	SetFW0Cmd2(0x32);	//set FW0 CMD2 for wrist_burst command
	SetFW0Offset(param->offset);
	//SetFW0Data1(fx0_data1);
	//SetFW0Data2(fx0_data2);

	SetFW0DataToFifo(param->data_len);	//Write data to buffer, only for case5

	TriggerFW0Req();
	WaitFW0Req();
	CheckReceiveCommand();		// <-ack
	//CheckReceiveData();				// <-data
	////CheckMSCErrCode();

	//Delay(300);
	//BusyWaiting(5000);
	if (CheckMSCErrCode() == 0)
	{
		printf("%s \n", "- write burst normal case\n");
		Test_SendMSCCommand_case_0_2_2a();
	}
	return CheckMSCErrCode();
}

int set_ddc_cmd_case0(CBUS_DDC_CMD_PARAM_ST *param)
{
	//UINT32 ret;
	printf("%s \n", "- ddc case 0 test");
	CheckCBUSBusy();
	CheckDDCReq();
	SetFW0_FIFO_clr();
	//GetDDCReqType(ddc_req_type);
	SetDDCReqType(param->cmd);
	//FillDDCAddr_0x60(ddc_addr_0x60);
	//FillDDCSegVal(ddc_seg_val);

	//FillDDCAddrW(ddc_addr_w);	merge to one
	FillDDCOffset(param->offset);
	//FillDDCAddrR(ddc_addr_r);	merge to one
	FillDDCAddrW_AddrR(param->addr_w,param->addr_r);

	SetFW0_FIFO_len(param->data_len);
	TriggerDDCReq();
	WaitDDCReq();
	return CheckDDCErrCode();
}

int set_ddc_cmd_case1(CBUS_DDC_CMD_PARAM_ST *param)
{
	//UINT32 ret;
	printf("%s \n", "- ddc case 1 test");
	CheckCBUSBusy();
	CheckDDCReq();
	SetFW0_FIFO_clr();
	//GetDDCReqType(ddc_req_type);
	SetDDCReqType(param->cmd);
	//FillDDCAddr_0x60(ddc_addr_0x60);	merge to one
	//FillDDCSegVal(ddc_seg_val);		merge to one
	//FillDDCAddrW(ddc_addr_w);		merge to one
	//FillDDCOffset(ddc_offset);			merge to one

	FillDDCAddr_0x60_SegVal_Offset(param->addr_0x60, param->seg_value, param->offset);
	//FillDDCAddrR(ddc_addr_r);	merge to one
	FillDDCAddrW_AddrR(param->addr_w,param->addr_r);

	SetFW0_FIFO_len(param->data_len);
	TriggerDDCReq();
	WaitDDCReq();
	return CheckDDCErrCode();
}

int set_ddc_cmd_case2(CBUS_DDC_CMD_PARAM_ST *param)
{
	//UINT32 ret;
	printf("%s \n", "- ddc case 2 test");
	CheckCBUSBusy();
	CheckDDCReq();
	SetFW0_FIFO_clr();
	//GetDDCReqType(ddc_req_type);
	SetDDCReqType(param->cmd);
	//FillDDCAddr_0x60(ddc_addr_0x60);
	//FillDDCSegVal(ddc_seg_val);
	//FillDDCAddrW(ddc_addr_w);
	//FillDDCOffset(ddc_offset);
	FillDDCAddrR(param->addr_r);
	SetFW0_FIFO_len(param->data_len);
	TriggerDDCReq();
	WaitDDCReq();
	return CheckDDCErrCode();
}

int set_ddc_cmd_case3(CBUS_DDC_CMD_PARAM_ST *param)
{
	//UINT32 ret;
	CheckCBUSBusy();
	CheckDDCReq();
	SetFW0_FIFO_clr();
	//GetDDCReqType(ddc_req_type);
	SetDDCReqType(param->cmd);
	//FillDDCAddr_0x60(ddc_addr_0x60);
	//FillDDCSegVal(ddc_seg_val);
	FillDDCAddrW(param->addr_w);
	FillDDCOffset(param->offset);
	//SetFW0Data(fx0_data); write data
	//FillDDCAddrR(ddc_addr_r);
	//SetFW0_FIFO_len(fw0_fifo_len);
	SetFW0DataToFifo(param->data_len);	//Write data to buffer, only for type3
	TriggerDDCReq();
	WaitDDCReq();

	printk("offset = %x, addr_w = %x, data_len = %x\n", param->offset, param->addr_w, param->data_len);
	return CheckDDCErrCode();
}

int test_send_ddc_case_3(CBUS_DDC_CMD_PARAM_ST *param)
{
	printf("%s \n", "- ddc case 3 test");
	CBUS_DDC_CMD_PARAM_ST tmp;
	int ret = 0;
	// UINT32 get_cmd, cnt = 0;
	// do {
	// 	get_cmd = CheckReceiveCommand();
	// 	if ((get_cmd == MSC_CMD_WRITE_STAT_SET_INT) ||
	// 		(get_cmd == MSC_CMD_SET_HPD))
	// 	{
	// 		cnt++;
	// 	}
	// 	printk("- cnt = %d\n", cnt);
	// } while(cnt != 3);
	//

	// printf("%s \n", "- busy wait");
	// BusyWaiting(5000);

	ret = set_ddc_cmd_case3(param);

	printf("%s \n", "- busy wait");
	BusyWaiting(5000);

	tmp.cmd = 3;
	tmp.offset = 0x12;
	tmp.seg_value = 0;
	tmp.addr_0x60 = 0;
	tmp.addr_w = DDC_HDCP_ADDR;
	tmp.addr_r = 0;
	tmp.data_len = 3;

	ret = set_ddc_cmd_case3(&tmp);
	return ret;
}

void get_data_from_fifo_to_buf(UINT8 len, UINT8* buf)
{
	UINT8 fetch_len, i;
	UINT32 regValue;

	fetch_len = (len > 16) ? 16 : len;

	for (i = 0; i < fetch_len; i++)
	{
		regValue = rtd_inl(MHL_FW0_REQ_08_reg);
		*(buf + i) = MHL_FW0_REQ_08_get_fw0_rdata(regValue);
	}
}