#include <qc_verify_common.h>
#include <peripheral_common.h>

UINT32 GetMaskAnd(UINT32 msb,UINT32 lsb);
#define RESULT_FAIL 1
#define RESULT_TRUE 0

#define ADDR_DRTCSEC   (0xb8061900)
#define ADDR_DRTCMIN   (0xb8061904)
#define ADDR_DRTCHR    (0xb8061908)
#define ADDR_DRTCDATE1 (0xb806190c)
#define ADDR_DRTCDATE2 (0xb8061910)
#define ADDR_DALMMIN   (0xb8061914)
#define ADDR_DALMHR    (0xb8061918)
#define ADDR_DALMDATE1 (0xb806191c)
#define ADDR_DALMDATE2 (0xb8061920)
#define ADDR_DRTCSTOP  (0xb8061924)
#define ADDR_ALMHSEC   (0xb8061928)
#define ADDR_DRTCEN    (0xb806192c)
#define ADDR_DRTCCR    (0xb8061930)

#define  ADDR_ISR_ISO_MISC 		0xb8061000 //BIT1; ARTC: BIT2 & BIT3

 typedef union _RTC_Time
 {
	UINT8	rtc_times[5];
	
 	struct
	{ 
		//total 32bit
		UINT8	rtc_date2;	//6bit
		UINT8	rtc_date1;	//8bit	
		UINT8	rtc_hour;	//5bit
		UINT8 	rtc_min;		//6bit
		UINT8	rtc_hsec;	//7bit
 	};
	
}RTC_Time;

RTC_Time RTCTimeFormat(UINT32 date, UINT8 hour, UINT8 min, UINT8 sec)
{
    	RTC_Time time;
	time.rtc_date2 = GetFieldValue(date, 13,8);
	time.rtc_date1 = GetFieldValue(date, 7, 0);
	time.rtc_hour = hour;
	time.rtc_min = min; 
	time.rtc_hsec = sec*2;

	return time;
}

UINT32 Get_RTC_Time_hSecs(RTC_Time time)
{
	UINT32 hsecs = 0;

	hsecs += ((time.rtc_date2<<8)  + time.rtc_date1)*24*60*60*2;
	hsecs += time.rtc_hour*60*60*2;
	hsecs += time.rtc_min*60*2;
	hsecs += time.rtc_hsec;

	return hsecs;
}

//time2 - time1
UINT32 Get_Time_Consuming(RTC_Time time1, RTC_Time time2)
{
	UINT32 getVal = 0;
	
	UINT32 getVal1, getVal2;
	getVal1 = Get_RTC_Time_hSecs(time1);
	getVal2 = Get_RTC_Time_hSecs(time2);
	getVal = getVal2 - getVal1;

	return getVal;
}

void DRTC_crt(void)
{
	//clken:0
	rtd_outl(0xb806013c,2);//clken_rtc
	//RSTN:0
	rtd_outl(0xb8060138,2);//rstn_rtc
	//clken:1
	rtd_outl(0xb806013c,3);//clken_rtc
	//clken:0
	rtd_outl(0xb806013c,2);//clken_rtc
	//RSTN:1
	rtd_outl(0xb8060138,3);//rstn_rtc
	//clken:1
	rtd_outl(0xb806013c,3);//clken_rtc
}
extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);
extern unsigned int rtd_part_inl(unsigned int addr, unsigned int B, unsigned int C);

void DRTC_status_clear(void)
{
	//Reset RTC Counter 
	rtd_part_outl(ADDR_DRTCEN,7,0,0x0);	//disable RTC
	rtd_part_outl(ADDR_DRTCCR,6,6,0x1);	//reset open
	rtd_part_outl(ADDR_DRTCCR,6,6,0x0);	//reset close
	//clear RTC interrupt
	rtd_outl(ADDR_DRTCCR,0xf8000000);
}

void DRTC_status(void)
{
	rtd_part_inl(ADDR_DRTCSTOP,0,0); //RTCSTOP: 1:RTC STOP;0:RTC Running

	//Check Counter time twice
	rtd_part_inl(ADDR_DRTCSEC,6,0);//get current second
	rtd_part_inl(ADDR_DRTCSEC,6,0);//get current second
	rtd_part_inl(ADDR_DRTCMIN,5,0);//get current minute
	rtd_part_inl(ADDR_DRTCMIN,5,0);//get current minute
	rtd_part_inl(ADDR_DRTCHR,4,0);//get current hour
	rtd_part_inl(ADDR_DRTCHR,4,0);//get current hour
	rtd_part_inl(ADDR_DRTCDATE1,7,0);//get current Date1(LSB)
	rtd_part_inl(ADDR_DRTCDATE1,7,0);//get current Date1(LSB)
	rtd_part_inl(ADDR_DRTCDATE2,6,0);//get current Date2(MSB)
	rtd_part_inl(ADDR_DRTCDATE2,6,0);//get current Date2(MSB)

	//Check local interrupt
	rtd_part_inl(ADDR_DRTCCR,31,31);//get alarm interrupt.
	rtd_part_inl(ADDR_DRTCCR,30,30);//get date interrupt.
	rtd_part_inl(ADDR_DRTCCR,29,29);//get hour interrupt.
	rtd_part_inl(ADDR_DRTCCR,28,28);//get min interrupt.
	rtd_part_inl(ADDR_DRTCCR,27,27);//get half_sec interrupt.

	//Check global interrupt
	rtd_part_inl(ADDR_ISR_ISO_MISC,1,1);//Global_RTC_INT(bit1:DRTC_INT)
}

void DRTC_status_para(void)
{
	//Check RTC parameters
	rtd_part_inl(ADDR_DRTCEN,7,0);	//RTCEN: 8h5A:Enabled;8h00:Disabled;else:RTC time not initialized.
	rtd_part_inl(ADDR_DRTCSTOP,0,0); //RTCSTOP: 1:RTC STOP;0:RTC Running
	
	//Check Alam time
	rtd_inl(ADDR_ALMHSEC);//read ALM half_sec
	rtd_inl(ADDR_DALMMIN);//read ALM minute
	rtd_inl(ADDR_DALMHR);//read ALM hour
	rtd_inl(ADDR_DALMDATE1);//read ALM Data LSB
	rtd_inl(ADDR_DALMDATE2);//read ALM Data MSB
}

UINT32 DRTC_Get_INT_status(void)
{
	UINT32 getVal = 0;
	
	//Check local interrupt
	getVal |= rtd_part_inl(ADDR_DRTCCR,27,27);		//get half_sec interrupt.
	getVal |= rtd_part_inl(ADDR_DRTCCR,28,28)<<1;	//get min interrupt.
	getVal |= rtd_part_inl(ADDR_DRTCCR,29,29)<<2;	//get hour interrupt.
	getVal |= rtd_part_inl(ADDR_DRTCCR,30,30)<<3;	//get date interrupt.
	getVal |= rtd_part_inl(ADDR_DRTCCR,31,31)<<4;	//get alarm interrupt.

	//Check global interrupt
	getVal |= rtd_part_inl(ADDR_ISR_ISO_MISC,1,1)<<5;		//Global_RTC_INT(bit1:DRTC_INT)

	return getVal;
}

void DRTC_Set_ALM(RTC_Time time)
{
	rtd_outl(ADDR_DRTCEN,0x00);//disable RTC	

	rtd_outl(ADDR_ALMHSEC,time.rtc_hsec);	
	rtd_outl(ADDR_DALMMIN,time.rtc_min);		//write ALM minute
	rtd_outl(ADDR_DALMHR,time.rtc_hour);		//write ALM hour
	rtd_outl(ADDR_DALMDATE1,time.rtc_date1);	//write ALM Data LSB
	rtd_outl(ADDR_DALMDATE2,time.rtc_date2);	//write ALM Data MSB

	rtd_part_outl(ADDR_DRTCCR,4,0,0x1f);//Enable Local int
	rtd_part_outl(ADDR_DRTCEN,7,0,0x5a);//Enable RTC
}

void DRTC_Set_Time(RTC_Time time)
{
	rtd_part_outl(ADDR_DRTCEN,7,0,0x00);	//disable RTC

	rtd_part_outl(ADDR_DRTCSEC,6,0,time.rtc_hsec);		//write second:59
	rtd_part_outl(ADDR_DRTCMIN,5,0,time.rtc_min);		//write minute:59
	rtd_part_outl(ADDR_DRTCHR,4,0,time.rtc_hour);		//write hour:23
	rtd_part_outl(ADDR_DRTCDATE1,7,0,time.rtc_date1); 	//write Date1(LSB)
	rtd_part_outl(ADDR_DRTCDATE2,6,0,time.rtc_date2);	//write Date2(MSB)

	rtd_part_outl(ADDR_DRTCEN,7,0,0x5a);	//Enable RTC
}

RTC_Time DRTC_Get_Time(void)
{	
	RTC_Time time;
	
	//rtd_part_outl(ADDR_DRTCEN,7,0,0x00);	//disable RTC

	time.rtc_hsec = rtd_part_inl(ADDR_DRTCSEC,6,0);	//get current second
	time.rtc_hsec = rtd_part_inl(ADDR_DRTCSEC,6,0);	//get current second
	time.rtc_min = rtd_part_inl(ADDR_DRTCMIN,5,0);		//get current minute
	time.rtc_min = rtd_part_inl(ADDR_DRTCMIN,5,0);		//get current minute
	time.rtc_hour = rtd_part_inl(ADDR_DRTCHR,4,0);	//get current hour
	time.rtc_hour = rtd_part_inl(ADDR_DRTCHR,4,0);	//get current hour
	time.rtc_date1 = rtd_part_inl(ADDR_DRTCDATE1,7,0);	//get current Date1(LSB)
	time.rtc_date1 = rtd_part_inl(ADDR_DRTCDATE1,7,0);	//get current Date1(LSB)
	time.rtc_date2 = rtd_part_inl(ADDR_DRTCDATE2,6,0);	//get current Date2(MSB)
	time.rtc_date2 = rtd_part_inl(ADDR_DRTCDATE2,6,0);	//get current Date2(MSB)

	//rtd_part_outl(ADDR_DRTCEN,7,0,0x5a);	//Enable RTC

	return time;
}

////////////////////////////////////////////////////////////////////////////////////
//----------------------------------------------------------------------------------

int Verify_DRTC(void)
{
    printk("[BEGIN] DRTC Verify.... \n");
	

	UINT32 getVal = 0;
	UINT32 tryTimes=0;
	
	RTC_Time set_times,alm_times, get_times;
	alm_times = RTCTimeFormat(4,0,0,0);	 	//date/hour/min/sec
	set_times = RTCTimeFormat(3,23,59,58);		//date/hour/min/sec
	
	//init crt
	DRTC_crt();
	
	
	//status & clear status
	DRTC_status_clear();

	//Set Alam time: 4date
	DRTC_Set_ALM(alm_times);
	//Set Counter begin time: 3date/23:59:58
	DRTC_Set_Time(set_times);

	mdelay(1000);	//delay 1s
	mdelay(800);	//delay 

	//Check in high-precision 
	tryTimes = 30;
	while(tryTimes>0)
	{
		getVal = rtd_part_inl(ADDR_DRTCCR,31,31);//get alarm interrupt.
		if(getVal)
		{
			rtd_part_outl(ADDR_DRTCEN,7,0,0x00);	//disable RTC

			//get status
			getVal =  DRTC_Get_INT_status();

			//get time
			get_times = DRTC_Get_Time();
			break;
		}

		tryTimes--;
		mdelay(10);	//delay 10ms	

	}

    	//---------check RTC -------------------------------------------			
	//check int
    	if(tryTimes==0)
    	{
  		printk("[END][Timeout!] alarm interrupt hasn't raised! \n");
  		return RESULT_FAIL;
	}	
	if(getVal != (1<<6)-1) //binary:111111 = pow(2,6) -1
	{
  		printk("[END][FAIL] RTC interrupt is not correct: total int = 0x%x!\n", getVal);		
		return RESULT_FAIL;
	}

	//check time 
	getVal = Get_Time_Consuming(set_times, get_times);
	printk("DRTC test result time length: %d sec. \n",getVal);
	if(getVal>8) 	// 2*delay*2hsec = 2*2*2
	{
  		printk("[END][FAIL] RTC Timing is not verify accurate: want 4*hsec, get %d*hsec \n", getVal);		
		return RESULT_FAIL;
	}

	//check reset & int clear
	DRTC_status_clear();
	
	get_times = DRTC_Get_Time();
	getVal = Get_RTC_Time_hSecs(get_times);
	if(getVal != 0) 
	{
  		printk("[END][FAIL] RTC time hasn't been reset: time = %d (hsecs) \n", getVal);		
		return RESULT_FAIL;
	}
	
	getVal = DRTC_Get_INT_status();
	if(getVal > 0x1)	//if has, only hsec int
	{
  		printk("[END][FAIL] RTC int hasn't been cleared correctly: total int = %d \n", getVal);		
		return RESULT_FAIL;
	}

    	printk("[END][Success] DRTC Verify \n");
	return RESULT_TRUE;

}



int Verify_RTC(void)
{
	int fail_result  = 0;
	fail_result |= Verify_DRTC();

	if(fail_result>0)
		return RESULT_FAIL;
	else
		return RESULT_TRUE;
		
}

