/*
 * (C) Copyright 2001
 * Denis Peter, MPL AG Switzerland
 *
 * Most of this source has been derived from the Linux USB
 * project.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

/*
 * How it works:
 *
 * Since this is a bootloader, the devices will not be automatic
 * (re)configured on hot/plug, but after a restart of the USB the
 * device should work.
 *
 * For each transfer (except "Interrupt") we wait for completion.
 */


#include <common.h>
#include "linux/compat.h"
#include <asm/arch/system.h>
#include <rtd_types_qc.h>
#include "usb/read_define.h"


//#include <rtd_system.h>
#include <usb/usb.h>
//#include <misc.h>
//--#include <asm/addrspace.h>

#if defined(CONFIG_BOOT_TIMER)
#include <timer.h>
#endif




#undef	USB_DEBUG
//#define USB_DEBUG 1

#ifdef	USB_DEBUG
#define	USB_PRINTF(fmt,args...)	printk (fmt ,##args)
#else
#define USB_PRINTF(fmt,args...)
#endif

#define USB_BUFSIZ	512

#define TEST_MAX_DEV	2

static struct usb_device usb_dev[USB_MAX_DEVICE];
static struct usb_device *pusb_dev;

static int dev_index;
static int running;
static int asynch_allowed;

int USB_Host_Type=USB_EHCI_HOST;

#if defined(CONFIG_USB_PATCH)
static struct devrequest setup_packet __attribute__ ((aligned (512)));
#else
static struct devrequest setup_packet;
#endif
// static struct devrequest *psetup_packet;

/**********************************************************************
 * some forward declerations...
 */
int usb_scan_devices(void);

int usb_hub_probe_qc(struct usb_device *dev, int ifnum);
void usb_hub_reset_qc(void);

/***********************************************************************
 * wait_ms
 */

void wait_ms(unsigned long ms)
{
	//printk("ms=%d\n",ms);
	while(ms-->0)
		udelay(1000);
}
/* --------------------------------------------------------------------------------------------------------- */


extern int ehci_lowlevel_init(void);
extern int ohci_lowlevel_init(void);
extern int xhci_lowlevel_init(void);

extern int ehci_lowlevel_stop(void);
extern int ohci_lowlevel_stop(void);
extern int xhci_lowlevel_stop(void);


extern int xhci_submit_control_msg(struct usb_device *usbdev, unsigned long pipe, void *buffer,
					int length, struct devrequest *setup);
		
extern int ehci_submit_control_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len, struct devrequest *setup);
		
extern int ehci_submit_bulk_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len);	
		
		
extern int ohci_submit_control_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len, struct devrequest *setup);
		
extern int ohci_submit_bulk_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len);		

extern int xhci_submit_bulk_msg(struct usb_device *usbdev, unsigned long pipe, void *buffer,int length);

extern long _read_reg(struct usb_device *dev, unsigned char mcu_type, unsigned int ocp_address, unsigned int length, void *buffer);
extern long _write_reg(struct usb_device *dev, unsigned char mcu_type, unsigned int ocp_address, unsigned int length, void *buffer);


extern void xhci_set_slot_id(int slot_id);

void usb_set_slot_id(int slot_id)
{
if (USB_Host_Type == USB_XHCI_HOST) {
	xhci_set_slot_id(slot_id);
}
	
}


static int submit_control_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len, struct devrequest *setup)
{

#if defined(CONFIG_BOOT_USB_XHCI)
	if (USB_Host_Type == USB_XHCI_HOST)	
		return xhci_submit_control_msg(dev,pipe,buffer,transfer_len,setup);
#endif


#if defined(CONFIG_BOOT_USB_EHCI)	
	if (USB_Host_Type == USB_EHCI_HOST)	
		return ehci_submit_control_msg(dev,pipe,buffer,transfer_len,setup);
#endif		

#if defined(CONFIG_BOOT_USB_OHCI)
	if (USB_Host_Type == USB_OHCI_HOST)	
		return ohci_submit_control_msg(dev,pipe,buffer,transfer_len,setup);
#endif		
	
	printk("%s() : Unknow USB host type = 0x%x\n",__FUNCTION__,USB_Host_Type); 
		
	return 0;		
}

static int submit_bulk_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len)
{

#if defined(CONFIG_BOOT_USB_XHCI)	
	if (USB_Host_Type == USB_XHCI_HOST)	
		return xhci_submit_bulk_msg(dev,pipe,buffer,transfer_len);
#endif	

#if defined(CONFIG_BOOT_USB_EHCI)	
	if (USB_Host_Type == USB_EHCI_HOST)	
		return ehci_submit_bulk_msg(dev,pipe,buffer,transfer_len);
#endif		

#if defined(CONFIG_BOOT_USB_OHCI)
	if (USB_Host_Type == USB_OHCI_HOST)	
		return ohci_submit_bulk_msg(dev,pipe,buffer,transfer_len);
#endif		
	
	printk("%s() : Unknow USB host type = 0x%x\n",__FUNCTION__,USB_Host_Type); 
		
	return 0;	
}	

int submit_int_msg_qc(struct usb_device *dev, unsigned long pipe, void *buffer,
		int transfer_len, int interval)
{
	// info("submit_int_msg");
	printk("%s()\n",__FUNCTION__);
	return -1;
}	
/* --------------------------------------------------------------------------------------------------------- */

/***************************************************************************
 * Init USB Device
 */
#define USB_HOST_N_PORT 		(2)

int usb_start(int usb_host)
{
	int result;
	running=0;
	dev_index=0;
	asynch_allowed=1;

#if defined(CONFIG_BOOT_USB_XHCI)
	if(usb_host == USB_XHCI_HOST){
		USB_Host_Type=USB_XHCI_HOST;
		usb_hub_reset_qc();
		/* init low_level USB */
		printk("USB:   \n");
		
		result = xhci_lowlevel_init();
	
		/* if lowlevel init is OK, scan the bus for devices i.e. search HUBs and configure them */
		if(result==0) {
			printk("USB3.0 scanning bus for devices... ");
			running=1;
			if(usb_scan_devices()<2){
				xhci_lowlevel_stop();
				printk("Could not find USB3.0 device by XHCI\n");
				return -1;
			}else
				return 0;
		}
		else {
			printk("Error, couldn't init Lowlevel part\n");
			return -1;
		}
	}
#endif


	
#if defined(CONFIG_BOOT_USB_EHCI)
	if(usb_host == USB_EHCI_HOST){
		USB_Host_Type=USB_EHCI_HOST;
		usb_hub_reset_qc();
		/* init low_level USB */
		printk("USB:   \n");
		result = ehci_lowlevel_init();
		/* if lowlevel init is OK, scan the bus for devices i.e. search HUBs and configure them */
		if(result==0) {
			printk("USB2.0 scanning bus for devices... ");
			running=1;
			if(usb_scan_devices()<2){
		//		ehci_lowlevel_stop();
				printk("Could not find USB2.0 device by EHCI\n");
				return -1;
			}else
				return 0;
		}
		else {
			printk("Error, couldn't init Lowlevel part\n");
			return -1;
		}
	}
#endif	

#if defined(CONFIG_BOOT_USB_OHCI)
	if(usb_host == USB_OHCI_HOST){
		USB_Host_Type=USB_OHCI_HOST;
		usb_hub_reset_qc();
		result = ohci_lowlevel_init();
				
		if(result==0) {
			printk("USB1.1 scanning bus for devices... ");
			running=1;
			if(usb_scan_devices()<2){
				printk("Could not find USB1.0 device by OHCI\n");
				ohci_lowlevel_stop();
				return -1;
			}else
				return 0;
		}
		else {
			printk("Error, couldn't init Lowlevel part\n");
			return -1;
		}
	}
#endif // CONFIG_BOOT_USB_OHCI
	printk("Error, Unknow usb host type\n");
	return -1;
}

/******************************************************************************
 * Stop USB this stops the LowLevel Part and deregisters USB devices.
 */
int usb_stop_qc(int host_type)
{
	asynch_allowed=1;
	usb_hub_reset_qc();	


#if defined(CONFIG_BOOT_USB_XHCI)	
	if (host_type == USB_XHCI_HOST) {
		printk("XHCI stopping ..\n");
		return xhci_lowlevel_stop();
	}	
#endif	

	
#if defined(CONFIG_BOOT_USB_EHCI)	
	if (host_type == USB_EHCI_HOST) {
		printk("EHCI stopping ..\n");
		return ehci_lowlevel_stop();
	}	
#endif	
	
#if defined(CONFIG_BOOT_USB_OHCI)	
	if (host_type == USB_OHCI_HOST) {
		printk("OHCI stopping ..\n");
		return ohci_lowlevel_stop();
	}	
#endif	

	printk("%s() : Unknow USB host type = 0x%x\n",__FUNCTION__,USB_Host_Type); 
	
	return 0;
}

/*
 * disables the asynch behaviour of the control message. This is used for data
 * transfers that uses the exclusiv access to the control and bulk messages.
 */
void usb_disable_asynch_qc(int disable)
{
	asynch_allowed=!disable;
}


/*-------------------------------------------------------------------
 * Message wrappers.
 *
 */

/*
 * submits an Interrupt Message
 */
int usb_submit_int_msg_qc(struct usb_device *dev, unsigned long pipe,
			void *buffer,int transfer_len, int interval)
{
#ifdef ENABLE_KSEG1ADDR
	if (buffer)
		buffer = (void *) KSEG1ADDR(buffer);
#endif /* ENABLE_KSEG1ADDR */
		
	return submit_int_msg_qc(dev,pipe,buffer,transfer_len,interval);
}

/*
 * submits a control message and waits for comletion (at least timeout * 1ms)
 * If timeout is 0, we don't wait for completion (used as example to set and
 * clear keyboards LEDs). For data transfers, (storage transfers) we don't
 * allow control messages with 0 timeout, by previousely resetting the flag
 * asynch_allowed (usb_disable_asynch(1)).
 * returns the transfered length if OK or -1 if error. The transfered length
 * and the current status are stored in the dev->act_len and dev->status.
 */
int usb_control_msg_qc(struct usb_device *dev, unsigned int pipe,
			unsigned char request, unsigned char requesttype,
			unsigned short value, unsigned short index,
			void *data, unsigned short size, int timeout)
{
#ifdef ENABLE_KSEG1ADDR
	struct devrequest *psetup_packet = (struct devrequest *) KSEG1ADDR(&setup_packet); 
#else /* ENABLE_KSEG1ADDR */
	struct devrequest *psetup_packet = (struct devrequest *) &setup_packet; 
#endif /* ENABLE_KSEG1ADDR */
	if((timeout==0)&&(!asynch_allowed)) /* request for a asynch control pipe is not allowed */
		return -1;
	/* set setup command */
	psetup_packet->requesttype = requesttype;
	psetup_packet->request = request;
	psetup_packet->value = swap_16(value);
	psetup_packet->index = swap_16(index);
	psetup_packet->length = swap_16(size);
 	USB_PRINTF("usb_control_msg: request: 0x%x, requesttype: 0x%x\nvalue 0x%x index 0x%x length 0x%x\n",
		request,requesttype,value,index,size);
	dev->status=USB_ST_NOT_PROC; /*not yet processed */
    
#ifdef ENABLE_KSEG1ADDR
	if (data)
		data = (void *) KSEG1ADDR(data);
#endif /* ENABLE_KSEG1ADDR */
 
	submit_control_msg(dev,pipe,data,size,psetup_packet);
	if(timeout==0) {
		return (int)size;
	}
	while(timeout--) {
		if(!((volatile unsigned long)dev->status & USB_ST_NOT_PROC))
			break;
		wait_ms(1);
	}
	if(dev->status==0)
		return dev->act_len;
	else {
		return -1;
	}
}

/*-------------------------------------------------------------------
 * submits bulk message, and waits for completion. returns 0 if Ok or
 * -1 if Error.
 * synchronous behavior
 */
int usb_bulk_msg_qc(struct usb_device *dev, unsigned int pipe,
			void *data, int len, int *actual_length, int timeout)
{
	// printk("%s : data = 0x%x\n",__FUNCTION__,data);
	if (len < 0)
		return -1;
	dev->status=USB_ST_NOT_PROC; /*not yet processed */

#ifdef ENABLE_KSEG1ADDR
	if (data)
		data = (void *) KSEG1ADDR(data);
#endif /* ENABLE_KSEG1ADDR */		
	
	submit_bulk_msg(dev,pipe,data,len);
	while(timeout--) {
		if(!((volatile unsigned long)dev->status & USB_ST_NOT_PROC))
			break;
		wait_ms(1);
	}
	
	*actual_length=dev->act_len;
	
	if(dev->status==0)
		return 0;
	else
		return -1;
}


/*-------------------------------------------------------------------
 * Max Packet stuff
 */

/*
 * returns the max packet size, depending on the pipe direction and
 * the configurations values
 */
int usb_maxpacket_qc(struct usb_device *dev,unsigned long pipe)
{
	if((pipe & USB_DIR_IN)==0) /* direction is out -> use emaxpacket out */
		return(dev->epmaxpacketout[((pipe>>15) & 0xf)]);
	else
		return(dev->epmaxpacketin[((pipe>>15) & 0xf)]);
}

/*
 * set the max packed value of all endpoints in the given configuration
 */
int usb_set_maxpacket(struct usb_device *dev)
{
	int i,ii,b;
	struct usb_endpoint_descriptor *ep;

	for(i=0; i<dev->config.bNumInterfaces;i++) {
		for(ii=0; ii<dev->config.if_desc[i].bNumEndpoints; ii++) {
#if defined(CONFIG_BOOT_USB_XHCI)
			ep=&dev->config.if_desc[i].ep_desc[ii].ep_desc;
#else
			ep=&dev->config.if_desc[i].ep_desc[ii];
#endif
			b=ep->bEndpointAddress & USB_ENDPOINT_NUMBER_MASK;

			if((ep->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK)==USB_ENDPOINT_XFER_CONTROL) {	/* Control => bidirectional */
				dev->epmaxpacketout[b] = ep->wMaxPacketSize;
				dev->epmaxpacketin [b] = ep->wMaxPacketSize;
				USB_PRINTF("##Control EP epmaxpacketout/in[%d] = %d\n",b,dev->epmaxpacketin[b]);
			}
			else {
				if ((ep->bEndpointAddress & 0x80)==0) { /* OUT Endpoint */
					if(ep->wMaxPacketSize > dev->epmaxpacketout[b]) {
						dev->epmaxpacketout[b] = ep->wMaxPacketSize;
						USB_PRINTF("##EP epmaxpacketout[%d] = %d\n",b,dev->epmaxpacketout[b]);
					}
				}
				else  { /* IN Endpoint */
					if(ep->wMaxPacketSize > dev->epmaxpacketin[b]) {
						dev->epmaxpacketin[b] = ep->wMaxPacketSize;
						USB_PRINTF("##EP epmaxpacketin[%d] = %d\n",b,dev->epmaxpacketin[b]);
					}
				} /* if out */
			} /* if control */
		} /* for each endpoint */
	}
	return 0;
}

/*******************************************************************************
 * Parse the config, located in buffer, and fills the dev->config structure.
 * Note that all little/big endian swapping are done automatically.
 */
int usb_parse_config(struct usb_device *dev, unsigned char *buffer, int cfgno)
{
	struct usb_descriptor_header *head;
	int index,ifno,epno;
	//int i = 0;
	ifno=-1;
	epno=-1;

	dev->configno=cfgno;
	head =(struct usb_descriptor_header *)&buffer[0];
	if(head->bDescriptorType!=USB_DT_CONFIG) {
		printk(" ERROR: NOT USB_CONFIG_DESC %x\n",head->bDescriptorType);
		return -1;
	}
	memcpy(&dev->config,buffer,buffer[0]);
	dev->config.wTotalLength=swap_16(dev->config.wTotalLength);
	dev->config.no_of_if=0;
	index=dev->config.bLength;
	/* Ok the first entry must be a configuration entry, now process the others */
	head=(struct usb_descriptor_header *)&buffer[index];

	while(index+1 < dev->config.wTotalLength) {
		switch(head->bDescriptorType) {
			case USB_DT_INTERFACE:
				ifno=dev->config.no_of_if;
				dev->config.no_of_if++; /* found an interface desc, increase numbers */
				memcpy(&dev->config.if_desc[ifno],&buffer[index],buffer[index]); /* copy new desc */
				dev->config.if_desc[ifno].no_of_ep=0;
				USB_PRINTF("interface..........\n");
				break;
			case USB_DT_ENDPOINT:
				
				epno=dev->config.if_desc[ifno].no_of_ep;
				dev->config.if_desc[ifno].no_of_ep++; /* found an endpoint */
#if defined(CONFIG_BOOT_USB_XHCI)
				memcpy(&dev->config.if_desc[ifno].ep_desc[epno].ep_desc,&buffer[index],buffer[index]);
#else
				memcpy(&dev->config.if_desc[ifno].ep_desc[epno],&buffer[index],buffer[index]);
#endif
				// cyhuang : Debug WORD alignment.
				// printk("dev->config.if_desc[ifno].ep_desc[epno].wMaxPacketSize = 0x%x\n", &dev->config.if_desc[ifno].ep_desc[epno].wMaxPacketSize);

#if defined(CONFIG_BOOT_USB_XHCI)
				dev->config.if_desc[ifno].ep_desc[epno].ep_desc.wMaxPacketSize
					=swap_16(dev->config.if_desc[ifno].ep_desc[epno].ep_desc.wMaxPacketSize);
#else
				dev->config.if_desc[ifno].ep_desc[epno].wMaxPacketSize
					=swap_16(dev->config.if_desc[ifno].ep_desc[epno].wMaxPacketSize);
#endif
				USB_PRINTF("if %d, ep %d\n",ifno,epno);
				break;


			case USB_DT_SS_ENDPOINT_COMP:
if (USB_Host_Type == USB_XHCI_HOST) {
				USB_PRINTF("DT_SS_ENDPOINT_COMP\n");
				memcpy(&(dev->config.if_desc[ifno].ep_desc[epno].ss_ep_comp),&buffer[index], buffer[index]);
}
				break;


			default:
				if(head->bLength==0)
					return 1;
				USB_PRINTF("unknown Description Type : %x\n",head->bDescriptorType);
				{
					#if USB_DEBUG
					int i;
					unsigned char *ch;
					ch=(unsigned char *)head;
					for(i=0;i<head->bLength; i++)
						USB_PRINTF("0x%x ",*ch++);
					USB_PRINTF("\n\n\n");
					#endif
				}
				break;
		}
		index+=head->bLength;
		head=(struct usb_descriptor_header *)&buffer[index];
	}
	return 1;
}

/***********************************************************************
 * Clears an endpoint
 * endp: endpoint number in bits 0-3;
 * direction flag in bit 7 (1 = IN, 0 = OUT)
 */
int usb_clear_halt_qc(struct usb_device *dev, int pipe)
{
	int result;
	
#if defined(CONFIG_USB_PATCH)	
	static unsigned short status __attribute__ ((aligned (512)));
#else
	unsigned short status;
#endif
	
	int endp=usb_pipeendpoint(pipe)|(usb_pipein(pipe)<<7);

	result = usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_CLEAR_FEATURE, USB_RECIP_ENDPOINT, 0, endp, NULL, 0, USB_CNTL_TIMEOUT * 3);

	/* don't clear if failed */
	if (result < 0)
		return result;
	result = usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
		USB_REQ_GET_STATUS, USB_DIR_IN | USB_RECIP_ENDPOINT, 0, endp,
		&status, sizeof(status), USB_CNTL_TIMEOUT * 3);
	if (result < 0)
		return result;
	USB_PRINTF("usb_clear_halt: status 0x%x\n",status);
	if (status & 1)
		return -1;		/* still halted */
	usb_endpoint_running(dev, usb_pipeendpoint(pipe), usb_pipeout(pipe));
	/* toggle is reset on clear */
	usb_settoggle(dev, usb_pipeendpoint(pipe), usb_pipeout(pipe), 0);
	return 0;
}


/**********************************************************************
 * get_descriptor type
 */
int usb_get_descriptor(struct usb_device *dev, unsigned char type, unsigned char index, void *buf, int size)
{
	int res;
 	res = usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
			USB_REQ_GET_DESCRIPTOR, USB_DIR_IN,
			(type << 8) + index, 0,
			buf, size, USB_CNTL_TIMEOUT);
	return res;
}

/**********************************************************************
 * gets configuration cfgno and store it in the buffer
 */
int usb_get_configuration_no_qc(struct usb_device *dev,unsigned char *buffer,int cfgno)
{
 	int result;
	unsigned int tmp;
	struct usb_config_descriptor *config;


	config=(struct usb_config_descriptor *)&buffer[0];
	result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, buffer, 8);
	if (result < 8) {
		if (result < 0)
			printk("unable to get descriptor, error 0x%lx\n",dev->status);
		else
			printk("config descriptor too short (expected %i, got %i)\n",8,result);
		return -1;
	}
	tmp=swap_16(config->wTotalLength);

	if (tmp > USB_BUFSIZ) {
		USB_PRINTF("usb_get_configuration_no: failed to get descriptor - too long: %d\n",
			tmp);
		return -1;
	}

	result = usb_get_descriptor(dev, USB_DT_CONFIG, cfgno, buffer, tmp);
	USB_PRINTF("get_conf_no %d Result %d, wLength %d\n",cfgno,result,tmp);
	return result;
}

/********************************************************************
 * set address of a device to the value in dev->devnum.
 * This can only be done by addressing the device via the default address (0)
 */
int usb_set_address(struct usb_device *dev)
{
	int res;

	USB_PRINTF("set address %d\n",dev->devnum);

	res=usb_control_msg_qc(dev, usb_snddefctrl(dev),
		USB_REQ_SET_ADDRESS, 0,
		(dev->devnum),0,
		NULL,0, USB_CNTL_TIMEOUT);
	return res;
}

/********************************************************************
 * set interface number to interface
 */
int usb_set_interface_qc(struct usb_device *dev, int interface, int alternate)
{
	struct usb_interface_descriptor *if_face = NULL;
	int ret, i;
	



#if defined(CONFIG_BOOT_USB_XHCI)

	printk("@@@@@@@@@@@@ set interface %x %x %x\n", dev->config.no_of_if, dev->config.bNumInterfaces,interface);
	for (i=0; i<dev->config.no_of_if; i++) {
		if ((dev->config.if_desc[i].bInterfaceNumber == interface) && (dev->config.if_desc[i].bAlternateSetting == alternate)) {
			if_face = &dev->config.if_desc[i];
			dev->cur_interface = i;
			break;
		}
	}
#else

		for (i=0; i<dev->config.bNumInterfaces; i++) {
			if (dev->config.if_desc[i].bInterfaceNumber == interface) {
				if_face = &dev->config.if_desc[i];
				break;
			}
		}
#endif

	if (!if_face) {
		printk("selecting invalid interface %d", interface);
		return -1;
	}


	if ((ret = usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
	    USB_REQ_SET_INTERFACE, USB_RECIP_INTERFACE, alternate,
	    interface, NULL, 0, USB_CNTL_TIMEOUT * 5)) < 0)
		return ret;

	if_face->act_altsetting = (unsigned char)alternate;
	usb_set_maxpacket(dev);
	return 0;
}

/********************************************************************
 * set configuration number to configuration
 */
int usb_set_configuration(struct usb_device *dev, int configuration)
{
	int res;
	USB_PRINTF("set configuration %d\n",configuration);
	/* set setup command */
	res=usb_control_msg_qc(dev, usb_sndctrlpipe(dev,0),
		USB_REQ_SET_CONFIGURATION, 0,
		configuration,0,
		NULL,0, USB_CNTL_TIMEOUT);
	if(res==0) {
		dev->toggle[0] = 0;
		dev->toggle[1] = 0;
		return 0;
	}
	else
		return -1;
}

/********************************************************************
 * set protocol to protocol
 */
int usb_set_protocol_qc(struct usb_device *dev, int ifnum, int protocol)
{
	return usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_SET_PROTOCOL, USB_TYPE_CLASS | USB_RECIP_INTERFACE,
		protocol, ifnum, NULL, 0, USB_CNTL_TIMEOUT);
}

/********************************************************************
 * set idle
 */
int usb_set_idle_qc(struct usb_device *dev, int ifnum, int duration, int report_id)
{
	return usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_SET_IDLE, USB_TYPE_CLASS | USB_RECIP_INTERFACE,
		(duration << 8) | report_id, ifnum, NULL, 0, USB_CNTL_TIMEOUT);
}

/********************************************************************
 * get report
 */
int usb_get_report_qc(struct usb_device *dev, int ifnum, unsigned char type, unsigned char id, void *buf, int size)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
		USB_REQ_GET_REPORT, USB_DIR_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE,
		(type << 8) + id, ifnum, buf, size, USB_CNTL_TIMEOUT);
}

/********************************************************************
 * get class descriptor
 */
int usb_get_class_descriptor_qc(struct usb_device *dev, int ifnum,
		unsigned char type, unsigned char id, void *buf, int size)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
		USB_REQ_GET_DESCRIPTOR, USB_RECIP_INTERFACE | USB_DIR_IN,
		(type << 8) + id, ifnum, buf, size, USB_CNTL_TIMEOUT);
}

/********************************************************************
 * get string index in buffer
 */
int usb_get_string(struct usb_device *dev, unsigned short langid, unsigned char index, void *buf, int size)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
		USB_REQ_GET_DESCRIPTOR, USB_DIR_IN,
		(USB_DT_STRING << 8) + index, langid, buf, size, USB_CNTL_TIMEOUT);
}

/********************************************************************
 * usb_string:
 * Get string index and translate it to ascii.
 * returns string length (> 0) or error (< 0)
 */
int usb_string_qc(struct usb_device *dev, int index, char *buf, size_t size)
{
#if defined(CONFIG_USB_PATCH)	
	static unsigned char mybuf[USB_BUFSIZ] __attribute__ ((aligned (1024)));
	static unsigned char *tbuf;
#else
	unsigned char mybuf[USB_BUFSIZ];
	unsigned char *tbuf;
#endif	
	int err;
	unsigned int u, idx;

	if (size <= 0 || !buf || !index)
		return -1;
	buf[0] = 0;

#ifdef ENABLE_KSEG1ADDR
	tbuf= (unsigned char *) KSEG1ADDR(&mybuf[0]);
#else /* ENABLE_KSEG1ADDR */
	tbuf= (unsigned char *) mybuf;
#endif /* ENABLE_KSEG1ADDR */

	/* get langid for strings if it's not yet known */
	if (!dev->have_langid) {
		err = usb_get_string(dev, 0, 0, tbuf, 4);
		if (err < 0) {
			USB_PRINTF("error getting string descriptor 0 (error=%x)\n",dev->status);
			return -1;
		} else if (tbuf[0] < 4) {
			USB_PRINTF("string descriptor 0 too short\n");
			return -1;
		} else {
			dev->have_langid = -1;
			dev->string_langid = tbuf[2] | (tbuf[3]<< 8);
				/* always use the first langid listed */
			USB_PRINTF("USB device number %d default language ID 0x%x\n",
				dev->devnum, dev->string_langid);
		}
	}
	/* Just ask for a maximum length string and then take the length
	 * that was returned. */
	err = usb_get_string(dev, dev->string_langid, index, tbuf, 4);
	if (err < 0)
		return err;
	u=tbuf[0];
	USB_PRINTF("Strn Len %d, index %d\n",u,index);

	if (u > USB_BUFSIZ) {
		USB_PRINTF("usb_string: failed to get string - too long: %d\n", u);
		return -1;
	}

	err = usb_get_string(dev, dev->string_langid, index, tbuf, u);
	if (err < 0)
		return err;
	size--;		/* leave room for trailing NULL char in output buffer */
	for (idx = 0, u = 2; u < err; u += 2) {
		if (idx >= size)
			break;
		if (tbuf[u+1])			/* high byte */
			buf[idx++] = '?';  /* non-ASCII character */
		else
			buf[idx++] = tbuf[u];
	}
	buf[idx] = 0;
	err = idx;
	return err;
}



/********************************************************************
 * USB device handling:
 * the USB device are static allocated [USB_MAX_DEVICE].
 */


/* returns a pointer to the device with the index [index].
 * if the device is not assigned (dev->devnum==-1) returns NULL
 */
struct usb_device * usb_get_dev_index_qc(int index)
{
	if(pusb_dev[index].devnum==-1)
		return NULL;
	else
		return &pusb_dev[index];
}


/* returns a pointer of a new device structure or NULL, if
 * no device struct is available
 */
struct usb_device * usb_alloc_new_device_qc(void)
{
	int i;
	USB_PRINTF("New Device %d\n",dev_index);
	if(dev_index==USB_MAX_DEVICE) {
		printk("ERROR, too many USB Devices, max=%d\n",USB_MAX_DEVICE);
		return NULL;
	}
	pusb_dev[dev_index].devnum=dev_index+1; /* default Address is 0, real addresses start with 1 */
	pusb_dev[dev_index].maxchild=0;
	for(i=0;i<USB_MAXCHILDREN;i++)
		pusb_dev[dev_index].children[i]=NULL;
	pusb_dev[dev_index].parent=NULL;
	dev_index++;
	return &pusb_dev[dev_index-1];
}


/* Allocate usb device */
int usb_alloc_dev(struct usb_device *dev)
{
	int res;

	printk("alloc device\n");
	#if 1
	res = usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
				USB_REQ_ALLOC_DEV, 0, 0, 0,
				NULL, 0, USB_CNTL_TIMEOUT);

	#else
	res = usb_control_msg_qc(dev, usb_sndctrlpipe(dev->parent, 0),
				USB_REQ_ALLOC_DEV, 0, 0, 0,
				NULL, 0, USB_CNTL_TIMEOUT);
	#endif
	return res;
}


/*
0x00 0x5AF8 
0x01 0x9003 
0x02 0xA048 
0x03 0x2770 
0x04 0x70AB 
0x05 0x60C2 
0x06 0x2170 
0x07 0x2E60 
0x08 0x5F31 
0x09 0x523C 
0x0A 0x9200 
0x0B 0x8B1D 
0x0C 0xD42B 
0x0D 0x14AA 
0x0E 0x98C5 
0x0F 0xA525 
0x10 0x000C 
0x11 0x4C00 
0x12 0xFC00 
0x13 0x0C81 
0x14 0xDE01 
0x15 0x0000 
0x16 0x0000 
0x17 0x0000 
0x18 0x0000 
0x19 0x9102 
0x1A 0x01CB 
0x1B 0xC5DF 
0x1C 0xCB00 
0x1D 0xA03F 
0x1E 0xC280 
0x1F @ 
0x20 0xF7F0 
0x21 0x028D 
0x22 0x0836 
0x23 0x0431 
0x24 0x93F1 
0x25 0x2BDD 
0x26 0xC06D 
0x27 0xC025 
0x28 0x48B0 
0x29 0x1000 
0x2A 0x3480 
0x2B 0x2078 
0x2C 0x80B0 
0x2D 0x6490 
0x2E 0x76AF 
0x2F @ 
0x30 0x4934 
0x31 0xB300 
0x32 0x1038 
0x33 0xC360 
*/

void write_phy_reg(struct usb_device*dev,int phyaddr, unsigned char lb, unsigned hb)
{
	unsigned char buf[4];
	buf[0] = lb;
	buf[1] = hb;
	buf[2] = phyaddr; //adddress[5:0]
	buf[3] = 0x40; //write
	_write_reg(dev,0x00, 0xD480,4,buf);  //USB/PLAiHNVendor Wr F800~FA00
}

void read_phy_reg(struct usb_device*dev,int phyaddr)
{
	unsigned char buf[4];
	unsigned char buf1[4];
	
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = phyaddr; //adddress[5:0]
	buf[3] = 0x80;  //read
	_write_reg(dev,0x00, 0xD480,4,buf);  //USB/PLAiHNVendor Wr F800~FA00
	wait_ms(1);
	_read_reg(dev, 0x00, 0xD480, 4,  buf1); // Ū MCU USB, Register CF00  4 bytee 
//--	printk("%2x %2x%2x \n",phyaddr,buf1[1], buf1[0]);
	
}

int usb_etn_dongle_phy(struct usb_device*dev)
{
 	//unsigned char buf[4];
	//unsigned char buf1[4];
	//int i = 0;

//	for(i = 0 ; i <= 0x33 ; ++i)
//	{
//		read_phy_reg(dev,i);

//	}
	


	write_phy_reg(dev,0x00, 0xF8,0x5A);  //0x00 0x5AF8 
	write_phy_reg(dev,0x01, 0x03,0x90);  //0x01 0x9003 
	write_phy_reg(dev,0x02, 0x48,0xa0);  //0x02 0xA048 
	write_phy_reg(dev,0x03, 0x70,0x27);  //0x03 0x2770 
	write_phy_reg(dev,0x04, 0xAB,0x70);  //0x04 0x70AB 
	write_phy_reg(dev,0x05, 0xC2,0x60);  //0x05 0x60C2 
	write_phy_reg(dev,0x06, 0x70,0x21);  //0x06 0x2170 
	write_phy_reg(dev,0x07, 0x60,0x2e);  //0x07 0x2E60 
	write_phy_reg(dev,0x08, 0x31,0x5f);  //0x08 0x5F31 
	write_phy_reg(dev,0x09, 0x3c,0x52);  //0x09 0x523C 
	write_phy_reg(dev,0x0a, 0x00,0x92);  //0x0A 0x9200 
	write_phy_reg(dev,0x0b, 0x1d,0x8b);  //0x0B 0x8B1D 
	write_phy_reg(dev,0x0c, 0x2b,0xd4);  //0x0C 0xD42B 
	write_phy_reg(dev,0x0d, 0xaa,0x14);  //0x0D 0x14AA 
	write_phy_reg(dev,0x0e, 0xc5,0x98);  //0x0E 0x98C5 
	write_phy_reg(dev,0x0f, 0x25,0xa5);  //0x0F 0xA525 
	write_phy_reg(dev,0x10, 0x0c,0x00);  //0x10 0x000C 
	write_phy_reg(dev,0x11, 0x00,0x4c);  //0x11 0x4C00 
	write_phy_reg(dev,0x12, 0x00,0xfc);  //0x12 0xFC00 
	write_phy_reg(dev,0x13, 0x81,0x0c);  //0x13 0x0C81 
	write_phy_reg(dev,0x14, 0x01,0xde);  //0x14 0xDE01 
	write_phy_reg(dev,0x15, 0x00,0x00);  //0x15 0x0000 
	write_phy_reg(dev,0x16, 0x00,0x00);  //0x16 0x0000 
	write_phy_reg(dev,0x17, 0x00,0x00);  //0x17 0x0000 
	write_phy_reg(dev,0x18, 0x00,0x00);  //0x18 0x0000 
	write_phy_reg(dev,0x19, 0x02,0x91);  //0x19 0x9102 
	write_phy_reg(dev,0x1a, 0xcb,0x01);  //0x1A 0x01CB 
	write_phy_reg(dev,0x1b, 0xdf,0xc5);  //0x1B 0xC5DF 
	write_phy_reg(dev,0x1c, 0x00,0xcb);  //0x1C 0xCB00 
	write_phy_reg(dev,0x1d, 0x3f,0xa0);  //0x1D 0xA03F 
	write_phy_reg(dev,0x1e, 0x80,0xc2);  //0x1E 0xC280 
	//write_phy_reg(dev,0x1f, 0x00,0x00);  //0x1F @     
	write_phy_reg(dev,0x20, 0xf0,0xf7);  //0x20 0xF7F0 
	write_phy_reg(dev,0x21, 0x8d,0x02);  //0x21 0x028D 
	write_phy_reg(dev,0x22, 0x36,0x08);  //0x22 0x0836 
	write_phy_reg(dev,0x23, 0x31,0x04);  //0x23 0x0431 
	write_phy_reg(dev,0x24, 0xf1,0x93);  //0x24 0x93F1 
	write_phy_reg(dev,0x25, 0xdd,0x2b);  //0x25 0x2BDD 
	write_phy_reg(dev,0x26, 0x6d,0xc0);  //0x26 0xC06D 
	write_phy_reg(dev,0x27, 0x25,0xc0);  //0x27 0xC025 
	write_phy_reg(dev,0x28, 0xb0,0x48);  //0x28 0x48B0 
	write_phy_reg(dev,0x29, 0x00,0x10);  //0x29 0x1000 
	write_phy_reg(dev,0x2a, 0x80,0x34);  //0x2A 0x3480 
	write_phy_reg(dev,0x2b, 0x78,0x20);  //0x2B 0x2078 
	write_phy_reg(dev,0x2c, 0xb0,0x80);  //0x2C 0x80B0 
	write_phy_reg(dev,0x2d, 0x90,0x64);  //0x2D 0x6490 
	write_phy_reg(dev,0x2e, 0xaf,0x76);  //0x2E 0x76AF 
	//write_phy_reg(dev,0x2f, 0x00,0x00);  //0x2F @     
	write_phy_reg(dev,0x30, 0x34,0x49);  //0x30 0x4934 
	write_phy_reg(dev,0x31, 0x00,0xb3);  //0x31 0xB300 
	write_phy_reg(dev,0x32, 0x38,0x10);  //0x32 0x1038 
	write_phy_reg(dev,0x33, 0x60,0xc3);  //0x33 0xC360
	return 0;


}




int usb_etn_dongle_control_test(struct usb_device*dev)
{
 	unsigned char buf[4];
	unsigned char buf1[4];
	int result = 0;
//	printk("usb_etn_dongle_control_test....\n");
	buf[0] = 0x00;
	buf[1] = 0xff;
	buf[2] = 0x5c;
	buf[3] = 0xc5;
	result = _write_reg(dev,0x00, 0xF800,4,buf);  //USB/PLAiHNVendor Wr F800~FA00
	if (result < 0)
		return -1;
//	printk("usb_etn_dongle_control_test write reg....\n");
	buf1[0] = 0x00;
	buf1[1] = 0x00;
	buf1[2] = 0x00;
	buf1[3] = 0x00;
	result = _read_reg(dev, 0x00, 0xF800, 4,  buf1); // Ū MCU USB, Register CF00  4 bytee 
	if (result < 0)
		return -1;
//	printk("usb_etn_dongle_control_test read reg....\n");
	if (memcmp(buf,buf1,4) != 0)		
		return -1;
	return 0;


}

#define SET_ETHERNET_PACKET_FILTER 0x43
unsigned char data_buffer0[512] __attribute__ ((aligned (4096)));
unsigned char data_buffer1[512] __attribute__ ((aligned (4096)));

int usb_etn_dongle_bulk_in_out(struct usb_device *dev)
{


 	unsigned char buf[4];
 	//unsigned char buf1[4];

	static int data_len __attribute__ ((aligned (512)));

	unsigned char	ep_in=0, ep_out=0;
	struct usb_interface_descriptor *if_face = NULL;
	unsigned char tmpbuf[USB_BUFSIZ];
	int partial;
	__attribute__ ((unused)) int result;
	int i = 0;
	int timeout = 5;
	       // 1. Control transfer, SetConfig 2

	//usb_get_configuration_no(dev,&tmpbuf[0],2);

		//usb_parse_config(dev,&tmpbuf[0],2);

	printk("usb_get_configuration_no start...\n");
	usb_get_configuration_no_qc(dev,&tmpbuf[0],1);
//	printk("usb_get_configuration_no end...\n");
	

	printk("usb_parse_config start...\n");
	usb_parse_config(dev,&tmpbuf[0],1);
//	printk("usb_parse_config end...\n");

	printk("usb_set_maxpacket start...\n");
	usb_set_maxpacket(dev);
//	printk("usb_set_maxpacket end...\n");
		//parse config
	//9 2 62 0 2 2 0 a0 20  descriptor (bCinfigurationValue = 2 bNumInterface=2)
	//9 4 0 0 1 2 6 0 5       interface0
	//5 24 0 10 1
	//5 24 6 0 1
	//d 24 f 3 0 0 0 0 e a 5 0 0
	//0
	//7 5 83 3 10 0 8  // ep0
	//6 30 0 0 8 0
	//9 4 1 0 0 a 0 0 0 //interface alternate0
	//9 4 1 1 2 a 0 0 4  //interface1  alternate1
	//7 5 81 2 0 4 0   //ep0 
	//6 30 3 0 0 0      //ss_ep0_comp
	//7 5 2 2 0 4 0   //ep1
	//6 30 3 0 0 0    // ss_ep1_comp

	printk("usb_set_configuration start...\n");
	if (usb_set_configuration(dev, dev->config.bConfigurationValue)) {
		printk("failed to set default configuration len %d, status 0x%lx\n",dev->act_len,dev->status);
		return -1;
	}
//		printk("usb_set_configuration end...\n");
	//set configuration link will re-connect , so we must relink


//relink to 100M
//write MCU PLA , Register 0xE86C = 0x07ffa000
//write MCU PLA , Register 0xB400 = 0x00002100    // force 100M
//read MCU PLA , Register 0xE908 bit[2] = linkok
// mdi looback YU force 100Mj 80ms iHŪ linkok = 1

	
	buf[0] = 0x00;
	buf[1] = 0xa0;
	buf[2] = 0xff;
	buf[3] = 0x07;
	_write_reg(dev,0x01, 0xE86C,4,buf);
//	printk("_write_reg 1 end...\n");
	buf[0] = 0x00;
	buf[1] = 0x21;
	buf[2] = 0x00;
	buf[3] = 0x00;
	_write_reg(dev,0x01, 0xB400,4,buf); 
//	printk("_write_reg 2 end...\n");
	while(timeout > 0)
	{
		_read_reg(dev, 0x01, 0xE908, 4,  buf);
		wait_ms(50);
		if ((buf[0] & 0x03) == 0x3)
			break;
		timeout--;
		if (timeout == 0)
		{
			printk("link timeout\n");
			return -1;
		}
	}

//	printk("usb_set_interface start...\n");
	// 2. Control transfer, Setinterface 1 alternative setting 1
	if (usb_set_interface_qc(dev, 0x1, 0x1)) {
			printk("Problems with device\n");
			return -1;
	}
//	printk("usb_set_interface end...\n");

	//iface = &dev->config.if_desc[ifnum];
 
	// 3. Control transfer, SetEthernetPacketFilter 
	//BmReq bReq wValueL wValueH wIndexL wIndexH wLengthL wLengthH 
	//   0x21 0x43 0x1F 0 0 0 0 0 
 
	usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0), SET_ETHERNET_PACKET_FILTER, USB_DIR_OUT|USB_TYPE_CLASS|USB_RECIP_INTERFACE, 0x1, 0x0 , NULL, 0, USB_CNTL_TIMEOUT * 3); 


	if_face = &dev->config.if_desc[dev->cur_interface];

	for (i = 0; i < if_face->bNumEndpoints; i++) {
		if ((if_face->ep_desc[i].ep_desc.bmAttributes &  USB_ENDPOINT_XFERTYPE_MASK) == USB_ENDPOINT_XFER_BULK) {
			if (if_face->ep_desc[i].ep_desc.bEndpointAddress & USB_DIR_IN)
				ep_in = if_face->ep_desc[i].ep_desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK;
			else
				ep_out = if_face->ep_desc[i].ep_desc.bEndpointAddress & USB_ENDPOINT_NUMBER_MASK;
		}
	}
 
	//printk("endpoint %x %x\n",ep_in, ep_out);

	data_len = 512;

	memset(data_buffer0,0x0,data_len);
	memset(data_buffer1,0x0,data_len);

	for(i = 0 ; i < data_len; ++i)
		data_buffer0[i] = i;


	// 4. Bulk out
	result = usb_bulk_msg_qc(dev, usb_sndbulkpipe(dev, ep_out), data_buffer0, data_len, &partial, USB_CNTL_TIMEOUT*5);


	//5. Bulk in
	result = usb_bulk_msg_qc(dev, usb_rcvbulkpipe(dev, ep_in), data_buffer1, data_len, &data_len, USB_CNTL_TIMEOUT * 5);



	if (memcmp(data_buffer1,data_buffer0,data_len) != 0)		
		return -1;
	
	return 0;


#if 0

for (i = 0; i < iface->bNumEndpoints; i++) {
		/* is it an BULK endpoint? */
		if ((iface->ep_desc[i].ep_desc.bmAttributes &  USB_ENDPOINT_XFERTYPE_MASK)
		    == USB_ENDPOINT_XFER_BULK) {
			if (iface->ep_desc[i].ep_desc.bEndpointAddress & USB_DIR_IN)
				ss->ep_in = iface->ep_desc[i].ep_desc.bEndpointAddress &
					USB_ENDPOINT_NUMBER_MASK;
			else
				ss->ep_out = iface->ep_desc[i].ep_desc.bEndpointAddress &
					USB_ENDPOINT_NUMBER_MASK;
		}

		/* is it an interrupt endpoint? */
		if ((iface->ep_desc[i].ep_desc.bmAttributes & USB_ENDPOINT_XFERTYPE_MASK)
		    == USB_ENDPOINT_XFER_INT) {
			ss->ep_int = iface->ep_desc[i].ep_desc.bEndpointAddress &
				USB_ENDPOINT_NUMBER_MASK;
			ss->irqinterval = iface->ep_desc[i].ep_desc.bInterval;
		}
	}


#if defined(CONFIG_USB_PATCH)
	static int partial __attribute__ ((aligned (512)));
#else
	int partial;
#endif
	int maxtry;
	int stat;

	/* determine the maximum packet size for these transfers */
	max_size = usb_maxpacket_qc(us->pusb_dev, pipe) * 16;

	/* while we have data left to transfer */
	while (length) {

		/* calculate how long this will be -- maximum or a remainder */
		this_xfer = length > max_size ? max_size : length;
		length -= this_xfer;

		/* setup the retry counter */
		maxtry = 10;

		/* set up the transfer loop */
		do {
			/* transfer the data */
			USB_STOR_PRINTF("Bulk xfer 0x%x(%d) try #%d\n",
				  (unsigned int)buf, this_xfer, 11 - maxtry);
			result = usb_bulk_msg(us->pusb_dev, pipe, buf,
					      this_xfer, &partial, USB_CNTL_TIMEOUT*5);
			USB_STOR_PRINTF("bulk_msg returned %d xferred %d/%d\n",
				  result, partial, this_xfer);
			if(us->pusb_dev->status!=0) {
				/* if we stall, we need to clear it before we go on */

#endif




}
/*
 * By the time we get here, the device has gotten a new device ID
 * and is in the default state. We need to identify the thing and
 * get the ball rolling..
 *
 * Returns 0 for success, != 0 for error.
 */
int usb_new_device_qc(struct usb_device *dev)
{
	
	int addr, err;
	int tmp;
	
#if defined(CONFIG_USB_PATCH)
	static unsigned char tmpbuf[USB_BUFSIZ] __attribute__ ((aligned (512)));
#else
	unsigned char tmpbuf[USB_BUFSIZ];
#endif	


	dev->descriptor.bMaxPacketSize0 = 8;  /* Start off at 8 bytes  */
	dev->maxpacketsize = 0;		/* Default to 8 byte max packet size */
	dev->epmaxpacketin [0] = 8;
	dev->epmaxpacketout[0] = 8;

	/* We still haven't set the Address yet */
	addr = dev->devnum;
	printk("dev num=%x\n",dev->devnum);
	dev->devnum = 0;


if (USB_Host_Type == USB_XHCI_HOST) {
		usb_alloc_dev(dev);
		usb_set_slot_id(addr - 1);
} else {

	err = usb_get_descriptor(dev, USB_DT_DEVICE, 0, &dev->descriptor, 8);

	
	if (err < 8) {
		printk("\n      USB device not responding, giving up (status=0x%lx)\n",dev->status);
		return 1;
	}


	dev->epmaxpacketin [0] = dev->descriptor.bMaxPacketSize0;
	dev->epmaxpacketout[0] = dev->descriptor.bMaxPacketSize0;	
	
	switch (dev->descriptor.bMaxPacketSize0) {
		case 8: dev->maxpacketsize = 0; break;
		case 16: dev->maxpacketsize = 1; break;
		case 32: dev->maxpacketsize = 2; break;
		case 64: dev->maxpacketsize = 3; break;
	}
}

	dev->devnum = addr;

	err = usb_set_address(dev); /* set address */
	if (err < 0) {
		printk("\n      USB device not accepting new address (error=0x%lx)\n", dev->status);
		return 1;
	}

	wait_ms(10);	/* Let the SET_ADDRESS settle */

	tmp = sizeof(dev->descriptor);


	err = usb_get_descriptor(dev, USB_DT_DEVICE, 0, &dev->descriptor, sizeof(struct usb_device_descriptor));
	if (err < tmp) {
		if (err < 0)
			printk("unable to get device descriptor (error=%d)\n",err);
		else
			printk("USB device descriptor short read (expected %d, got %d)\n",tmp,err);
		return 1;
	}
	/* correct le values */
	dev->descriptor.bcdUSB=swap_16(dev->descriptor.bcdUSB);
	dev->descriptor.idVendor=swap_16(dev->descriptor.idVendor);
	dev->descriptor.idProduct=swap_16(dev->descriptor.idProduct);
	dev->descriptor.bcdDevice=swap_16(dev->descriptor.bcdDevice);
	
	/* only support for one config for now */
	usb_get_configuration_no_qc(dev,&tmpbuf[0],0);

	usb_parse_config(dev,&tmpbuf[0],0);
	usb_set_maxpacket(dev);

	/* we set the default configuration here */
	if (usb_set_configuration(dev, dev->config.bConfigurationValue)) {
		printk("failed to set default configuration len %d, status 0x%lx\n",dev->act_len,dev->status);
		return -1;
	}
	
	USB_PRINTF("new device strings: idVendor=0x%x, idProduct=0x%x, bcdDevice=0x%x\n",
		dev->descriptor.idVendor, dev->descriptor.idProduct, dev->descriptor.bcdDevice);
		
	USB_PRINTF("new device strings: Mfr=%d, Product=%d, SerialNumber=%d\n",
		dev->descriptor.iManufacturer, dev->descriptor.iProduct, dev->descriptor.iSerialNumber);
	memset(dev->mf, 0, sizeof(dev->mf));
	memset(dev->prod, 0, sizeof(dev->prod));
	memset(dev->serial, 0, sizeof(dev->serial));
	if (dev->descriptor.iManufacturer)
		usb_string_qc(dev, dev->descriptor.iManufacturer, dev->mf, sizeof(dev->mf));
	if (dev->descriptor.iProduct)
		usb_string_qc(dev, dev->descriptor.iProduct, dev->prod, sizeof(dev->prod));
	if (dev->descriptor.iSerialNumber)
		usb_string_qc(dev, dev->descriptor.iSerialNumber, dev->serial, sizeof(dev->serial));
	USB_PRINTF("Manufacturer %s\n", dev->mf);
	USB_PRINTF("Product      %s\n", dev->prod);
	USB_PRINTF("SerialNumber %s\n", dev->serial);
	usb_hub_probe_qc(dev,0);

	return 0;
}
extern unsigned int xhci_port_speed(struct usb_device *usbdev, unsigned int port_status);
extern void xhci_poll_and_HandleEvent(struct usb_device *usbdev);

#if 0

int usb_ft_descriptor_test(struct usb_device *dev)
{
	int addr, err;
	int tmp;
	int port_status = 0;
	int port_id = 3;
	int i = 0;

#if 0
        usb=usb_alloc_new_device_qc();
        usb->slow = (portstatus & USB_PORT_STAT_LOW_SPEED) ? 1 : 0;

        dev->children[port] = usb;
        usb->parent=dev;
        usb->port_id = port + 1;
        // while(1);

        /* Run it through the hoops (find a driver, etc) */
        if (usb_new_device_qc(usb)) {
                /* Woops, disable the port */
                USB_HUB_PRINTF("hub: disabling port %d\n", port + 1);
                usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_ENABLE);
        }


	dev = usb_alloc_new_device_qc();
#endif	
	printk("descriptor test start\n");
	
	if (port_id == 0)
	{
		rtd_outl(0xb8050420,0x206f1);
		wait_ms(50);
		xhci_poll_and_HandleEvent(dev); //trb will not complete if no call this function
		port_status = rtd_inl(0xb8050420);
	}
	else if (port_id == 1)
	{
		rtd_outl(0xb8050430,0x206f1);
		wait_ms(50);
		xhci_poll_and_HandleEvent(dev); //trb will not complete if no call this function
		port_status = rtd_inl(0xb8050430);
	}
	else if (port_id == 2)
	{
		rtd_outl(0xb8050440,0x206f1);
		wait_ms(50);
		xhci_poll_and_HandleEvent(dev); //trb will not complete if no call this function
		port_status = rtd_inl(0xb8050440);
	}
	else if (port_id == 3) //usb3
	{
		port_status = rtd_inl(0xb8050450);
	}

//	printk("ft port_status=%x\n",port_status);
	xhci_port_speed(dev,port_status);
	//memset(dev,0,sizeof(struct usb_device));
	dev->slow = 0;  // 0 high speed , 1 low speed
	dev->port_id = port_id + 1;  //need to set correct
        dev->descriptor.bMaxPacketSize0 = 8;  /* Start off at 8 bytes  */
        dev->maxpacketsize = 0;         /* Default to 8 byte max packet size */
        dev->epmaxpacketin [0] = 8;
        dev->epmaxpacketout[0] = 8;

	dev->devnum = 3;
	addr = dev->devnum;
	dev->devnum = 0;
	usb_alloc_dev(dev);
	usb_set_slot_id(addr - 1);
	dev->devnum = addr;

	err = usb_set_address(dev); /* set address */

        if (err < 0) {
                printk("\n      USB device not accepting new address (error=0x%lx)\n", dev->status);
                return 1;
        }

        wait_ms(10);    /* Let the SET_ADDRESS settle */

        tmp = sizeof(dev->descriptor);

	for ( i = 0 ; i < 10 ; ++i)
	{
		memset(&dev->descriptor,0,sizeof(struct usb_device_descriptor));	
        	err = usb_get_descriptor(dev, USB_DT_DEVICE, 0, &dev->descriptor, sizeof(struct usb_device_descriptor));

		if (dev->descriptor.bLength == 0)
		{
			printk("descriptor length is not correct %d\n",dev->descriptor.bLength);
			return 1;
		}
        	if (err < tmp) {
                	if (err < 0)
                        	printk("unable to get device descriptor (error=%d)\n",err);
                	else
                        	printk("USB device descriptor short read (expected %d, got %d)\n",tmp,err);
                	
			return 1;
		}
        }
	return 0;
}
#endif

/* build device Tree  */
int usb_scan_devices(void)
{
	int i;
	struct usb_device *dev;


#ifdef ENABLE_KSEG1ADDR
	pusb_dev = (struct usb_device *) KSEG1ADDR(&usb_dev[0]);
#else /* ENABLE_KSEG1ADDR */
	pusb_dev = (struct usb_device *) usb_dev;
#endif /* ENABLE_KSEG1ADDR */

	/* first make all devices unknown */
	for(i=0;i<USB_MAX_DEVICE;i++) {
		memset(&pusb_dev[i],0,sizeof(struct usb_device));
		pusb_dev[i].devnum=-1;
	}
	dev_index=0;
	/* device 0 is always present (root hub, so let it analyze) */
	dev=usb_alloc_new_device_qc();
	usb_new_device_qc(dev);
	printk("%d USB Devices found\n",dev_index);
	return dev_index;
	/* insert "driver" if possible */
}


/****************************************************************************
 * HUB "Driver"
 * Probes device for being a hub and configurate it
 */

#undef	USB_HUB_DEBUG
//#define	USB_HUB_DEBUG 1

#ifdef	USB_HUB_DEBUG
#define	USB_HUB_PRINTF(fmt,args...)	printk (fmt ,##args)
#else
#define USB_HUB_PRINTF(fmt,args...)
#endif


static struct usb_hub_device hub_dev[USB_MAX_HUB];
static int usb_hub_index;


int usb_get_hub_descriptor(struct usb_device *dev, void *data, int size)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
		USB_REQ_GET_DESCRIPTOR, USB_DIR_IN | USB_RT_HUB,
		USB_DT_HUB << 8, 0, data, size, USB_CNTL_TIMEOUT);
}

int usb_clear_hub_feature(struct usb_device *dev, int feature)
{
	return usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_CLEAR_FEATURE, USB_RT_HUB, feature, 0, NULL, 0, USB_CNTL_TIMEOUT);
}

int usb_clear_port_feature(struct usb_device *dev, int port, int feature)
{
	return usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_CLEAR_FEATURE, USB_RT_PORT, feature, port, NULL, 0, USB_CNTL_TIMEOUT);
}

int usb_set_port_feature(struct usb_device *dev, int port, int feature)
{
	return usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0),
		USB_REQ_SET_FEATURE, USB_RT_PORT, feature, port, NULL, 0, USB_CNTL_TIMEOUT);
}

int usb_get_hub_status(struct usb_device *dev, void *data)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
			USB_REQ_GET_STATUS, USB_DIR_IN | USB_RT_HUB, 0, 0,
			data, sizeof(struct usb_hub_status), USB_CNTL_TIMEOUT);
}

int usb_get_port_status(struct usb_device *dev, int port, void *data)
{
	return usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0),
			USB_REQ_GET_STATUS, USB_DIR_IN | USB_RT_PORT, 0, port,
			data, sizeof(struct usb_hub_status), USB_CNTL_TIMEOUT);
}


static void usb_hub_power_on(struct usb_hub_device *hub)
{
	int i;
	struct usb_device *dev;

	dev=hub->pusb_dev;
	/* Enable power to the ports */
	USB_HUB_PRINTF("enabling power on all ports\n");
	
	for (i = 0; i < dev->maxchild; i++) {
		usb_set_port_feature(dev, i + 1, USB_PORT_FEAT_POWER);
		USB_HUB_PRINTF("port %d returns 0x%lx\n",i+1,dev->status);

		#ifdef usb_speed_up
		wait_ms(5);
		#else
		wait_ms(hub->desc.bPwrOn2PwrGood * 2);  //hub->desc.bPwrOn2PwrGood  = 10
		#endif
	}
	
}

void usb_hub_reset_qc(void)
{
	usb_hub_index=0;
}

struct usb_hub_device *usb_hub_allocate(void)
{
	if(usb_hub_index<USB_MAX_HUB) {
		return &hub_dev[usb_hub_index++];
	}
	printk("ERROR: USB_MAX_HUB (%d) reached\n",USB_MAX_HUB);
	return NULL;
}

#define MAX_TRIES 5


void usb_hub_port_connect_change_qc_xhci(struct usb_device *dev, int port)
{
	struct usb_device *usb;
	struct usb_port_status portsts;
	unsigned short portstatus, portchange;
	int tries;
	int do_port_reset = 0;

	/* Check status */
	if (usb_get_port_status(dev, port + 1, &portsts)<0) {
		USB_HUB_PRINTF("get_port_status failed\n");
		return;
	}	
	portstatus = swap_16(portsts.wPortStatus);
	portchange = swap_16(portsts.wPortChange);


	#if 0
	// cyhuang 
	if (portstatus&(1<<USB_PORT_FEAT_HIGHSPEED)) {
		dev->speed = USB_SPEED_HIGH;		
		USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
			 "High Speed");
	}
	else 
		USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
			portstatus&(1<<USB_PORT_FEAT_LOWSPEED) ? "Low Speed" : "Full Speed");
	#endif

		
	if(portstatus & (1 << USB_PORT_FEAT_SUPERSPEED))
	{
		printk("usb super speed....\n");
		dev->speed = USB_SPEED_SUPER;
		
	}
	else
		do_port_reset = 1;
		
	/* Clear the connection change status */
	usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_C_CONNECTION);


#if 0
	/* Disconnect any existing devices under this port */
	if (((!(portstatus & USB_PORT_STAT_CONNECTION)) &&
	     (!(portstatus & USB_PORT_STAT_ENABLE)))|| (dev->children[port])) {
		USB_HUB_PRINTF("usb_disconnect(&hub->children[port]);\n");
		/* Return now if nothing is connected */
		if (!(portstatus & USB_PORT_STAT_CONNECTION))
			return;
	}
#endif

	#ifdef usb_speed_up
	wait_ms(5);	
	#else
	wait_ms(200);
	#endif

	if (do_port_reset) {	

	/* Reset the port */
		for(tries=0;tries<MAX_TRIES;tries++) {

			usb_set_port_feature(dev, port + 1, USB_PORT_FEAT_RESET);
			#if 0 //def usb_speed_up (some usb device can reduce delay time )
			wait_ms(50);	
			#else
			wait_ms(200);
			#endif
			if (usb_get_port_status(dev, port + 1, &portsts)<0) {
				USB_HUB_PRINTF("get_port_status failed status 0x%lx\n",dev->status);
				return;
			}
			portstatus = swap_16(portsts.wPortStatus);
			portchange = swap_16(portsts.wPortChange);

			// cyhuang 
			if (portstatus&(1<<USB_PORT_FEAT_HIGHSPEED)) {
				dev->speed = USB_SPEED_HIGH;
				USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
				 	"High Speed");
			}
			else		
				USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus ,portchange,
					portstatus&(1<<USB_PORT_FEAT_LOWSPEED) ? "Low Speed" : "Full Speed");
					
			USB_HUB_PRINTF("STAT_C_CONNECTION = %d STAT_CONNECTION = %d  USB_PORT_STAT_ENABLE %d\n",
				(portchange & USB_PORT_STAT_C_CONNECTION) ? 1 : 0,
				(portstatus & USB_PORT_STAT_CONNECTION) ? 1 : 0,
				(portstatus & USB_PORT_STAT_ENABLE) ? 1 : 0);
			if ((portchange & USB_PORT_STAT_C_CONNECTION) ||
			    !(portstatus & USB_PORT_STAT_CONNECTION))
				return;

			if (portstatus & USB_PORT_STAT_ENABLE)
			{
				#ifdef usb_speed_up
				wait_ms(5);	
				#else
				wait_ms(200);
				#endif
				break;

			}
			#ifdef usb_speed_up
			wait_ms(5);	
			#else
			wait_ms(200);
			#endif
		}



		if (tries==MAX_TRIES) {
			USB_HUB_PRINTF("Cannot enable port %i after %i retries, disabling port.\n", port+1, MAX_TRIES);
			USB_HUB_PRINTF("Maybe the USB cable is bad?\n");
			return;
		}

		usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_C_RESET);
		#ifdef usb_speed_up
		wait_ms(20);	
		#else
		wait_ms(200);
		#endif
	}

	/* Allocate a new device struct for it */
	usb=usb_alloc_new_device_qc();
	usb->slow = (portstatus & USB_PORT_STAT_LOW_SPEED) ? 1 : 0;

	dev->children[port] = usb;
	usb->parent=dev;	
	usb->port_id = port + 1;
	// while(1);
	
	/* Run it through the hoops (find a driver, etc) */
	if (usb_new_device_qc(usb)) {
		/* Woops, disable the port */
		USB_HUB_PRINTF("hub: disabling port %d\n", port + 1);
		usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_ENABLE);
	}
}




void usb_hub_port_connect_change_qc_ehci(struct usb_device *dev, int port)
{
	struct usb_device *usb;
	struct usb_port_status portsts;
	unsigned short portstatus, portchange;
	int tries;

	/* Check status */
	if (usb_get_port_status(dev, port + 1, &portsts)<0) {
		USB_HUB_PRINTF("get_port_status failed\n");
		return;
	}	
	portstatus = swap_16(portsts.wPortStatus);
	portchange = swap_16(portsts.wPortChange);
	
	// cyhuang 
	if (portstatus&(1<<USB_PORT_FEAT_HIGHSPEED)) {
		dev->speed = USB_SPEED_HIGH;		
		USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
			 "High Speed");
	}
	else 
		USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
			portstatus&(1<<USB_PORT_FEAT_LOWSPEED) ? "Low Speed" : "Full Speed");

	

	/* Clear the connection change status */
	usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_C_CONNECTION);

	/* Disconnect any existing devices under this port */
	if (((!(portstatus & USB_PORT_STAT_CONNECTION)) &&
	     (!(portstatus & USB_PORT_STAT_ENABLE)))|| (dev->children[port])) {
		USB_HUB_PRINTF("usb_disconnect(&hub->children[port]);\n");
		/* Return now if nothing is connected */
		if (!(portstatus & USB_PORT_STAT_CONNECTION))
			return;
	}

	#ifdef usb_speed_up
	wait_ms(5);	
	#else
	wait_ms(200);
	#endif

	/* Reset the port */

	for(tries=0;tries<MAX_TRIES;tries++) {

		usb_set_port_feature(dev, port + 1, USB_PORT_FEAT_RESET);
		#if 0 //def usb_speed_up (some usb device can reduce delay time )
		wait_ms(50);	
		#else
		wait_ms(200);
		#endif
		if (usb_get_port_status(dev, port + 1, &portsts)<0) {
			USB_HUB_PRINTF("get_port_status failed status 0x%lx\n",dev->status);
			return;
		}
		portstatus = swap_16(portsts.wPortStatus);
		portchange = swap_16(portsts.wPortChange);
		
		// cyhuang 
		if (portstatus&(1<<USB_PORT_FEAT_HIGHSPEED)) {
			dev->speed = USB_SPEED_HIGH;
			USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus, portchange,
			 	"High Speed");
		}
		else		
			USB_HUB_PRINTF("portstatus %x, change %x, %s\n", portstatus ,portchange,
				portstatus&(1<<USB_PORT_FEAT_LOWSPEED) ? "Low Speed" : "Full Speed");
				
		USB_HUB_PRINTF("STAT_C_CONNECTION = %d STAT_CONNECTION = %d  USB_PORT_STAT_ENABLE %d\n",
			(portchange & USB_PORT_STAT_C_CONNECTION) ? 1 : 0,
			(portstatus & USB_PORT_STAT_CONNECTION) ? 1 : 0,
			(portstatus & USB_PORT_STAT_ENABLE) ? 1 : 0);
		if ((portchange & USB_PORT_STAT_C_CONNECTION) ||
		    !(portstatus & USB_PORT_STAT_CONNECTION))
			return;

		if (portstatus & USB_PORT_STAT_ENABLE)
		{
			#ifdef usb_speed_up
			wait_ms(5);	
			#else
			wait_ms(200);
			#endif
			break;

		}

		#ifdef usb_speed_up
		wait_ms(5);	
		#else
		wait_ms(200);
		#endif
	}

	if (tries==MAX_TRIES) {
		USB_HUB_PRINTF("Cannot enable port %i after %i retries, disabling port.\n", port+1, MAX_TRIES);
		USB_HUB_PRINTF("Maybe the USB cable is bad?\n");
		return;
	}

	usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_C_RESET);
	#ifdef usb_speed_up
	wait_ms(50);	
	#else
	wait_ms(200);
	#endif

	/* Allocate a new device struct for it */
	usb=usb_alloc_new_device_qc();
	usb->slow = (portstatus & USB_PORT_STAT_LOW_SPEED) ? 1 : 0;

	dev->children[port] = usb;
	usb->parent=dev;	
	
	// while(1);
	
	/* Run it through the hoops (find a driver, etc) */
	if (usb_new_device_qc(usb)) {
		/* Woops, disable the port */
		USB_HUB_PRINTF("hub: disabling port %d\n", port + 1);
		usb_clear_port_feature(dev, port + 1, USB_PORT_FEAT_ENABLE);
	}
}


#define U2PORT_IN_U3 1  //0xb8050430
#define U3PORT_REG_ADDRESS 0xb8050440
#define XHCI_PORT_CNT (2)
#define EHCI_PORT_CNT (2)


int usb_hub_configure(struct usb_device *dev)
{
#if defined(CONFIG_USB_PATCH)		
	static unsigned char buffer[USB_BUFSIZ] __attribute__ ((aligned (512)));
#else
	unsigned char buffer[USB_BUFSIZ];
#endif	
	unsigned char *bitmap;
	struct usb_hub_descriptor *descriptor;
#ifdef USB_DEBUG
	struct usb_hub_status *hubsts;
#endif
	int i;
	struct usb_hub_device *hub;
#if CONFIG_EQC
	int init_port = 0;
#endif

	struct usb_port_status portsts;
	unsigned short portstatus, portchange;

	/* "allocate" Hub device */
	hub=usb_hub_allocate();
	if(hub==NULL)
		return -1;
	hub->pusb_dev=dev;
	/* Get the the hub descriptor */
	if (usb_get_hub_descriptor(dev, buffer, 4) < 0) {
		USB_HUB_PRINTF("usb_hub_configure: failed to get hub descriptor, giving up 0lx%lx\n",dev->status);
		return -1;
	}
	descriptor = (struct usb_hub_descriptor *)buffer;

	if (descriptor->bLength > USB_BUFSIZ) {
		USB_HUB_PRINTF("usb_hub_configure: failed to get hub descriptor - too long: %d\n",
			descriptor->bLength);
		return -1;
	}

	if (usb_get_hub_descriptor(dev, buffer, descriptor->bLength) < 0) {
		USB_HUB_PRINTF("usb_hub_configure: failed to get hub descriptor 2nd giving up 0x%lx\n",dev->status);
		return -1;
	}

	printk("hub removable %d %d\n",descriptor->DeviceRemovable[0],descriptor->DeviceRemovable[1]);

	memcpy((unsigned char *)&hub->desc,buffer,descriptor->bLength);
	/* adjust 16bit values */
	hub->desc.wHubCharacteristics=swap_16(descriptor->wHubCharacteristics);
	/* set the bitmap */
	bitmap=(unsigned char *)&hub->desc.DeviceRemovable[0];
	memset(bitmap,0xff,(USB_MAXCHILDREN+1+7)/8); /* devices not removable by default */
	bitmap=(unsigned char *)&hub->desc.PortPowerCtrlMask[0];
	memset(bitmap,0xff,(USB_MAXCHILDREN+1+7)/8); /* PowerMask = 1B */
	for(i=0;i<((hub->desc.bNbrPorts + 1 + 7)/8);i++) {
		hub->desc.DeviceRemovable[i]=descriptor->DeviceRemovable[i];
	}
	for(i=0;i<((hub->desc.bNbrPorts + 1 + 7)/8);i++) {
		hub->desc.DeviceRemovable[i]=descriptor->PortPowerCtrlMask[i];
	}
	dev->maxchild = descriptor->bNbrPorts;
	USB_HUB_PRINTF("%d ports detected\n", dev->maxchild);

	switch (hub->desc.wHubCharacteristics & HUB_CHAR_LPSM) {
		case 0x00:
			USB_HUB_PRINTF("ganged power switching\n");
			break;
		case 0x01:
			USB_HUB_PRINTF("individual port power switching\n");
			break;
		case 0x02:
		case 0x03:
			USB_HUB_PRINTF("unknown reserved power switching mode\n");
			break;
	}

	if (hub->desc.wHubCharacteristics & HUB_CHAR_COMPOUND)
		USB_HUB_PRINTF("part of a compound device\n");
	else
		USB_HUB_PRINTF("standalone hub\n");

	switch (hub->desc.wHubCharacteristics & HUB_CHAR_OCPM) {
		case 0x00:
			USB_HUB_PRINTF("global over-current protection\n");
			break;
		case 0x08:
			USB_HUB_PRINTF("individual port over-current protection\n");
			break;
		case 0x10:
		case 0x18:
			USB_HUB_PRINTF("no over-current protection\n");
      break;
	}
	USB_HUB_PRINTF("power on to power good time: %dms\n", descriptor->bPwrOn2PwrGood * 2);
	USB_HUB_PRINTF("hub controller current requirement: %dmA\n", descriptor->bHubContrCurrent);
	
	for (i = 0; i < dev->maxchild; i++)
		USB_HUB_PRINTF("port %d is%s removable\n", i + 1,
			hub->desc.DeviceRemovable[(i + 1)/8] & (1 << ((i + 1)%8)) ? " not" : "");
	if (sizeof(struct usb_hub_status) > USB_BUFSIZ) {
		USB_HUB_PRINTF("usb_hub_configure: failed to get Status - too long: %d\n",
			descriptor->bLength);
		return -1;
	}

	if (usb_get_hub_status(dev, buffer) < 0) {
		USB_HUB_PRINTF("usb_hub_configure: failed to get Status 0x%lx\n",dev->status);
		return -1;
	}

#ifdef USB_DEBUG
	hubsts = (struct usb_hub_status *)buffer;

	USB_HUB_PRINTF("get_hub_status returned status 0x%x, change 0x%x\n",
		swap_16(hubsts->wHubStatus),swap_16(hubsts->wHubChange));
	USB_HUB_PRINTF("local power source is %s\n",
		(swap_16(hubsts->wHubStatus) & HUB_STATUS_LOCAL_POWER) ? "lost (inactive)" : "good");
	USB_HUB_PRINTF("%sover-current condition exists\n",
		(swap_16(hubsts->wHubStatus) & HUB_STATUS_OVERCURRENT) ? "" : "no ");
#endif
	usb_hub_power_on(hub);
	/* fox add 080701 for stable power on */
	#ifdef usb_speed_up
	wait_ms(10);	
	#else
	wait_ms(200);
	#endif

	//root hub port reset
if (USB_Host_Type == USB_XHCI_HOST) {
	//0xb8050440
	usb_set_port_feature(dev, U2PORT_IN_U3 + 1 , USB_PORT_FEAT_RESET);
}


#if CONFIG_EQC  //reduece FT time
	wait_ms(500);
#else
	#ifdef NEW_FT
	wait_ms(20); //2018/10/11 modify
	#else
	wait_ms(100);
	#endif
#endif

	if (USB_Host_Type == USB_XHCI_HOST) {
		usb_clear_port_feature(dev, U2PORT_IN_U3 + 1 , USB_PORT_FEAT_C_RESET);
		//rtd_outl(0xb8050420, rtd_inl(0xb8050420) | _BIT4);
		printk("[0]usb2 status %x\n", rtd_inl(0xb8050430));
		printk("[0]usb3 status %x\n", rtd_inl(0xb8050440));

	    	if ((rtd_inl(0xb8050440) & 0xfff) == 0x2e0)
	    	{
	        	printk("polling state...\n");
	        	rtd_outl(0xb8050440, 0x80400340); // warm reset
				wait_ms(150);
				printk("[1]usb3 status %x\n", rtd_inl(0xb8050440));
	        	//goto verify_fail;
	    	}

		printf("$$$$$$$$$$$$$$$$$ start....\n");

	}

/* ==================================
Port information of MER6
xhci  dev->maxchild = 3
	index = 1 (i=0)  b8050420
	index = 2 (i=1)  b8050430  U2/U2U3
	index = 3 (i=2)  b8050440  U3/U2U3

ehci dev->maxchild = 2
	index = 1 (i=0)  b8012054
	index = 2 (i=1)  b8012058
================================== */

#if CONFIG_EQC 
	if (USB_Host_Type == USB_XHCI_HOST) {
		init_port = 2;
	} else if (USB_Host_Type == USB_EHCI_HOST) {
		init_port = 1;
	}

	for (i = init_port; i >= 0 ; i--) {  
		if (USB_Host_Type == USB_XHCI_HOST) {
			if (i == 1)
				continue;
		}
#else
	//FT flow check U3 only
	if(1){
		i = 2;
#endif

		if (usb_get_port_status(dev, i + 1, &portsts) < 0) {
			USB_HUB_PRINTF("get_port_status failed\n");
#if CONFIG_EQC
			continue;
#endif
		}

		portstatus = swap_16(portsts.wPortStatus);
		portchange = swap_16(portsts.wPortChange);
		USB_HUB_PRINTF("Port %d Status 0x%x Change 0x%x\n",i+1,portstatus,portchange);

		printk("$$$$$$$$$$$$$$$  [portid=%d] port stauts=%x %x\n", i + 1 , portstatus,portchange);

		printk("u2 0x%x 0x%x \n", rtd_inl(0xb8050420),rtd_inl(0xb8050430));
		printk("u3 0x%x\n",rtd_inl(0xb8050440));

if (USB_Host_Type == USB_XHCI_HOST) {
			printk("%s(%d)USB_XHCI_HOST\n",__func__,__LINE__);
		if (portchange & USB_PORT_STAT_C_CONNECTION) {
			USB_HUB_PRINTF("port %d connection change\n", i + 1);
			usb_hub_port_connect_change_qc_xhci(dev, i);
		}
		else if ((portstatus & 0x1) == 1) {		//ehci not have must check	
			USB_HUB_PRINTF("port %d connection change1\n", i + 1);
			usb_hub_port_connect_change_qc_xhci(dev, i); 		
		}

		if (portchange & USB_PORT_STAT_C_ENABLE) {
			USB_HUB_PRINTF("port %d enable change, status 0x%x\n", i + 1, portstatus);
			usb_clear_port_feature(dev, i + 1, USB_PORT_FEAT_C_ENABLE);

			/* EM interference sometimes causes bad shielded USB devices to
			 * be shutdown by the hub, this hack enables them again.
			 * Works at least with mouse driver */
			if (!(portstatus & USB_PORT_STAT_ENABLE) &&
				(portstatus & USB_PORT_STAT_CONNECTION) && (dev->children[i])) {
				USB_HUB_PRINTF("already running port %i disabled by hub (EMI?), re-enabling...\n",
					i + 1);
					usb_hub_port_connect_change_qc_xhci(dev, i);
			}
		}
		
		if ((rtd_inl(U3PORT_REG_ADDRESS) & 0x0000ffff) != 0x1203) {
			printk("usb3.0 port not recognized u3\n");
			return -1;
		}

} else {
			printk("%s(%d)USB_EHCI_HOST\n",__func__,__LINE__);
		if (portchange & USB_PORT_STAT_C_CONNECTION) {
			USB_HUB_PRINTF("port %d connection change\n", i + 1);
			usb_hub_port_connect_change_qc_ehci(dev, i);
		}

		if (portchange & USB_PORT_STAT_C_ENABLE) {
			USB_HUB_PRINTF("port %d enable change, status 0x%x\n", i + 1, portstatus);
			usb_clear_port_feature(dev, i + 1, USB_PORT_FEAT_C_ENABLE);

			/* EM interference sometimes causes bad shielded USB devices to
			 * be shutdown by the hub, this hack enables them again.
			 * Works at least with mouse driver */
			if (!(portstatus & USB_PORT_STAT_ENABLE) &&
				(portstatus & USB_PORT_STAT_CONNECTION) && (dev->children[i])) {
				USB_HUB_PRINTF("already running port %i disabled by hub (EMI?), re-enabling...\n",
					i + 1);
					usb_hub_port_connect_change_qc_ehci(dev, i);
			}
		}
		
}

		if (portstatus & USB_PORT_STAT_SUSPEND) {
			USB_HUB_PRINTF("port %d suspend change\n", i + 1);
			usb_clear_port_feature(dev, i + 1,  USB_PORT_FEAT_SUSPEND);
		}

		if (portchange & USB_PORT_STAT_C_OVERCURRENT) {
			USB_HUB_PRINTF("port %d over-current change\n", i + 1);
			usb_clear_port_feature(dev, i + 1, USB_PORT_FEAT_C_OVER_CURRENT);
			usb_hub_power_on(hub);
		}

		if (portchange & USB_PORT_STAT_C_RESET) {
			USB_HUB_PRINTF("port %d reset change\n", i + 1);
			usb_clear_port_feature(dev, i + 1, USB_PORT_FEAT_C_RESET);
		}

if (USB_Host_Type == USB_XHCI_HOST) {
	
		if (portchange & USB_PORT_STAT_C_LINK_STATE) {
			usb_clear_port_feature(dev, i + 1,USB_PORT_FEAT_C_PORT_LINK_STATE);
		}
		
		if ( ((portstatus & USB_PORT_STAT_LINK_STATE) == USB_SS_PORT_LS_SS_INACTIVE) || 
			((portstatus & USB_PORT_STAT_LINK_STATE) == USB_SS_PORT_LS_COMP_MOD) ) 
		{
				usb_set_port_feature(dev, i + 1, USB_PORT_FEAT_BH_PORT_RESET);  //warm reset
				
				printk("warm reset...\n");
				wait_ms(100);
				usb_hub_port_connect_change_qc_xhci(dev, i);
		}
}

	} /* end for i all ports */
	
	return 0;
}

int usb_hub_probe_qc(struct usb_device *dev, int ifnum)
{
	struct usb_interface_descriptor *iface;
	struct usb_endpoint_descriptor *ep;
	int ret;

	iface = &dev->config.if_desc[ifnum];
	/* Is it a hub? */
	if (iface->bInterfaceClass != USB_CLASS_HUB)
		return 0;
	/* Some hubs have a subclass of 1, which AFAICT according to the */
	/*  specs is not defined, but it works */
	if ((iface->bInterfaceSubClass != 0) &&
	    (iface->bInterfaceSubClass != 1))
		return 0;
	/* Multiple endpoints? What kind of mutant ninja-hub is this? */
	if (iface->bNumEndpoints != 1)
		return 0;

#if defined(CONFIG_BOOT_USB_XHCI)
	ep = &iface->ep_desc[0].ep_desc;
#else
	ep = &iface->ep_desc[0];
#endif
	/* Output endpoint? Curiousier and curiousier.. */
	if (!(ep->bEndpointAddress & USB_DIR_IN))
		return 0;
	/* If it's not an interrupt endpoint, we'd better punt! */
	if ((ep->bmAttributes & 3) != 3)
		return 0;
	/* We found a hub */
	USB_HUB_PRINTF("USB hub found\n");
	ret=usb_hub_configure(dev);
	return ret;
}
void usb_init_qc(void)
{
	int i;
        // struct usb_device *dev;
   #ifdef ENABLE_KSEG1ADDR
        pusb_dev = (struct usb_device *) KSEG1ADDR(&usb_dev[0]);
#else /* ENABLE_KSEG1ADDR */
        pusb_dev = (struct usb_device *) usb_dev;
#endif /* ENABLE_KSEG1ADDR */

    /* first make all devices unknown */
    for(i=0;i<USB_MAX_DEVICE;i++) {
            memset(&pusb_dev[i],0,sizeof(struct usb_device));
            pusb_dev[i].devnum=-1;
    }

}

/* ==========================================================================================================
 No.          USB Command   BmReq    bReq   wValueL        wValueH   wIndexL    wIndexH  wLengthL  wLengthH
   1   Vendor Memory Read      C0       5   regoffsetL  regoffsetH         0   Mcu_type   Lengh_L   Lengh_H
   2   Vendor Memory Write     40       5   regoffsetL  regoffsetH    Byteen   Mcu_type   Lengh_L   Lengh_H
========================================================================================================== */
#define USB_REQ_RTK_VENDORW  (0x05)
 
typedef struct _Vendor_command_args
{
	 unsigned char mcu_type;
	 unsigned short int ocp_address; 
	 unsigned short int length;
	 unsigned char byte_enable; // will be ignored in vendor-read
	 unsigned char *user_buffer;
}Vendor_command_args;
 



// Read operation
static long _vendor_read_operation(struct usb_device *dev, Vendor_command_args *info)
{
 //struct usb_skel *dev;
 // unsigned char *user_buffer = NULL;
 unsigned char *data_buffer = NULL;
 unsigned short int index; 
 long result = 0;
 
 //printk(KERN_DEBUG "[FT] Vendor read\n");
 //printk(KERN_DEBUG "[FT]  #mcuType = %X, Add = %04X, length = %u\n", info->mcu_type, info->ocp_address, info->length);
 //dev = filp->private_data;
 index = (info->mcu_type<<8)|0x00; // vendor read should ignore byteEn
 
 //printk(KERN_DEBUG "[FT]  #kmalloc data buffer\n");  
 data_buffer =  info->user_buffer; //((uint8_t *)info) + sizeof(Vendor_command_args);
 //data_buffer = kmalloc(info->length, GFP_KERNEL);
 //if(data_buffer == NULL)
 //{
 // return -ENOMEM;
// }
 
 //mutex_lock(&dev->io_mutex);
 //printk(KERN_DEBUG "[FT] #usb_control_msg\n");
 result = usb_control_msg_qc(dev, usb_rcvctrlpipe(dev, 0), USB_REQ_RTK_VENDORW, 0xC0, info->ocp_address, index, data_buffer, info->length, USB_CNTL_TIMEOUT * 3);
 //mutex_unlock(&dev->io_mutex);  

 
 
 //printk(KERN_DEBUG "[FT]  #copy_to_user\n"); 
 //if(copy_to_user(user_buffer, data_buffer, info->length))
 //{
  //kfree(data_buffer);
  //return -EFAULT;
 //}
 
//-- printk("[FT]  data : %02X %02X %02X %02X, end : %02X\n", *data_buffer, *(data_buffer+1), *(data_buffer+2), *(data_buffer+3),*(data_buffer+info->length-1));
 
 //printk("[FT]Read Result = %ld\n", result);
 //kfree(data_buffer);
 return result;
}



// write operation
static long _vendor_write_operation(struct usb_device *dev, Vendor_command_args *info)
{
 //struct usb_skel *dev;
 //uint8_t *user_buffer = NULL;
 unsigned char *data_buffer = NULL;
 unsigned short int  index; 
 long result = 0;
  
 //printk(KERN_DEBUG "[FT] Vendor write\n");
 //printk(KERN_DEBUG "[FT]  #mcuType = %X, Add = %04X, length = %u, byteEn = %02X\n", info->mcu_type, info->ocp_address, info->length, info->byte_enable);
 //dev = filp->private_data;
 index = (info->mcu_type<<8)|info->byte_enable;
 
 //printk(KERN_DEBUG "[FT]  #kmalloc data buffer\n");  
 data_buffer = info->user_buffer; //((uint8_t *)info) + sizeof(Vendor_command_args);
// data_buffer = kmalloc(info->length, GFP_KERNEL);
// if(data_buffer == NULL)
// {
//  return -ENOMEM;
// }
  
// printk(KERN_DEBUG "[FT]  #copy_from_user\n"); 
// if(copy_from_user(data_buffer, user_buffer, info->length))
// {
//  kfree(data_buffer);
//  return -EFAULT;
// }
  
// mutex_lock(&dev->io_mutex);
// printk(KERN_DEBUG "[FT] #usb_control_msg\n");
 result = usb_control_msg_qc(dev, usb_sndctrlpipe(dev, 0), USB_REQ_RTK_VENDORW, 0x40, info->ocp_address, index, data_buffer, info->length, USB_CNTL_TIMEOUT * 3);
// mutex_unlock(&dev->io_mutex);  
 
//printk("[FT]  data : %02X %02X %02X %02X, end : %02X\n", *data_buffer, *(data_buffer+1), *(data_buffer+2), *(data_buffer+3),*(data_buffer+info->length-1));
   
// printk("[FT] Result = %ld\n", result);
// kfree(data_buffer);
 return result;
}




 
long _read_reg(struct usb_device *dev, unsigned char mcu_type, unsigned int ocp_address, unsigned int length, void *buffer)
{
 	Vendor_command_args info;
 
//-- 	printk("_read_reg MCU %d, Add %04X\n", mcu_type, ocp_address);
 
 	 info.mcu_type = mcu_type;
	 info.ocp_address = ocp_address;
 	 info.length = length;
	 info.byte_enable = 0x00;
	 info.user_buffer = (unsigned char*)buffer;


	 return _vendor_read_operation(dev, &info);

}


long _write_reg(struct usb_device *dev, unsigned char mcu_type, unsigned int ocp_address, unsigned int length, void *buffer)
{
 	Vendor_command_args info;
 
 //--	printk("_write_reg MCU %d, Add %04X\n", mcu_type, ocp_address);
 
 	 info.mcu_type = mcu_type;
	 info.ocp_address = ocp_address;
 	 info.length = length;
	 info.byte_enable = 0xff;
	 info.user_buffer = (unsigned char*)buffer;

	 return _vendor_write_operation(dev, &info);

}




#if 0
// Read operation
static long _vendor_read_operation(struct file *filp, Vendor_command_args *info)
{
 struct usb_skel *dev;
 uint8_t *user_buffer = NULL;
 uint8_t *data_buffer = NULL;
 uint16_t index; 
 long result = 0;
 
 printk(KERN_DEBUG "[FT] Vendor read\n");
 printk(KERN_DEBUG "[FT]  #mcuType = %X, Add = %04X, length = %u\n", info->mcu_type, info->ocp_address, info->length);
 dev = filp->private_data;
 index = (info->mcu_type<<8)|0x00; // vendor read should ignore byteEn
 
 printk(KERN_DEBUG "[FT]  #kmalloc data buffer\n");  
 user_buffer =  info->user_buffer; //((uint8_t *)info) + sizeof(Vendor_command_args);
 data_buffer = kmalloc(info->length, GFP_KERNEL);
 if(data_buffer == NULL)
 {
  return -ENOMEM;
 }
 
 mutex_lock(&dev->io_mutex);
 printk(KERN_DEBUG "[FT] #usb_control_msg\n");
 result = usb_control_msg(dev->udev, usb_rcvctrlpipe(dev->udev, 0),
   USB_REQ_RTK_VENDORW, 0xC0,
   info->ocp_address, index, data_buffer, info->length, CONTROL_IO_TIMEOUT);
 mutex_unlock(&dev->io_mutex);  
 
 printk(KERN_DEBUG "[FT]  #copy_to_user\n"); 
 if(copy_to_user(user_buffer, data_buffer, info->length))
 {
  kfree(data_buffer);
  return -EFAULT;
 }
 
 printk(KERN_DEBUG "[FT]  data : %02X %02X %02X %02X, end : %02X\n", 
   *data_buffer, *(data_buffer+1), *(data_buffer+2), *(data_buffer+3),
   *(data_buffer+info->length-1));
 
 printk(KERN_DEBUG "[FT] Result = %ld\n", result);
 kfree(data_buffer);
 return result;
}

#endif

#if 0
// write operation
static long _vendor_write_operation(struct file *filp, Vendor_command_args *info)
{
 struct usb_skel *dev;
 uint8_t *user_buffer = NULL;
 uint8_t *data_buffer = NULL;
 uint16_t index; 
 long result = 0;
  
 printk(KERN_DEBUG "[FT] Vendor write\n");
 printk(KERN_DEBUG "[FT]  #mcuType = %X, Add = %04X, length = %u, byteEn = %02X\n", info->mcu_type, info->ocp_address, info->length, info->byte_enable);
 dev = filp->private_data;
 index = (info->mcu_type<<8)|info->byte_enable;
 
 printk(KERN_DEBUG "[FT]  #kmalloc data buffer\n");  
 user_buffer = info->user_buffer; //((uint8_t *)info) + sizeof(Vendor_command_args);
 data_buffer = kmalloc(info->length, GFP_KERNEL);
 if(data_buffer == NULL)
 {
  return -ENOMEM;
 }
  
 printk(KERN_DEBUG "[FT]  #copy_from_user\n"); 
 if(copy_from_user(data_buffer, user_buffer, info->length))
 {
  kfree(data_buffer);
  return -EFAULT;
 }
  
 mutex_lock(&dev->io_mutex);
 printk(KERN_DEBUG "[FT] #usb_control_msg\n");
 result = usb_control_msg(dev->udev, usb_sndctrlpipe(dev->udev, 0),
   USB_REQ_RTK_VENDORW, 0x40,
   info->ocp_address, index, data_buffer, info->length, CONTROL_IO_TIMEOUT);
 mutex_unlock(&dev->io_mutex);  
 
 printk(KERN_DEBUG "[FT]  data : %02X %02X %02X %02X, end : %02X\n", 
   *data_buffer, *(data_buffer+1), *(data_buffer+2), *(data_buffer+3),
   *(data_buffer+info->length-1));
   
 printk(KERN_DEBUG "[FT] Result = %ld\n", result);
 kfree(data_buffer);
 return result;
}
 
#endif








// #endif /* (CONFIG_COMMANDS & CFG_CMD_USB) */

/* EOF */
