#ifndef __EMCU_H__
#define __EMCU_H__

typedef enum {
	PP_HDMI1 = 0,
	PP_HDMI2,
	PP_HDMI3,
	PP_VGA1,
	PP_VGA2,
	PP_VGA3,
	PP_AV1,
	PP_AV2,
	PP_AV3,
	PP_SOURCE_MAX,
} PP_SOURCE_T;

typedef enum {
    WKSOR_UNDEF,
    WKSOR_KEYPAD,
    WKSOR_WUT,
    WKSOR_IRDA,
    WKSOR_CEC,
    WKSOR_PPS,
    WKSOR_WOW,
} WAKE_UP_T;


typedef struct {
    const char* name;
    const char* description;
    int (*debug_func)(int argc, char *argv[]);
}EMCU_FUNC;


extern int emcu_show_share_mem(int argc, char *argv[]);
extern int emcu_set_sharemem(int argc, char *argv[]);
extern int emcu_clr_share_mem(int argc, char *argv[]);
extern int emcu_clr_pm_task(int argc, char *argv[]);
extern int emcu_set_STM(int argc, char *argv[]);
extern int emcu_set_RTC(int argc, char *argv[]);

extern int emcu_set_wifi_dg(int argc, char *argv[]);
extern int emcu_set_WOL(int argc, char *argv[]);
extern int emcu_get_WOL(int argc, char *argv[]);
extern int emcu_set_WOWLAN(int argc, char *argv[]);

extern int emcu_setup_cec(int argc, char *argv[]);
extern int emcu_setup_mhl(int argc, char *argv[]);
extern int emcu_setup_uart(int argc, char *argv[]);
extern int emcu_setup_ppsource(int argc, char *argv[]);

extern int emcu_setup_irda(int argc, char *argv[]);
extern int emcu_setup_irda_in_code(int argc, char *argv[]);
extern int emcu_setup_power_en(int argc, char *argv[]);
extern int emcu_setup_power_en_in_name(int argc, char *argv[]);
extern int emcu_set_cur_time(int argc, char *argv[]);
extern int emcu_get_rec_time(int argc, char *argv[]);
extern int emcu_get_sleep_dur(int argc, char *argv[]);

extern int emcu_setup_keypad_multi(int argc, char *argv[]);
extern int emcu_setup_wakeup_timer(int argc, char *argv[]);
extern int emcu_setup_wakeup_timer1(int argc, char *argv[]);
extern int emcu_setup_wakeup_min(int argc, char *argv[]);
extern int emcu_setup_wakeup_sec(int argc, char *argv[]);

extern int emcu_get_wakeup_source(int argc, char *argv[]);
extern int emcu_get_wakeup_status(int argc, char *argv[]);


EMCU_FUNC emcu_debug_func_list[] =
{
    {"show_share_mem",	"display all share memory value.",
	emcu_show_share_mem},
    {"set_sharemem",	"fill data to certian share memory.",
    	emcu_set_sharemem},
    {"clr_share_mem",	"claer all share memory immediately.",
    	emcu_clr_share_mem},
    {"clr_pm_task",	"clear assigned operation, not clear all share memory.",
    	emcu_clr_pm_task},
    {"set_STM",		"enable suspend to RAM.",
    	emcu_set_STM},
    {"set_RTC",		"enable RTC wakeup function.",
    	emcu_set_RTC},
    {"set_WIFI_DG",	"enable WIFI dongle wake up func.",
    	emcu_set_wifi_dg},
    {"set_WOL",	"enable wake on LAN(WOL) func.",
    	emcu_set_WOL},
    {"get_WOL",	"get info about weke on LAN(WOL) func.",
    	emcu_get_WOL},
    {"set_WOWLAN",	"enable weke on wide LAN(WOWLAN) func.",
    	emcu_set_WOWLAN},
    {"setup_cec",	"enable and setting keypad parameter.",
    	emcu_setup_cec},
    {"setup_mhl",	"enable mhl func.",
    	emcu_setup_mhl},
    {"setup_uart",	"enable uart func.",
    	emcu_setup_uart},
    {"setup_ppsource",	"enable pervious power on source.",
    	emcu_setup_ppsource},
    {"setup_irda",	"set IRDA key to wakeup.",
    	emcu_setup_irda},
    {"setup_irda_in_code",	"set IRDA key to wakeup.The format of input is in user_code.",
        emcu_setup_irda_in_code},
    {"setup_power_en",	"set power pin information to EMCU.",
    	emcu_setup_power_en},
    {"setup_power_en_in_name",	"set power pin information to EMCU.The format is in name",
        emcu_setup_power_en_in_name},
    {"set_cur_time",	"set current time to emcu.",
    	emcu_set_cur_time},
    {"get_rec_time",	"get recorded time from emcu.",
    	emcu_get_rec_time},
    {"get_sleep_dur",	"get how long sleeping from emcu.",
    	emcu_get_sleep_dur},
    {"setup_keypad_multi",	"enable and setting keypad parameter.",
    	emcu_setup_keypad_multi},
    {"setup_wakeup_timer",	"set wake up timers, minimum is minute.",
    	emcu_setup_wakeup_timer},
    {"setup_wakeup_timer1",	"set wake up timers, minimum is sec.",
    	emcu_setup_wakeup_timer1},
    {"setup_wakeup_min",	"set wake up timers and reason, minimum is min.",
    	emcu_setup_wakeup_min},
    {"setup_wakeup_sec",	"set wake up timers and reason, minimum is sec.",
    	emcu_setup_wakeup_sec},
    {"get_wakeup_source",	"get wake up source from emcu.",
    	emcu_get_wakeup_source},
    {"get_wakeup_status",	"get wake up status.",
    	emcu_get_wakeup_status},
};

#define EMCU_FUNC_COUNT    (sizeof(emcu_debug_func_list)/sizeof(EMCU_FUNC))

#endif  //__EMCU_H__
