#include <linux/string.h>
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <linux/compiler.h>
#include <asm/io.h>
#include "alc1310.h"
#include <drivers/i2c/rtk_i2c_lib.h>

//*************************************************************
// PUBLIC FUNCTIONS
//*************************************************************

#define END_FLAG 	(0xff)
#define AMP_DEBUG 	(0)

typedef struct
{
    unsigned char bAddr;
    unsigned char bArray[2];
}ALC1310_REG;

static ALC1310_REG ALC1310_InitTbl[] = 
{
    //init
    {ALC_1310_MX05_PLL_SETTING,{0x40,0x3e}},
    {ALC_1310_MX02_SEQ_CTRL,{0x80,0x00}},
    {ALC_1310_MX09_DRC_SETTING_1,{0x80,0x00}},
    //DC
    {ALC_1310_PR6A_REG_INDEX,{0x00,0xdc}},
    {ALC_1310_PR6C_REG_DATA,{0x00,0x24}},
    //D9
    {0x14,{0x00,0xd9}},
    {0x50,{0xa4,0x10}},

    //DRC
    {ALC_1310_MX09_DRC_SETTING_1,{0xC8,0x00}},//DRC Enable
    {ALC_1310_MX0A_DRC_SETTING_2,{0x05,0x00}},
    {ALC_1310_MX0C_DRC_SETTING_4,{0x04,0x02}},
    {ALC_1310_MX0E_DRC_SETTING_5,{0x06,0x00}},

    /* down layer register and value */
    {ALC_1310_PR6A_REG_INDEX,{0x00,0x9d}},
    {ALC_1310_PR6C_REG_DATA,{0x46,0x00}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x9e}},
    {ALC_1310_PR6C_REG_DATA,{0x26,0x26}},

    //b0
    {ALC_1310_PR6A_REG_INDEX,{0x00,0xb0}},
    {ALC_1310_PR6C_REG_DATA,{0x88,0x03}},//Silence detection:level is -80dB

    {ALC_1310_MX13_VOLUME_SETTING_2,{0x00,0x0c}},//unmute
    {ALC_1310_MX13_VOLUME_SETTING_2,{0x00,0x0c}},//unmute

    //{0x12,{0xff,0xff}},					//L/R Volume
    //{0x12,{0x00,0x00}},					//L/R Volume

    //BQ1 BQ1Ty= 4 BQ1Fs=44100 BQ1Fc=100BQ1Ga=1 BQ1Fb=120
    {ALC_1310_PR6A_REG_INDEX,{0x00,0x00}}, 
    {ALC_1310_PR6C_REG_DATA,{0x02,0x00}},
                            
    {ALC_1310_PR6A_REG_INDEX,{0x00,0x01}},
    {ALC_1310_PR6C_REG_DATA,{0x87,0x8E}},
                            
    {ALC_1310_PR6A_REG_INDEX,{0x00,0x02}},
    {ALC_1310_PR6C_REG_DATA,{0x1C,0x09}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x03}},
    {ALC_1310_PR6C_REG_DATA,{0xD4,0xC5}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x04}},
    {ALC_1310_PR6C_REG_DATA,{0x01,0xF6}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x05}},
    {ALC_1310_PR6C_REG_DATA,{0x45,0x96}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x06}},
    {ALC_1310_PR6C_REG_DATA,{0x03,0xF7}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x07}},
    {ALC_1310_PR6C_REG_DATA,{0x37,0xBD}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x08}},
    {ALC_1310_PR6C_REG_DATA,{0x1E,0x08}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x09}},
    {ALC_1310_PR6C_REG_DATA,{0xAD,0xE1}},

    //BQ2 BQ2Ty= 4 BQ2Fs=44100 BQ2Fc=6000 BQ2Ga=0.5 BQ2Fb=1500
    {ALC_1310_PR6A_REG_INDEX,{0x00,0xa}}, 
    {ALC_1310_PR6C_REG_DATA,{0x02,0x02}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x0b}},
    {ALC_1310_PR6C_REG_DATA,{0x9D,0x55}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x0c}},
    {ALC_1310_PR6C_REG_DATA,{0x1D,0x9C}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x0d}},
    {ALC_1310_PR6C_REG_DATA,{0xF6,0x0A}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x0e}},
    {ALC_1310_PR6C_REG_DATA,{0x01,0xA2}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x0f}},
    {ALC_1310_PR6C_REG_DATA,{0xFE,0xEA}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x10}},
    {ALC_1310_PR6C_REG_DATA,{0x02,0x66}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x11}},
    {ALC_1310_PR6C_REG_DATA,{0x28,0xC3}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x12}},
    {ALC_1310_PR6C_REG_DATA,{0x1E,0x58}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x13}},
    {ALC_1310_PR6C_REG_DATA,{0x40,0x03}},

    //BQ3 BQ3Ty= 4 BQ3Fs=44100 BQ3Fc=11000 BQ3Ga=1.5 BQ3Fb=14000
    {ALC_1310_PR6A_REG_INDEX,{0x00,0x14}},
    {ALC_1310_PR6C_REG_DATA,{0x02,0x25}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x15}},
    {ALC_1310_PR6C_REG_DATA,{0x88,0x62}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x16}},
    {ALC_1310_PR6C_REG_DATA,{0x1F,0xFD}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x17}},
    {ALC_1310_PR6C_REG_DATA,{0xEC,0x29}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x18}},
    {ALC_1310_PR6C_REG_DATA,{0x00,0x47}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x19}},
    {ALC_1310_PR6C_REG_DATA,{0x09,0xFB}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x1a}},
    {ALC_1310_PR6C_REG_DATA,{0x00,0x02}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x1b}},
    {ALC_1310_PR6C_REG_DATA,{0x3A,0xD3}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x1c}},
    {ALC_1310_PR6C_REG_DATA,{0x1F,0x8E}},

    {ALC_1310_PR6A_REG_INDEX,{0x00,0x1d}},
    {ALC_1310_PR6C_REG_DATA,{0x38,0x7E}},

    {ALC_1310_MX04_BQ_SETUP,{0xc7,0x00}},


    {END_FLAG ,{0x00}},
};

void alc1310_amp_mute(int enable)
{
    IO_Set(PIN_AMP_RESET, enable);
}

void alc1310_func(int bus_id, unsigned short addr)
{
    unsigned char data[8];
    unsigned char regaddr;
    unsigned char index;
    for(index = 0; ; index ++)
    {
        regaddr = ALC1310_InitTbl[index].bAddr;
        data[0] = ALC1310_InitTbl[index].bAddr;
        if(regaddr == END_FLAG)
        {
            break;
        }
        data[1] = ALC1310_InitTbl[index].bArray[0];
        data[2] = ALC1310_InitTbl[index].bArray[1];
#if AMP_DEBUG
        printf("\nwrite addr:0x%x----data[1]=0x%x----data[2]=0x%x\n",regaddr,data[1],data[2]);
#endif
        if (I2C_Write_EX(bus_id, addr,  3, data ) < 0)
            printf("program ACL1310 failed\n");

#if AMP_DEBUG
        unsigned char readArray[2] = {0,0};
        if (I2C_Read_EX(bus_id, addr, 1,  &regaddr, 2, readArray)!= S_OK)
        {
            printf("\n i2c read fail \n"  );
        }
        printf("\nread addr:0x%x----data[0]=0x%x----data[1]=0x%x\n",regaddr,readArray[0],readArray[1]);
#endif

    }
    //After 10msec, Relase PD Pin
    mdelay(10);
    alc1310_amp_mute(0);
}

