#ifndef __EHCI_PLAT_RTD288O_CONFIG_H__
#define __USB2_PLAT_CONFIG_H__

#define U2_PHY_SETTING_REVISION     20200323

/////////////////////////////////////////////////////////////////
// U2 Phy Setting
/////////////////////////////////////////////////////////////////

typedef struct {
    unsigned char page;
    unsigned char addr;
} U2_PHY_REG;


typedef struct {
    int port1;
    unsigned char addr;
    unsigned char val;
} U2_PHY_REGISTER;

#define Z0_GEAR(val)   (3+((val&0xf)*4))
//[GEAR]->[Value]:
//0x0->0x03;0x1->0x07;0x2->0x0b;0x3->0x0f;0x4->0x13;0x5->0x17;0x6->0x1b;0x7->0x1f
//0x8->0x23;0x9->0x27;0xa->0x2b;0xb->0x2f;0xc->0x33;0xd->0x37;0xe->0x3b;0xf->0x3f

#if defined(CONFIG_TV030_PLATFORM)
//2.2R XHCI : 0xa , EHCI : 0x9
#define Z0_VAL_P1_EHCI  Z0_GEAR(0x6)
#define Z0_VAL_P2_EHCI  Z0_GEAR(0x6)
#define Z0_VAL_P1_EHCI_OLED  Z0_GEAR(0x6)
#define Z0_VAL_P2_EHCI_OLED  Z0_GEAR(0x6)
#elif defined(CONFIG_CUSTOMER_TV006)
// 4.7R XHCI : 0x9 , EHCI : 0x8
#define Z0_VAL_P1_EHCI  Z0_GEAR(0x5)
#define Z0_VAL_P2_EHCI  Z0_GEAR(0x5)
#define Z0_VAL_P1_EHCI_OLED  Z0_GEAR(0x5)
#define Z0_VAL_P2_EHCI_OLED  Z0_GEAR(0x5)
#else   //CONFIG_TV001_PLATFORM for default
//public 2.2R XHCI : 0xa , EHCI : 0x9
#define Z0_VAL_P1_EHCI  Z0_GEAR(0x5)
#define Z0_VAL_P2_EHCI  Z0_GEAR(0x5)
#define Z0_VAL_P1_EHCI_OLED  Z0_GEAR(0x5)
#define Z0_VAL_P2_EHCI_OLED  Z0_GEAR(0x5)
#endif

// First version: 2020/0321
U2_PHY_REGISTER u2_phy_setting[] = {
    /*
     * 0xF4: select page.
     *   0x9b: page 0
     *   0xbb: page 1
     *   0xdb: page 2
     */

    {1, 0xF4, 0xbb},
    {2, 0xF4, 0xbb},

    {1, 0xE5, 0x0f},
    {2, 0xE5, 0x0f},

    {1, 0xE6, 0x58},
    {2, 0xE6, 0x58},

    {1, 0xE7, 0xe3},
    {2, 0xE7, 0xe3},

    {1, 0xF4, 0x9b},  // to page0
    {2, 0xF4, 0x9b},

    {1, 0xE0, Z0_VAL_P1_EHCI},  //Z0
    {2, 0xE0, Z0_VAL_P2_EHCI},

    {1, 0XE1, 0x18},
    {2, 0XE1, 0x18},

    {1, 0xE2, 0x79},
    {2, 0xE2, 0x79},

    {1, 0xE3, 0xcd},
    {2, 0xE3, 0xcd},

    {1, 0xE5, 0x63},  // REG_DBS_SEL[1]=1: Double sensitivity 1.5X
    {2, 0xE5, 0x63},

    {1, 0xF4, 0xbb},  // to page1
    {2, 0xF4, 0xbb},

    {1, 0xE1, 0x77},
    {2, 0xE1, 0x77},

    {1, 0xF4, 0xdb},  // to page2
    {2, 0xF4, 0xdb},

    {1, 0xE7, 0x44},    //bit[7:4]=4,SENH OBJ, bit[3:0]=8,SEND OBJ
    {2, 0xE7, 0x44},

    {1, 0xF4, 0x9b},  // to page0
    {2, 0xF4, 0x9b},

    {1, 0xE4, 0x5d},  // (first) bit[7:4]=disconnect level, bit[3:0]=swing;
    {2, 0xE4, 0x5d},

    {1, 0xE7, 0x51},  // (first) bit[7:4]=deivce disconnect level (squelch)
    {2, 0xE7, 0x51},

    {1, 0xF4, 0xbb},  // to page1
    {2, 0xF4, 0xbb},

    {1, 0xE0, 0x22},    //bit[2]=0,Calibration sensitivity
    {2, 0xE0, 0x22},

    {1, 0xE0, 0x26},
    {2, 0xE0, 0x26},

    {1, 0xF4, 0x9b},  // to page0
    {2, 0xF4, 0x9b},

    {1, 0xE4, 0x8d},  // port0 (second) bit[7:4]=6(disconnect level), bit[3:0]=8(swing)
    {2, 0xE4, 0x8d},  // port1 (second) bit[7:4]=6(disconnect level), bit[3:0]=8(swing)

    {1, 0xE7, 0x71},  // (second) bit[7:4]=deivce disconnect level (squelch)
    {2, 0xE7, 0x71},

    {1, 0xE6, 0x01},
    {2, 0xE6, 0x01},

    {1, 0xF0, 0xfc},
    {2, 0xF0, 0xfc},

    {1, 0xF1, 0x8c},
    {2, 0xF1, 0x8c},

    {1, 0xF2, 0x00},
    {2, 0xF2, 0x00},

    {1, 0xF3, 0x11},
    {2, 0xF3, 0x11},

    {1, 0xF5, 0x95},    //[bit7]deglitch _en=1
    {2, 0xF5, 0x95},    //[bit7]deglitch _en=1

    {1, 0xF6, 0x00},
    {2, 0xF6, 0x00},

    {1, 0xF7, 0xba},    //NSQ_HST=0x3; enable REG_tolerance(bit8)
    {2, 0xF7, 0xba},

    {1, 0xF4, 0xbb},  // to page1
    {2, 0xF4, 0xbb},  // to page1

    {1, 0xE2, 0x00},
    {2, 0xE2, 0x00},

    {1, 0xE3, 0x03},
    {2, 0xE3, 0x03},

    {1, 0xE4, 0x48},
    {2, 0xE4, 0x48},

    {1, 0xF4, 0xdb},  // to page2
    {2, 0xF4, 0xdb},  // to page2

    {1, 0xE0, 0xff},
    {2, 0xE0, 0xff},

    {1, 0xE1, 0xff},
    {2, 0xE1, 0xff},

    {1, 0xE2, 0x00},
    {2, 0xE2, 0x00},

    {1, 0xE3, 0x00},
    {2, 0xE3, 0x00},

    {1, 0xE4, 0x15},
    {2, 0xE4, 0x15},

    {1, 0xE5, 0x81},
    {2, 0xE5, 0x81},

    {1, 0xE6, 0x53},
    {2, 0xE6, 0x53},

    {1, 0xF4, 0x9b},  // to page0
    {2, 0xF4, 0x9b},
};


typedef struct {
    U2_PHY_REGISTER* p_reg_table;
    unsigned int     n_reg;
} U2_PHY_CONFIG;


#endif // __USB2_PLAT_CONFIG_H__
